import 'package:fis_common/extensions/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_i18n/modules/error_codes.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/cupertino.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/consultation.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/managers/interfaces/ipatientController.dart';
import 'package:flyinsonolite/consultation/records/models/patientinfo.dart';
import 'package:flyinsonolite/consultation/records/models/selectmodel.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/routeconstants.dart';
import 'package:get/get.dart';
import 'package:universal_html/html.dart' as html;

class EmergencyPreparednessController extends GetxController
    implements IPatientController {
  final ILiveConsultationManager _liveConsultationManager =
      Get.find<ILiveConsultationManager>();
  final _appointmentManager = Get.find<IAppointmentManager>();

  ConsultationServerConfig? emergencyConfig;
  List<EmergencyDeviceInfoDTO> deviceInfoList = [];
  List<UserDTO> expertList = [];
  Rx<EmergencyDeviceInfoDTO?> selectedDevice = Rx(null);
  Rx<UserDTO?> selectedExpert = Rx(null);
  Map<String, TextEditingController> textEditingControllers = {};
  @override
  Map<String, String> patient = {'AgeUnits': "Year"};
  @override
  Rx<List<FISSelectModel>> sexList = Rx([]);
  @override
  Rx<FISSelectModel?> selectedSex = Rx(null);
  @override
  Rx<List<FISSelectModel>> ageUnitList = Rx([]);
  @override
  Rx<FISSelectModel?> selectedAgeUnit = Rx(null);
  @override
  PatientInfo currentPatient = PatientInfo(
    isValid: true,
    createTime: DateTime.now(),
    patientCode: '',
    ageUnits: 'Year',
  );
  @override
  RxBool isEdit = false.obs;
  EmergencyPreparednessController(
      this.deviceInfoList, this.expertList, this.emergencyConfig);

  @override
  void onReady() async {
    super.onReady();
    if (deviceInfoList.isNotEmpty) {
      selectedDevice.value = deviceInfoList.firstWhereOrNull(
          (element) => element.deviceCode == Storage.usDeviceCode);
    }

    if (expertList.isNotEmpty) {
      selectedExpert.value = expertList.first;
    }
  }

  @override
  TextEditingController getTextEditingController(
      String key, String defaultValue) {
    if (textEditingControllers.containsKey(key)) {
      var existTextEditController = textEditingControllers[key]!;
      existTextEditController.text = defaultValue;
      return existTextEditController;
    }

    var newTextEditingController = TextEditingController(text: defaultValue);
    textEditingControllers[key] = newTextEditingController;
    return newTextEditingController;
  }

  void updatePatientInfo(PatientInfo patientInfo) {
    currentPatient = patientInfo;
    patient['Name'] = patientInfo.name ?? '';
    var nameTextEditController = getTextEditingController("Name", "");
    nameTextEditController.text = patient['Name']!;
    patient['Phone'] = patientInfo.phone ?? '';
    var phoneTextEditController = getTextEditingController("Phone", "");
    phoneTextEditController.text = patient['Phone']!;
    patient['AgeUnits'] = patientInfo.ageUnits.isNullOrWhiteSpace
        ? 'Year'
        : patientInfo.ageUnits!;
    selectedAgeUnit.value = ageUnitList.value
        .firstWhereNullable((element) => element.key == patient["AgeUnits"]);
    var ageYearTextEditController = getTextEditingController("AgeYear", "");
    ageYearTextEditController.text = patientInfo.ageYear ?? "";
    var ageMonthTextEditController = getTextEditingController("AgeMonth", "");
    ageMonthTextEditController.text = patientInfo.ageMonth ?? "";
    var ageWeekTextEditController = getTextEditingController("AgeWeek", "");
    ageWeekTextEditController.text = patientInfo.ageWeek ?? "";
    var ageDayTextEditController = getTextEditingController("AgeDay", "");
    ageDayTextEditController.text = patientInfo.ageDay ?? "";
    switch (patientInfo.ageUnits) {
      case 'YearAndMonth':
        patient["AgeYear"] = patientInfo.ageYear ?? "";
        patient["AgeMonth"] = patientInfo.ageMonth ?? "";
        patient.removeWhere((key, value) => key == "AgeWeek");
        patient.removeWhere((key, value) => key == "AgeDay");
        break;
      case 'MonthAndWeek':
        patient["AgeMonth"] = patientInfo.ageMonth ?? "";
        patient["AgeWeek"] = patientInfo.ageWeek ?? "";
        patient.removeWhere((key, value) => key == "AgeYear");
        patient.removeWhere((key, value) => key == "AgeDay");
        break;
      case 'WeekAndDay':
        patient["AgeWeek"] = patientInfo.ageDay ?? "";
        patient["AgeDay"] = patientInfo.ageWeek ?? "";
        patient.removeWhere((key, value) => key == "AgeYear");
        patient.removeWhere((key, value) => key == "AgeMonth");
        break;
      default:
        patient["AgeYear"] = patientInfo.ageYear ?? "";
        patient.removeWhere((key, value) => key == "AgeMonth");
        patient.removeWhere((key, value) => key == "AgeWeek");
        patient.removeWhere((key, value) => key == "AgeDay");
        break;
    }
    patient['Sex'] = patientInfo.sex ?? '';
    selectedSex.value = sexList.value
        .firstWhereNullable((element) => element.key == patient["Sex"]);
    patient['Anamnesis'] = patientInfo.anamnesis ?? '';
    var anamnesisTextEditController = getTextEditingController("Anamnesis", "");
    anamnesisTextEditController.text = patient['Anamnesis']!;
    patient['IdentityCard'] = patientInfo.identityCard ?? '';
    var identityCardTextEditController =
        getTextEditingController("IdentityCard", "");
    identityCardTextEditController.text = patient['IdentityCard']!;
  }

  ///开启急诊
  Future<void> startEmergencyConsultationAsync() async {
    if (Storage.isConsultating.value) {
      PromptBox.toast(i18nBook.errorCodes.errorCode852.t);
      Get.close(1);
      return;
    }
    if (emergencyConfig != null) {
      if (!_isCanApplyConsultation()) {
        PromptBox.toast(
            i18nBook.realTimeConsultation.requiredItemsNotCompleted.t);
        return;
      }
    }

    var consultationId = await _liveConsultationManager.applyEmergencyTreatment(
        selectedExpert.value!.userCode!,
        selectedDevice.value!.deviceCode!,
        emergencyConfig != null ? currentPatient.patientCode : null,
        emergencyConfig != null
            ? _appointmentManager.getPatientDatas(patient)
            : null);
    if (consultationId.isNotEmpty) {
      if (Storage.usDeviceCode == selectedDevice.value!.deviceCode!) {
        try {
          Get.close(1);
          var result = await jsonRpcProxyForFISLib.consultation
              .startLiveConsultationFromFlutter(FISLiveConsultationRequestDTO(
                  consultationCode: consultationId,
                  token: Storage.user.token,
                  isEmergency: true,
                  isInvite: true,
                  isJoin: false));
          if (!result.isSuccess) {
            await loggerAsyn(FISDeviceLogCategory.Error,
                "jsonRpcProxyForFISLib startLiveConsultationFromFlutter Fail,FailMessage:${result.failMessage}，ErrorCode:${result.errorCode}");
            if (result.errorCode > 0) {
              var text = i18nBook
                  .getTrItem(ErrorCodesModule.ModuleName,
                      'errorCode${result.errorCode}')
                  ?.t;
              PromptBox.toast(text ?? "Unkonwn RPC Exception");
            } else {
              PromptBox.toast(
                  "jsonRpcProxyForFISLib startLiveConsultationFromFlutter Fail,FailMessage:${result.failMessage}");
            }
            return;
          } else {
            _liveConsultationManager.currentConsultationCode = consultationId;
            Storage.isConsultating.value = true;
            Storage.isUseFISForConsultation.value = true;
          }
        } catch (ex) {
          await handlerErrorAsync(ex,
              "EmergencyPreparednessController jsonRpcProxyForFISLib startLiveConsultationFromFlutter Fail$ex");
        }
      } else {
        Get.close(1);
        var result = await _liveConsultationManager
            .startEmergencyConsultationAsync(consultationId);
        if (result) {
          if (Storage.hasMultiScreens) {
            //final url = html.window.location.origin;
            //html.window.open(
            // '$url/#/liveconsultation?page=liveconsultation&consultationCode=${consultationDetail.value!.consultationCode}&isJoin=false&token=${Storage.user.token}&language=${FI18n.ins.currentBook.locale.languageCode}&theme=${Storage.currentTheme.themeName}',
            //  '');
            //Storage.liveConsultationInSecondView.value=true;
            //return;
            Storage.isConsultating.value = true;
            await sendFlutterLiveConsultationInfo(consultationId);
          } else {
            Get.toNamed(
                RouteConstants.consultationRouteConstants.liveconsultation);
          }
        }
      }
    } else {
      await handlerErrorAsync(FISDeviceLogCategory.Error,
          "applyEmergencyTreatment Fail,the consultaitonId is empty");
      PromptBox.toast(
          "Apply Emergency Treatment Fail,the consultaitonId is empty");
    }
  }

  /// 是否能够申请预约单
  bool _isCanApplyConsultation() {
    // 遍历病人信息中的必填项列表
    var patientKeyList = emergencyConfig!.patientKeyList;
    for (int i = 0; i < patientKeyList.length; i++) {
      // 如果当前项目是必填项
      if (patientKeyList[i].required) {
        if (patient[patientKeyList[i].key].isNullOrWhiteSpace) {
          // 如果必填字段为空，返回 false
          return false;
        }
      }
    }
    // 如果所有必填字段都已填写，返回 true
    return true;
  }

  Future<void> sendFlutterLiveConsultationInfo(String consultationId) async {
    try {
      final url = html.window.location.origin;
      var result = await jsonRpcProxyForFISLib.consultation
          .sendFlutterLiveConsultationInfo(FISFlutterLiveConsultationInfo(
              isStartLiveConsultation: true,
              token: Storage.user.token,
              consultationCode: consultationId,
              liveConsultationFlutterUrl:
                  '$url/#/liveconsultation?page=liveconsultation&consultationCode=$consultationId&isJoin=false&token=${Storage.user.token}&language=${FI18n.ins.currentBook.locale.languageCode}&theme=${Storage.currentTheme.themeName}'));
      if (!result.isSuccess) {
        await loggerAsyn(FISDeviceLogCategory.Error,
            "ConsultationRequestController sendFlutterLiveConsultationInfoAsync ex:${result.failMessage}");
        Get.toNamed(RouteConstants.consultationRouteConstants.liveconsultation);
      }
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationRequestController sendFlutterLiveConsultationInfo Error:$ex");
      Get.toNamed(RouteConstants.consultationRouteConstants.liveconsultation);
    }
  }
}
