import 'dart:convert';

import 'package:fis_common/extensions/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flyinsonolite/consultation/records/views/choosepatientdialog.dart';
import 'package:flyinsonolite/consultation/emergency/controllers/emergencypreparednesscontroller.dart';
import 'package:flyinsonolite/controls/custom/customcheckbox.dart';
import 'package:flyinsonolite/controls/custom/customdropdownbutton.dart';
import 'package:flyinsonolite/controls/select/fisdropdownbutton.dart';
import 'package:flyinsonolite/controls/select/fbuildagetype.dart';
import 'package:flyinsonolite/controls/dialog/fisdialog.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fislabelinputfield.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/fistextfield.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/index.dart';
import 'package:flyinsonolite/localization/localization.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/consultation/records/models/selectmodel.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/utils.dart';
import 'package:get/get.dart';

class EmergencyPreparednessDialog
    extends GetView<EmergencyPreparednessController> {
  QueryEmergencyDataResult data;
  ConsultationServerConfig? emergencyConfig;
  String get _consultationConstant => 'consultation';
  double get _labelWidth => 90.s;
  EdgeInsets get _labelPadding => EdgeInsets.only(top: 12.s);
  EdgeInsets get _labelFieldMargin => EdgeInsets.all(5.s);
  EmergencyPreparednessDialog(this.data, GetEmergencySettingResult setting,
      {super.key}) {
    try {
      if (setting.emergencyNeedFill &&
          setting.emergencySettingJson.isNotNullOrWhiteSpace) {
        var jsonObject = jsonDecode(setting.emergencySettingJson!);
        emergencyConfig = ConsultationServerConfig.fromJson(
            jsonObject['ConsultationInfo'], "1.0");
      }
    } catch (ex) {
      loggerAsyn(FISDeviceLogCategory.Error,
          "JsonDecode ${setting.emergencySettingJson} Error:${ex.toString()}");
    }
  }

  @override
  Widget build(BuildContext context) {
    var deviceInfo = data.deviceInfoList?.firstWhereOrNull(
        (element) => element.deviceCode == Storage.usDeviceCode);
    List<EmergencyDeviceInfoDTO> deviceInfoList =
        Storage.platform == Platform.Windows
            ? (deviceInfo == null ? [] : [deviceInfo])
            : (data.deviceInfoList ?? []);
    Get.lazyPut<EmergencyPreparednessController>(() =>
        EmergencyPreparednessController(
            deviceInfoList, data.expertList ?? [], emergencyConfig));
    // Get.lazyPut<EmergencyPreparednessController>(() =>
    //     EmergencyPreparednessController(
    //         data.deviceInfoList ?? [], data.expertList ?? [], emergencyConfig));
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      final double contentWidth = Storage.isPortraitMode.value
          ? (Get.width > 600.s ? 600.s : Get.width / 10 * 9)
          : 600.s;
      return FISDialog(
        i18nBook.realTimeConsultation.emergencyConfig.t,
        _createContentView(contentWidth),
        false,
        false,
      );
    });
  }

  Widget _createContentView(double contentWidth) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        SizedBox(
          height: (Get.height - 150.s) / 10 * 9,
          width: contentWidth,
          child: CustomScrollView(
            scrollDirection: Axis.vertical,
            slivers: [
              _buildPatientInfo(contentWidth),
              SliverToBoxAdapter(
                child: Container(
                  margin: EdgeInsets.symmetric(horizontal: 5.s),
                  child: FISText(
                    i18nBook.realTimeConsultation.selectExpert.t,
                    style: Storage.currentTheme.dialogStyle.messageTextStyle
                        .copyWith(
                            fontSize: Storage.currentTheme.dialogStyle
                                .messageTextStyle.fontSize!.s),
                  ),
                ),
              ),
              Obx(() => SliverGrid.builder(
                    gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                      crossAxisCount: Get.width > 600.s ? 2 : 1,
                      mainAxisSpacing: 10.s, // 垂直间距
                      crossAxisSpacing: 10.s, // 水平间距
                      childAspectRatio: Get.width > 600.s ? 2.5 : 4, // 宽高比
                    ),
                    itemCount: controller.expertList.length,
                    itemBuilder: (context, index) {
                      var itemInfo = controller.expertList[index];
                      return InkWell(
                          onTap: () {
                            controller.selectedExpert.value = itemInfo;
                          },
                          child: Container(
                            margin: EdgeInsets.all(0.s),
                            child: Card(
                              elevation: 5, // 阴影效果
                              shadowColor: Colors.black54, // 阴影颜色
                              shape: RoundedRectangleBorder(
                                borderRadius: BorderRadius.circular(8.s),
                              ),
                              child: Row(
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  SizedBox(
                                    width: 10.s,
                                  ),
                                  CircleAvatar(
                                    radius: 35.s,
                                    backgroundImage: buildUserAvatarProvider(
                                        itemInfo.headImageUrl),
                                  ),
                                  SizedBox(
                                    width: 10.s,
                                  ),
                                  Expanded(
                                      child: Column(
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: [
                                      SizedBox(
                                        height: 10.s,
                                      ),
                                      FISText(
                                        itemInfo.displayName ?? "",
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            color: Colors.black,
                                            fontSize: 16.s),
                                        softWrap: true,
                                      ),
                                      SizedBox(height: 8.s),
                                      FISText(
                                        itemInfo.rootOrganizationName ?? "",
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            color: Colors.black,
                                            fontSize: 12.s),
                                        softWrap: true,
                                      ),
                                      SizedBox(height: 8.s),
                                      FISText(
                                        itemInfo.fieldList == null
                                            ? ""
                                            : itemInfo.fieldList!.join("; "),
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            color: Colors.black,
                                            fontSize: 12.s),
                                        overflow: TextOverflow.ellipsis,
                                        softWrap: true,
                                      )
                                    ],
                                  )),
                                  Transform.scale(
                                    scale: 1.5,
                                    child: Obx(() => CustomCheckbox(
                                          value: controller.selectedExpert.value
                                                  ?.userCode ==
                                              itemInfo.userCode,
                                          onChanged: ((value) {
                                            if (value!) {
                                              controller.selectedExpert.value =
                                                  itemInfo;
                                            }
                                          }),
                                          shape: const CircleBorder(),
                                        )),
                                  ),
                                  SizedBox(
                                    width: 10.s,
                                  ),
                                ],
                              ),
                            ),
                          ));
                    },
                  )),
              SliverToBoxAdapter(
                child: Container(
                  margin: EdgeInsets.symmetric(horizontal: 5.s),
                  child: FISText(
                    i18nBook.realTimeConsultation.selectDevice.t,
                    style: Storage.currentTheme.dialogStyle.messageTextStyle
                        .copyWith(
                            fontSize: Storage.currentTheme.dialogStyle
                                .messageTextStyle.fontSize!.s),
                  ),
                ),
              ),
              Obx(() => SliverGrid.builder(
                    gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
                      crossAxisCount: Get.width > 600.s ? 2 : 1,
                      mainAxisSpacing: 10.s, // 垂直间距
                      crossAxisSpacing: 10.s, // 水平间距
                      childAspectRatio: Get.width > 600.s ? 2.5 : 4, // 宽高比
                    ),
                    itemCount: controller.deviceInfoList.length,
                    itemBuilder: (context, index) {
                      var itemInfo = controller.deviceInfoList[index];
                      return InkWell(
                          onTap: () {
                            controller.selectedDevice.value = itemInfo;
                          },
                          child: Container(
                            margin: EdgeInsets.all(0.s),
                            child: Card(
                              shape: RoundedRectangleBorder(
                                borderRadius: BorderRadius.circular(8.s),
                              ),
                              child: Row(
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  SizedBox(
                                    width: 10.s,
                                  ),
                                  Container(
                                    width: 80.s,
                                    height: 80.s,
                                    decoration: BoxDecoration(
                                      shape: BoxShape.rectangle,
                                      image: DecorationImage(
                                        image: buildUserAvatarProvider(
                                            itemInfo.headPicUrl,
                                            isUS: true),
                                        fit: BoxFit.cover,
                                      ),
                                    ),
                                  ),
                                  SizedBox(
                                    width: 10.s,
                                  ),
                                  Expanded(
                                      child: Column(
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    crossAxisAlignment:
                                        CrossAxisAlignment.start,
                                    children: [
                                      SizedBox(
                                        height: 10.s,
                                      ),
                                      FISText(
                                        itemInfo.displayName ?? "",
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            color: Colors.black,
                                            fontSize: 16.s),
                                      ),
                                      SizedBox(height: 8.s),
                                      FISText(
                                        itemInfo.shortCode ?? "",
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            color: Colors.black,
                                            fontSize: 12.s),
                                      ),
                                      SizedBox(height: 8.s),
                                      FISText(
                                        itemInfo.description ?? "",
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            color: Colors.black,
                                            fontSize: 12.s),
                                        overflow: TextOverflow.ellipsis,
                                      )
                                    ],
                                  )),
                                  Transform.scale(
                                    scale: 1.5,
                                    child: Obx(() => CustomCheckbox(
                                          value: controller.selectedDevice.value
                                                  ?.deviceCode ==
                                              itemInfo.deviceCode,
                                          onChanged: ((value) {
                                            if (value!) {
                                              controller.selectedDevice.value =
                                                  itemInfo;
                                            }
                                          }),
                                          shape: const CircleBorder(),
                                        )),
                                  ),
                                  SizedBox(
                                    width: 10.s,
                                  ),
                                ],
                              ),
                            ),
                          ));
                    },
                  )),
            ],
          ),
        ),
        SizedBox(
          height: 10.s,
        ),
        FISLinearGradientElevatedButtonAsync(() async {
          await controller.startEmergencyConsultationAsync();
        }, i18nBook.realTimeConsultation.startEmergency.t,
            Storage.currentTheme.dialogStyle.buttonGradient,
            textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s)),
        SizedBox(
          height: 10.s,
        ),
      ],
    );
  }

  Widget _buildPatientInfo(double contentWidth) {
    return emergencyConfig != null
        ? SliverToBoxAdapter(
            child: Column(
            children: [
              Container(
                margin: EdgeInsets.symmetric(horizontal: 5.s),
                alignment: Alignment.centerLeft,
                child: FISText(
                  i18nBook.remedical.patientDetail.t,
                  style: Storage.currentTheme.dialogStyle.messageTextStyle
                      .copyWith(
                          fontSize: Storage.currentTheme.dialogStyle
                              .messageTextStyle.fontSize!.s),
                ),
              ),
              ...emergencyConfig!.patientKeyList.map((e) {
                switch (e.showType) {
                  case "input":
                    return FISLabelInputField(
                      e.required,
                      getLanguageValues(_consultationConstant, e.key),
                      FISTextField(
                        uniqueKey: GlobalKey(),
                        enabled: true,
                        borderColor: Storage.currentTheme
                            .fisLabelInputFieldStyle.inputTextBorderColor,
                        enabledBorderColor: Storage.currentTheme
                            .fisLabelInputFieldStyle.inputTextBorderColor,
                        textStyle: Storage
                            .currentTheme.fisLabelInputFieldStyle.inputTextStyle
                            .copyWith(
                                fontSize: Storage
                                    .currentTheme
                                    .fisLabelInputFieldStyle
                                    .inputTextStyle
                                    .fontSize!
                                    .s),
                        controller: controller.getTextEditingController(
                            e.key, controller.patient[e.key] ?? ""),
                        onChanged: (value) {
                          controller.patient[e.key] = value.toString().trim();
                        },
                        fillColor: Storage
                            .currentTheme.fisLabelInputFieldStyle.fillColor,
                        inputFormatters: [
                          _getInputFormatter(e.key),
                          LengthLimitingTextInputFormatter(20),
                        ],
                      ),
                      contentWidth,
                      padding: _labelFieldMargin,
                      labelPadding: _labelPadding,
                      labelWidth: _labelWidth,
                    );

                  case "AgeType":
                    _handleAgeUnitSource(e.source ?? []);
                    return FISLabelInputField(
                      e.required,
                      getLanguageValues(_consultationConstant, e.key),
                      const FBuildAgeType(true),
                      contentWidth,
                      padding: _labelFieldMargin,
                      labelPadding: _labelPadding,
                      labelWidth: _labelWidth,
                    );

                  case "select":
                    _handleSexSource(e.source ?? []);
                    return FISLabelInputField(
                      e.required,
                      getLanguageValues(_consultationConstant, e.key),
                      Obx(() => Row(children: [
                            Expanded(
                                child: FISDropdownButton(
                              value: controller.selectedSex.value,
                              items: controller.sexList.value
                                  .map<CustomDropdownMenuItem<FISSelectModel>>(
                                      (FISSelectModel selectModel) {
                                return CustomDropdownMenuItem<FISSelectModel>(
                                  value: selectModel,
                                  child: FISText(selectModel.value),
                                );
                              }).toList(),
                              onChanged: (value) {
                                controller.selectedSex.value = value;
                                controller.patient[e.key] =
                                    (value as FISSelectModel).key;
                              },
                            )),
                            controller.selectedSex.value == null
                                ? const SizedBox()
                                : FISIconButton(
                                    icon: const Icon(Icons.close),
                                    onPressed: () {
                                      controller.selectedSex.value = null;
                                      controller.patient[e.key] = '';
                                    }),
                          ])),
                      contentWidth,
                      padding: _labelFieldMargin,
                      labelPadding: _labelPadding,
                      labelWidth: _labelWidth,
                    );
                  case "inputButton":
                    return FISLabelInputField(
                      e.required,
                      getLanguageValues(_consultationConstant, e.key),
                      FISTextField(
                        uniqueKey: null,
                        enabled: true,
                        readOnly: true, //病人姓名不支持输入
                        borderColor: Storage.currentTheme
                            .fisLabelInputFieldStyle.inputTextBorderColor,
                        enabledBorderColor: Storage.currentTheme
                            .fisLabelInputFieldStyle.inputTextBorderColor,
                        textStyle: Storage
                            .currentTheme.fisLabelInputFieldStyle.inputTextStyle
                            .copyWith(
                                fontSize: Storage
                                    .currentTheme
                                    .fisLabelInputFieldStyle
                                    .inputTextStyle
                                    .fontSize!
                                    .s),
                        controller: controller.getTextEditingController(
                            e.key, controller.patient[e.key] ?? ""),
                        onTap: () {
                          Get.dialog(const ChoosePatientDialog(true),
                              barrierDismissible: false);
                        },
                        fillColor: Storage
                            .currentTheme.fisLabelInputFieldStyle.fillColor,
                      ),
                      contentWidth,
                      padding: _labelFieldMargin,
                      labelPadding: _labelPadding,
                      labelWidth: _labelWidth,
                    );
                  case 'textarea':
                    return FISLabelInputField(
                      e.required,
                      getLanguageValues(_consultationConstant, e.key),
                      FISTextField(
                        uniqueKey: GlobalKey(),
                        enabled: true,
                        borderColor: Storage.currentTheme
                            .fisLabelInputFieldStyle.inputTextBorderColor,
                        enabledBorderColor: Storage.currentTheme
                            .fisLabelInputFieldStyle.inputTextBorderColor,
                        textStyle: Storage
                            .currentTheme.fisLabelInputFieldStyle.inputTextStyle
                            .copyWith(
                                fontSize: Storage
                                    .currentTheme
                                    .fisLabelInputFieldStyle
                                    .inputTextStyle
                                    .fontSize!
                                    .s),
                        controller: controller.getTextEditingController(
                            e.key, controller.patient[e.key] ?? ""),
                        onChanged: (value) {
                          controller.patient[e.key] = value.toString().trim();
                        },
                        maxLines: 6,
                        contentPadding:
                            EdgeInsets.fromLTRB(12.s, 10.s, 12.s, 10.s),
                        fillColor: Storage
                            .currentTheme.fisLabelInputFieldStyle.fillColor,
                      ),
                      contentWidth,
                      padding: _labelFieldMargin,
                      labelWidth: _labelWidth,
                      labelPadding: _labelPadding,
                      height: 150.s,
                    );
                  default:
                    return const SizedBox();
                }
              }).toList(),
              SizedBox(
                height: 10.s,
              ),
            ],
          ))
        : const SliverToBoxAdapter(
            child: SizedBox(),
          );
  }

  void _handleSexSource(List source) {
    List<FISSelectModel> selectList = [];
    for (int i = 0; i < source.length; i++) {
      selectList.add(
        FISSelectModel(
          source[i],
          getLanguageValues(
            "exam",
            source[i],
          ),
        ),
      );
    }
    controller.selectedSex.value = selectList.firstWhereNullable(
        (element) => element.key == controller.patient["Sex"]);
    controller.sexList.value = selectList;
  }

  void _handleAgeUnitSource(List source) {
    List<FISSelectModel> selectList = [];
    for (int i = 0; i < source.length; i++) {
      selectList.add(
        FISSelectModel(
          source[i],
          getLanguageValues(
            "exam",
            source[i],
          ),
        ),
      );
    }
    controller.ageUnitList.value = selectList;
    controller.selectedAgeUnit.value = selectList.firstWhereNullable(
        (element) => element.key == controller.patient["AgeUnits"]);
  }

  TextInputFormatter _getInputFormatter(String key) {
    if (key.toLowerCase() == "phone") {
      return FilteringTextInputFormatter.digitsOnly;
    } else if (key.toLowerCase() == "identitycard") {
      return FilteringTextInputFormatter.allow(RegExp(r'[A-Za-z0-9]'));
    } else {
      return FilteringTextInputFormatter.allow(RegExp(r'[A-Za-z0-9]'));
    }
  }
}
