import 'package:fis_common/extensions/string.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationlistcontroller.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/consultation/records/models/selectmodel.dart';
import 'package:get/get.dart';

class FilterDrawerController extends GetxController {
  final _appointmentManager = Get.find<IAppointmentManager>();
  final consultationListController = Get.find<ConsultationListController>();
  Rx<QueryEvaluateGradeEnum?> selectedEvaluateGrade = Rx(null);
  Rx<QueryConsultationStatusEnum?> selectedConsultationStatus = Rx(null);
  Rx<ConsultationQueryTypeEnum?> selectedConsultationQueryType = Rx(null);
  List<FISSelectModel> consultationInstitutionList = [];
  Rx<FISSelectModel?> selectedInstitution = Rx(null);
  Rx<List<FISSelectModel>> expertList = Rx([]);
  Rx<FISSelectModel?> selectedExpert = Rx(null);
  TextEditingController keywordTextEditController = TextEditingController();
  TextEditingController diseasesNameTextEditingController =
      TextEditingController();
  TextEditingController primaryDiagnosisTextEditingController =
      TextEditingController();
  Future initializeAsync() async {
    await initializeParentOrganizationsAsync();
    selectedConsultationStatus.value ??= QueryConsultationStatusEnum.All;
    selectedEvaluateGrade.value ??= QueryEvaluateGradeEnum.All;
    selectedConsultationQueryType.value ??= ConsultationQueryTypeEnum.All;
    if (expertList.value.isEmpty) {
      expertList.value.add(FISSelectModel('', i18nBook.common.all.t));
      selectedExpert.value = expertList.value.first;
    }
  }

  void reset() {
    selectedConsultationStatus.value = QueryConsultationStatusEnum.All;
    selectedEvaluateGrade.value = QueryEvaluateGradeEnum.All;
    selectedConsultationQueryType.value = ConsultationQueryTypeEnum.All;
    if (consultationInstitutionList.isNotEmpty) {
      selectedInstitution.value = consultationInstitutionList.first;
    }
    if (expertList.value.isNotEmpty) {
      selectedExpert.value = expertList.value.first;
    }
    keywordTextEditController.text = '';
    diseasesNameTextEditingController.text = '';
    primaryDiagnosisTextEditingController.text = '';
  }

  /// 获取会诊机构
  Future<void> initializeParentOrganizationsAsync() async {
    EasyLoading.show();
    try {
      consultationInstitutionList.clear();
      consultationInstitutionList
          .add(FISSelectModel('', i18nBook.common.all.t));
      final result =
          await consultationListController.findParentOrganizationsAsync();

      for (int i = 0; i < result.length; i++) {
        consultationInstitutionList.add(
          FISSelectModel(
            result[i].organizationCode ?? '',
            result[i].organizationName ?? '',
          ),
        );
      }
      if (selectedInstitution.value == null) {
        selectedInstitution.value = consultationInstitutionList.first;
      } else {
        selectedInstitution.value =
            consultationInstitutionList.firstWhereOrNull(
                (element) => element.key == selectedInstitution.value?.key);
      }
    } catch (e) {
      printError(
          info:
              "FilterDrawerController initializeParentOrganizationsAsync exception:$e");
      await handlerErrorAsync(e,
          "FilterDrawerController initializeParentOrganizationsAsync exception: $e");
    } finally {
      EasyLoading.dismiss();
    }
  }

  /// 获取会诊专家
  Future<void> findOrganizationExpertsAsync(String? organizationCode) async {
    try {
      List<FISSelectModel> newExpertList = [];
      newExpertList.add(FISSelectModel('', i18nBook.common.all.t));
      if (!organizationCode.isNullOrWhiteSpace &&
          selectedInstitution.value != null) {
        final result = await _appointmentManager
            .findOrganizationExpertsAsync(organizationCode!);
        var pageData = result.pageData ?? [];
        for (int i = 0; i < pageData.length; i++) {
          newExpertList.add(
            FISSelectModel(
              pageData[i].userCode ?? '',
              pageData[i].displayName!,
            ),
          );
        }
      }
      selectedExpert.value = null;
      expertList.value = newExpertList;
      selectedExpert.value = expertList.value.first;
    } catch (e) {
      printError(
          info:
              "FilterDrawerController findOrganizationExpertsAsync exception:$e");
      await handlerErrorAsync(e,
          "FilterDrawerController findOrganizationExpertsAsync exception: $e");
    }
  }

  Future searchAsync() async {
    final consultationFilter = Get.find<ConsultationFilter>();
    consultationFilter.consultationStatus = selectedConsultationStatus.value;
    consultationFilter.evaluateGrade = selectedEvaluateGrade.value;
    consultationFilter.consultationQueryType =
        selectedConsultationQueryType.value != null
            ? selectedConsultationQueryType.value!
            : ConsultationQueryTypeEnum.All;
    consultationFilter.expertCodes.clear();
    if (selectedExpert.value != null) {
      consultationFilter.expertCodes = [selectedExpert.value!.key];
    }

    consultationFilter.expertOrganizationCodes.clear();
    if (selectedInstitution.value != null) {
      consultationFilter.expertOrganizationCodes = [
        selectedInstitution.value!.key
      ];
    }
    consultationFilter.keyword = keywordTextEditController.text.trim();
    consultationFilter.diseaseName =
        diseasesNameTextEditingController.text.trim();
    consultationFilter.primaryDiagnosis =
        primaryDiagnosisTextEditingController.text.trim();

    await consultationListController.refreshAsync();
  }
}
