import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/filters/controllers/filterdrawercontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationlistcontroller.dart';
import 'package:flyinsonolite/controls/custom/customdropdownbutton.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/controls/select/fisdropdownbutton.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fislabelinputfield.dart';
import 'package:flyinsonolite/controls/text/datetimetextfield.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/fistextfield.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/index.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/consultation/records/models/selectmodel.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class FilterDrawer extends GetView<FilterDrawerController> {
  final VoidCallback? closeDrawer;
  EdgeInsets get _labelFieldMargin => EdgeInsets.fromLTRB(0, 8.s, 0, 8.s);
  double get _labelWidth => i18nBook.isCurrentChinese ? 90.s : 140.s;
  EdgeInsets get _labelPadding => EdgeInsets.only(top: 12.s);
  const FilterDrawer({
    super.key,
    this.closeDrawer,
  });

  @override
  Widget build(BuildContext context) {
    final double contentWidth = Storage.isPortraitMode.value
        ? (Get.width > 500.s ? 500.s : Get.width / 5 * 4)
        : 500.s;
    return Drawer(
        backgroundColor: Storage.currentTheme.listViewStyle.itembgColor,
        width: contentWidth,
        child: Container(
            padding: EdgeInsets.fromLTRB(20.s, 10.s, 20.s, 10.s),
            child: Column(
              children: [
                _buildCloseDrawer(),
                Expanded(
                    child: SingleChildScrollView(
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: [
                      ///关键字
                      FISLabelInputField(
                        false,
                        fisI18nBook.common.keyword.t,
                        FISTextField(
                          uniqueKey: GlobalKey(),
                          textStyle: Storage.currentTheme.normalTextStyle
                              .copyWith(
                                  color: Colors.black,
                                  fontSize: Storage.currentTheme.normalTextStyle
                                      .fontSize!.s),
                          controller: controller.keywordTextEditController,
                          onSubmitted: onSubmittedAsync,
                        ),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelWidth: _labelWidth,
                        labelPadding: _labelPadding,
                      ),

                      ///开始时间
                      FISLabelInputField(
                        false,
                        i18nBook.realTimeConsultation.startTime.t,
                        Obx(() => DateTimeTextField(
                              i18nBook.realTimeConsultation.startTime.t,
                              controller.consultationListController
                                  .consultationFilter.startDate.value,
                              dateChanged: (value) {
                                controller.consultationListController
                                    .consultationFilter.startDate.value = value;
                              },
                            )),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelPadding: _labelPadding,
                        labelWidth: _labelWidth,
                      ),

                      ///结束时间
                      FISLabelInputField(
                        false,
                        i18nBook.realTimeConsultation.endTime.t,
                        Obx(() => DateTimeTextField(
                              i18nBook.realTimeConsultation.endTime.t,
                              controller.consultationListController
                                  .consultationFilter.endDate.value,
                              dateChanged: (value) {
                                controller.consultationListController
                                    .consultationFilter.endDate.value = value;
                              },
                            )),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelPadding: _labelPadding,
                        labelWidth: _labelWidth,
                      ),

                      ///会诊评价
                      FISLabelInputField(
                        false,
                        i18nBook.realTimeConsultation.consultationEvaluation.t,
                        Obx(() => FISDropdownButton(
                              value: controller.selectedEvaluateGrade.value,
                              items: QueryEvaluateGradeEnum.values.map<
                                      CustomDropdownMenuItem<
                                          QueryEvaluateGradeEnum>>(
                                  (QueryEvaluateGradeEnum typeEnum) {
                                return CustomDropdownMenuItem<
                                    QueryEvaluateGradeEnum>(
                                  value: typeEnum,
                                  child: FISText(
                                      _getQueryEvaluateGradeEnumTranslate(
                                          typeEnum)),
                                );
                              }).toList(),
                              onChanged: (value) {
                                controller.selectedEvaluateGrade.value =
                                    value as QueryEvaluateGradeEnum;
                              },
                            )),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelPadding: _labelPadding,
                        labelWidth: _labelWidth,
                      ),

                      ///会诊状态
                      FISLabelInputField(
                        false,
                        i18nBook.realTimeConsultation.consultationStatus.t,
                        Obx(() => FISDropdownButton(
                              value:
                                  controller.selectedConsultationStatus.value,
                              items: QueryConsultationStatusEnum.values.map<
                                      CustomDropdownMenuItem<
                                          QueryConsultationStatusEnum>>(
                                  (QueryConsultationStatusEnum typeEnum) {
                                return CustomDropdownMenuItem<
                                    QueryConsultationStatusEnum>(
                                  value: typeEnum,
                                  child: FISText(
                                      _getQueryConsultationStatusEnumTranslate(
                                          typeEnum)),
                                );
                              }).toList(),
                              onChanged: (value) {
                                controller.selectedConsultationStatus.value =
                                    value as QueryConsultationStatusEnum;
                              },
                            )),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelPadding:
                            i18nBook.isCurrentChinese ? _labelPadding : null,
                        labelWidth: _labelWidth,
                      ),

                      ///申请状态
                      FISLabelInputField(
                        false,
                        i18nBook
                            .realTimeConsultation.applicationClassification.t,
                        Obx(() => FISDropdownButton(
                              value: controller
                                  .selectedConsultationQueryType.value,
                              items: ConsultationQueryTypeEnum.values.map<
                                      CustomDropdownMenuItem<
                                          ConsultationQueryTypeEnum>>(
                                  (ConsultationQueryTypeEnum typeEnum) {
                                return CustomDropdownMenuItem<
                                    ConsultationQueryTypeEnum>(
                                  value: typeEnum,
                                  child: FISText(
                                      _getConsultationQueryTypeEnumTranslate(
                                          typeEnum)),
                                );
                              }).toList(),
                              onChanged: (value) {
                                controller.selectedConsultationQueryType.value =
                                    value;
                              },
                            )),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelPadding:
                            i18nBook.isCurrentChinese ? _labelPadding : null,
                        labelWidth: _labelWidth,
                      ),

                      ///会诊医院
                      FISLabelInputField(
                        false,
                        i18nBook.realTimeConsultation.consultationInstitution.t,
                        Obx(() => FISDropdownButton(
                              value: controller.selectedInstitution.value,
                              items: controller.consultationInstitutionList
                                  .map<CustomDropdownMenuItem<FISSelectModel>>(
                                      (FISSelectModel selectModel) {
                                return CustomDropdownMenuItem<FISSelectModel>(
                                    value: selectModel,
                                    child: FISText(selectModel.value));
                              }).toList(),
                              onChanged: (value) async {
                                var newSelectedItem = value as FISSelectModel;
                                if (controller.selectedInstitution.value?.key ==
                                    newSelectedItem.key) {
                                  return;
                                }
                                controller.selectedInstitution.value =
                                    newSelectedItem;
                                await controller.findOrganizationExpertsAsync(
                                    controller.selectedInstitution.value?.key);
                              },
                            )),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelPadding:
                            i18nBook.isCurrentChinese ? _labelPadding : null,
                        labelWidth: _labelWidth,
                      ),

                      ///会诊专家
                      FISLabelInputField(
                        false,
                        i18nBook.realTimeConsultation.competentExperts.t,
                        Obx(() => FISDropdownButton(
                              value: controller.selectedExpert.value,
                              items: controller.expertList.value
                                  .map<CustomDropdownMenuItem<FISSelectModel>>(
                                      (FISSelectModel selectModel) {
                                return CustomDropdownMenuItem<FISSelectModel>(
                                    value: selectModel,
                                    child: FISText(selectModel.value));
                              }).toList(),
                              onChanged: (value) {
                                controller.selectedExpert.value =
                                    value as FISSelectModel;
                              },
                            )),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelPadding:
                            i18nBook.isCurrentChinese ? _labelPadding : null,
                        labelWidth: _labelWidth,
                      ),

                      ///疾病名称
                      FISLabelInputField(
                        false,
                        i18nBook.realTimeConsultation.nameOfDisease.t,
                        FISTextField(
                          uniqueKey: GlobalKey(),
                          textStyle: Storage.currentTheme.normalTextStyle
                              .copyWith(
                                  color: Colors.black,
                                  fontSize: Storage.currentTheme.normalTextStyle
                                      .fontSize!.s),
                          controller:
                              controller.diseasesNameTextEditingController,
                          onSubmitted: onSubmittedAsync,
                        ),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelPadding: _labelPadding,
                        labelWidth: _labelWidth,
                      ),

                      ///初步诊断
                      FISLabelInputField(
                        false,
                        i18nBook.realTimeConsultation.preliminaryDiagnosis.t,
                        FISTextField(
                          uniqueKey: GlobalKey(),
                          textStyle: Storage.currentTheme.normalTextStyle
                              .copyWith(
                                  color: Colors.black,
                                  fontSize: Storage.currentTheme.normalTextStyle
                                      .fontSize!.s),
                          controller:
                              controller.primaryDiagnosisTextEditingController,
                          onSubmitted: onSubmittedAsync,
                        ),
                        contentWidth,
                        padding: _labelFieldMargin,
                        labelPadding:
                            i18nBook.isCurrentChinese ? _labelPadding : null,
                        labelWidth: _labelWidth,
                      ),
                    ],
                  ),
                )),
                SizedBox(
                  height: 20.s,
                ),
                Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: [
                    FISLinearGradientElevatedButtonAsync(() async {
                      await onSubmittedAsync('');
                    }, i18nBook.common.search.t,
                        Storage.currentTheme.dialogStyle.buttonGradient,
                        textStyle: Storage.currentTheme.normalTextStyle
                            .copyWith(
                                fontSize: Storage
                                    .currentTheme.normalTextStyle.fontSize!.s)),
                    SizedBox(
                      width: 10.s,
                    ),
                    FISLinearGradientElevatedButtonAsync(() async {
                      var listController =
                          Get.find<ConsultationListController>();
                      await listController.resetSearchFilterAsync();
                      closeDrawer!();
                    }, fisI18nBook.common.reset.t,
                        Storage.currentTheme.dialogStyle.buttonGradient,
                        textStyle: Storage.currentTheme.normalTextStyle
                            .copyWith(
                                fontSize: Storage
                                    .currentTheme.normalTextStyle.fontSize!.s)),
                  ],
                ),
                SizedBox(
                  height: 20.s,
                ),
              ],
            )));
  }

  String _getQueryEvaluateGradeEnumTranslate(QueryEvaluateGradeEnum typeEnum) {
    switch (typeEnum) {
      case QueryEvaluateGradeEnum.All:
        return i18nBook.common.all.t;
      case QueryEvaluateGradeEnum.UnSet:
        return i18nBook.realTimeConsultation.notSet.t;
      case QueryEvaluateGradeEnum.Dissatisfaction:
        return i18nBook.realTimeConsultation.unsatisfactory.t;
      case QueryEvaluateGradeEnum.General:
        return i18nBook.realTimeConsultation.general.t;
      case QueryEvaluateGradeEnum.Satisfaction:
        return i18nBook.realTimeConsultation.satisfactory.t;
      default:
        return "";
    }
  }

  String _getQueryConsultationStatusEnumTranslate(
      QueryConsultationStatusEnum statusEnum) {
    switch (statusEnum) {
      case QueryConsultationStatusEnum.All:
        return i18nBook.common.all.t;
      case QueryConsultationStatusEnum.Applied:
        return i18nBook.user.underVerification.t;
      case QueryConsultationStatusEnum.Withdrawn:
        return i18nBook.realTimeConsultation.withdrawn.t;
      case QueryConsultationStatusEnum.Rejected:
        return i18nBook.realTimeConsultation.rejected.t;
      case QueryConsultationStatusEnum.ToStart:
        return i18nBook.realTimeConsultation.toBeStarted.t;
      case QueryConsultationStatusEnum.InProgress:
        return i18nBook.realTimeConsultation.haveInHand.t;
      case QueryConsultationStatusEnum.PendingReport:
        return i18nBook.realTimeConsultation.toBeReported.t;
      case QueryConsultationStatusEnum.Expired:
        return i18nBook.realTimeConsultation.expired.t;
      case QueryConsultationStatusEnum.End:
        return i18nBook.realTimeConsultation.closed.t;
      default:
        return "";
    }
  }

  String _getConsultationQueryTypeEnumTranslate(
      ConsultationQueryTypeEnum typeEnum) {
    switch (typeEnum) {
      case ConsultationQueryTypeEnum.All:
        return i18nBook.common.all.t;
      case ConsultationQueryTypeEnum.MyApply:
        return fisI18nBook.common.iApplied.t;
      case ConsultationQueryTypeEnum.MyArrange:
        return fisI18nBook.common.iOrganized.t;
      case ConsultationQueryTypeEnum.MyJoin:
        return fisI18nBook.common.iInvolved.t;
      default:
        return "";
    }
  }

  // 关闭按钮
  Widget _buildCloseDrawer() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.end,
      children: [
        FISIconButton(
          icon: const Icon(
            Icons.close,
          ),
          onPressed: () {
            closeDrawer!();
          },
        )
      ],
    );
  }

  Future onSubmittedAsync(String value) async {
    if (!onFilterDateComparison()) {
      return;
    }
    try {
      if (Storage.platform == Platform.Windows && Storage.isShowSoftKeyBoard) {
        Storage.isShowSoftKeyBoard = false;
        await jsonRpcProxyForFISLib.consultation.operateSoftKeyBoard(
            FISControlPositionInfo(isShowSoftwareKeyBoard: false));
      }
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "OperateSoftKeyBoard error: $ex");
    }
    await controller.searchAsync();
    closeDrawer!();
  }

  /// 校验表格时间筛选 开始时间不能大于结束时间
  bool onFilterDateComparison() {
    if (controller.consultationListController.consultationFilter.startDate
                .value !=
            null &&
        controller
                .consultationListController.consultationFilter.endDate.value !=
            null) {
      DateTime startDate = controller
          .consultationListController.consultationFilter.startDate.value!;
      DateTime endDate = controller
          .consultationListController.consultationFilter.endDate.value!;
      if (startDate.isAfter(endDate)) {
        PromptBox.toast(i18nBook.errorCodes.errorCode4022.t);
        return false;
      }
    }

    return true;
  }
}
