import 'dart:async';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/authentication.m.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/fispanel/micandspeakerpanel.dart';
import 'package:flyinsonolite/consultation/fispanel/micandspeakerpanelformobile.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationlistcontroller.dart';
import 'package:flyinsonolite/controls/button/fiselevatedbutton.dart';
import 'package:flyinsonolite/controls/custom/customelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/utils.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/index.dart';
import 'package:flyinsonolite/helpers/dialoghelper.dart';
import 'package:flyinsonolite/home/controllers/homecontroller.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/consultation/records/models/consultationmember.dart';
import 'package:flyinsonolite/consultation/records/models/liveconsultation.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/routeconstants.dart';
import 'package:get/get.dart';

class ConsulationFISPanel extends StatefulWidget {
  const ConsulationFISPanel({
    Key? key,
  }) : super(key: key);

  @override
  State<ConsulationFISPanel> createState() => _ConsulationFISPanelState();
}

class _ConsulationFISPanelState extends State<ConsulationFISPanel> {
  final HomeController homeController = Get.find<HomeController>();
  Timer? _fisCheckTimer;
  int micVolume = 0;
  int speakerVolume = 0;
  bool isSpeakerMute = false;
  bool isMicMute = false;
  bool hasInit = false;
  final _liveConsultationManager = Get.find<ILiveConsultationManager>();

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      return Container(
          width: 240.s,
          height: 200.s,
          decoration: BoxDecoration(
              color: Storage.currentTheme.themeData.scaffoldBackgroundColor,
              borderRadius: BorderRadius.all(Radius.circular(5.s)),
              border: Border.all(
                  width: 1.s,
                  color: Storage.currentTheme.loginContainStyle.borderColor)),
          child: (Column(
            children: [
              SizedBox(
                height: 40.s,
                child: Container(
                  color: Storage.currentTheme.homepageHeaderColor,
                  child: Center(
                      child: FISText(
                    i18nBook.realTimeConsultation.realTimeDiagnosis.t,
                    style: Storage.currentTheme.titleTextStyle.copyWith(
                        fontSize:
                            Storage.currentTheme.titleTextStyle.fontSize!.s),
                  )),
                ),
              ),
              Storage.platform == Platform.Windows
                  ? const MicAndSpeakerPanel()
                  : const MicAndSpeakerPanelForMobile(),
              SizedBox(
                height: 10.s,
              ),
              FISElevatedButton(() {
                DialogHelper.showConfirmDialog(Get.context!,
                    i18nBook.realTimeConsultation.confirmLeaveConsultation.t,
                    () async {
                  Get.close(1);
                  await leaveLiveConsultationAsync(true);
                });
              }, i18nBook.realTimeConsultation.exitConsultation.t,
                  buttonStyle: CustomElevatedButton.styleFrom(
                      backgroundColor: Colors.redAccent,
                      padding: kIsMobile
                          ? EdgeInsets.symmetric(
                              vertical: 8.s, horizontal: 20.s)
                          : EdgeInsets.symmetric(
                              vertical: 18.s, horizontal: 20.s)),
                  textStyle: TextStyle(fontSize: 20.s, color: Colors.white)),
            ],
          )));
    });
  }

  @override
  void initState() {
    addListeners();
    _fisCheckTimer = Timer(const Duration(seconds: 3), () {
      fisConsultationCheck();
    });
    super.initState();
  }

  Future initAsync() async {
    try {
      if (hasInit) {
        return;
      }
      var hardwareInfo =
          await jsonRpcProxyForFISLib.hardware.getCurrentHardwareInfo();
      if (hardwareInfo.micInfo != null) {
        micVolume = hardwareInfo.micInfo!.volume;
        isMicMute = hardwareInfo.micInfo!.isMute;
      }
      if (hardwareInfo.speakerInfo != null) {
        speakerVolume = hardwareInfo.speakerInfo!.volume;
        isSpeakerMute = hardwareInfo.speakerInfo!.isMute;
      }
      hasInit = true;
    } catch (ex) {
      loggerAsyn(
          FISDeviceLogCategory.Error, 'GetCurrentHardwareInfo Error:$ex');
    }
  }

  @override
  void dispose() {
    removeListeners();
    _fisCheckTimer?.cancel();
    super.dispose();
  }

  ///监听成员状态变化
  void addListeners() {
    _liveConsultationManager.onCloseConsultation
        .addListener(onCloseConsultation);
    _liveConsultationManager.onCloseConsultationDueToChange
        .addListener(onCloseConsultation);
    _liveConsultationManager.onChangeConsultation
        .addListener(onChangeConsultation);
    _liveConsultationManager.onLeaveConsultation
        .addListener(onLeaveConsultation);
    _liveConsultationManager.onJoinInConsultation
        .addListener(onJoinInConsultation);
    _liveConsultationManager.onWhiteboardDataReceive
        .addListener(onWhiteboardDataReceive);
    _liveConsultationManager.onSwitchVideoPlay.addListener(onSwitchVideoPlay);
  }

  void removeListeners() {
    _liveConsultationManager.onCloseConsultation
        .removeListener(onCloseConsultation);
    _liveConsultationManager.onCloseConsultationDueToChange
        .removeListener(onCloseConsultation);
    _liveConsultationManager.onChangeConsultation
        .removeListener(onChangeConsultation);
    _liveConsultationManager.onLeaveConsultation
        .removeListener(onLeaveConsultation);
    _liveConsultationManager.onJoinInConsultation
        .removeListener(onJoinInConsultation);
    _liveConsultationManager.onWhiteboardDataReceive
        .removeListener(onWhiteboardDataReceive);
    _liveConsultationManager.onSwitchVideoPlay
        .removeListener(onSwitchVideoPlay);
  }

  Future<void> onCloseConsultation(Object sender, LiveConsultation e) async {
    if (_liveConsultationManager.currentConsultationCode !=
        e.consultationCode) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "CurrentConsultationCode is ${_liveConsultationManager.currentConsultationCode},not${e.consultationCode}");
      return;
    }
    await leaveLiveConsultationAsync(false);
  }

  Future<void> onChangeConsultation(Object sender, LiveConsultation e) async {
    await changeConsultationAsync(e);
  }

  Future<void> onJoinInConsultation(Object sender, LiveConsultation e) async {
    await joinInConsultationAsync(e);
  }

  Future<void> onLeaveConsultation(Object sender, LiveConsultation e) async {
    await leaveConsultationAsync(e);
  }

  Future<void> onWhiteboardDataReceive(Object sender, String e) async {
    await whiteboardDataReceiveAsync(e);
  }

  Future<void> onSwitchVideoPlay(Object sender, ConsultationMember e) async {
    await switchVideoPlayAsync(e);
  }

//切换病人
  Future changeConsultationAsync(LiveConsultation e) async {
    try {
      if (e.memberLiveDatas.isEmpty) {
        return;
      }
      var fisChangeLiveConsultationDTO =
          convertToFISChangeLiveConsultationDTO(e);
      await jsonRpcProxyForFISLib.consultation
          .changeLiveConsultationFromFlutter(fisChangeLiveConsultationDTO);
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel changeConsultationAsync error: $ex");
    }
  }

//某人加入会诊
  Future joinInConsultationAsync(LiveConsultation e) async {
    try {
      if (e.memberLiveDatas.isEmpty) {
        return;
      }
      var fisConsultationMemberChangeDTO =
          convertToFISConsultationMemberChangeDTO(
              e.consultationCode, e.memberLiveDatas[0], true);

      await jsonRpcProxyForFISLib.consultation
          .liveConsultationMemberChangedFromFlutter(
              fisConsultationMemberChangeDTO);
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel joinInConsultationAsync error:$ex");
    }
  }

  Future leaveLiveConsultationAsync(bool isInitiativeExit) async {
    try {
      await jsonRpcProxyForFISLib.consultation
          .hangupLiveConsultationFromFlutter(
              FISHangupLiveConsultationDTO(isInitiativeExit: isInitiativeExit));
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel leaveLiveConsultationAsync error: $ex");
    } finally {
      Storage.isUseFISForConsultation.value = false;
      Storage.isConsultating.value = false;
      if (!Storage.isLogouting && Get.currentRoute != RouteConstants.home) {
        var liveConsultationManager = Get.find<ILiveConsultationManager>();
        liveConsultationManager.currentConsultationCode = "";
        await Future.delayed(const Duration(milliseconds: 500));
        var consultationListController = Get.find<ConsultationListController>();
        await consultationListController.refreshAsync();
        var consultationDetailController =
            Get.find<ConsultationDetailController>();
        await consultationDetailController.onTabChangedAsync(0);
      }
    }
  }

  Future whiteboardDataReceiveAsync(String e) async {
    try {
      if (e.isEmpty) {
        return;
      }
      var fisWhiteBoardDataDTO = FISWhiteBoardDataDTO(whiteBoardData: e);
      await jsonRpcProxyForFISLib.consultation
          .receiveWhiteBoardDataFromFlutter(fisWhiteBoardDataDTO);
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel whiteboardDataReceiveAsync error: $ex");
    }
  }

  FISChangeLiveConsultationDTO convertToFISChangeLiveConsultationDTO(
      LiveConsultation e) {
    List<FISConsultationMemberInfo> memberInfoList = [];
    for (var member in e.memberLiveDatas) {
      if (member.roleType == MemberRoleType.Device) {
        continue;
      } //因为设备信息不变，只有人员的变动
      FISConsultationRoleType roleType = FISConsultationRoleType.Others;
      switch (member.sortLevel) {
        case 0:
          roleType = FISConsultationRoleType.Initiator;
          break;
        case 1:
          roleType = FISConsultationRoleType.Recipient;
          break;
        case 2:
          roleType = FISConsultationRoleType.Others;
          break;
        case 3:
          roleType = FISConsultationRoleType.AssitantDoctor;
          break;
        case 4:
          roleType = FISConsultationRoleType.Assistant;
          break;
      }
      var fisConsultationMemberInfo = FISConsultationMemberInfo(
          operationType: FISClientMessageOperationType.Add,
          roleType: roleType,
          loginSource: FISLoginSource.PersonalComputer,
          state: FISLiveStates.OK,
          pushUrl: member.liveData?.rtmpPushUrl,
          pullRtmpUrl: member.liveData?.rtmpPullUrl,
          pullFlvUrl: member.liveData?.httpPullUrl,
          pullHlsUrl: member.liveData?.hlsPullUrl,
          pushLiveState: FISLiveStates.OK,
          userSign: null,
          id: member.id,
          name: member.name,
          displayName: member.name,
          avatar: null);
      memberInfoList.add(fisConsultationMemberInfo);
    }
    return FISChangeLiveConsultationDTO(
        token: Storage.user.token,
        consultationCode: e.consultationCode,
        members: memberInfoList);
  }

  Future switchVideoPlayAsync(ConsultationMember e) async {
    try {
      var fisSwtichVideoPlayDTO =
          FISSwtichVideoPlayDTO(userId: e.id, isVideoOpen: e.videoOpend);
      await jsonRpcProxyForFISLib.consultation
          .switchVideoPlayFromFlutter(fisSwtichVideoPlayDTO);
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel switchVideoPlayAsync error: $ex");
    }
  }

  void fisConsultationCheck() async {
    try {
      var result =
          await jsonRpcProxyForFISLib.consultation.checkLiveConsultationState();
      if (!result.isConsultingOrSecondWindowMode) {
        Storage.isConsultating.value = false;
        Storage.isUseFISForConsultation.value = false;
        var liveConsultationManager = Get.find<ILiveConsultationManager>();
        liveConsultationManager.currentConsultationCode = "";
      } else {
        _fisCheckTimer = Timer(const Duration(seconds: 3), () {
          fisConsultationCheck();
        });
      }
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "fisConsultationCheck Error:$ex");
      _fisCheckTimer?.cancel();
    }
  }

//某人离开会诊
  Future<void> leaveConsultationAsync(LiveConsultation e) async {
    try {
      if (e.memberLiveDatas.isEmpty) {
        return;
      }
      var fisConsultationMemberChangeDTO =
          convertToFISConsultationMemberChangeDTO(
              e.consultationCode, e.memberLiveDatas[0], false);
      await jsonRpcProxyForFISLib.consultation
          .liveConsultationMemberChangedFromFlutter(
              fisConsultationMemberChangeDTO);
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel leaveLiveConsultationAsync error: $ex");
    }
  }

  FISConsultationMemberChangeDTO convertToFISConsultationMemberChangeDTO(
      String? consultationCode,
      ConsultationMember memberLiveData,
      bool isJoin) {
    FISConsultationRoleType roleType = FISConsultationRoleType.Others;
    switch (memberLiveData.sortLevel) {
      case 0:
        roleType = FISConsultationRoleType.Initiator;
        break;
      case 1:
        roleType = FISConsultationRoleType.Recipient;
        break;
      case 2:
        roleType = FISConsultationRoleType.Others;
        break;
      case 3:
        roleType = FISConsultationRoleType.AssitantDoctor;
        break;
      case 4:
        roleType = FISConsultationRoleType.Assistant;
        break;
    }
    var fisConsultationMemberInfo = FISConsultationMemberInfo(
        operationType: isJoin
            ? FISClientMessageOperationType.Add
            : FISClientMessageOperationType.Delete,
        roleType: roleType,
        loginSource: FISLoginSource.PersonalComputer,
        state: FISLiveStates.OK,
        pushUrl: memberLiveData.liveData?.rtmpPushUrl,
        pullRtmpUrl: memberLiveData.liveData?.rtmpPullUrl,
        pullFlvUrl: memberLiveData.liveData?.httpPullUrl,
        pullHlsUrl: memberLiveData.liveData?.hlsPullUrl,
        pushLiveState: FISLiveStates.OK,
        userSign: null,
        id: memberLiveData.id,
        name: memberLiveData.name,
        displayName: memberLiveData.name,
        avatar: null);
    return FISConsultationMemberChangeDTO(
        consultationCode: consultationCode,
        memberInfo: fisConsultationMemberInfo);
  }
}
