import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/volumeslider.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/index.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';

class MicAndSpeakerPanel extends StatefulWidget {
  const MicAndSpeakerPanel({
    Key? key,
  }) : super(key: key);

  @override
  State<MicAndSpeakerPanel> createState() => _MicAndSpeakerPanelState();
}

class _MicAndSpeakerPanelState extends State<MicAndSpeakerPanel> {
  int micVolume = 0;
  int speakerVolume = 0;
  bool isSpeakerMute = false;
  bool isMicMute = false;
  bool hasInit = false;

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: initAsync(),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return LayoutBuilder(
                builder: (BuildContext context, BoxConstraints constraints) {
              return Container(
                  margin: EdgeInsets.only(top: 10.s, bottom: 10.s),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: [
                      SizedBox(width: 10.s),
                      VolumeSlider(
                        extraOffsetX: 30.s,
                        initVolume: micVolume,
                        child: _buildIconButton(
                            isMicMute ? Icons.mic_off : Icons.mic, () async {
                          await setMicMuteAsync(!isMicMute);
                          setState(() {
                            isMicMute = !isMicMute;
                          });
                        }),
                        onVolumeChanged: (v) async {
                          await setMicVolumeAsync(v);
                          micVolume = v;
                          if (isMicMute) {
                            setState(() {
                              isMicMute = false;
                            });
                          }
                        },
                      ),
                      SizedBox(width: 10.s),
                      VolumeSlider(
                        extraOffsetX: 30.s,
                        initVolume: speakerVolume,
                        child: _buildIconButton(
                            isSpeakerMute ? Icons.volume_off : Icons.volume_up,
                            () async {
                          await setSpeakerMuteAsync(!isSpeakerMute);
                          setState(() {
                            isSpeakerMute = !isSpeakerMute;
                          });
                        }),
                        onVolumeChanged: (v) async {
                          await setSpeakerVolumeAsync(v);
                          speakerVolume = v;
                          if (isSpeakerMute) {
                            setState(() {
                              isSpeakerMute = false;
                            });
                          }
                        },
                      ),
                      SizedBox(width: 10.s),
                    ],
                  ));
            });
          } else {
            return SizedBox(
              height: 85.s,
            );
          }
        });
  }

  @override
  void initState() {
    super.initState();
  }

  Future initAsync() async {
    try {
      if (hasInit) {
        return;
      }
      var hardwareInfo =
          await jsonRpcProxyForFISLib.hardware.getCurrentHardwareInfo();
      if (hardwareInfo.micInfo != null) {
        micVolume = hardwareInfo.micInfo!.volume;
        isMicMute = hardwareInfo.micInfo!.isMute;
      }
      if (hardwareInfo.speakerInfo != null) {
        speakerVolume = hardwareInfo.speakerInfo!.volume;
        isSpeakerMute = hardwareInfo.speakerInfo!.isMute;
      }
      hasInit = true;
    } catch (ex) {
      loggerAsyn(
          FISDeviceLogCategory.Error, 'GetCurrentHardwareInfo Error:$ex');
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  Widget _buildIconButton(IconData icon, GestureTapCallback onLongPress) {
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      child: GestureDetector(
          onLongPress: onLongPress,
          child: Padding(
              padding: EdgeInsets.all(15.s),
              child: Icon(
                icon,
                color: const Color.fromARGB(255, 192, 194, 201),
                size: 35.s,
              ))),
    );
  }

  Future<void> setSpeakerVolumeAsync(int v) async {
    try {
      await jsonRpcProxyForFISLib.hardware.setCurrentSpeakerVolume(v);
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel setSpeakerVolumeAsync error: $ex");
    }
  }

  Future<void> setMicVolumeAsync(int v) async {
    try {
      await jsonRpcProxyForFISLib.hardware.setCurrentMicVolume(v);
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel setMicVolumeAsync error: $ex");
    }
  }

  Future<void> setSpeakerMuteAsync(bool isMute) async {
    try {
      await jsonRpcProxyForFISLib.hardware.setCurrentSpeakerMute(isMute);
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel setSpeakerMuteAsync error: $ex");
    }
  }

  Future<void> setMicMuteAsync(bool isMute) async {
    try {
      await jsonRpcProxyForFISLib.hardware.setCurrentMicMute(isMute);
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationFISPanel setMicMuteAsync error: $ex");
    }
  }
}
