import 'package:fis_common/event/event_type.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/images/views/contentimageview.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

///单张云端图像
class CloudImageItem extends StatelessWidget {
  final int index;
  final double? imageWidth;
  final RemedicalInfoDTO imageInfo;

  /// 图片双击事件
  final VoidCallback? onDoubleTap;

  /// 图片单机事件
  final VoidCallback? onTap;

  ///描述
  final String? description;

  ///边框颜色
  final bool? isSelected;

  ///选中图像选择框时事件通知
  final FEventHandler<List<String>>? onSelectedInputChange;

  Rx<Color?> borderColor = Rx(null);

  CloudImageItem(
    this.index,
    this.imageInfo, {
    super.key,
    this.imageWidth,
    this.onTap,
    this.description,
    this.onDoubleTap,
    this.isSelected = false,
    this.onSelectedInputChange,
  });

  @override
  Widget build(BuildContext context) {
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      child: Container(
        width: imageWidth,
        decoration: BoxDecoration(
          border: Border.all(
            width: 4.s,
            color: (isSelected ?? false)
                ? Storage.currentTheme.themeData.indicatorColor
                : Colors.transparent,
          ),
          color: Colors.black,
        ),
        margin: EdgeInsets.all(10.s),
        child: Stack(
          children: [
            Center(
              child: ContentImage(
                remedicalInfo: imageInfo,
                onTap: () {
                  if (onTap != null) {
                    onTap!.call();
                  }
                },
                onDoubleTap: onDoubleTap,
                serialNo: index + 1,
                description: description,
              ),
            ),
          ],
        ),
      ),
    );
  }
}
