import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/reports/controllers/reporteditcontroller.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

/// 会诊图像
class ConsultationContentImage extends StatelessWidget {
  /// 图片参数
  final ConsultationFileDTO? consultationFileInfo;

  /// 是否是测量页面
  final bool? isMeasure;

  /// 图片类型
  late RemedicalFileDataTypeEnum? fileDataType;

  /// 改变图片地址
  final VoidCallback? onDoubleTap;

  /// 图片单机事件
  final VoidCallback? onTap;

  /// 是否是选择状态
  final bool isPureImage;

  ///是否显示左上角序号
  final bool? ifShowIndex;

  ///图像宽度
  final double? contentWidth;

  ///图像高度
  final double? contentHeight;

  ///序号
  final int serialNo;

  ///图片文字描述
  final String? description;

  ///插件
  final Widget? plugin;

  final ReportEditController? controller;

  ConsultationContentImage(
      {Key? key,
      this.isMeasure = false,
      this.onDoubleTap,
      this.onTap,
      this.isPureImage = false,
      this.ifShowIndex = true,
      this.serialNo = 0,
      this.description = '',
      this.contentHeight,
      this.contentWidth,
      this.consultationFileInfo,
      this.fileDataType,
      this.plugin,
      this.controller})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    fileDataType ??= consultationFileInfo?.fileDataType;

    Widget child;
    if (isPureImage) {
      child = Container(
        color: Colors.black,
        decoration: BoxDecoration(
          border: Border.all(
            width: 4.s,
            color: Colors.transparent,
          ),
          color: Colors.black,
        ),
        margin: EdgeInsets.all(10.s),
        child: Image.network(
          consultationFileInfo?.previewImageUrl ?? "",
          width: contentWidth ?? 300.s,
          height: contentHeight ?? 200.s,
          errorBuilder: ((context, error, stackTrace) {
            return FISText(i18nBook.common.error.t,
                style: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
                    color: Colors.black));
          }),
        ),
      );
    } else {
      child = _buildFContentImage(
        fileDataType!,
        consultationFileInfo?.previewImageUrl ?? "",
      );
    }
    return Stack(
      children: [
        Positioned(child: child),
        Positioned(
          right: 10.s,
          top: 5.s,
          child: plugin ?? const SizedBox(),
        ),
      ],
    );
  }

  Widget _buildFContentImage(
    RemedicalFileDataTypeEnum fileDataType,
    String previewUrl,
  ) {
    switch (fileDataType) {
      case RemedicalFileDataTypeEnum.Image:
      case RemedicalFileDataTypeEnum.ThirdVidSingle:
      case RemedicalFileDataTypeEnum.VinnoVidSingle:
        return _buildImageCard(
          previewUrl,
        );
      case RemedicalFileDataTypeEnum.ThirdVidMovie:
      case RemedicalFileDataTypeEnum.VinnoVidMovie:
        return _buildVidMovieCard(
          previewUrl,
        );
      default:
        return FISText(i18nBook.common.error.t,
            style: Storage.currentTheme.normalTextStyle.copyWith(
                fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
                color: Colors.black));
    }
  }

  Widget _buildImageCard(
    String previewUrl,
  ) {
    return InkWell(
        onDoubleTap: () {
          onDoubleTap?.call();
        },
        onTap: () {
          onTap?.call();
        },
        child: controller == null
            ? Container(
                width: 190.s,
                height: 160.s,
                decoration: BoxDecoration(
                  border: Border.all(
                    width: 4.s,
                    color: Colors.transparent,
                  ),
                  color: Colors.black,
                ),
                margin: EdgeInsets.all(10.s),
                child: Stack(
                  children: [
                    Center(
                      child: Image.network(
                        previewUrl,
                        errorBuilder: ((context, error, stackTrace) {
                          return FISText(i18nBook.common.error.t,
                              style: Storage.currentTheme.normalTextStyle
                                  .copyWith(
                                      fontSize: Storage.currentTheme
                                          .normalTextStyle.fontSize!.s,
                                      color: Colors.black));
                        }),
                      ),
                    ),
                    if (ifShowIndex!) _buildIndex(),
                    Positioned(
                      bottom: 5.s,
                      left: 5.s,
                      child: _buildApplication(),
                    ),
                  ],
                ),
              )
            : Obx(
                () => Container(
                  width: 190.s,
                  height: 160.s,
                  decoration: BoxDecoration(
                    border: Border.all(
                      width: 4.s,
                      color: controller!.state.selectedImages
                              .contains(consultationFileInfo!.coverImageUrl)
                          ? Storage.currentTheme.themeData.indicatorColor
                          : Colors.transparent,
                    ),
                    color: Colors.black,
                  ),
                  margin: EdgeInsets.all(10.s),
                  child: Stack(
                    children: [
                      Center(
                        child: Image.network(
                          previewUrl,
                          errorBuilder: ((context, error, stackTrace) {
                            return FISText(i18nBook.common.error.t,
                                style: Storage.currentTheme.normalTextStyle
                                    .copyWith(
                                        fontSize: Storage.currentTheme
                                            .normalTextStyle.fontSize!.s,
                                        color: Colors.black));
                          }),
                        ),
                      ),
                      if (ifShowIndex!) _buildIndex(),
                      Positioned(
                        bottom: 5.s,
                        left: 5.s,
                        child: _buildApplication(),
                      ),
                    ],
                  ),
                ),
              ));
  }

  ///创建序号
  Widget _buildIndex() {
    return Positioned(
      left: 5.s,
      child: FISText((serialNo).toString()),
    );
  }

  Widget _buildVidMovieCard(String previewUrl) {
    return controller == null
        ? Container(
            width: 190.s,
            height: 160.s,
            decoration: BoxDecoration(
              border: Border.all(
                width: 4.s,
                color: Colors.transparent,
              ),
              color: Colors.black,
            ),
            margin: EdgeInsets.all(10.s),
            child: Stack(
              children: [
                Center(
                  child: Image.network(
                    previewUrl,
                    errorBuilder: ((context, error, stackTrace) {
                      return FISText(i18nBook.common.error.t,
                          style: Storage.currentTheme.normalTextStyle.copyWith(
                              fontSize: Storage
                                  .currentTheme.normalTextStyle.fontSize!.s,
                              color: Colors.black));
                    }),
                  ),
                ),
                InkWell(
                  onDoubleTap: () {
                    onDoubleTap?.call();
                  },
                  onTap: () {
                    onTap?.call();
                  },
                  child: Center(
                    child: Icon(
                      Icons.play_circle_outline_rounded,
                      color: Colors.white,
                      size: 50.s,
                    ),
                  ),
                ),
                if (ifShowIndex!) _buildIndex(),
                Positioned(
                  bottom: 5,
                  left: 5,
                  child: _buildApplication(),
                ),
              ],
            ),
          )
        : Obx(() => Container(
              width: 190.s,
              height: 160.s,
              decoration: BoxDecoration(
                border: Border.all(
                  width: 4.s,
                  color: controller!.state.selectedImages
                          .contains(consultationFileInfo!.coverImageUrl)
                      ? Storage.currentTheme.themeData.indicatorColor
                      : Colors.transparent,
                ),
                color: Colors.black,
              ),
              margin: EdgeInsets.all(10.s),
              child: Stack(
                children: [
                  Center(
                    child: Image.network(
                      previewUrl,
                      errorBuilder: ((context, error, stackTrace) {
                        return FISText(i18nBook.common.error.t,
                            style: Storage.currentTheme.normalTextStyle
                                .copyWith(
                                    fontSize: Storage.currentTheme
                                        .normalTextStyle.fontSize!.s,
                                    color: Colors.black));
                      }),
                    ),
                  ),
                  InkWell(
                    onDoubleTap: () {
                      onDoubleTap?.call();
                    },
                    onTap: () {
                      onTap?.call();
                    },
                    child: Center(
                      child: Icon(
                        Icons.play_circle_outline_rounded,
                        color: Colors.white,
                        size: 50.s,
                      ),
                    ),
                  ),
                  if (ifShowIndex!) _buildIndex(),
                  Positioned(
                    bottom: 5.s,
                    left: 5.s,
                    child: _buildApplication(),
                  ),
                ],
              ),
            ));
  }

  ///构建描述字段
  Widget _buildApplication() {
    return FISText(
      description ?? "",
      style: TextStyle(
        fontSize: 10.s,
        overflow: TextOverflow.ellipsis,
      ),
    );
  }
}
