import 'package:fis_common/event/event_type.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_vid/data_host/data_host.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/custom/customiconbutton.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/consultation/images/views/freeslider.dart';
import 'package:flyinsonolite/consultation/images/vid/vidplayer.dart';
import 'package:flyinsonolite/consultation/images/vid/vidplayercontrollernosharing.dart';
import 'package:flyinsonolite/consultation/images/vid/vidplayerevent.dart';
import 'package:get/get.dart';

class ConsultationImagePage extends StatefulWidget {
  final ConsultationFileDTO imageInfo;
  const ConsultationImagePage(this.imageInfo, {super.key});

  @override
  State<StatefulWidget> createState() {
    return _ConsultationImagePageState();
  }
}

class _ConsultationImagePageState extends State<ConsultationImagePage> {
  double get _playIconSize => 30.s;
  final FEventHandler<bool> onPause = FEventHandler<bool>();
  final FEventHandler<double> onProcessChange = FEventHandler<double>();
  late final String _sourceUrl;
  late final String _coverImageUrl;
  late final RemedicalFileDataTypeEnum _fileDataType;
  late final VidPlayerControllerNoSharing _playerController;
  VidPlayStatus _status = VidPlayStatus.init;
  bool _isInitialized = false;
  int _processValue = 0;
  int _totalFramesCount = 0;
  late ConsultationFileTypeEnum _consultationFileType;
  double closeBtnHeight = 36.s;

  ///是否为图像
  bool get _isImage => [
        RemedicalFileDataTypeEnum.Image,
        RemedicalFileDataTypeEnum.VinnoVidSingle,
        RemedicalFileDataTypeEnum.ThirdVidSingle,
      ].contains(_fileDataType);

  ///是否显示播放Icon
  bool get _isNeedShowPlayIcon => [
        VidPlayStatus.pause,
        VidPlayStatus.stop,
      ].contains(_playerController.status);

  @override
  void initState() {
    super.initState();
    _initParameters();
    _loadVidDataHost(_sourceUrl);
    _processValue = 0;
    if (_consultationFileType == ConsultationFileTypeEnum.MeasurementImage) {
      _isInitialized = true;
    } else {
      _clickPlay();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.black,
      child: SafeArea(
        child: Stack(
          children: [
            _buildMainVideoContentView(),
            if (!_isInitialized) ...[
              _buildLoading(),
            ],
            _buildCloseIconButton(),
            if (!_isImage && _isNeedShowPlayIcon) ...[
              _buildPlayCircleIcon(),
            ],
            if (!_isImage) ...[
              ///视频显示进度条
              _buildProcessBar(),
            ],
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    super.dispose();
    _playerController.eventHandler.removeListener(_onStatusChanged);
  }

  ///视频画面
  Widget _buildVideoView() {
    return InkWell(
      mouseCursor: SystemMouseCursors.basic,
      onTap: _videoClick,
      child: VidPlayer(_playerController),
    );
  }

  ///播放Icon
  Widget _buildPlayCircleIcon() {
    return Positioned(
      right: 0,
      bottom: 0,
      left: 0,
      top: closeBtnHeight,
      child: Container(
        alignment: Alignment.center,
        child: MouseRegion(
          cursor: SystemMouseCursors.click,
          child: GestureDetector(
            onTap: _clickPlay,
            child: Icon(
              Icons.play_circle_outline,
              size: 50.s,
              color: Colors.white,
            ),
          ),
        ),
      ),
    );
  }

  ///关闭Icon按钮
  Widget _buildCloseIconButton() {
    return Positioned(
        right: 0,
        top: 0,
        child: FISIconButton(
          onPressed: () {
            Get.close(1);
          },
          iconSize: closeBtnHeight,
          icon: const Icon(Icons.close),
        ));
  }

  ///视频播放
  Widget _buildMainVideoContentView() {
    Widget imageContent;
    if (_isImage) {
      imageContent = Image.network(
        _coverImageUrl,
        fit: BoxFit.contain,
      );
    } else {
      imageContent = _buildVideoView();
    }
    return Positioned(
      right: 0,
      bottom: 0,
      left: 0,
      top: closeBtnHeight,
      child: Container(
        color: Colors.black,
        child: imageContent,
      ),
    );
  }

  ///进度条栏
  Widget _buildProcessBar() {
    if (_totalFramesCount < 1) {
      return const SizedBox();
    }
    return Positioned(
      bottom: 20.s,
      right: 10.s,
      left: 10.s,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          CustomIconButton(
            onPressed: _videoClick,
            icon: Icon(
              _status == VidPlayStatus.pause
                  ? Icons.play_circle_outline
                  : Icons.pause,
              size: _playIconSize,
              color: Colors.white,
            ),
            padding: const EdgeInsets.all(0),
            iconSize: _playIconSize,
            constraints: const BoxConstraints(),
          ),
          Expanded(
            child: Container(
              height: _playIconSize,
              alignment: Alignment.center,
              child: FFreeSlider(
                onProcessChange: onProcessChange,
                min: 0,
                max: _totalFramesCount.toDouble(),
                step: 1,
                value: _processValue.toDouble(),
                onChanged: (value) {
                  _playerController.gotoFrame(value.toInt() - 1);
                },
              ),
            ),
          ),
        ],
      ),
    );
  }

//----------------------UI逻辑处理-----------------------

  ///点击播放
  void _clickPlay() async {
    if (_isInitialized) {
      setState(() {
        _totalFramesCount = _playerController.totalFramesCount;
      });
      _playerController.play();
    } else {
      int retryCount = 0;
      while (retryCount < 3) {
        try {
          bool success = await _playerController.load();
          if (!success) {
            throw "loadVidDataHost failed";
          }
          if (_playerController.status != VidPlayStatus.ready) {
            PromptBox.toast(i18nBook.common.NetImageError.t);
            await loggerAsyn(FISDeviceLogCategory.Error,
                '_playerController play error:${_playerController.status.name}');
            return;
          }
          Future.delayed(const Duration(milliseconds: 100), () {
            setState(() {
              _totalFramesCount = _playerController.totalFramesCount;
            });
            _playerController.play();
          });
          _isInitialized = true;
          return;
        } catch (e) {
          retryCount++;
          if (retryCount == 3) {
            PromptBox.toast(i18nBook.common.NetImageError.t);
            if (_playerController.status != VidPlayStatus.ready) {
              await loggerAsyn(FISDeviceLogCategory.Error,
                  '_playerController play error:${_playerController.status.name}');
              return;
            }
          } else if (!mounted) {
            return; // 如果组件已经被销毁，则停止重试
          }
        }
      }
    }
  }

  void _loadVidDataHost(String url) {
    final dataHost = VidDataHost(url);
    _playerController = VidPlayerControllerNoSharing(dataHost: dataHost);
    _playerController.eventHandler.addListener(_onStatusChanged);
  }

  ///初始化入参
  void _initParameters() {
    var imageInfo = widget.imageInfo;
    _sourceUrl = imageInfo.sourceUrl ?? '';
    _coverImageUrl = imageInfo.coverImageUrl ?? '';
    _fileDataType = imageInfo.fileDataType;
    _consultationFileType = imageInfo.consultationFileType;
  }

  ///状态改变
  void _onStatusChanged(Object sender, VidPlayerEvent e) {
    if (e is VidPlayerStatusChangeEvent) {
      setState(() {
        _status = e.status;
      });
    } else if (e is VidPlayerFrameIndexChangeEvent) {
      _processValue = e.index + 1;
      onProcessChange.emit(this, _processValue.toDouble());
    }
  }

  ///点击视频逻辑处理
  void _videoClick() {
    if (_isNeedShowPlayIcon) {
      _clickPlay();
      onPause.emit(this, false);
    } else if (_playerController.playing) {
      _playerController.pause();
      onPause.emit(this, true);
      setState(() {
        _status = VidPlayStatus.pause;
      });
    }
  }

  Widget _buildLoading() {
    return const Align(
      child: CircularProgressIndicator(),
    );
  }
}
