import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/images/views/cloudimageview.dart';
import 'package:flyinsonolite/consultation/images/views/consultationcontentimageview.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/images/views/consultationimagepage.dart';
import 'package:flyinsonolite/consultation/reports/controllers/reporteditcontroller.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';

///会诊图像
class ConsultationImageView extends StatelessWidget {
  final ScrollController _scrollController = ScrollController();
  double get _leftMargin => 20.s;
  double get _topMargin => 8.s;
  double get _imageWidth => 190.s;
  int _index = 0;
  ConsultationDetailController controller;

  ConsultationImageView(this.controller, {super.key});

  @override
  Widget build(BuildContext context) {
    return Obx(() => controller.realTimeConsultationImageList.isEmpty
        ? _mockEmptyView()
        : _buildImagesView());
  }

  Widget _buildItemImages(int index) {
    _index = 0;
    var imageInfos = controller.realTimeConsultationImageList[index];
    var consultationImages = imageInfos.screenshotList ?? [];
    var coloudImags = imageInfos.ultrasoundImageList ?? [];
    var measureImags = imageInfos.measurementImageList ?? [];

    return Container(
        color: Storage.currentTheme.listViewStyle.itembgColor,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Container(
              margin: EdgeInsets.only(
                left: _leftMargin,
                top: 15.s,
              ),
              child: FISText(
                DateFormat("yyyy-MM-dd").format(
                    controller.realTimeConsultationImageList[index].imageDate ??
                        DateTime.now()),
                style: TextStyle(
                  fontSize: 18.s,
                  fontWeight: FontWeight.bold,
                ),
              ),
            ),
            if (consultationImages.isNotEmpty) ...[
              Container(
                margin: EdgeInsets.only(left: _leftMargin, top: _topMargin),
                child: FISText(
                    i18nBook.realTimeConsultation.consultationScreenshot.t),
              ),
              ScreenshotImags(consultationImages),
            ],
            if (coloudImags.isNotEmpty) ...[
              Container(
                margin: EdgeInsets.only(left: _leftMargin, top: _topMargin),
                child: FISText(i18nBook.remedical.cloudImage.t),
              ),
              _buildCloudImages(coloudImags),
            ],
            if (measureImags.isNotEmpty) ...[
              Container(
                margin: EdgeInsets.only(left: _leftMargin, top: _topMargin),
                child: FISText(i18nBook.measure.measureImage.t),
              ),
              _buildMeasureImages(measureImags),
            ],
          ],
        ));
  }

  Widget _buildMeasureImages(List<RemedicalMeasuredInfoDTO> measureImags) {
    return Container(
      margin: EdgeInsets.only(
        left: _leftMargin,
        top: _topMargin,
      ),
      child: Wrap(
        spacing: 8.s, // 主轴(水平)方向间距
        runSpacing: 8.s, // 纵轴（垂直）方向间距
        direction: Axis.horizontal,
        alignment: WrapAlignment.start,
        children: List<Widget>.generate(
          measureImags.length,
          (i) {
            var imageInfo = measureImags[i];
            var consultationFileDTO =
                convertRemedicalMeasuredInfoDTOToconsultationFileDTO(imageInfo);
            final image = InkWell(
                onTap: () {},
                onDoubleTap: () {
                  Get.to(ConsultationImagePage(consultationFileDTO));
                },
                child: Container(
                    width: _imageWidth,
                    decoration: BoxDecoration(
                      border: Border.all(
                        width: 4.s,
                        color: Colors.transparent,
                      ),
                      color: Colors.black,
                    ),
                    margin: EdgeInsets.all(10.s),
                    child: Stack(
                      children: [
                        Center(
                            child: (Image.network(
                          imageInfo.previewFileToken ?? '',
                          width: _imageWidth,
                          height: 160.s,
                        ))),
                        Positioned(
                          left: 5.s,
                          top: 5.s,
                          child: FISText(
                            (_index + 1).toString(),
                            style:
                                TextStyle(color: Colors.white, fontSize: 14.s),
                          ),
                        ),
                      ],
                    )));
            _index++;
            return image;
          },
        ),
      ),
    );
  }

  Widget _buildCloudImages(List<RemedicalInfoDTO> images) {
    return Container(
      margin: EdgeInsets.only(
        left: _leftMargin,
        top: _topMargin,
      ),
      child: Wrap(
        spacing: 8.s, // 主轴(水平)方向间距
        runSpacing: 8.s, // 纵轴（垂直）方向间距
        direction: Axis.horizontal,
        alignment: WrapAlignment.start,
        children: List<Widget>.generate(
          images.length,
          (i) {
            var imageInfo = images[i];
            var consultationFileDTO =
                convertRemedicalInfoDTOToconsultationFileDTO(imageInfo);
            final image = CloudImageItem(
              _index,
              imageInfo,
              onTap: () {},
              onDoubleTap: () {
                Get.to(ConsultationImagePage(consultationFileDTO));
              },
              imageWidth: _imageWidth,
            );
            _index++;
            return image;
          },
        ),
      ),
    );
  }

  ConsultationFileDTO convertRemedicalInfoDTOToconsultationFileDTO(
      RemedicalInfoDTO imageInfo) {
    return ConsultationFileDTO(
        sourceUrl: imageInfo.terminalImages?.imageUrl,
        previewImageUrl: imageInfo.terminalImages?.previewUrl,
        coverImageUrl: imageInfo.terminalImages?.coverImageUrl,
        createTime: imageInfo.createTime,
        creatorCode: imageInfo.deviceCode,
        creatorName: "",
        fileDataType: imageInfo.fileDataType,
        consultationFileType: ConsultationFileTypeEnum.UltrasoundImage,
        remedicalCode: imageInfo.remedicalCode,
        remedicalMeasureCode: "");
  }

  ConsultationFileDTO convertRemedicalMeasuredInfoDTOToconsultationFileDTO(
      RemedicalMeasuredInfoDTO imageInfo) {
    return ConsultationFileDTO(
        sourceUrl: "",
        previewImageUrl: imageInfo.previewFileToken,
        coverImageUrl: imageInfo.measuredFileToken,
        createTime: imageInfo.createTime,
        creatorCode: imageInfo.userCode,
        creatorName: "",
        fileDataType: RemedicalFileDataTypeEnum.VinnoVidSingle,
        consultationFileType: ConsultationFileTypeEnum.MeasurementImage,
        remedicalCode: imageInfo.remedicalCode,
        remedicalMeasureCode: imageInfo.remedicalMeasuredInfoCode);
  }

  Widget _buildImagesView() {
    return Expanded(
        child: SingleChildScrollView(
            child: ListView.builder(
                shrinkWrap: true,
                controller: _scrollController,
                itemCount: controller.realTimeConsultationImageList.length,
                itemBuilder: (BuildContext context, int index) {
                  return _buildItemImages(index);
                })));
  }
}

///缺省显示
Widget _mockEmptyView() {
  return Container(
    margin: EdgeInsets.only(top: 40.s),
    child: Column(
      children: [
        Center(
          child: Image.asset(
            "assets/images/no_data.png",
            width: 300.s,
            height: 300.s,
            fit: BoxFit.cover,
          ),
        ),
        FISText(i18nBook.common.noneDataTip.t),
      ],
    ),
  );
}

class ScreenshotImags extends StatelessWidget {
  final List<ConsultationFileDTO> imageList;

  /// 图片单机事件
  final ReportEditController? controller;
  int _index = 0;
  ScreenshotImags(this.imageList, {super.key, this.controller});
  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: Alignment.topLeft,
      margin: EdgeInsets.only(
        left: 20.s,
        top: 8.s,
      ),
      child: Wrap(
        spacing: 8.s, // 主轴(水平)方向间距
        runSpacing: 8.s, // 纵轴（垂直）方向间距
        direction: Axis.horizontal,
        alignment: WrapAlignment.start,
        children: List<Widget>.generate(
          imageList.length,
          (i) {
            var imageInfo = imageList[i];
            final image = ConsultationContentImage(
                controller: controller,
                consultationFileInfo: imageInfo,
                onDoubleTap: () {
                  Get.to(ConsultationImagePage(imageInfo));
                },
                onTap: () {
                  controller?.onConsultationImageSelected(
                      imageInfo.coverImageUrl ?? '');
                },
                serialNo: _index + 1,
                contentWidth: 300.s,
                description:
                    " ${i18nBook.realTimeConsultation.uploader.t} ${imageList[i].creatorName!} ");
            _index++;
            return image;
          },
        ),
      ),
    );
  }
}
