import 'package:fis_i18n/i18n.dart';
import 'package:fis_i18n/modules/error_codes.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/consultation.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/consultation/records/models/consultationmember.dart';
import 'package:flyinsonolite/consultation/records/models/liveconsultation.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/helpers/parameterhelper.dart';
import 'package:flyinsonolite/infrastructure/routeconstants.dart';
import 'package:get/get.dart';
import 'package:universal_html/html.dart' as html;

class ConsultationRequestController extends GetxController {
  final ILiveConsultationManager _liveConsultationManager =
      Get.find<ILiveConsultationManager>();
  LiveConsultation liveConsultation;
  bool isJoin;
  late ConsultationMember initiaor;
  ConsultationRequestController(this.liveConsultation, this.isJoin) {
    ///获取发起者信息
    initiaor = liveConsultation.memberLiveDatas[0];
    if (liveConsultation.operator != null) {
      initiaor = ConsultationMember.fromDto(liveConsultation.operator!)
          .fixMergedVideoOutputSize()
          .fixOnlyCamera();
    }
  }

  Future rejectAsync() async {
    try {
      await _liveConsultationManager
          .rejectLiveConsultationAsync(liveConsultation.consultationCode!);
    } catch (ex) {
      await handlerErrorAsync(
          ex, "ConsultationRequestController rejectAsync error:$ex");
    }
  }

  Future startConsultationAsync() async {
    final IAppointmentManager appointmentManager =
        Get.find<IAppointmentManager>();
    var consultationDetaioDto = await appointmentManager
        .findConsultationDetailAsync(liveConsultation.consultationCode!);
    var result = await _liveConsultationManager.startConsultationAsync(
        consultationDetaioDto!,
        checkOnly: true,
        checkMedia: true,
        isJoin: isJoin);
    var deviceCode = consultationDetaioDto.deviceCode;
    if (result != null) {
      if (deviceCode == Storage.usDeviceCode) {
        try {
          var startResult = await jsonRpcProxyForFISLib.consultation
              .startLiveConsultationFromFlutter(FISLiveConsultationRequestDTO(
                  consultationCode: consultationDetaioDto.consultationCode,
                  token: Storage.user.token,
                  isJoin: isJoin,
                  isInvite: false,
                  isEmergency: consultationDetaioDto.isEmergency));
          if (!startResult.isSuccess) {
            await loggerAsyn(FISDeviceLogCategory.Error,
                "jsonRpcProxyForFISLib startLiveConsultationFromFlutter Fail,FailMessage:${startResult.failMessage}");
            if (startResult.errorCode > 0) {
              var text = i18nBook
                  .getTrItem(ErrorCodesModule.ModuleName,
                      'errorCode${startResult.errorCode}')
                  ?.t;
              PromptBox.toast(text ?? "Unkonwn RPC Exception");
            } else {
              PromptBox.toast(
                  "jsonRpcProxyForFISLib startLiveConsultationFromFlutter Fail,FailMessage:${startResult.failMessage}");
            }
            return;
          } else {
            _liveConsultationManager.consultationDetail = consultationDetaioDto;
            _liveConsultationManager.liveConsultation = result;
            _liveConsultationManager.currentConsultationCode =
                consultationDetaioDto.consultationCode!;
            Storage.isConsultating.value = true;
            Storage.isUseFISForConsultation.value = true;
          }
        } catch (ex) {
          await handlerErrorAsync(ex,
              "ConsultationRequestController jsonRpcProxyForFISLib startLiveConsultationFromFlutter Fail$ex");
        }
      } else if (consultationDetaioDto.deviceCode != Storage.usDeviceCode &&
          Storage.hasMultiScreens) {
        _liveConsultationManager.consultationDetail = consultationDetaioDto;
        _liveConsultationManager.liveConsultation = result;
        Storage.isConsultating.value = true;
        // html.window.open(
        //'$url/#/liveconsultation?page=liveconsultation&consultationCode=${liveConsultation.consultationCode}&isJoin=false&token=${Storage.user.token}&language=${FI18n.ins.currentBook.locale.languageCode}&theme=${Storage.currentTheme.themeName}',
        //  '');
        await sendFlutterLiveConsultationInfo();
      } else {
        _liveConsultationManager.consultationDetail = consultationDetaioDto;
        _liveConsultationManager.liveConsultation = result;
        ParameterHelper.parameterMap['isJoin'] = isJoin.toString();
        Get.toNamed(RouteConstants.consultationRouteConstants.liveconsultation);
      }
    }
  }

  Future<void> sendFlutterLiveConsultationInfo() async {
    try {
      final url = html.window.location.origin;
      var result = await jsonRpcProxyForFISLib.consultation
          .sendFlutterLiveConsultationInfo(FISFlutterLiveConsultationInfo(
              isStartLiveConsultation: true,
              token: Storage.user.token,
              consultationCode: liveConsultation.consultationCode,
              liveConsultationFlutterUrl:
                  '$url/#/liveconsultation?page=liveconsultation&consultationCode=${liveConsultation.consultationCode}&isJoin=$isJoin&token=${Storage.user.token}&language=${FI18n.ins.currentBook.locale.languageCode}&theme=${Storage.currentTheme.themeName}'));
      if (!result.isSuccess) {
        await loggerAsyn(FISDeviceLogCategory.Error,
            "ConsultationRequestController sendFlutterLiveConsultationInfoAsync ex:${result.failMessage}");
        ParameterHelper.parameterMap['isJoin'] = isJoin.toString();
        Get.toNamed(RouteConstants.consultationRouteConstants.liveconsultation);
      }
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationRequestController sendFlutterLiveConsultationInfo Error:$ex");
      ParameterHelper.parameterMap['isJoin'] = isJoin.toString();
      Get.toNamed(RouteConstants.consultationRouteConstants.liveconsultation);
    }
  }
}
