import 'dart:async';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/consultation/records/controllers/addparticipantcontroller.dart';
import 'package:flyinsonolite/consultation/records/views/addparticipantdialog.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/liveconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/liveconsultationstate.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/consultation/records/models/consultationmember.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/routeconstants.dart';
import 'package:get/get.dart';

class LiveMembersController {
  final _liveConsultationManager = Get.find<ILiveConsultationManager>();
  LiveConsultationController liveConsultationController;
  LiveConsultationState get state => liveConsultationController.state;
  ConsultationDetailDTO get _consultationDetail =>
      liveConsultationController.consultationDetail!;
  //会诊心跳
  Timer? _heartbeatTimer;
  //会诊计时
  Timer? _timer;
  int retryCount = 0;
  LiveMembersController(this.liveConsultationController);

  ///开启会诊心跳
  Future<void> startHeartbeatAsync() async {
    var heartbeatSerconds = Storage.user.liveConsultationRateSeconds;
    await _liveConsultationManager
        .heartRateAsync(_consultationDetail.consultationCode!);
    _heartbeatTimer = Timer.periodic(
      Duration(seconds: heartbeatSerconds ?? 10),
      (timer) async {
        try {
          await _liveConsultationManager
              .heartRateAsync(_consultationDetail.consultationCode!);
          retryCount = 0;
        } on JsonRpcException catch (e) {
          //833是会诊已结束的错误码
          retryCount++; //切换病人时，若碰巧还没有及时更换ConsultationCode，会出现该报错。
          if (e.data.code == 833 && retryCount > 1) {
            if (Get.currentRoute.contains(
                RouteConstants.consultationRouteConstants.liveconsultation)) {
              Get.close(1);
            }
          }
        } catch (e) {
          await loggerAsyn(FISDeviceLogCategory.Error,
              'LiveConsultationManager heartRateAsync:$e');
        }
      },
    );

    _startConsultationTimer();
  }

  void _startConsultationTimer() {
    _timer = Timer.periodic(
      const Duration(seconds: 1),
      (timer) {
        if (state.seconds.value == 59) {
          state.seconds.value = 0;
          state.minute++;
        } else {
          state.seconds++;
        }
      },
    );
  }

  ///初始化待会诊病人列表
  Future<void> initWaitInLineForConsultationPatientsAsync() async {
    try {
      if (_consultationDetail.expertUserCode != Storage.user.userCode) {
        return;
      }

      final consultationPageDatas =
          await _liveConsultationManager.findCanSwitchConsultationsAsync(
              _consultationDetail.consultationCode!);
      state.waitInLineForConsultationPatients.value = consultationPageDatas;
      var currentPatinet = ConsultationPageDTO(
        consultationCode: _consultationDetail.consultationCode,
        patientName: _consultationDetail.patientName,
        consultationStatus: _consultationDetail.consultationStatus,
      );
      state.waitInLineForConsultationPatients.add(currentPatinet);
      state.selectedPatient.value = currentPatinet;
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          'LiveMembersController _initWaitInLineForConsultationOatients error: $e');
    }
  }

  ///取消呼叫
  Future<void> cancelCall(String targetCode) async {
    var result = await _liveConsultationManager
        .cancelInvitingInLiveConsultationAsync(
            state.liveConsultation.value!.consultationCode ?? '', [targetCode]);
    final consultationCode = result.consultationCode ?? '';
    if (consultationCode.isNotEmpty) {
      var existMember = state.liveConsultation.value!.memberLiveDatas
          .firstWhereOrNull((element) => element.id == targetCode);
      if (existMember != null) {
        existMember.status.value = MemberStatus.Cancel;
      }
    }
  }

  ///再次呼叫
  Future<void> callAgain(String targetId) async {
    final result = await _liveConsultationManager.inviteInLiveConsultationAsync(
      state.liveConsultation.value!.consultationCode ?? '',
      [targetId],
      state.liveConsultation.value!.roomNo,
    );
    var resultCode = result.consultationCode ?? '';
    if (resultCode.isNotEmpty) {
      var existMember = state.liveConsultation.value!.memberLiveDatas
          .firstWhereOrNull((element) => element.id == targetId);
      if (existMember != null) {
        existMember.status.value = MemberStatus.Calling;
      }
    }
  }

  ///添加成员
  Future<void> addMember() async {
    Get.lazyPut<AddParticipantController>(() => AddParticipantController());
    var addParticipantController = Get.find<AddParticipantController>();
    addParticipantController.currentConsultation = _consultationDetail;
    await addParticipantController.initializeAsync();
    Get.dialog(AddParticipantDialog(addConsultationMemberAsync),
        barrierDismissible: false);
  }

  ///添加会诊成员
  Future<void> addConsultationMemberAsync(
      List<UserExtendDTO> selectedExpertList) async {
    List<String> userCodes = [];
    if (selectedExpertList.isNotEmpty) {
      for (var expert in selectedExpertList) {
        userCodes.add(expert.userCode!);
      }
    }

    //if (mobileUserCodes != null) {
    //  userCodes = mobileUserCodes;
    //} else {
    //  userCodes = realTimeConsultationController.state.applicationInfo.participantsTotal;
    //}
    await _liveConsultationManager.inviteInLiveConsultationAsync(
      state.liveConsultation.value!.consultationCode ?? '',
      userCodes,
      state.liveConsultation.value!.roomNo,
    );
    for (var expert in selectedExpertList) {
      if (!state.liveConsultation.value!.memberLiveDatas
          .any((element) => element.id == expert.userCode)) {
        var newMemeber = ConsultationMember(
          id: expert.userCode,
          headImageToken: expert.headImageUrl,
          name: expert.displayName ?? '',
          sortLevel: 2,
        );
        newMemeber.status.value = MemberStatus.Calling;
        state.liveConsultation.value!.memberLiveDatas.add(newMemeber);
      }
    }
  }

  void destroyMemory() {
    _heartbeatTimer?.cancel();
    _heartbeatTimer = null;
    _timer?.cancel();
    _timer = null;
  }
}
