import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/liveconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/liveconsultationstate.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/managers/interfaces/irtmpmockrtcmanager.dart';
import 'package:flyinsonolite/consultation/records/models/consultationmember.dart';
import 'package:flyinsonolite/consultation/records/models/liveconsultation.dart';
import 'package:get/get.dart';

class MemberListeners {
  LiveConsultationController liveConsultationController;
  LiveConsultationState get state => liveConsultationController.state;
  final _liveConsultationManager = Get.find<ILiveConsultationManager>();

  MemberListeners(this.liveConsultationController);

  ///监听成员状态变化
  void addListeners() {
    //接受邀请
    _liveConsultationManager.onAcceptConsultation
        .addListener(_onMemberAcceptConsultation);
    //拒绝邀请
    _liveConsultationManager.onRejectConsultation
        .addListener(_onMemberRejectApplayConsultation);
    //成员启用或关闭静音
    _liveConsultationManager.onMemberMute.addListener(_onMemberMute);
    //成员启用或关闭摄像头
    _liveConsultationManager.onSwitchVideoPlay.addListener(_onSwitchVideoPlay);
    _liveConsultationManager.onLeaveConsultation.addListener(_onMemberLeave);
    _liveConsultationManager.onJoinInConsultation.addListener(_onMemberJoin);
    _liveConsultationManager.onNetworkErrConsultation
        .addListener(_onMemberNetworkError);
    _liveConsultationManager.onJoinConsultation.addListener(_onMemberJoin);
    _liveConsultationManager.onChangeConsultation
        .addListener(_onConsultationChange);
    _liveConsultationManager.onCloseConsultationDueToChange
        .addListener(_onCloseConsultation);
    _liveConsultationManager.onCancelInvitingConsultation
        .addListener(_onCancelInviting);
    _liveConsultationManager.onCloseConsultation
        .addListener(_onCloseConsultation);
  }

  void dispose() {
    _liveConsultationManager.onCancelInvitingConsultation
        .removeListener(_onCancelInviting);
    _liveConsultationManager.onAcceptConsultation
        .removeListener(_onMemberAcceptConsultation);
    _liveConsultationManager.onRejectConsultation
        .removeListener(_onMemberRejectApplayConsultation);
    _liveConsultationManager.onMemberMute.removeListener(_onMemberMute);
    _liveConsultationManager.onSwitchVideoPlay
        .removeListener(_onSwitchVideoPlay);
    _liveConsultationManager.onJoinInConsultation.removeListener(_onMemberJoin);
    _liveConsultationManager.onNetworkErrConsultation
        .removeListener(_onMemberNetworkError);
    _liveConsultationManager.onJoinConsultation.removeListener(_onMemberJoin);
    _liveConsultationManager.onChangeConsultation
        .removeListener(_onConsultationChange);
    _liveConsultationManager.onCloseConsultationDueToChange
        .removeListener(_onCloseConsultation);
    _liveConsultationManager.onCloseConsultation
        .removeListener(_onCloseConsultation);
  }

  //接受会诊
  void _onMemberAcceptConsultation(Object sender, LiveConsultation e) {
    var acceptMembers = e.memberLiveDatas;
    if (acceptMembers.isEmpty) {
      loggerAsyn(FISDeviceLogCategory.Error,
          'MemberListeners _onMemberAcceptConsultation error: acceptMember is null');
      return;
    }

    var accepter = acceptMembers[0];
    var memberStatus = state.liveConsultation.value?.memberLiveDatas;
    if (memberStatus == null || memberStatus.isEmpty) {
      return;
    }
    var member =
        memberStatus.firstWhereOrNull((element) => element.id == accepter.id);
    if (member != null) {
      member.status.value = MemberStatus.Accepted;
    }
  }

  ///成员拒绝
  void _onMemberRejectApplayConsultation(Object sender, LiveConsultation e) {
    var rejectorMembers = e.memberLiveDatas;
    if (rejectorMembers.isEmpty) {
      loggerAsyn(FISDeviceLogCategory.Error,
          'MemberListeners _onMemberRejectApplayConsultation error: rejectorMember is null');
      return;
    }
    var rejector = rejectorMembers[0];
    var memberStatus = state.liveConsultation.value?.memberLiveDatas;
    if (memberStatus == null) {
      return;
    }

    var member =
        memberStatus.firstWhereOrNull((element) => element.id == rejector.id);
    if (member != null) {
      member.status.value = MemberStatus.Rejected;
    }
  }

  ///收到某个成员静音的通知
  void _onMemberMute(Object sender, ConsultationMember e) {
    var mute = e.mute;
    var memberStatus = state.liveConsultation.value?.memberLiveDatas;
    if (memberStatus == null) {
      return;
    }

    var member = memberStatus.firstWhereOrNull((element) => element.id == e.id);
    if (member != null) {
      member.mute = mute;
    }
  }

  ///切换视频播放
  void _onSwitchVideoPlay(Object sender, ConsultationMember e) {
    var id = e.id ?? '';
    var videoOpend = e.videoOpend;
    var memberStatus = state.liveConsultation.value?.memberLiveDatas;
    if (memberStatus == null) {
      return;
    }
    var member = memberStatus.firstWhereOrNull((element) => element.id == e.id);
    if (member != null) {
      member.videoOpend = videoOpend;
    }
  }

  ///成员离开
  void _onMemberLeave(Object sender, LiveConsultation e) {
    var members = e.memberLiveDatas;
    if (members.isEmpty) {
      return;
    }
    var leaveMember = members[0];
    var memberStatus = state.liveConsultation.value?.memberLiveDatas;
    if (memberStatus == null) {
      return;
    }

    var member = memberStatus
        .firstWhereOrNull((element) => element.id == leaveMember.id);
    if (member != null) {
      member.status.value = MemberStatus.Left;
    }
  }

  ///成员加入
  void _onMemberJoin(Object sender, LiveConsultation e) {
    if (e.memberLiveDatas.isEmpty) {
      return;
    }

    var joinMember = e.memberLiveDatas[0];
    var memberStatus = state.liveConsultation.value?.memberLiveDatas;
    if (memberStatus == null) {
      return;
    }

    LiveMemberEnum curMemberType = LiveMemberEnum.User;
    var member =
        memberStatus.firstWhereOrNull((element) => element.id == joinMember.id);
    if (member != null) {
      member.status.value = MemberStatus.Joined;
      member.mergedChannel = joinMember.mergedChannel;
      curMemberType = joinMember.memberType;
      memberStatus.remove(member);
      memberStatus.add(member);
      if (joinMember.consultationDeviceInfo != null) {
        member.videoDeviceInfos =
            joinMember.consultationDeviceInfo!.videoDeviceOutputList ?? [];
        member.mergedVideoOutputHeight = joinMember.mergedVideoOutputHeight;
        member.mergedVideoOutputWidth = joinMember.mergedVideoOutputWidth;

        liveConsultationController.mediaController.cameraDeviceInfo =
            member.videoDeviceInfos!.firstWhereOrNull((element) =>
                element.videoDeviceSourceType ==
                VideoDeviceSourceTypeEnum.Camera);
      }

      /// 如果是 RTMP ，需要发送虚拟【加入房间】事件
      if (curMemberType == LiveMemberEnum.Device) {
        liveConsultationController.loadingController.stopLoading();
        Get.find<IRtmpMockRtcManager>().updateDevice(LiveConsultationMember(
            id: member.id,
            videoDeviceInfos:
                member.consultationDeviceInfo!.videoDeviceOutputList ?? [],
            memberType: member.memberType,
            mergedChannel: member.mergedChannel,
            mergedVideoOutputHeight: member.mergedVideoOutputHeight,
            mergedVideoOutputWidth: member.mergedVideoOutputWidth,
            mute: member.mute,
            videoOpend: member.videoOpend));
      }
    } else {
      joinMember.status.value = MemberStatus.Joined;
      state.liveConsultation.value?.memberLiveDatas.add(joinMember);
    }
  }

  ///成员网络异常
  void _onMemberNetworkError(Object sender, LiveConsultation e) {
    if (e.memberLiveDatas.isEmpty) {
      return;
    }
    var targetMember = e.memberLiveDatas[0];
    var memberStatus = state.liveConsultation.value?.memberLiveDatas;
    if (memberStatus == null) {
      return;
    }

    var member = memberStatus
        .firstWhereOrNull((element) => element.id == targetMember.id);
    if (member != null) {
      member.status.value = MemberStatus.NetError;
    }
  }

  void _onCancelInviting(Object sender, LiveConsultation e) {
    var targetMemberId = e.userCode ?? '';
    var memberStatus = state.liveConsultation.value?.memberLiveDatas;
    if (memberStatus == null) {
      return;
    }
    var member = memberStatus
        .firstWhereOrNull((element) => element.id == targetMemberId);
    if (member != null) {
      member.status.value = MemberStatus.Cancel;
    }
  }

  ///关闭会诊
  void _onCloseConsultation(Object sender, LiveConsultation e) {
    if (_liveConsultationManager.currentConsultationCode !=
        e.consultationCode) {
      loggerAsyn(FISDeviceLogCategory.Error,
          "CurrentConsultationCode is ${_liveConsultationManager.currentConsultationCode},not${e.consultationCode}");
      return;
    }
    liveConsultationController.leaveConsultationAsync();
  }

  ///切换会诊
  void _onConsultationChange(Object sender, LiveConsultation e) {
    liveConsultationController.reload(e);
  }
}
