import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/liveconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/liveconsultationstate.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/consultation/records/models/consultationmember.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class WhiteboardController {
  LiveConsultationController liveConsultationController;
  LiveConsultationState get state => liveConsultationController.state;

  WhiteboardController(this.liveConsultationController);

  /// 通过位序获取颜色，如果无位序则返回透明色
  Color getColorByIndex(int? index) {
    if (index == null) {
      return Colors.transparent;
    }
    assert(index < _patientColors.length, 'only support 8 colors');
    return _patientColors[index];
  }

  static const List<Color> _patientColors = [
    Color.fromARGB(255, 145, 223, 18),
    Color.fromARGB(255, 255, 0, 0),
    Color.fromARGB(255, 252, 212, 82),
    Color.fromARGB(255, 18, 100, 223),
    Color.fromARGB(255, 2, 213, 250),
    Color.fromARGB(255, 190, 101, 101),
    Color.fromARGB(255, 255, 192, 241),
    Color.fromARGB(255, 119, 14, 247),
  ];

  /// 当前颜色
  Color currectColor() {
    try {
      var memberLiveDatas = state.liveConsultation.value?.memberLiveDatas;
      if (memberLiveDatas == null || memberLiveDatas.isEmpty) {
        return const Color(0xFF1283DF);
      }
      List<ConsultationMember> consultationMembers = memberLiveDatas
          .where((element) => element.memberType == LiveMemberEnum.User)
          .toList();
      ConsultationMember? currectConsultationMember = consultationMembers
          .firstWhereOrNull((element) => element.id == Storage.user.userCode);
      int currectIndex = 0;
      if (currectConsultationMember != null) {
        currectIndex = currectConsultationMember.index ?? 0;
      }
      return _patientColors[currectIndex];
    } catch (e) {
      loggerAsyn(
          FISDeviceLogCategory.Error, 'WhiteboardController currectColor: $e');
      return const Color(0xFF1283DF);
    }
  }
}
