import 'dart:typed_data';

import 'package:fis_common/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter_sound/flutter_sound.dart';
import 'package:flyinsonolite/consultation/liveconsultation/views/consultationrequestdialog.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/popuptimer.dart';
import 'package:flyinsonolite/consultation/records/models/consultationmember.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/global.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/consultation/records/models/liveconsultation.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';
import 'package:logger/logger.dart';

class ConsultationPopup {
  static bool _stopTimer = false;
  static bool _isPlaying = false;
  static PopupTimer? _timer;
  static const String audioPath = 'assets/audio/LiveTalkingMessage.opus';
  static final FlutterSoundPlayer _flutterSound =
      FlutterSoundPlayer(logLevel: Level.nothing);

  static void startListening() {
    var liveConsultationManager = Get.find<ILiveConsultationManager>();

    ///开始会诊通知事件 11
    liveConsultationManager.onInviteLiveConsultation
        .addListener(onInviteLiveConsultation);
    //邀请者取消接听会诊事件
    liveConsultationManager.onCancelConsultation
        .addListener(onCancelConsultation);
    //取消会诊过程中邀请成员的通知(通知被邀请者) 7
    liveConsultationManager.onCancelInvitingConsultation
        .addListener(onCancelInvitingConsultation);

    //会诊过程中邀请成员的通知(通知被邀请者) 8
    liveConsultationManager.onInviteeInLiveConsultation
        .addListener(onJoinLiveConsultation);
  }

  /// 会诊过程中取消呼叫
  static void onCancelInvitingConsultation(Object sender, LiveConsultation e) {
    if (e.userCode != Storage.user.userCode) {
      return;
    }
    if (_isPlaying) {
      _isPlaying = false;
      _flutterSound.stopPlayer();
      _timer?.resetTimer();
      Get.close(1);
    }
  }

  /// 会诊开始前取消呼叫
  static void onCancelConsultation(Object sender, LiveConsultation e) {
    if (_isPlaying) {
      _isPlaying = false;
      _flutterSound.stopPlayer();
      _timer?.resetTimer();
      Get.close(1);
    }
  }

  ///接受会诊弹窗
  static Future<void> onInviteLiveConsultation(
      Object sender, LiveConsultation e) async {
    ///Felix临时屏蔽非本机会诊
    if (!(await isSupportThisConsultation(e))) {
      return;
    }
    //if (_isPlaying) {
    //  _stopTimer = true;
    //  _isPlaying = false;
    //  _flutterSound.stopPlayer();
    // _timer?.resetTimer();
    // router.back();
    //}
    await _informUSToFlyinsonoPageAsync();
    await _requestDialogAsync(e);
  }

  ///加入会诊弹窗
  static void onJoinLiveConsultation(Object sender, LiveConsultation e) async {
    if (!(await isSupportThisConsultation(e))) {
      return;
    }

    if (_isPlaying) {
      _stopTimer = true;
      _isPlaying = false;
      _flutterSound.stopPlayer();
      _timer?.resetTimer();
      Get.close(1);
    }
    await _informUSToFlyinsonoPageAsync();
    await _requestDialogAsync(e, isJoin: true);
  }

  static Future<void> _requestDialogAsync(LiveConsultation e,
      {bool isJoin = false}) async {
    _timer ??= PopupTimer();
    _timer!.startTimer(onTimerComplete: () {
      _timer?.resetTimer();
      Get.close(1);
    });
    try {
      Global.assetBundle.load(audioPath).then((buf) {
        var buffer = buf.buffer.asUint8List();
        _startPlayer(_flutterSound, buffer);
        _isPlaying = true;
      });

      await Get.dialog(
          ConsultationRequestDialog(() {
            _stopTimer = true;
            _isPlaying = false;
            _timer?.resetTimer();
            _flutterSound.stopPlayer();
          }, e, isJoin),
          barrierDismissible: false);

      //if (isPc) {
      //  Get.find<FWebviewInteractiveController>().onDialogDispose();
      //}
    } catch (e) {
      _isPlaying = false;
      await loggerAsyn(FISDeviceLogCategory.Error,
          'ConsultationPopup _requestDialogAsync erro: $e');
    }
    _stopTimer = true;
    _isPlaying = false;
    _timer?.resetTimer();
    _flutterSound.stopPlayer();
  }

  //播放铃声
  static void _startPlayer(FlutterSoundPlayer flutterSound, Uint8List buffer) {
    _stopTimer = false;
    flutterSound.openPlayer();
    flutterSound.startPlayer(
      fromDataBuffer: buffer,
      codec: FPlatform.isIOS ? Codec.pcm16WAV : Codec.opusWebM,
      whenFinished: () {
        if (_stopTimer) {
          return;
        }
        Future.delayed(const Duration(milliseconds: 200), () {
          if (_stopTimer) {
            _isPlaying = false;
            return;
          }
          _startPlayer(flutterSound, buffer);
        });
      },
    );
  }

  static Future<void> _informUSToFlyinsonoPageAsync() async {
    try {
      await jsonRpcProxyForFISLib.consultation.switchToFlyinsonoPage();
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "_informUSToFlyinsonoPageAsync error:$ex");
    }
  }

  static Future<bool> isSupportThisConsultation(LiveConsultation e) async {
    try {
      final IAppointmentManager appointmentManager =
          Get.find<IAppointmentManager>();
      var consultationDetaioDto = await appointmentManager
          .findConsultationDetailAsync(e.consultationCode!);
      if (Storage.usDeviceCode != consultationDetaioDto?.deviceCode &&
          Storage.platform == Platform.Windows) {
        var initiaor = e.memberLiveDatas[0];
        if (e.operator != null) {
          initiaor = ConsultationMember.fromDto(e.operator!)
              .fixMergedVideoOutputSize()
              .fixOnlyCamera();
        }
        PromptBox.toast(fisI18nBook.common.currentConsultationNotSupport
            .translate([initiaor.name]));
        return false;
      }
      return true;
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "isSupportThisConsultation error:$ex");
      return false;
    }
  }
}
