import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/consultationrequestcontroller.dart';
import 'package:flyinsonolite/controls/dialog/fisdialog.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/consultation/records/models/liveconsultation.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/utils.dart';
import 'package:get/get.dart';

class ConsultationRequestDialog extends GetView<ConsultationRequestController> {
  final VoidCallback? clickCallBack;
  final LiveConsultation liveConsultation;
  final bool isJoin;
  const ConsultationRequestDialog(
      this.clickCallBack, this.liveConsultation, this.isJoin,
      {super.key});

  @override
  Widget build(BuildContext context) {
    Get.lazyPut<ConsultationRequestController>(
        () => ConsultationRequestController(liveConsultation, isJoin));
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      return FISDialog(
          i18nBook.homePage.consultation.t, _createContentView(), false, false,
          showCloseButton: false);
    });
  }

  Widget _createContentView() {
    return Column(
      children: [
        SizedBox(
          height: 10.s,
        ),
        CircleAvatar(
          radius: 50.s,
          backgroundImage:
              buildUserAvatarProvider(controller.initiaor.headImageToken),
        ),
        SizedBox(
          height: 10.s,
        ),
        FISText(
          i18nBook.notification.consultationInvitation
              .translate([controller.initiaor.name]),
          style: Storage.currentTheme.dialogStyle.messageTextStyle.copyWith(
              fontSize: Storage
                  .currentTheme.dialogStyle.messageTextStyle.fontSize!.s),
        ),
        SizedBox(
          height: 10.s,
        ),
        Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            FISLinearGradientElevatedButtonAsync(() async {
              clickCallBack?.call();
              Get.close(1);
              await controller.startConsultationAsync();
            }, i18nBook.realTimeConsultation.accept.t,
                Storage.currentTheme.dialogStyle.buttonGradient,
                textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize:
                        Storage.currentTheme.normalTextStyle.fontSize!.s)),
            SizedBox(
              width: 10.s,
            ),
            FISLinearGradientElevatedButtonAsync(() async {
              clickCallBack?.call();
              Get.close(1);
              await controller.rejectAsync();
            }, i18nBook.user.refuse.t,
                Storage.currentTheme.dialogStyle.buttonGradient,
                textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize:
                        Storage.currentTheme.normalTextStyle.fontSize!.s)),
          ],
        )
      ],
    );
  }
}
