import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/liveconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/liveconsultationstate.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:get/get.dart';

class ImageCaptureList extends StatelessWidget {
  LiveConsultationController liveConsultationController;
  LiveConsultationState get state => liveConsultationController.state;
  ImageCaptureList(this.liveConsultationController, {super.key});

  @override
  Widget build(BuildContext context) {
    final scrollController = ScrollController();
    return Obx(() {
      if (state.isShowCaptureImageList.value) {
        return Container(
          width: 120.s,
          margin: EdgeInsets.symmetric(vertical: 80.s, horizontal: 20.s),
          padding: EdgeInsets.symmetric(horizontal: 3.s, vertical: 10.s),
          decoration: BoxDecoration(
            color: const Color.fromARGB(255, 45, 48, 50),
            borderRadius: BorderRadius.all(Radius.circular(10.s)),
            border: Border.all(
                width: 1.5.s, color: const Color.fromARGB(255, 170, 170, 170)),
          ),
          child: Column(
            children: [
              _buildListHead(),
              SizedBox(height: 10.s),
              Expanded(
                child: _buildListBody(scrollController, context),
              ),
            ],
          ),
        );
      }

      return _buildExpandButton();
    });
  }

  /// 列表主体
  Widget _buildListBody(
      ScrollController scrollController, BuildContext context) {
    return ScrollConfiguration(
        behavior: ScrollConfiguration.of(context).copyWith(scrollbars: false),
        child: Obx(() {
          List<CaptureImage> displayImagesList = [];
          if (state.captureImageUploadingCount.value > 0) {
            scrollController.animateTo(0,
                duration: const Duration(milliseconds: 200),
                curve: Curves.easeIn);
            displayImagesList.addAll(
              List.generate(
                state.captureImageUploadingCount.value,
                (index) => CaptureImage(type: CaptureImageType.uploading),
              ),
            );
            displayImagesList.addAll(
              List.generate(
                state.captureImageUploadOnceSum.value -
                    state.captureImageUploadingCount.value,
                (index) => CaptureImage(type: CaptureImageType.success),
              ),
            );
          }

          displayImagesList.addAll(state.captureImageList
              .map((e) => CaptureImage(
                  type:
                      e.fileDataType == RemedicalFileDataTypeEnum.ThirdVidSingle
                          ? CaptureImageType.image
                          : CaptureImageType.video,
                  previewImageUrl: e.previewImageUrl,
                  videoCoverUrl: e.coverImageUrl))
              .toList()
              .reversed
              .toList());
          if (displayImagesList.isEmpty) {
            return Container(
              padding: EdgeInsets.symmetric(vertical: 20.s),
              child: Center(
                child: FISText(
                  i18nBook.realTimeConsultation.noCaptureImage.t,
                  style: TextStyle(color: Colors.grey, fontSize: 16.s),
                ),
              ),
            );
          }
          return ListView.builder(
            controller: scrollController,
            itemCount: displayImagesList.length,
            prototypeItem: _buildImageInProgressing(''),
            padding: EdgeInsets.zero,
            itemBuilder: (context, index) {
              return Container(
                padding: EdgeInsets.symmetric(horizontal: 8.s),
                child: _buildPreviewImage(displayImagesList[index]),
              );
            },
          );
        }));
  }

  /// 预览图
  Widget _buildPreviewImage(CaptureImage image) {
    if (image.type == CaptureImageType.uploading) {
      return _buildImageInProgressing(
          i18nBook.realTimeConsultation.uploading.t);
    }
    if (image.type == CaptureImageType.success) {
      return _buildImageInProgressing(
          i18nBook.realTimeConsultation.uploadFinished.t);
    }
    return _buildCaptureImagePreview(image);
  }

  /// 截图预览图
  Widget _buildCaptureImagePreview(CaptureImage image) {
    if (image.previewImageUrl == null || image.previewImageUrl!.isEmpty) {
      return const SizedBox();
    }
    return Container(
      margin: EdgeInsets.only(bottom: 10.s),
      decoration: BoxDecoration(
        color: Colors.white.withOpacity(0.1),
        border: Border.all(
            width: 1.5.s, color: const Color.fromARGB(255, 200, 200, 200)),
      ),
      child: Stack(
        children: [
          Container(
            height: 60.s,
            color: Colors.black,
            child: Center(
              child: Image.network(
                image.previewImageUrl ?? '',
                fit: BoxFit.cover,
              ),
            ),
          ),
          Positioned(
            right: 0,
            bottom: 0,
            child: SizedBox(
              width: 20.s,
              height: 20.s,
              child: Center(
                child: Icon(
                  image.type == CaptureImageType.video
                      ? Icons.play_arrow
                      : Icons.image,
                  color: Colors.white,
                  size: 16.s,
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  /// 未准备好的图片
  Widget _buildImageInProgressing(String text) {
    return Container(
      margin: EdgeInsets.only(bottom: 10.s),
      child: Container(
        width: 80.s,
        height: 60.s,
        decoration: BoxDecoration(
          color: Colors.white.withOpacity(0.1),
          border: Border.all(
              width: 1.5.s, color: const Color.fromARGB(255, 200, 200, 200)),
        ),
        child: Center(
          child: FISText(
            text,
            style: TextStyle(
                color: const Color.fromARGB(255, 200, 200, 200),
                fontSize: 14.s),
          ),
        ),
      ),
    );
  }

  /// 列表头部
  Widget _buildListHead() {
    return Container(
      padding: EdgeInsets.symmetric(horizontal: 8.s),
      child: Row(
        children: [
          FISText(
            i18nBook.realTimeConsultation.captureImageList.t,
            style: TextStyle(color: Colors.white, fontSize: 16.s),
          ),
          Expanded(child: Container()),
          InkWell(
            onTap: () => state.isShowCaptureImageList.value = false,
            child: Icon(
              Icons.expand_more,
              color: Colors.white,
              size: 20.s,
            ),
          )
        ],
      ),
    );
  }

  /// 展开截图列表按钮
  Widget _buildExpandButton() {
    return InkWell(
      onTap: () async {
        state.isShowCaptureImageList.value = true;
        await liveConsultationController.recordingController
            .initCaptureImageListAsync();
      },
      child: Container(
        width: 60.s,
        height: 40.s,
        margin: EdgeInsets.symmetric(vertical: 80.s, horizontal: 20.s),
        decoration: BoxDecoration(
          color: const Color.fromARGB(255, 45, 48, 50),

          /// 右上右下圆角
          borderRadius: BorderRadius.all(Radius.circular(10.s)),
          border: Border.all(
              width: 1.5.s, color: const Color.fromARGB(255, 170, 170, 170)),
        ),
        child: Icon(
          Icons.perm_media,
          color: Colors.white,
          size: 20.s,
        ),
      ),
    );
  }
}

enum CaptureImageType {
  /// 图片
  image,

  /// 视频
  video,

  /// 上传中
  uploading,

  /// 上传成功
  success,
}

class CaptureImage {
  /// 图片类型
  CaptureImageType type;

  /// 预览图地址
  String? previewImageUrl;

  /// 首帧图地址
  String? videoCoverUrl;

  CaptureImage({
    required this.type,
    this.previewImageUrl,
    this.videoCoverUrl,
  }) : assert(
            type == CaptureImageType.image || type == CaptureImageType.video
                ? previewImageUrl != null && videoCoverUrl != null
                : true,
            'image or video must have previewImageUrl and videoCoverUrl');
}
