import 'dart:convert';
import 'package:fis_common/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_lib_media_rt/fis_lib_media_rt.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/fispanel/micandspeakerpanel.dart';
import 'package:flyinsonolite/consultation/fispanel/micandspeakerpanelformobile.dart';
import 'package:flyinsonolite/consultation/liveconsultation/views/imagecapturelist.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/liveconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/liveconsultation/views/operatesdropdownbutton.dart';
import 'package:flyinsonolite/consultation/liveconsultation/views/previewmembercallingstatusview.dart';
import 'package:flyinsonolite/consultation/liveconsultation/views/previewmemberstatusview.dart';
import 'package:flyinsonolite/controls/button/fiselevatedbutton.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/custom/customdropdownbutton.dart';
import 'package:flyinsonolite/controls/custom/customelevatedbutton.dart';
import 'package:flyinsonolite/controls/custom/customtextbutton.dart';
import 'package:flyinsonolite/controls/select/fisselect.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/videoloading.dart';
import 'package:flyinsonolite/consultation/liveconsultation/views/fiswhiteboard.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/consultation/records/models/consultationmember.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/utils.dart';
import 'package:get/get.dart';

class LiveConsultationPage extends GetView<LiveConsultationController> {
  double get _smallLivePreviewWidth => 250.s;
  double get _smallLivePreviewHeight => 190.s;
  double get _callingViewHeight => 25.s;
  double get _previewMemberHeadTitle => 40.s;
  double get _smallLiveDeviceHeight => 190.s;
  double get _smallLiveDeviceWidth => 250.s;
  BoxDecoration get _smallLivePrevieWDecoration => BoxDecoration(
        border: Border.all(
          width: 1.5.s,
          color: const Color.fromARGB(255, 55, 56, 57),
        ),
        borderRadius: BorderRadius.all(Radius.circular(5.s)),
        color: const Color.fromARGB(255, 14, 16, 17),
      );
  EdgeInsetsGeometry get _smallLivePreviewMarginLast => EdgeInsets.all(2.s);

  bool hasInit = false;

  LiveConsultationPage({super.key});

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: initAsync(),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return LayoutBuilder(builder: (context, constraint) {
              return Row(
                children: [
                  ///左侧区域
                  Expanded(
                      child: Container(
                    color: Colors.black,
                    child: Stack(
                      fit: StackFit.expand,
                      children: [
                        Obx(() {
                          if (controller.state.isLoading.value) {
                            return const VideoLoadingWidget();
                          }
                          if (controller.state.showRetryButton.value) {
                            return Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                FISIconButtonAsync(
                                  icon: const Icon(Icons.refresh),
                                  iconSize: 40.s,
                                  label: i18nBook.college.retry.t,
                                  onPressed: () async {
                                    await controller.retryLoadDevice();
                                  },
                                )
                              ],
                            );
                          }
                          if (controller.state.isSwitchCameraDevice.value &&
                              !controller.state.isMergedMode.value) {
                            return FRTPlayChannelView(
                              controller.state.deviceCameraChannel.value!,
                              size: const Size(1500, 900),
                            );
                          }
                          ConsultationMember? device = controller
                              .state.liveConsultation.value!.memberLiveDatas
                              .firstWhereOrNull((element) =>
                                  element.memberType == LiveMemberEnum.Device);
                          if (device == null) {
                            return const VideoLoadingWidget();
                          }
                          if (controller.state.isMergedMode.value) {
                            return _buildMainViewInMergedMode(device);
                          }

                          return FRTDeviceChannelView(
                            viewSize: Size(
                              getVideoDeviceInfoWidth(
                                  videoDeviceInfos: device.videoDeviceInfos!),
                              getVideoDeviceInfoHeight(
                                  videoDeviceInfos: device.videoDeviceInfos!),
                            ),
                            controller.state.deviceDesktopChannel.value!,
                          );
                        }),
                        Obx(() {
                          if (!controller.state.isLoading.value &&
                              !controller.state.showRetryButton.value) {
                            return _buildWhiteboardView();
                          } else {
                            return const SizedBox();
                          }
                        }),
                        _buildDeviceCameraView(),
                        Positioned(
                          right: 20.s,
                          bottom: 20.s,
                          child: OperatesDropdownButton(
                            onClearCanavsHandler:
                                controller.onClearCanavsHandler,
                          ),
                        ),
                        Align(
                          alignment: Alignment.bottomLeft,
                          child: ImageCaptureList(controller),
                        ),
                        controller.consultationDetail!.expertUserCode !=
                                    Storage.user.userCode ||
                                controller.isEmergency
                            ? const SizedBox()
                            : Positioned(
                                left: 5.s,
                                bottom: 5.s,
                                child: _buildWaitInLineForConsultation(),
                              ),
                      ],
                    ),
                  )),

                  ///右侧区域
                  _buildRightContentView()
                ],
              );
            });
          } else {
            return Container();
          }
        });
  }

  Future initAsync() async {
    try {
      if (hasInit) {
        return;
      }
      if (Storage.isSecondView) {
        Storage.initlizeTheme();
        await Storage.intializeLocaleAsync();
      }
      var isJoin = false;
      if (Get.parameters.isNotEmpty && Get.parameters.containsKey("isJoin")) {
        isJoin = jsonDecode(Get.parameters["isJoin"]!);
      }
      Get.lazyPut<LiveConsultationController>(
          () => LiveConsultationController(isJoin: isJoin));
      hasInit = true;
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error, 'LoginPage Init Error:$ex');
    }
  }

  ///白板
  Widget _buildWhiteboardView() {
    return Obx(() {
      final IPlayChannel? channel = controller.state.deviceDesktopChannel.value;
      if (channel == null) {
        return const SizedBox();
      }
      var memberLiveDatas =
          controller.state.liveConsultation.value?.memberLiveDatas;
      if (memberLiveDatas == null || memberLiveDatas.isEmpty) {
        return const SizedBox();
      }

      ConsultationMember device = memberLiveDatas
          .firstWhere((element) => element.memberType == LiveMemberEnum.Device);
      if (device.videoDeviceInfos == null) {
        return const SizedBox();
      }

      return ClipRRect(
        child: Center(
          child: AspectRatio(
            aspectRatio: getVideoDeviceInfoWidth(
                    videoDeviceInfos: device.videoDeviceInfos!) /
                getVideoDeviceInfoHeight(
                    videoDeviceInfos: device.videoDeviceInfos!),
            child: FISWhiteBoard(
              sendData: (value) async {
                await controller.sendInteractiveBoardDataAsync(value);
              },
              patientColor: controller.currectColor(),
              paintType: controller.state.paintType.value,
              userId: Storage.user.userCode!,
              onReceiveData: controller.onWhiteboardDataReceive,
              onClearCanavs: controller.onClearCanavsHandler,
              initData: controller.state.initBoardDatas,
            ),
          ),
        ),
      );
    });
  }

  String getWaitInLineString(ConsultationPageDTO patient) {
    var name = patient.patientName ?? '';
    var status = i18nBook.realTimeConsultation.toBeStarted.t;
    if (patient.consultationCode ==
        controller.consultationDetail!.consultationCode) {
      status = i18nBook.realTimeConsultation.haveInHand.t;
    }
    return ' $name $status';
  }

  ///待会诊病人
  Widget _buildWaitInLineForConsultation() {
    return Obx(() {
      var patienets = controller.state.waitInLineForConsultationPatients;
      if (patienets.isEmpty) {
        return const SizedBox();
      }
      return FISSelect(
        patienets.map<CustomDropdownMenuItem<ConsultationPageDTO>>(
            (ConsultationPageDTO patient) {
          return CustomDropdownMenuItem<ConsultationPageDTO>(
            value: patient,
            child: FISText(
              getWaitInLineString(patient),
              style: Storage.currentTheme.normalTextStyle.copyWith(
                  fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
                  color: Colors.black),
            ),
          );
        }).toList(),
        (value) {
          controller.updateSelectedPatient(value);
        },
        borderColor: Storage.currentTheme.loginContainStyle.borderColor,
        backgroundColor: Colors.white,
        value: controller.state.selectedPatient.value,
      );
    });
  }

  Widget _buildMainViewInMergedMode(ConsultationMember device) {
    final deviceWidth =
        getVideoDeviceInfoWidth(videoDeviceInfos: device.videoDeviceInfos!);
    final deviceHeight =
        getVideoDeviceInfoHeight(videoDeviceInfos: device.videoDeviceInfos!);
    return Obx(() {
      if (!controller.state.isSwitchCameraDevice.value) {
        return LayoutBuilder(builder: (context, constraint) {
          return Center(
            child: AspectRatio(
              aspectRatio: deviceWidth / deviceHeight,
              child: FRTMergedChannelView(
                isSubView: false,
                mainViewContainerMaxSize: Size(
                  constraint.maxWidth,
                  constraint.maxHeight,
                ),
                videoSize: Size(
                  device.mergedVideoOutputWidth.toDouble(),
                  device.mergedVideoOutputHeight.toDouble(),
                ),
                viewSize: Size(deviceWidth, deviceHeight),
                viewAlignment: Alignment.topLeft,
                controller.state.deviceDesktopChannel.value!,
              ),
            ),
          );
        });
      }
      if (device.videoDeviceInfos != null &&
          device.videoDeviceInfos!.length > 1) {
        return Center(
          child: AspectRatio(
              aspectRatio: getVideoDeviceInfoWidth(
                      videoDeviceInfos: device.videoDeviceInfos!,
                      videoDeviceSourceType: VideoDeviceSourceTypeEnum.Camera) /
                  getVideoDeviceInfoHeight(
                      videoDeviceInfos: device.videoDeviceInfos!,
                      videoDeviceSourceType: VideoDeviceSourceTypeEnum.Camera),
              child: FRTMergedChannelSubView(
                  device: device, controller: controller)),
        );
      }

      return const SizedBox();
    });
  }

  Widget _buildRightContentView() {
    return Container(
      padding: EdgeInsets.all(5.s),
      color: const Color.fromARGB(255, 65, 65, 65),
      width: _smallLivePreviewWidth + 10.s,
      child: Column(children: [
        Expanded(
          child: _buildMembersPreviewView(),
        ),
        Column(
          children: [
            Storage.isSecondView
                ? const SizedBox()
                : Storage.platform == Platform.Windows
                    ? const MicAndSpeakerPanel()
                    : const MicAndSpeakerPanelForMobile(),
            Obx(() => FISText(
                '${i18nBook.measure.playLocationDurationLabel.t} ${controller.state.minute.value.padLeft2()}:${controller.state.seconds.value.padLeft2()}')),
            SizedBox(height: 10.s),
            FISElevatedButton(() {
              controller.exitConsultation();
            }, i18nBook.realTimeConsultation.exitConsultation.t,
                buttonStyle: CustomElevatedButton.styleFrom(
                    backgroundColor: Colors.redAccent,
                    padding: kIsMobile
                        ? EdgeInsets.symmetric(vertical: 8.s, horizontal: 20.s)
                        : EdgeInsets.symmetric(
                            vertical: 18.s, horizontal: 20.s)),
                textStyle: TextStyle(fontSize: 20.s, color: Colors.white)),
          ],
        )
      ]),
    );
  }

  Widget _buildMembersPreviewView() {
    return Obx(() {
      if (controller.state.liveConsultation.value?.memberLiveDatas == null) {
        return const SizedBox();
      }
      var memberLiveDatas =
          controller.state.liveConsultation.value!.memberLiveDatas;
      var consultationMembers = memberLiveDatas
          .where((element) => element.memberType == LiveMemberEnum.User)
          .toList();
      consultationMembers.sort(((a, b) => a.sortLevel.compareTo(b.sortLevel)));
      List<Widget> children =
          consultationMembers.map((e) => _buildMemberPreviewView(e)).toList();

      if (Storage.user.userCode ==
          controller.consultationDetail!.expertUserCode) {
        children.add(_buildAddMemberButton());
      }
      return ListView(
        children: children,
      );
    });
  }

  ///邀请新成员
  Widget _buildAddMemberButton() {
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      child: GestureDetector(
        onTap: () {
          controller.addMember();
        },
        child: Container(
          margin: _smallLivePreviewMarginLast,
          decoration: _smallLivePrevieWDecoration,
          height: _smallLivePreviewHeight,
          width: _smallLivePreviewWidth,
          child: Icon(
            Icons.add,
            color: Colors.grey,
            size: 26.s,
          ),
        ),
      ),
    );
  }

  Widget _buildMemberPreviewView(ConsultationMember member) {
    return Container(
      decoration: _smallLivePrevieWDecoration,
      margin: _smallLivePreviewMarginLast,
      width: _smallLivePreviewWidth,
      child: Stack(
        children: [
          _buildLiveViewContent(member),
          PreviewMemberStatus(member),
          Positioned(
              top: 5.s,
              left: 5.s,
              width: _smallLivePreviewWidth,
              child: _buildMemberName(member)),
          _buildMemberIcon(
              controller.whiteboardController.getColorByIndex(member.index)),
          Positioned(
            top: _smallLivePreviewHeight,
            width: _smallLivePreviewWidth,
            child: PreviewMemberCallingStatus(member),
          ),
        ],
      ),
    );
  }

  Widget _buildMemberIcon(Color currectPatientColor) {
    return Positioned(
      top: 5.s,
      right: 5.s,
      child: Align(
        alignment: Alignment.centerRight,
        child: Icon(
          Icons.circle,
          color: currectPatientColor,
          size: 14.s,
        ),
      ),
    );
  }

  Widget _buildMemberName(ConsultationMember member) {
    return Align(
      alignment: Alignment.centerLeft,
      child: FISText(
        member.name ?? '',
      ),
    );
  }

  ///摄像头内容
  Widget _buildLiveViewContent(ConsultationMember member) {
    return Obx(() {
      IChannel? channel = controller.getUserMemberPreviewChannel(member);
      var memberStatus = controller
          .state.liveConsultation.value!.memberLiveDatas
          .firstWhereOrNull((element) => element.id == member.id);
      bool videoOpend = true;
      if (memberStatus != null && !memberStatus.videoOpend) {
        videoOpend = false;
      }
      if (channel != null) {
        var isUserMember = member.memberType == LiveMemberEnum.User;
        double height;
        if (isUserMember) {
          height = _smallLivePreviewHeight + _callingViewHeight;
        } else {
          height = _smallLiveDeviceHeight + 10.s;
        }
        if (videoOpend) {
          return _buildChannel(isUserMember, height, channel);
        }
        // else {
        //  return Stack(
        //    children: [
        //      _buildChannel(isUserMember, height, channel),
        //      _buildUserHeadImage(member),
        //    ],
        //  );
        //}
      }
      return _buildUserHeadImage(member);
    });
  }

  Widget _buildUserHeadImage(ConsultationMember member) {
    var headImageToken = member.headImageToken ?? '';
    var height = _smallLivePreviewHeight;
    var isUserMember = member.memberType == LiveMemberEnum.User;
    if (isUserMember) {
      height = _smallLivePreviewHeight + _callingViewHeight;
    }

    Widget child = Image(
      image: buildUserAvatarProvider(headImageToken),
      fit: BoxFit.fill,
      height: height,
      width: _smallLivePreviewWidth,
    );

    return Container(
      height: height,
      padding: EdgeInsets.only(
        top: _previewMemberHeadTitle,
        bottom: isUserMember ? _callingViewHeight : 0,
      ),
      width: _smallLivePreviewWidth,
      child: child,
    );
  }

  Widget _buildChannel(bool isUserMember, double height, IChannel channel) {
    Widget channelView;
    if (channel is IPublishChannel) {
      channelView = FRTPublishPreviewView(
        channel,
        //size: size,
      );
    } else {
      channelView = FRTPlayChannelView(
        channel as IPlayChannel,
        // size: size,
      );
    }
    return Container(
      height: height,
      width: isUserMember ? _smallLivePreviewWidth : _smallLiveDeviceWidth,
      padding: EdgeInsets.only(
        top: _previewMemberHeadTitle + 1.s, //+1是防止有锯齿的缝隙
        bottom: isUserMember ? _callingViewHeight : 0,
      ),
      child: channelView,
    );
  }

  ///超声机摄像头（扫查手法）
  Widget _buildDeviceCameraView() {
    return Obx(() {
      if (controller.state.isLoading.value ||
          controller.state.showRetryButton.value) {
        return const SizedBox();
      }

      ConsultationMember? consultationMember = controller
          .state.liveConsultation.value!.memberLiveDatas
          .firstWhereOrNull(
              (element) => element.memberType == LiveMemberEnum.Device);

      if (consultationMember == null ||
          consultationMember.videoDeviceInfos == null ||
          consultationMember.videoDeviceInfos!.firstWhereOrNull((element) =>
                  element.videoDeviceSourceType ==
                  VideoDeviceSourceTypeEnum.Camera) ==
              null) {
        return const SizedBox();
      }
      if (controller.state.isShowDeviceCamera.value) {
        return Positioned(
          right: 0,
          top: 0,
          child: _buildDeviceSmallLiveView(consultationMember),
        );
      } else {
        return Positioned(
          top: 0,
          right: 0,
          child: CustomTextButton(
              onPressed: () {
                controller.state.isShowDeviceCamera.value =
                    !controller.state.isShowDeviceCamera.value;
              },
              child: FISText(controller.state.isSwitchCameraDevice.value
                  ? i18nBook.realTimeConsultation.devicePicture.t
                  : i18nBook.realTimeConsultation.scanningTechnique.t)),
        );
      }
    });
  }

  Widget _buildDeviceSmallLiveView(ConsultationMember device) {
    return Container(
        decoration: _smallLivePrevieWDecoration,
        height: _smallLiveDeviceHeight + _previewMemberHeadTitle,
        width: _smallLiveDeviceWidth,
        child: Column(
          children: [
            Align(
              alignment: Alignment.topRight,
              child: Container(
                height: _previewMemberHeadTitle,
                padding: EdgeInsets.only(left: 5.s),
                color: Colors.black,
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    _buildMemberName(device),
                    Row(
                      children: [
                        FISIconButton(
                          icon: const Icon(Icons.swap_horiz),
                          onPressed: () {
                            controller.state.isSwitchCameraDevice.value =
                                !controller.state.isSwitchCameraDevice.value;
                          },
                        ),
                        FISIconButton(
                          icon: const Icon(Icons.minimize),
                          onPressed: () {
                            controller.state.isShowDeviceCamera.value =
                                !controller.state.isShowDeviceCamera.value;
                          },
                        )
                      ],
                    ),
                  ],
                ),
              ),
            ),
            _buildDeviceChannelView(device),
          ],
        ));
  }

  Widget _buildDeviceChannelView(ConsultationMember device) {
    Size viewSize = Size(
      getVideoDeviceInfoWidth(videoDeviceInfos: device.videoDeviceInfos!),
      getVideoDeviceInfoHeight(videoDeviceInfos: device.videoDeviceInfos!),
    );
    Size cameraSize = Size(
      getVideoDeviceInfoWidth(
          videoDeviceInfos: device.videoDeviceInfos!,
          videoDeviceSourceType: VideoDeviceSourceTypeEnum.Camera),
      getVideoDeviceInfoHeight(
          videoDeviceInfos: device.videoDeviceInfos!,
          videoDeviceSourceType: VideoDeviceSourceTypeEnum.Camera),
    );
    double aspectRatio = 1;
    if (controller.state.isSwitchCameraDevice.value) {
      aspectRatio = viewSize.width / viewSize.height;
    } else {
      aspectRatio = cameraSize.width / cameraSize.height;
    }

    return Obx(() {
      if (controller.state.isMergedMode.value &&
          controller.state.deviceDesktopChannel.value != null) {
        return Obx(() {
          if (controller.state.isSwitchCameraDevice.value) {
            return Center(
              child: AspectRatio(
                aspectRatio: aspectRatio,
                child: FRTMergedChannelView(
                  isSubView: false,
                  videoSize: Size(
                    device.mergedVideoOutputWidth.toDouble(),
                    device.mergedVideoOutputHeight.toDouble(),
                  ),
                  viewSize: viewSize,
                  viewAlignment: Alignment.topLeft,
                  controller.state.deviceDesktopChannel.value!,
                ),
              ),
            );
          }
          if (device.videoDeviceInfos != null &&
              device.videoDeviceInfos!.length > 1) {
            return Center(
              child: AspectRatio(
                aspectRatio: aspectRatio,
                child: FRTMergedChannelSubView(
                    device: device, controller: controller),
              ),
            );
          } else {
            return const SizedBox();
          }
        });
      }

      if (controller.state.deviceCameraChannel.value == null) {
        return const VideoLoadingWidget();
      }
      if (controller.state.isSwitchCameraDevice.value) {
        return FRTDeviceChannelView(
          viewSize: viewSize,
          controller.state.deviceDesktopChannel.value!,
        );
      }

      return Center(
        child: AspectRatio(
          aspectRatio: aspectRatio,
          child: FRTPlayChannelView(
            controller.state.deviceCameraChannel.value!,
            size: Size(_smallLiveDeviceHeight, _smallLiveDeviceWidth),
          ),
        ),
      );
    });
  }
}

/// 超声机合图模式下的子画面（摄像头画面）
class FRTMergedChannelSubView extends StatelessWidget {
  const FRTMergedChannelSubView({
    super.key,
    required this.device,
    required this.controller,
  });

  final ConsultationMember device;
  final LiveConsultationController controller;

  @override
  Widget build(BuildContext context) {
    var cameraSize = Size(
        getVideoDeviceInfoWidth(
            videoDeviceInfos: device.videoDeviceInfos!,
            videoDeviceSourceType: VideoDeviceSourceTypeEnum.Camera),
        getVideoDeviceInfoHeight(
            videoDeviceInfos: device.videoDeviceInfos!,
            videoDeviceSourceType: VideoDeviceSourceTypeEnum.Camera));
    if (device.isOldPlatform) {
      return FRTMergedChannelSubViewOldVersion(
        videoSize: Size(
          device.mergedVideoOutputWidth.toDouble(),
          device.mergedVideoOutputHeight.toDouble(),
        ),
        mainViewSize: Size(
          getVideoDeviceInfoWidth(videoDeviceInfos: device.videoDeviceInfos!),
          getVideoDeviceInfoHeight(videoDeviceInfos: device.videoDeviceInfos!),
        ),
        cameraViewSize: cameraSize,
        viewAlignment: Alignment.topLeft,
        controller.state.deviceDesktopChannel.value!,
      );
    } else {
      return FRTMergedChannelView(
        isSubView: true,
        videoSize: Size(
          device.mergedVideoOutputWidth.toDouble(),
          device.mergedVideoOutputHeight.toDouble(),
        ),
        viewSize: cameraSize,
        viewAlignment: Alignment.topRight,
        controller.state.deviceDesktopChannel.value!,
      );
    }
  }
}
