import 'package:fis_common/event/event_type.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/liveconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/whiteboardstructure.dart';
import 'package:flyinsonolite/consultation/liveconsultation/views/recordbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class OperatesDropdownButton extends StatelessWidget {
  /// 获取到清除事件通知
  FEventHandler<String> onClearCanavsHandler = FEventHandler<String>();
  // 按钮组宽度
  double get _buttonGroupWidth => 68.s;

  // 按钮组容器底部内边距 30 + 30
  double get groupContainerPadding => 5.s;
  static const Color _buttonTextColor = Color.fromARGB(255, 199, 198, 201);

  /// 获取当前组件最大高度
  double _maxGroupHeight = 0;
  final RxDouble _groupHeight = 0.0.obs;
  final RxBool _isExpanded = true.obs;
  final controller = Get.find<LiveConsultationController>();

  OperatesDropdownButton({
    super.key,
    required this.onClearCanavsHandler,
  });

  @override
  Widget build(BuildContext context) {
    final BoxDecoration buttonDecoration = BoxDecoration(
        borderRadius: BorderRadius.all(
          Radius.circular(_buttonGroupWidth / 2),
        ),
        border: Border.all(
            width: 1.5.s, color: const Color.fromARGB(255, 170, 170, 170)),
        color: const Color.fromARGB(255, 14, 16, 17));
    _maxGroupHeight = Get.height - 200.s;
    _groupHeight.value = _maxGroupHeight;
    return Column(
      children: [
        Transform.translate(
            offset: Offset(0, _buttonGroupWidth / 2),
            child: Obx(() => AnimatedContainer(
                  duration: const Duration(milliseconds: 300),
                  padding: EdgeInsets.only(
                    top: 0,
                    bottom: _buttonGroupWidth / 2,
                  ),
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(_buttonGroupWidth / 2),
                      topRight: Radius.circular(_buttonGroupWidth / 2),
                    ),
                  ),
                  clipBehavior: Clip.antiAlias,
                  width: _buttonGroupWidth,
                  constraints: BoxConstraints(maxHeight: _groupHeight.value),
                  child: ListView(
                    shrinkWrap: true,
                    children: _buildMainLiveOperateButtons(),
                  ),
                ))),
        InkWell(
          onHover: (isHover) {
            if (isHover && !_isExpanded.value) {
              _groupHeight.value = _maxGroupHeight;
              _isExpanded.value = true;
            }
          },
          onTap: () {
            if (_isExpanded.value) {
              _groupHeight.value = groupContainerPadding;
              _isExpanded.value = false;
            } else {
              _groupHeight.value = _maxGroupHeight;
              _isExpanded.value = true;
            }
          },
          child: Container(
            decoration: buttonDecoration,
            height: _buttonGroupWidth,
            width: _buttonGroupWidth,
            child: Obx(() => Icon(
                  _isExpanded.value ? Icons.expand_more : Icons.more_vert,
                  color: _buttonTextColor,
                  size: 40.s,
                )),
          ),
        ),
      ],
    );
  }

  List<Widget> _buildMainLiveOperateButtons() {
    return [
      Obx(() => _buildRoundButton(i18nBook.realTimeConsultation.lineSegment.t,
              () {
            controller.state.paintType.value = PaintType.straightLine;
          },
              isSelect:
                  controller.state.paintType.value == PaintType.straightLine)),
      Obx(() => _buildRoundButton(i18nBook.realTimeConsultation.paintBrush.t,
              () {
            controller.state.paintType.value = PaintType.curvedLine;
          },
              isSelect:
                  controller.state.paintType.value == PaintType.curvedLine)),
      _buildRoundButton(i18nBook.measure.clear.t, () {
        controller.sendInteractiveBoardDataAsync('', true);
        onClearCanavsHandler.emit(this, Storage.user.userCode!);
      }),
      Obx(() {
        if (controller.state.isVidFileCapturing.value) {
          return Container(
            width: _buttonGroupWidth,
            height: _buttonGroupWidth,
            margin: EdgeInsets.only(bottom: 10.s),
            decoration: BoxDecoration(
                borderRadius: BorderRadius.all(
                  Radius.circular(_buttonGroupWidth / 2),
                ),
                color: Colors.white),
            child: Center(
              child: Container(
                padding: EdgeInsets.all(5.s),
                child: CircularProgressIndicator(
                    strokeWidth: 4.s,
                    valueColor:
                        const AlwaysStoppedAnimation<Color>(Colors.black)),
              ),
            ),
          );
        } else {
          return RecordButton(
            onTap: () async {
              await controller.captureOneFrameAsync();
            },
            onStartRecord: () async {
              await controller.startRecordingAsync();
            },
            onStopRecord: () async {
              await controller.endRecordingAsync();
            },
            buttonGroupWidth: _buttonGroupWidth,
          );
        }
      })
    ];
  }

  Widget _buildRoundButton(String title, GestureTapCallback? onTap,
      {bool isSelect = false}) {
    final BoxDecoration buttonDecoration = BoxDecoration(
        borderRadius: BorderRadius.all(
          Radius.circular(_buttonGroupWidth / 2),
        ),
        border: Border.all(
            width: 1.5.s, color: const Color.fromARGB(255, 170, 170, 170)),
        color: const Color.fromARGB(255, 14, 16, 17));
    final BoxDecoration buttonSelectDecoration = BoxDecoration(
        borderRadius: BorderRadius.all(
          Radius.circular(_buttonGroupWidth / 2),
        ),
        border: Border.all(width: 1.5.s, color: Colors.blue),
        color: const Color.fromARGB(255, 0, 36, 54));
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      child: GestureDetector(
        onTap: onTap,
        child: Container(
          width: _buttonGroupWidth,
          height: _buttonGroupWidth,
          margin: EdgeInsets.only(bottom: 10.s),
          decoration: isSelect ? buttonSelectDecoration : buttonDecoration,
          child: Center(
            child: FISText(
              title,
              style: TextStyle(
                fontWeight: FontWeight.w300,
                color: _buttonTextColor,
                fontSize: 14.s,
              ),
            ),
          ),
        ),
      ),
    );
  }
}
