import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/liveconsultation/controllers/liveconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/records/models/consultationmember.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:get/get.dart';

class PreviewMemberStatus extends StatelessWidget {
  final ConsultationMember member;
  final controller = Get.find<LiveConsultationController>();
  double get _smallLivePreviewHeight => 190.s;
  TextStyle get _previewMemberTextStyle =>
      TextStyle(color: Colors.grey, fontSize: 14.s);
  double get _previewMemberHeadTitle => 30.s;

  PreviewMemberStatus(this.member, {super.key});

  @override
  Widget build(BuildContext context) {
    return Obx(() {
      var status = controller.state.liveConsultation.value!.memberLiveDatas;
      var consultationMember =
          status.firstWhereOrNull((element) => element.id == member.id);
      if (consultationMember != null) {
        switch (consultationMember.status.value) {
          case MemberStatus.Calling:
            return _buildCallingText(i18nBook.realTimeConsultation.calling.t);
          case MemberStatus.Rejected:
            return _buildCallingText(i18nBook.realTimeConsultation.rejected.t);
          case MemberStatus.Left:
            return _buildCallingText(i18nBook.realTimeConsultation.hasLeft.t);
          case MemberStatus.Busyline:
            return _buildCallingText(i18nBook.realTimeConsultation.busy.t);
          case MemberStatus.Offline:
            return _buildCallingText(i18nBook.user.notOnline.t);
          case MemberStatus.Cancel:
            return _buildCallingText(
                i18nBook.realTimeConsultation.callCanceled.t);
          case MemberStatus.NetError:
            return _buildCallingText(i18nBook.common.networkInstability.t);
          default:
            return const SizedBox();
        }
      }
      return const SizedBox();
    });
  }

  ///呼叫中蒙层显示
  Widget _buildCallingText(String content) {
    return Positioned(
      left: 0,
      right: 0,
      top: _previewMemberHeadTitle,
      child: Container(
        height: _smallLivePreviewHeight,
        alignment: Alignment.center,
        decoration: BoxDecoration(
          gradient: LinearGradient(
            colors: [
              Colors.black.withOpacity(0.7),
              Colors.black.withOpacity(0.7),
            ],
            begin: Alignment.bottomCenter,
            end: Alignment.topCenter,
          ),
        ),
        child: FISText(
          content,
          style: _previewMemberTextStyle,
        ),
      ),
    );
  }
}
