import 'dart:async';

import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:get/get.dart';

class RecordButton extends StatelessWidget {
  final RxString _title = i18nBook.realTimeConsultation.captureVid.t.obs;
  BoxDecoration get _buttonDecoration => BoxDecoration(
      borderRadius: BorderRadius.all(
        Radius.circular(buttonGroupWidth / 2),
      ),
      border: Border.all(
          width: 1.5.s, color: const Color.fromARGB(255, 170, 170, 170)),
      color: const Color.fromARGB(255, 14, 16, 17));

  /// 录制一帧
  final Function onTap;

  /// 开始录制
  final Function onStartRecord;

  /// 结束录制
  final Function onStopRecord;

  /// 按钮组宽度
  final double buttonGroupWidth;

  /// 录制开始时间
  DateTime? startTime;

  /// 最大录制时长 s
  static const int maxRecordTime = 30;

  /// 录制剩余时间百分比
  RxDouble recordCountDown = 0.0.obs;

  /// 是否正在录制
  RxBool isRecording = false.obs;

  /// 录制计时器
  Timer? recordTimer;

  RecordButton({
    super.key,
    required this.onTap,
    required this.onStartRecord,
    required this.onStopRecord,
    required this.buttonGroupWidth,
  });

  @override
  Widget build(BuildContext context) {
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      child: GestureDetector(
        onTap: () {
          _title.value = i18nBook.realTimeConsultation.captureVid.t;
          onTap();
        },
        onLongPressStart: (details) {
          startRecord();
        },
        onLongPressEnd: (details) {
          stopRecord();
        },
        child: Container(
          width: buttonGroupWidth,
          height: buttonGroupWidth,
          margin: EdgeInsets.only(bottom: 10.s),
          decoration: _buttonDecoration,
          child: Stack(
            children: [
              Center(
                child: Obx(() => FISText(
                      _title.value,
                      style: TextStyle(
                        fontWeight: FontWeight.w300,
                        fontSize: 14.s,
                      ),
                    )),
              ),
              Obx(() => isRecording.value
                  ? Center(
                      child: SizedBox(
                        width: buttonGroupWidth - 5.s,
                        height: buttonGroupWidth - 5.s,
                        child: CircularProgressIndicator(
                          strokeWidth: 6.s,
                          valueColor:
                              const AlwaysStoppedAnimation<Color>(Colors.blue),
                          value: recordCountDown.value,
                        ),
                      ),
                    )
                  : const SizedBox())
            ],
          ),
        ),
      ),
    );
  }

  void startRecord() {
    _title.value = i18nBook.realTimeConsultation.recording.t;
    recordCountDown.value = 1;
    isRecording.value = true;
    onStartRecord();
    startTime = DateTime.now();

    /// 改用 recordTimer ，倒计时结束后取消 recordTimer
    recordTimer = Timer.periodic(const Duration(milliseconds: 100), (timer) {
      if (startTime != null) {
        recordCountDown.value = 1 -
            (DateTime.now().difference(startTime!).inMilliseconds /
                maxRecordTime /
                1000);
        if (recordCountDown <= 0) {
          stopRecord();
        }
      }
    });
  }

  void stopRecord() {
    recordTimer?.cancel();
    recordTimer = null;
    _title.value = i18nBook.realTimeConsultation.captureVid.t;
    isRecording.value = false;
    onStopRecord();
  }
}
