import 'package:fis_common/extensions/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationlistcontroller.dart';
import 'package:flyinsonolite/consultation/records/models/appointmentinformation.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:get/get.dart';

class AcceptConsultationController extends GetxController {
  final _appointmentManager = Get.find<IAppointmentManager>();
  List<OrganizationExpertDTO> expertList = [];
  Rx<OrganizationExpertDTO?> selectedExpert = Rx(null);
  String? assistantDoctorCode;
  Rx<String?> assistantDoctorName = Rx(null);
  String? description;
  Rx<TimeOfDay?> startTime = Rx(null);
  Rx<TimeOfDay?> endTime = Rx(null);

  /// 会诊时间
  late DateTime appointmentTime;

  late ConsultationDetailDTO currentConsultation;

  List<TimeOfDay?> timeSelectModelList = [];

  //提醒时间
  RxList<ConsultationReminderTimeEnum> selectedReminderTimes = RxList([]);
  //提醒方式
  RxList<ConsultationReminderMode> selectedReminderModes = RxList([]);

  RxList<ConsultationMemberDTO> paticipants = RxList([]);

  Future initializeAsync() async {
    try {
      var currentTime = DateTime.now();
      var delayedTime = currentTime.add(const Duration(minutes: 30));
      startTime.value = TimeOfDay.fromDateTime(_roundTime(currentTime)); //开始时间
      endTime.value = TimeOfDay.fromDateTime(_roundTime(delayedTime)); //结束时间
      _setTimePickerSource();
      selectedReminderTimes.value = [
        ConsultationReminderTimeEnum.FifteenMinutes
      ];
      selectedReminderModes.value = [ConsultationReminderMode.Application];
      selectedExpert.value = null;
      appointmentTime = currentConsultation.consultationTime!;
      description = currentConsultation.description;
      paticipants.value =
          currentConsultation.consultationMembers?.toList() ?? []; //否则
      if (currentConsultation.expertUserCode.isNullOrWhiteSpace) {
        await _updateExpertsAsync(currentConsultation.expertOrganizationCode!);
        assistantDoctorName.value = i18nBook.realTimeConsultation.noAssistant.t;
        assistantDoctorCode = '';
      } else {
        var result = await _appointmentManager
            .getUserByCodeAsync(currentConsultation.expertUserCode);
        assistantDoctorName.value = result.bindAssistantDoctorUserName ??
            i18nBook.realTimeConsultation.noAssistant.t;
        assistantDoctorCode = result.bindAssistantDoctorUserCode ?? '';
      }
    } catch (ex) {
      await handlerErrorAsync(
          ex, "AcceptConsultationController initializeAsync erro: $ex");
    }
  }

  DateTime _roundTime(DateTime currentTime) {
    int minutes = currentTime.minute;
    int hours = currentTime.hour;

    if (minutes < 15) {
      minutes = 15;
    } else if (minutes < 30) {
      minutes = 30;
    } else if (minutes < 45) {
      minutes = 45;
    } else {
      minutes = 0;
      hours = (hours + 1) % 24;
    }

    return DateTime(
        currentTime.year, currentTime.month, currentTime.day, hours, minutes);
  }

  Future _updateExpertsAsync(String organizationCode) async {
    var result = await _appointmentManager
        .findOrganizationExpertsAsync(organizationCode);
    expertList = result.pageData ?? [];
  }

  Future updateSelectedExpertAsync(OrganizationExpertDTO? expert) async {
    try {
      selectedExpert.value = expert;
      assistantDoctorName.value = '';
      assistantDoctorCode = '';
      currentConsultation.expertUserCode = expert?.userCode;
      if (selectedExpert.value != null) {
        var result = await _appointmentManager
            .getUserByCodeAsync(selectedExpert.value!.userCode);
        assistantDoctorName.value = result.bindAssistantDoctorUserName ??
            i18nBook.realTimeConsultation.noAssistant.t;
        assistantDoctorCode = result.bindAssistantDoctorUserCode ?? '';
      }
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          'AcceptConsultationController updateSelectedExpertAsync ex:$ex');
    }
  }

  Future approveAsync() async {
    if (startTime.value == null ||
        endTime.value == null ||
        (currentConsultation.expertUserCode.isNullOrWhiteSpace &&
            selectedExpert.value == null)) {
      PromptBox.toast(i18nBook.user.fullFillRequirements.t);
      return;
    }

    ApproveConsultationInfo approveInfo = ApproveConsultationInfo();
    approveInfo.consultationCode = currentConsultation.consultationCode;
    DateTime startDateTime = DateTime(
        appointmentTime.year,
        appointmentTime.month,
        appointmentTime.day,
        startTime.value!.hour,
        startTime.value!.minute);
    DateTime endDateTime = DateTime(appointmentTime.year, appointmentTime.month,
        appointmentTime.day, endTime.value!.hour, endTime.value!.minute);
    approveInfo.startTime = startDateTime;
    approveInfo.endTime = endDateTime;
    approveInfo.competentExperts = currentConsultation.expertUserCode;
    if (selectedExpert.value != null) {
      approveInfo.competentExperts = selectedExpert.value!.userCode;
    }
    approveInfo.participants =
        paticipants.map((element) => element.memberCode!).toList();
    approveInfo.description = description;
    approveInfo.consultationReminders = _setConsultationReminders();

    if (startDateTime.compareTo(DateTime.now()) == -1) {
      PromptBox.toast(i18nBook.realTimeConsultation.dateErrorNotice.t);
      return;
    }
    if (endDateTime.compareTo(startDateTime) == -1) {
      PromptBox.toast(i18nBook.realTimeConsultation.timeErrorNotice.t);
      return;
    }
    if (endDateTime == startDateTime) {
      PromptBox.toast(i18nBook.realTimeConsultation.timeErrorNoticeEqual.t);
      return;
    }
    try {
      var result =
          await _appointmentManager.approveConsultationAsync(approveInfo);
      if (result) {
        var controller = Get.find<ConsultationDetailController>();
        await controller.onTabChangedAsync(0);
        if (controller.consultationDetail.value != null) {
          var listController = Get.find<ConsultationListController>();
          listController.updateItem(controller.consultationDetail.value!);
        }
        Get.close(1);
      }
    } catch (ex) {
      await handlerErrorAsync(
          ex, 'AcceptConsultationController apprvoeAsyn error: $ex');
    }
  }

  ///设置时间下拉框的数据源
  void _setTimePickerSource() {
    timeSelectModelList = [];
    List<TimeOfDay> timeList = [];
    int totalMinute = 0;
    int minute = 0;
    int hour = 0;
    totalMinute = hour * 60 + minute;
    TimeOfDay time = TimeOfDay(hour: hour, minute: minute);
    for (int i = 0; i < 24 * 4; i++) {
      time = time.replacing(hour: hour, minute: minute);
      timeList.add(time);
      totalMinute += 15;
      minute = totalMinute % 60;
      hour = totalMinute ~/ 60;
      timeSelectModelList.add(timeList[i]);
    }
  }

  void updateEndTime() {
    var startTimeIndex = timeSelectModelList.indexOf(startTime.value);
    var totalCount = timeSelectModelList.length;
    var expectedIndex = startTimeIndex + 2;
    if (expectedIndex > totalCount - 1) {
      endTime.value = timeSelectModelList.last;
    } else {
      endTime.value = timeSelectModelList[expectedIndex];
    }
  }

  ///创建提醒方式和提醒时间的统一集合
  List<ConsultationReminderDTO> _setConsultationReminders() {
    List<ConsultationReminderDTO> reminderDtoList = [];
    for (var reminderMode in selectedReminderModes) {
      for (var reminderTime in selectedReminderTimes) {
        reminderDtoList.add(ConsultationReminderDTO(
            consultationReminderMode: reminderMode,
            consultationReminderTime: reminderTime));
      }
    }

    return reminderDtoList;
  }

  void updatePaticipants(List<UserExtendDTO> selectedExpertList) {
    paticipants.clear();
    for (var expert in selectedExpertList) {
      paticipants.add(ConsultationMemberDTO(
          memberCode: expert.userCode,
          memberOrganizationCode: expert.rootOrganizationCode,
          name: expert.displayName ?? expert.userName,
          headImageToken: expert.headImageUrl));
    }
  }
}
