import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class AddParticipantController extends GetxController {
  final _appointmentManager = Get.find<IAppointmentManager>();
  late ConsultationDetailDTO currentConsultation;
  List<ConsultationMemberDTO>? existExpertList;
  List<OrganizationBaseDTO> hospitals = [];
  Rx<OrganizationBaseDTO?> selectedHospital = Rx(null);
  RxList<UserExtendDTO> selectedExpertList = RxList();
  RxList<UserExtendDTO> selectableExpertList = RxList();
  Future initializeAsync() async {
    await _initReferOrganizationAsync();
  }

  ///获取所有可转诊机构的信息和所有可选择的参与人的信息
  Future _initReferOrganizationAsync() async {
    try {
      hospitals = await _appointmentManager.findParentOrganizationsAsync();
      hospitals.insert(
          0,
          OrganizationBaseDTO(
              organizationCode: Storage.user.organizationCode,
              organizationName:
                  i18nBook.realTimeConsultation.thisInstitution.t));
      selectedHospital.value = hospitals.first;
      await loadAllExistParticipantsAsync();
    } catch (e) {
      await handlerErrorAsync(e,
          "AddParticipantController initReferOrganizationAsync exception: $e");
    }
  }

  ///加载所有已参与人员及当前选择医院可选择的参与人员
  Future<void> loadAllExistParticipantsAsync() async {
    selectedExpertList.value = [];
    List<String> usersNeedToRemove = [
      currentConsultation.expertUserCode ?? "",
      currentConsultation.assistantDoctorUserCode ?? "",
      currentConsultation.applyUserCode ?? "",
    ];
    for (var hospital in hospitals) {
      if (selectedHospital.value == null) {
        selectableExpertList.value = [];
        continue;
      }
      var expertList = await _appointmentManager.getInviteableUserListAsyn(
          currentConsultation.consultationCode!, hospital.organizationCode!);
      usersNeedToRemove.removeWhere((element) => element.isEmpty);
      expertList.removeWhere(
          (element) => usersNeedToRemove.contains(element.userCode));
      if (selectedHospital.value == hospital) {
        selectableExpertList.value = expertList;
      }
      if (currentConsultation.consultationMembers != null) {
        for (var member in currentConsultation.consultationMembers!) {
          var exist = expertList.firstWhereOrNull(
              (element) => element.userCode == member.memberCode);
          if (exist != null && !selectedExpertList.contains(exist)) {
            selectedExpertList.add(exist);
          }
        }
      }
      if (existExpertList != null) {
        for (var expert in existExpertList!) {
          var exist = expertList.firstWhereOrNull(
              (element) => element.userCode == expert.memberCode);
          if (exist != null && !selectedExpertList.contains(exist)) {
            selectedExpertList.add(exist);
          }
        }
      }
    }
  }

  ///获取所有可选择的参与人员
  Future<void> getAllSelectableParticipantsAsync() async {
    if (selectedHospital.value == null) {
      selectableExpertList.value = [];
      return;
    }

    List<String> usersNeedToRemove = [
      currentConsultation.expertUserCode ?? "",
      currentConsultation.assistantDoctorUserCode ?? "",
      currentConsultation.applyUserCode ?? "",
    ];
    selectableExpertList.value =
        await _appointmentManager.getInviteableUserListAsyn(
            currentConsultation.consultationCode!,
            selectedHospital.value!.organizationCode!);
    usersNeedToRemove.removeWhere((element) => element.isEmpty);
    selectableExpertList
        .removeWhere((element) => usersNeedToRemove.contains(element.userCode));
  }
}
