import 'package:fis_common/extensions/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_jsonrpc/services/liveConsultation.m.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationlistcontroller.dart';
import 'package:flyinsonolite/services/patientdataservice.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/localization/localization.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/managers/interfaces/ipatientController.dart';
import 'package:flyinsonolite/consultation/records/models/appointmentinformation.dart';
import 'package:flyinsonolite/consultation/records/models/patientinfo.dart';
import 'package:flyinsonolite/consultation/records/models/selectmodel.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class ApplyConsultationController extends GetxController
    implements IPatientController {
  final _consulttationConstant = 'consultation';
  final _appointmentManager = Get.find<IAppointmentManager>();
  final _consultationListController = Get.find<ConsultationListController>();
  ConsultationDetailDTO? _currentConsultationDetailDTO;
  @override
  Map<String, String> patient = {'AgeUnits': "Year"};
  @override
  Rx<List<FISSelectModel>> sexList = Rx([]);
  @override
  Rx<FISSelectModel?> selectedSex = Rx(null);
  @override
  Rx<List<FISSelectModel>> ageUnitList = Rx([]);
  @override
  Rx<FISSelectModel?> selectedAgeUnit = Rx(null);
  Map<String, TextEditingController> textEditingControllers = {};
  List<DeviceInfoDTO> deviceInfoList = [];
  Rx<DeviceInfoDTO?> selectedDeviceInfo = Rx(null);
  AppointmentInformation appointmentInformation = AppointmentInformation();
  List<UserExtendDTO> scanDoctorList = [];
  Rx<UserExtendDTO?> selectedScanDoctor = Rx(null);
  List<OrganizationBaseDTO> parentOrganizationList = [];
  Rx<OrganizationBaseDTO?> selectedParentOrganization = Rx(null);
  List<String> scanPositionList = [];
  Rx<List<String>> selectedScanPositionList = Rx<List<String>>([]);
  Rx<List<OrganizationExpertDTO>> expertList =
      Rx<List<OrganizationExpertDTO>>([]);
  Rx<OrganizationExpertDTO?> selectedExpert = Rx(null);
  List<OrganizationExpertDTO> applicantList = [];
  Rx<OrganizationExpertDTO?> selectedApplicant = Rx(null);
  @override
  PatientInfo currentPatient = PatientInfo(
    isValid: true,
    createTime: DateTime.now(),
    patientCode: '',
    ageUnits: 'Year',
  );
  @override
  RxBool isEdit = false.obs;
  RxBool isImprove = false.obs;
  TextEditingController scanPositionTextEditingController =
      TextEditingController();

  void updatePatientInfo(PatientInfo patientInfo) {
    currentPatient = patientInfo;
    patient['Name'] = patientInfo.name ?? '';
    var nameTextEditController = getTextEditingController("Name", "");
    nameTextEditController.text = patient['Name']!;
    patient['Phone'] = patientInfo.phone ?? '';
    var phoneTextEditController = getTextEditingController("Phone", "");
    phoneTextEditController.text = patient['Phone']!;
    patient['AgeUnits'] = patientInfo.ageUnits.isNullOrWhiteSpace
        ? 'Year'
        : patientInfo.ageUnits!;
    selectedAgeUnit.value = ageUnitList.value
        .firstWhereNullable((element) => element.key == patient["AgeUnits"]);
    var ageYearTextEditController = getTextEditingController("AgeYear", "");
    ageYearTextEditController.text = patientInfo.ageYear ?? "";
    var ageMonthTextEditController = getTextEditingController("AgeMonth", "");
    ageMonthTextEditController.text = patientInfo.ageMonth ?? "";
    var ageWeekTextEditController = getTextEditingController("AgeWeek", "");
    ageWeekTextEditController.text = patientInfo.ageWeek ?? "";
    var ageDayTextEditController = getTextEditingController("AgeDay", "");
    ageDayTextEditController.text = patientInfo.ageDay ?? "";
    switch (patientInfo.ageUnits) {
      case 'YearAndMonth':
        patient["AgeYear"] = patientInfo.ageYear ?? "";
        patient["AgeMonth"] = patientInfo.ageMonth ?? "";
        patient.removeWhere((key, value) => key == "AgeWeek");
        patient.removeWhere((key, value) => key == "AgeDay");
        break;
      case 'MonthAndWeek':
        patient["AgeMonth"] = patientInfo.ageMonth ?? "";
        patient["AgeWeek"] = patientInfo.ageWeek ?? "";
        patient.removeWhere((key, value) => key == "AgeYear");
        patient.removeWhere((key, value) => key == "AgeDay");
        break;
      case 'WeekAndDay':
        patient["AgeWeek"] = patientInfo.ageDay ?? "";
        patient["AgeDay"] = patientInfo.ageWeek ?? "";
        patient.removeWhere((key, value) => key == "AgeYear");
        patient.removeWhere((key, value) => key == "AgeMonth");
        break;
      default:
        patient["AgeYear"] = patientInfo.ageYear ?? "";
        patient.removeWhere((key, value) => key == "AgeMonth");
        patient.removeWhere((key, value) => key == "AgeWeek");
        patient.removeWhere((key, value) => key == "AgeDay");
        break;
    }
    patient['Sex'] = patientInfo.sex ?? '';
    selectedSex.value = sexList.value
        .firstWhereNullable((element) => element.key == patient["Sex"]);
    patient['Anamnesis'] = patientInfo.anamnesis ?? '';
    var anamnesisTextEditController = getTextEditingController("Anamnesis", "");
    anamnesisTextEditController.text = patient['Anamnesis']!;
    patient['IdentityCard'] = patientInfo.identityCard ?? '';
    var identityCardTextEditController =
        getTextEditingController("IdentityCard", "");
    identityCardTextEditController.text = patient['IdentityCard']!;
  }

  @override
  TextEditingController getTextEditingController(
      String key, String defaultValue) {
    if (textEditingControllers.containsKey(key)) {
      var existTextEditController = textEditingControllers[key]!;
      existTextEditController.text = defaultValue;
      return existTextEditController;
    }

    var newTextEditingController = TextEditingController(text: defaultValue);
    textEditingControllers[key] = newTextEditingController;
    return newTextEditingController;
  }

  Future initializeAsync() async {
    EasyLoading.show(status: i18nBook.common.loading.t);
    try {
      scanPositionTextEditingController.text = '';
      _currentConsultationDetailDTO = null;
      appointmentInformation = AppointmentInformation();
      patient = {'AgeUnits': "Year"};
      selectedSex.value = null;
      selectedAgeUnit.value = null;
      selectedDeviceInfo.value = null;
      selectedScanDoctor.value = null;
      selectedApplicant.value = null;
      selectedScanPositionList.value = [];
      selectedParentOrganization.value = null;
      selectedExpert.value = null;
      expertList.value = [];
      currentPatient = PatientInfo(
        isValid: true,
        createTime: DateTime.now(),
        patientCode: '',
        ageUnits: 'Year',
      );
      var deviceList =
          await _appointmentManager.getDeviceListByPersonRoleAsync();
      if (deviceList.pageData != null) {
        deviceInfoList = deviceList.pageData!;
      }
      scanDoctorList = await _appointmentManager.getScanDoctorsAsync();
      parentOrganizationList =
          await _consultationListController.findParentOrganizationsAsync();
      scanPositionList = await _appointmentManager.getScanPositionsAsync();
      applicantList = await _appointmentManager.findAssistantExpertsAsync();
      if (isEdit.value || isImprove.value) {
        var consultationDetailController =
            Get.find<ConsultationDetailController>();
        _currentConsultationDetailDTO =
            consultationDetailController.consultationDetail.value;
        _getPatientValue(_currentConsultationDetailDTO);
        currentPatient = PatientInfo(
            createTime:
                _currentConsultationDetailDTO?.createTime ?? DateTime.now(),
            isValid: true,
            patientCode: _currentConsultationDetailDTO?.patientCode ?? "",
            phone: patient["Phone"],
            name: patient["Name"],
            insuranceCode: patient['InsuranceCode'],
            birthday: patient['Birthday'],
            anamnesis: patient['Anamnesis'],
            ageUnits: patient['AgeUnits'],
            identityCard: patient['IdentityCard'],
            headImgUrl: patient['HeadImgUrl'],
            species: patient['Species'],
            breed: patient['Breed'],
            sex: patient['Sex'],
            ageYear: patient["AgeYear"],
            ageMonth: patient["AgeMonth"],
            ageWeek: patient["AgeWeek"],
            ageDay: patient["AgeDay"]);
        appointmentInformation.appointmentDevice =
            _currentConsultationDetailDTO?.deviceCode;
        selectedDeviceInfo.value = deviceInfoList.firstWhereOrNull((element) =>
            element.deviceCode == appointmentInformation.appointmentDevice);
        appointmentInformation.appointmentTime =
            _currentConsultationDetailDTO?.consultationTime;
        selectedScanDoctor.value = scanDoctorList.firstWhereOrNull((element) =>
            element.userCode == _currentConsultationDetailDTO?.scanUserCode);
        appointmentInformation.appointmentScanDoctor =
            selectedScanDoctor.value?.userCode;
        appointmentInformation.appointmentDiseaseName =
            _currentConsultationDetailDTO?.diseases;
        appointmentInformation.appointmentScanningPosition =
            _currentConsultationDetailDTO?.scanPositions;
        selectedScanPositionList.value =
            _currentConsultationDetailDTO?.scanPositions ?? [];
        appointmentInformation.appointmentConsultationInstitution =
            _currentConsultationDetailDTO?.expertOrganizationCode;
        selectedParentOrganization.value =
            parentOrganizationList.firstWhereOrNull((element) =>
                element.organizationCode ==
                appointmentInformation.appointmentConsultationInstitution);
        if (_currentConsultationDetailDTO?.expertOrganizationCode != null) {
          await updateExpertsAsync(
              _currentConsultationDetailDTO!.expertOrganizationCode!);
        }

        appointmentInformation.appointmentCompetentExperts =
            _currentConsultationDetailDTO?.expertUserCode;
        selectedExpert.value = expertList.value.firstWhereOrNull((element) =>
            element.userCode == _currentConsultationDetailDTO?.expertUserCode);
        selectedApplicant.value = applicantList.firstWhereOrNull((element) =>
            element.userCode == _currentConsultationDetailDTO?.applyUserCode);
        appointmentInformation.appointmentApplicant =
            selectedApplicant.value?.userCode;
        appointmentInformation.preliminaryDiagnosis =
            _currentConsultationDetailDTO?.primaryDiagnosis;
        scanPositionTextEditingController.text = selectedScanPositionList.value
            .map((data) => getLanguageValues(_consulttationConstant, data))
            .join(";");
      } else {
        appointmentInformation.appointmentDevice = Storage.usDeviceCode;
        selectedDeviceInfo.value = deviceInfoList.firstWhereOrNull(
            (element) => element.deviceCode == Storage.usDeviceCode);
        selectedScanDoctor.value = scanDoctorList.firstWhereOrNull(
            (element) => element.userCode == Storage.user.userCode);
        appointmentInformation.appointmentScanDoctor =
            selectedScanDoctor.value?.userCode;
        selectedApplicant.value = applicantList.firstWhereOrNull(
            (element) => element.userCode == Storage.user.userCode);
        appointmentInformation.appointmentApplicant =
            selectedApplicant.value?.userCode;
        if (parentOrganizationList.length == 1) {
          selectedParentOrganization.value = parentOrganizationList.first;
          appointmentInformation.appointmentConsultationInstitution =
              selectedParentOrganization.value!.organizationCode;
          await updateExpertsAsync(
              selectedParentOrganization.value?.organizationCode ?? "");
          if (expertList.value.length == 1) {
            selectedExpert.value = expertList.value.first;
            appointmentInformation.appointmentCompetentExperts =
                selectedExpert.value!.userCode;
          }
        }
      }
    } catch (ex) {
      await handlerErrorAsync(ex, 'ConsultationManager initializeAsync ex:$ex');
    } finally {
      EasyLoading.dismiss();
    }
  }

  Future updateExpertsAsync(String organizationCode) async {
    var result = await _appointmentManager
        .findOrganizationExpertsAsync(organizationCode);
    selectedExpert.value = null;
    expertList.value = result.pageData ?? [];
  }

  /// 创建申请
  Future<void> createOrUpdateOrImproveConsultationAsync() async {
    try {
      /// 如果不是专家助理，而是专家或者医生，那申请人是自己
      if (!isImprove.value && !Storage.user.isApplyUserField) {
        appointmentInformation.appointmentApplicant = Storage.user.userCode!;
      }

      if (!_isCanApplyConsultation()) {
        PromptBox.toast(
            i18nBook.realTimeConsultation.requiredItemsNotCompleted.t);
        return;
      }

      if (isEdit.value) {
        //获取今天的时间，从0点开始
        DateTime dateTimeNow = DateTime(DateTime.now().year,
            DateTime.now().month, DateTime.now().day, 0, 0, 0);
        //如果会诊时间早于当前时间则弹出提示
        if (appointmentInformation.appointmentTime!.compareTo(dateTimeNow) ==
            -1) {
          PromptBox.toast(i18nBook.realTimeConsultation.dateErrorNotice.t);
          return;
        }
      }

      EasyLoading.show(status: i18nBook.common.loading.t);
      if (isEdit.value) {
        final result = await _appointmentManager.updateConsultationAsync(
            _currentConsultationDetailDTO?.consultationCode,
            appointmentInformation);
        if (result) {
          PromptBox.toast(i18nBook.user.updateSuccess.t);
          await Future.delayed(const Duration(milliseconds: 500));
          Get.close(1);
          var controller = Get.find<ConsultationDetailController>();
          await controller.onTabChangedAsync(0);
          if (controller.consultationDetail.value != null) {
            var listController = Get.find<ConsultationListController>();
            listController.updateItem(controller.consultationDetail.value!);
          }
        }
      } else if (isImprove.value) {
        final result = await _appointmentManager.improveConsultationInfoAsync(
            currentPatient.patientCode,
            _currentConsultationDetailDTO!.consultationCode!,
            appointmentInformation,
            _appointmentManager.getPatientDatas(patient));
        if (result) {
          PromptBox.toast(i18nBook.user.updateSuccess.t);
          Get.close(1);
          await _consultationListController.refreshAsync();
          var consultationDetailController =
              Get.find<ConsultationDetailController>();
          await consultationDetailController.onTabChangedAsync(0);
        }
      } else {
        final result = await _appointmentManager.applyConsultationAsync(
            currentPatient.patientCode,
            appointmentInformation,
            _appointmentManager.getPatientDatas(patient));
        if (result.isNotEmpty) {
          PromptBox.toast(i18nBook.remedical.createSuccess.t);
          await Future.delayed(const Duration(milliseconds: 500));
          Get.close(1);
          await _consultationListController.refreshAsync();
          var consultationDetailController =
              Get.find<ConsultationDetailController>();
          consultationDetailController.onTabChangedAsync(0);
        }
      }
    } catch (e) {
      var errorMsg =
          "ApplyConsultationController createOrUpdateOrImproveConsultationAsync exception:$e";
      printError(info: errorMsg);
      await handlerErrorAsync(e, errorMsg);
    }
  }

  /// 是否能够申请预约单
  bool _isCanApplyConsultation() {
    // 遍历预约信息中的必填项列表
    var appointmentKeyList = Storage.configurationFromerver
            ?.consultationServerConfig.appointmentKeyList ??
        [];
    for (int i = 0; i < appointmentKeyList.length; i++) {
      // 如果当前项目是必填项
      if (appointmentKeyList[i].required) {
        // 检查对应的预约信息字段是否为空
        switch (appointmentKeyList[i].key) {
          case 'Device':
            if (appointmentInformation.appointmentDevice?.isEmpty ?? true) {
              // 如果必填字段为空，返回 false
              return false;
            }
            break;
          case 'ConsultationTime':
            if (appointmentInformation.appointmentTime == null) {
              return false;
            }
            break;
          case 'ScanUser':
            if (appointmentInformation.appointmentScanDoctor?.isEmpty ?? true) {
              return false;
            }
            break;
          case 'Diseases':
            if (appointmentInformation.appointmentDiseaseName?.isEmpty ??
                true) {
              return false;
            }
            break;
          case 'ScanPosition':
            if (appointmentInformation.appointmentScanningPosition?.isEmpty ??
                true) {
              return false;
            }
            break;
          case 'ExpertOrganization':
            if (appointmentInformation
                    .appointmentConsultationInstitution?.isEmpty ??
                true) {
              return false;
            }
            break;
          case 'Expert':
            if (appointmentInformation.appointmentCompetentExperts?.isEmpty ??
                true) {
              return false;
            }
            break;
          case 'ApplyUser':
            if (appointmentInformation.appointmentApplicant?.isEmpty ?? true) {
              return false;
            }
            break;

          case 'PrimaryDiagnosis':
            if (appointmentInformation.preliminaryDiagnosis?.isEmpty ?? true) {
              return false;
            }
            break;

          case 'ApplyOrganization':
            break;
        }
      }
    }
    // 遍历病人信息中的必填项列表
    var patientKeyList = Storage
            .configurationFromerver?.consultationServerConfig.patientKeyList ??
        [];
    for (int i = 0; i < patientKeyList.length; i++) {
      // 如果当前项目是必填项
      if (patientKeyList[i].required) {
        if (patient[patientKeyList[i].key].isNullOrWhiteSpace) {
          // 如果必填字段为空，返回 false
          return false;
        }
      }
    }

    // 如果所有必填字段都已填写，返回 true
    return true;
  }

  /// 获取病人数据
  void _getPatientValue(ConsultationDetailDTO? consultationDetail) {
    patient.clear();
    if (consultationDetail == null) {
      return;
    }
    var patientData = consultationDetail.patientDatas ?? [];
    patient = PatientDataService.getPatientValue(patientData);
  }
}
