import 'dart:convert';
import 'package:fis_common/extensions/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_i18n/modules/error_codes.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationlistcontroller.dart';
import 'package:flyinsonolite/helpers/dialoghelper.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/report_template_document.dart';
import 'package:flyinsonolite/services/patientdataservice.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/consultation.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/managers/interfaces/ireportmanager.dart';
import 'package:flyinsonolite/consultation/records/models/datetime.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/helpers/pdfhelper.dart';
import 'package:flyinsonolite/infrastructure/routeconstants.dart';
import 'package:flyinsonolite/infrastructure/filestorage.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:universal_html/html.dart' as html;

class ConsultationDetailController extends GetxController
    with GetTickerProviderStateMixin {
  static const String C_REPORTFORMAT = ".pdf";
  final _reportManager = Get.find<IReportManager>();
  final _appointmentManager = Get.find<IAppointmentManager>();
  final _consultationListController = Get.find<ConsultationListController>();
  bool _isJoin = false;
  bool get isJoin => _isJoin;
  final fileStorage = FileStorage.createInstance("remedical/report");
  ReportDataPipe get reportDataPipe => ReportDataPipe(
      cacheGet: fileStorage.getFile, cacheSave: fileStorage.setFile);
  Future<bool> Function(List<int> data, String fileName)? get shellExporter =>
      TextMediaFileExporterForShell.exportFile;

  /// tab选择器
  late TabController tabController = TabController(
    vsync: this,
    length: tabTitles.length,
    initialIndex: 0,
  );

  /// 标签页列表
  List<String> get tabTitles => [
        fisI18nBook.common.consultationDetail.t,
        fisI18nBook.common.consultationImage.t,
        fisI18nBook.common.consultationReport.t,
        fisI18nBook.common.consultationFollowUp.t,
      ];
  Rx<Map<String, String>> patient = Rx({});
  Rx<ConsultationDetailDTO?> consultationDetail = Rx(null);
  Rx<int> currentTabIndex = Rx(0);

  ///实时会诊的图片
  RxList<ConsultationImagesDTO> realTimeConsultationImageList = RxList([]);

  //报告
  RxList<ReportDTO> reports = RxList([]);

  //随访
  RxList<FollowUpVisitDTO> followUpVisits = RxList([]);

  Future onTabChangedAsync(int index) async {
    var selectedItem = _consultationListController.selectedItem.value;
    if (selectedItem == null) {
      return;
    }
    EasyLoading.show(status: i18nBook.common.loading.t);
    try {
      tabController.index = index;
      consultationDetail.value = await _appointmentManager
          .findConsultationDetailAsync(selectedItem.consultationCode);
      switch (index) {
        ///获取会诊详情
        case 0:
          _getPatientValue();
          break;

        ///获取会诊图像
        case 1:
          realTimeConsultationImageList.value = await _appointmentManager
              .getConsultationImagesAsync(selectedItem.consultationCode);
          break;

        ///获取报告信息
        case 2:
          reports.value = await _reportManager
              .findReportsAsync(selectedItem.consultationCode);
          break;

        ///获取随访信息
        case 3:
          followUpVisits.value = await _appointmentManager
              .getFollowUpVisitInfo(selectedItem.consultationCode);
          break;
      }

      currentTabIndex.value = index;
    } finally {
      EasyLoading.dismiss();
    }
  }

  Future updateDataAsync() async {
    await onTabChangedAsync(currentTabIndex.value);
  }

  void withdraw() {
    DialogHelper.showConfirmDialog(Get.context!,
        i18nBook.realTimeConsultation.sureCancelCurrentConsultation.t,
        () async {
      try {
        Get.close(1);
        var result = await _appointmentManager.withdrawConsultationAsync(
            consultationDetail.value!.consultationCode);
        if (result) {
          PromptBox.toast(i18nBook.common.revokeSuccess.t);
          await onTabChangedAsync(0);
          if (consultationDetail.value != null) {
            _consultationListController.updateItem(consultationDetail.value!);
          }
        }
      } catch (ex) {
        printError(
            info: "ConsultationDetailController withdrawAsync exception:$ex");
        await handlerErrorAsync(
            ex, "ConsultationDetailController withdrawAsync exception: ex");
      }
    });
  }

  Future refuseConsultationAsync(String? reason, bool isThirdParty) async {
    try {
      if (consultationDetail.value == null) {
        return;
      }
      if (reason.isNullOrWhiteSpace) {
        PromptBox.toast(i18nBook.realTimeConsultation.reasonsCannotBeNull.t);
        return;
      }
      bool result;
      if (isThirdParty) {
        result = await _appointmentManager.acceptInvitationAsync(
            consultationDetail.value!.consultationCode, false, reason);
      } else {
        result = await _appointmentManager.refuseConsultationAsync(
            consultationDetail.value!.consultationCode, reason);
      }
      if (result) {
        Get.close(1);
        if (isThirdParty) {
          PromptBox.toast(
              i18nBook.realTimeConsultation.declinedTheInvitation4Toast.t);
        } else {
          PromptBox.toast(i18nBook.remedical.refused.t);
        }
        await onTabChangedAsync(0);
        if (consultationDetail.value != null) {
          _consultationListController.updateItem(consultationDetail.value!);
        }
      }
    } catch (ex) {
      var errorMsg =
          "ConsultationDetailController refuseConsultationAsync fail,isThirdParty:$isThirdParty, exception:$ex";
      printError(info: errorMsg);
      await handlerErrorAsync(ex, errorMsg);
    }
  }

  /// 获取病人数据
  void _getPatientValue() {
    patient.value.clear();
    if (consultationDetail.value == null) {
      return;
    }
    var patientData = consultationDetail.value!.patientDatas ?? [];
    patient.value = PatientDataService.getPatientValue(patientData);
  }

  /// 是否展示会诊描述和参与人员
  bool isShowDescriptionAndParticipants() {
    if (consultationDetail.value == null ||
        consultationDetail.value!.description.isNullOrWhiteSpace) {
      return false;
    }

    List<TransactionStatusEnum> consultationStatus = [
      TransactionStatusEnum.Applied,
      TransactionStatusEnum.Rejected,
      TransactionStatusEnum.Withdrawn,
    ];
    return !consultationStatus.any(
        (element) => element == consultationDetail.value!.consultationStatus);
  }

  Future startConsultationAsyn({bool isJoin = false}) async {
    try {
      _isJoin = isJoin;
      var liveConsultationManager = Get.find<ILiveConsultationManager>();
      var result = await liveConsultationManager.startConsultationAsync(
          consultationDetail.value!,
          checkMedia: true,
          checkOnly: true,
          isJoin: isJoin);
      if (result == null) {
        return;
      }
      if (Get.isDialogOpen == true) {
        Get.close(1);
      }
      if (Storage.usDeviceCode == consultationDetail.value!.deviceCode!) {
        try {
          var result = await jsonRpcProxyForFISLib.consultation
              .startLiveConsultationFromFlutter(FISLiveConsultationRequestDTO(
                  consultationCode: consultationDetail.value!.consultationCode,
                  token: Storage.user.token,
                  isEmergency: consultationDetail.value!.isEmergency,
                  isInvite: !isJoin,
                  isJoin: isJoin));
          if (!result.isSuccess) {
            await loggerAsyn(FISDeviceLogCategory.Error,
                "jsonRpcProxyForFISLib startLiveConsultationFromFlutter Fail,FailMessage:${result.failMessage}");
            if (result.errorCode > 0) {
              var text = i18nBook
                  .getTrItem(ErrorCodesModule.ModuleName,
                      'errorCode${result.errorCode}')
                  ?.t;
              PromptBox.toast(text ?? "Unkonwn RPC Exception");
            } else {
              PromptBox.toast(
                  "jsonRpcProxyForFISLib startLiveConsultationFromFlutter Fail,FailMessage:${result.failMessage}");
            }
            return;
          } else {
            liveConsultationManager.consultationDetail =
                consultationDetail.value!;
            liveConsultationManager.currentConsultationCode =
                consultationDetail.value!.consultationCode!;
            Storage.isConsultating.value = true;
            Storage.isUseFISForConsultation.value = true;
          }
        } catch (ex) {
          await loggerAsyn(FISDeviceLogCategory.Error,
              "jsonRpcProxyForFISLib startLiveConsultationFromFlutter Fail: $ex");
        }
      } else if (consultationDetail.value!.deviceCode != Storage.usDeviceCode &&
          Storage.hasMultiScreens) {
        //final url = html.window.location.origin;
        //html.window.open(
        // '$url/#/liveconsultation?page=liveconsultation&consultationCode=${consultationDetail.value!.consultationCode}&isJoin=false&token=${Storage.user.token}&language=${FI18n.ins.currentBook.locale.languageCode}&theme=${Storage.currentTheme.themeName}',
        //  '');
        //Storage.liveConsultationInSecondView.value=true;
        //return;
        Storage.isConsultating.value = true;
        liveConsultationManager.consultationDetail = consultationDetail.value!;
        await sendFlutterLiveConsultationInfo();
      } else {
        liveConsultationManager.consultationDetail = consultationDetail.value!;
        Get.toNamed(RouteConstants.consultationRouteConstants.liveconsultation,
            parameters: {"isjoin": isJoin.toString()});
      }
    } catch (ex) {
      await handlerErrorAsync(
          ex, 'ConsultationDetailController startConsultationAsyn error: $ex');
    }
  }

  Future<void> sendFlutterLiveConsultationInfo() async {
    try {
      final url = html.window.location.origin;
      var result = await jsonRpcProxyForFISLib.consultation
          .sendFlutterLiveConsultationInfo(FISFlutterLiveConsultationInfo(
              isStartLiveConsultation: true,
              token: Storage.user.token,
              consultationCode: consultationDetail.value!.consultationCode,
              liveConsultationFlutterUrl:
                  '$url/#/liveconsultation?page=liveconsultation&consultationCode=${consultationDetail.value!.consultationCode}&isJoin=$isJoin&token=${Storage.user.token}&language=${FI18n.ins.currentBook.locale.languageCode}&theme=${Storage.currentTheme.themeName}'));
      if (!result.isSuccess) {
        await loggerAsyn(FISDeviceLogCategory.Error,
            "ConsultationDetailController sendFlutterLiveConsultationInfoAsync ex:${result.failMessage}");
        Get.toNamed(RouteConstants.consultationRouteConstants.liveconsultation,
            parameters: {"isjoin": isJoin.toString()});
      }
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationDetailController sendFlutterLiveConsultationInfo Error:$ex");
      Get.toNamed(RouteConstants.consultationRouteConstants.liveconsultation,
          parameters: {"isjoin": isJoin.toString()});
    }
  }

  /// 获取预约会诊时间
  String getConsultationTime() {
    //如果会诊申请没有通过，会诊状态是待审核或者已拒绝或者已撤回，则只显示日期，不显示具体时间
    if (consultationDetail.value == null) return '--';
    if (!_isConsultationAccepted(
        consultationDetail.value?.consultationStatus)) {
      return DateFormat('yyyy-MM-dd')
          .format(consultationDetail.value!.consultationTime!.toLocal());
    } else {
      if (consultationDetail.value?.consultationTimeEnd == null) return '--';
      return formatDateRangeWithYear(
          consultationDetail.value?.consultationTime!.toLocal(),
          consultationDetail.value?.consultationTimeEnd!.toLocal());
    }
  }

  /// 会诊是否被接受
  bool _isConsultationAccepted(TransactionStatusEnum? e) {
    //会诊状态
    List<TransactionStatusEnum> consultationStatus = [
      TransactionStatusEnum.Applied,
      TransactionStatusEnum.Rejected,
      TransactionStatusEnum.Withdrawn,
    ];
    return !consultationStatus.any((element) => element == e);
  }

  /// 直接打印报告
  Future printReportDirectlyAsync(ReportDTO reportInfo) async {
    try {
      if (reportInfo.reportPreviewList == null ||
          reportInfo.reportPreviewList!.isEmpty) {
        reportInfo =
            await _reportManager.findReportByCodeAsync(reportInfo.reportCode!);
        await _initPreviewReport(reportInfo);
        PDFHelper.printPDF(
            _generateReaportFileName(reportInfo), reportDataPipe);
      } else {
        PDFHelper.printPDFJpges(_generateReaportFileName(reportInfo),
            _generateReportUrl(reportInfo));
      }
    } catch (e) {
      await handlerErrorAsync(
          e, "ConsultationDetailController printReportDirectly error: $e");
    }
  }

  Future<void> _initPreviewReport(ReportDTO reportInfo) async {
    try {
      final jsonStr = reportInfo.reportDatasJson;
      if (jsonStr != null && jsonStr.isNotNullOrWhiteSpace) {
        final json = jsonDecode(jsonStr);

        var reportTemplate = reportInfo.reportTemplateJson;

        var reportTempalteDoc =
            ReportTemplateDocument.fromJson(jsonDecode(reportTemplate!));
        FReportInfo.instance.init(
          reportTempalteDoc,
          DateTime.now(),
          "",
          revoke: i18nBook.common.revoke.t,
          selectEntry: i18nBook.remedical.selectWord.t,
          selectImageHint: i18nBook.remedical.clickAndSelectImage.t,
        );
        FReportInfo.instance.fromJson(json);
      }
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "ConsultationDetailController _initPreviewReport error: $e");
    }
  }

  ///不打开报告预览页面，直接导出
  Future exportReportDirectlyAsync(ReportDTO reportInfo) async {
    try {
      if (reportInfo.reportPreviewList == null ||
          reportInfo.reportPreviewList!.isEmpty) {
        reportInfo =
            await _reportManager.findReportByCodeAsync(reportInfo.reportCode!);
        await _initPreviewReport(reportInfo);
        PDFHelper.downloadPDF(_generateReaportFileName(reportInfo),
            reportDataPipe, shellExporter);
      } else {
        PDFHelper.downloadPDFJpges(_generateReaportFileName(reportInfo),
            _generateReportUrl(reportInfo), shellExporter);
      }
    } catch (e) {
      await handlerErrorAsync(
          e, 'ConsultationDetailController exportReportDirectly ex: $e');
    }
  }

  String _generateReaportFileName(ReportDTO reportInfo) {
    DateFormat format = DateFormat("yyyMMddHHmmss");
    var dateStr = format.format(reportInfo.reportTime!.toLocal());
    var patientName = consultationDetail.value?.patientName ?? "";
    var fileName =
        "[${patientName.replaceAll("*", "x")}]-[$dateStr]-[${reportInfo.reportCode!}]$C_REPORTFORMAT";
    return fileName;
  }

  List<String>? _generateReportUrl(ReportDTO reportDTO) {
    List<String>? reportImageList;
    var images = reportDTO.reportPreviewList
            ?.where((element) => element.fileType == UploadFileTypeEnum.JPG) ??
        [];
    if (images.isNotEmpty) {
      reportImageList = [];
      var imageList = images.map((e) => e.fileToken ?? '').toList();
      for (var image in imageList) {
        if (image.contains(';')) {
          reportImageList
              .addAll(image.split(';').where((element) => element.isNotEmpty));
        } else {
          reportImageList.add(image);
        }
      }
    }

    return reportImageList;
  }

  /// 同意预约会诊邀约
  Future<void> agreeInvitationAsync() async {
    try {
      if (consultationDetail.value == null) {
        return;
      }
      var result = await _appointmentManager.acceptInvitationAsync(
          consultationDetail.value!.consultationCode, true, null);
      if (result) {
        PromptBox.toast(
            i18nBook.realTimeConsultation.acceptedTheInvitation4Toast.t);
        await onTabChangedAsync(0);
        if (consultationDetail.value != null) {
          _consultationListController.updateItem(consultationDetail.value!);
        }
      }
    } catch (e) {
      var errorMsg =
          'ConsultationDetailController agreeInvitationAsync error: $e';
      printError(info: errorMsg);
      await handlerErrorAsync(e, errorMsg);
    }
  }
}
