import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/localization/localization.dart';
import 'package:flyinsonolite/consultation/records/models/datetime.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';

class ConsultationViewModel {
  /// 会诊code
  String consultationCode;

  /// 病人名
  String? patientName;
  String? patientCode;
  String? age;
  String? gender;
  String? phone;
  String? expertUserName;
  DateTime? consultationTime;
  DateTime? consultationTimeEnd;
  String? applyOrganizationName;
  Rx<TransactionStatusEnum?> state = Rx(null);
  EvaluateGradeEnum? consultationEvaluate;

  String get displayInfo => _getDisplayInfo();

  String get timeInfo => _getConsultationTime();

  ConsultationViewModel({
    required this.consultationCode,
    this.patientName,
    this.patientCode,
    this.gender,
    this.age,
    this.phone,
    this.expertUserName,
    this.consultationTime,
    this.consultationTimeEnd,
    this.applyOrganizationName,
    this.consultationEvaluate,
  });

  String _getDisplayInfo() {
    return '${getLanguageValues('consultation', "ApplyOrganization")}：$applyOrganizationName';
  }

  /// 获取预约会诊时间
  String _getConsultationTime() {
    //如果会诊申请没有通过，会诊状态是待审核或者已拒绝或者已撤回，则只显示日期，不显示具体时间
    if (consultationTime == null) return '--';
    if (state.value == TransactionStatusEnum.Applied) {
      return DateFormat('MM-dd').format(consultationTime!.toLocal());
    }
    if (_isConsultationAccepted(state.value)) {
      if (consultationTimeEnd == null) return '--';
      return formatDateRangeWithYear(
          consultationTime!.toLocal(), consultationTimeEnd!.toLocal(),
          simpleFormat: true);
    } else {
      return '--';
    }
  }

  /// 会诊是否被接受
  bool _isConsultationAccepted(TransactionStatusEnum? e) {
    //会诊状态
    List<TransactionStatusEnum> consultationStatus = [
      TransactionStatusEnum.Applied,
      TransactionStatusEnum.Rejected,
      TransactionStatusEnum.Withdrawn,
    ];
    return !consultationStatus.any((element) => element == e);
  }
}
