import 'package:fis_common/extensions/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/index.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/controllers/acceptconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/addparticipantcontroller.dart';
import 'package:flyinsonolite/consultation/records/views/addparticipantdialog.dart';
import 'package:flyinsonolite/controls/custom/customdropdownbutton.dart';
import 'package:flyinsonolite/controls/custom/customtextbutton.dart';
import 'package:flyinsonolite/controls/select/fisdropdownbutton.dart';
import 'package:flyinsonolite/controls/select/fischeckbox.dart';
import 'package:flyinsonolite/controls/dialog/fisdialog.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/select/fisselect.dart';
import 'package:flyinsonolite/controls/text/fislabelinputfield.dart';
import 'package:flyinsonolite/controls/text/datetimetextfield.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/fistextfield.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/consultation/records/models/datetime.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/utils.dart';
import 'package:get/get.dart';

class AcceptConsultationDialog extends GetView<AcceptConsultationController> {
  EdgeInsets get _labelFieldMargin => EdgeInsets.fromLTRB(0, 8.s, 0, 8.s);
  double get _labelWidth => i18nBook.isCurrentChinese ? 90.s : 140.s;
  double get _contentWidth => 570.s;
  const AcceptConsultationDialog({super.key});
  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
        future: controller.initializeAsync(),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return LayoutBuilder(builder: (context, constraint) {
              return FISDialog(
                i18nBook.realTimeConsultation.confirmApplication.t,
                _createContentView(context),
                true,
                true,
                contentPadding:
                    EdgeInsets.symmetric(vertical: 10.s, horizontal: 20.s),
                onConfirm: () async {
                  await controller.approveAsync();
                },
              );
            });
          } else {
            return const SizedBox();
          }
        });
  }

  Widget _createContentView(BuildContext context) {
    return Column(
      children: [
        FISLabelInputField(
          true,
          i18nBook.realTimeConsultation.dateTime.t,
          _getDateTimePickerView(),
          _contentWidth,
          padding: _labelFieldMargin,
          labelWidth: _labelWidth,
          labelColor: Storage.currentTheme.dialogStyle.messageTextStyle.color,
          labelPadding: EdgeInsets.only(top: 12.s),
        ),
        FISLabelInputField(
          true,
          fisI18nBook.common.competentExperts.t,
          _getExpertView(),
          _contentWidth,
          padding: _labelFieldMargin,
          labelWidth: _labelWidth,
          labelColor: Storage.currentTheme.dialogStyle.messageTextStyle.color,
        ),
        FISLabelInputField(
          false,
          i18nBook.realTimeConsultation.assistantPhysician.t,
          Obx(() => FISText(
                controller.assistantDoctorName.value ?? '',
                style: Storage.currentTheme.dialogStyle.messageTextStyle
                    .copyWith(
                        fontSize: Storage.currentTheme.dialogStyle
                            .messageTextStyle.fontSize!.s),
              )),
          _contentWidth,
          padding: _labelFieldMargin,
          labelWidth: _labelWidth,
          labelColor: Storage.currentTheme.dialogStyle.messageTextStyle.color,
        ),
        FISLabelInputField(
          false,
          i18nBook.realTimeConsultation.participants.t,
          Obx(
            () => Wrap(
              spacing: 5.s,
              runSpacing: 10.s,
              children: _createPaticipantsWidget(),
            ),
          ),
          _contentWidth,
          padding: _labelFieldMargin,
          labelWidth: _labelWidth,
          labelColor: Storage.currentTheme.dialogStyle.messageTextStyle.color,
        ),
        FISLabelInputField(
          false,
          fisI18nBook.common.consultationDescription.t,
          FISTextField(
            uniqueKey: GlobalKey(),
            borderColor: Storage
                .currentTheme.fisLabelInputFieldStyle.inputTextBorderColor,
            enabledBorderColor: Storage
                .currentTheme.fisLabelInputFieldStyle.inputTextBorderColor,
            controller: TextEditingController(text: controller.description),
            onChanged: (value) {
              controller.description = value.toString().trim();
            },
            maxLines: 6,
            contentPadding: EdgeInsets.fromLTRB(12.s, 10.s, 12.s, 10.s),
          ),
          height: 150.s,
          _contentWidth,
          padding: _labelFieldMargin,
          labelWidth: _labelWidth,
          labelColor: Storage.currentTheme.dialogStyle.messageTextStyle.color,
        ),
        FISLabelInputField(
          true,
          i18nBook.realTimeConsultation.operator.t,
          FISText(
            controller.currentConsultation.operateUserName!,
            style: Storage.currentTheme.dialogStyle.messageTextStyle.copyWith(
                fontSize: Storage
                    .currentTheme.dialogStyle.messageTextStyle.fontSize!.s),
          ),
          _contentWidth,
          padding: _labelFieldMargin,
          labelWidth: _labelWidth,
          labelColor: Storage.currentTheme.dialogStyle.messageTextStyle.color,
        ),
      ],
    );
  }

  Widget _getDateTimePickerView() {
    return Row(
      children: [
        Expanded(
          child: DateTimeTextField(
            null,
            controller.appointmentTime,
            borderColor: Storage
                .currentTheme.fisLabelInputFieldStyle.inputTextBorderColor,
            enabledBorderColor: Storage
                .currentTheme.fisLabelInputFieldStyle.inputTextBorderColor,
            dateChanged: (value) {
              controller.appointmentTime = value!;
            },
          ),
        ),
        SizedBox(
          width: 20.s,
        ),
        Obx(() => FISSelect(
              controller.timeSelectModelList
                  .map<CustomDropdownMenuItem<TimeOfDay?>>(
                      (TimeOfDay? timeItem) {
                return CustomDropdownMenuItem<TimeOfDay?>(
                  value: timeItem,
                  child: FISText(
                    timeItem == null ? '' : transformFromTimeOfDay(timeItem),
                    style: Storage.currentTheme.dialogStyle.messageTextStyle
                        .copyWith(
                            fontSize: Storage.currentTheme.dialogStyle
                                .messageTextStyle.fontSize!.s,
                            color: Colors.black),
                  ),
                );
              }).toList(),
              (value) {
                controller.startTime.value = value;
                controller.updateEndTime();
              },
              borderColor: Storage.currentTheme.loginContainStyle.borderColor,
              backgroundColor: Colors.white,
              value: controller.startTime.value,
              width: 100.s,
            )),
        SizedBox(
          width: 10.s,
        ),
        FISText(
          "—",
          style: Storage.currentTheme.dialogStyle.messageTextStyle.copyWith(
              fontSize: Storage
                  .currentTheme.dialogStyle.messageTextStyle.fontSize!.s),
        ),
        SizedBox(
          width: 10.s,
        ),
        Obx(() => FISSelect(
              controller.timeSelectModelList
                  .map<CustomDropdownMenuItem<TimeOfDay?>>(
                      (TimeOfDay? timeItem) {
                return CustomDropdownMenuItem<TimeOfDay?>(
                  value: timeItem,
                  child: FISText(
                    timeItem == null ? '' : transformFromTimeOfDay(timeItem),
                    style: Storage.currentTheme.dialogStyle.messageTextStyle
                        .copyWith(
                            fontSize: Storage.currentTheme.dialogStyle
                                .messageTextStyle.fontSize!.s,
                            color: Colors.black),
                  ),
                );
              }).toList(),
              (value) {
                controller.endTime.value = value;
              },
              borderColor: Storage.currentTheme.loginContainStyle.borderColor,
              backgroundColor: Colors.white,
              value: controller.endTime.value,
              width: 100.s,
            )),
      ],
    );
  }

  Widget _getExpertView() {
    Widget expertWidget;
    if (controller.currentConsultation.expertUserName.isNullOrWhiteSpace) {
      expertWidget = Obx(() => FISDropdownButton(
            value: controller.selectedExpert.value,
            items: controller.expertList
                .map<CustomDropdownMenuItem<OrganizationExpertDTO>>(
                    (OrganizationExpertDTO selectedExpert) {
              return CustomDropdownMenuItem<OrganizationExpertDTO>(
                  value: selectedExpert,
                  child: FISText(selectedExpert.displayName!));
            }).toList(),
            onChanged: (value) async {
              await controller
                  .updateSelectedExpertAsync(value as OrganizationExpertDTO);
            },
          ));
    } else {
      expertWidget = FISText(controller.currentConsultation.expertUserName!,
          style: Storage.currentTheme.dialogStyle.messageTextStyle.copyWith(
              fontSize: Storage
                  .currentTheme.dialogStyle.messageTextStyle.fontSize!.s));
    }

    return expertWidget;
  }

  ///提醒时间的widget
  Widget _reminderTimeWidget() {
    return SizedBox(
      height: 70,
      child: GridView.count(
          physics: const NeverScrollableScrollPhysics(),
          childAspectRatio: 7,
          crossAxisCount: 2,
          children: ConsultationReminderTimeEnum.values
              .map((e) => Obx(
                    () => FISCheckBox(
                      controller.selectedReminderTimes.contains(e),
                      (value) {
                        if (value!) {
                          controller.selectedReminderTimes.add(e);
                        } else {
                          controller.selectedReminderTimes.remove(e);
                        }
                      },
                      label: _translateConsultationReminderTimeEnum(e),
                      labelTextSty:
                          Storage.currentTheme.dialogStyle.messageTextStyle,
                    ),
                  ))
              .toList()),
    );
  }

  String _translateConsultationReminderTimeEnum(
      ConsultationReminderTimeEnum consultationReminderTimeEnum) {
    switch (consultationReminderTimeEnum) {
      case ConsultationReminderTimeEnum.FifteenMinutes:
        return i18nBook.realTimeConsultation.minBeforeStart15.t;
      case ConsultationReminderTimeEnum.OneDay:
        return i18nBook.realTimeConsultation.beforeStart1day.t;
      case ConsultationReminderTimeEnum.SixtyMinutes:
        return i18nBook.realTimeConsultation.minBeforeStart60.t;
      case ConsultationReminderTimeEnum.ThirtyMinutes:
        return i18nBook.realTimeConsultation.minBeforeStart30.t;
      default:
        return "";
    }
  }

  ///提醒方式的Widget
  Widget _remindModeWidget() {
    return SizedBox(
      height: 25.s,
      child: GridView.count(
          physics: const NeverScrollableScrollPhysics(),
          crossAxisCount: 2,
          childAspectRatio: 10,
          children: ConsultationReminderMode.values
              .map((e) => Obx(() => FISCheckBox(
                    controller.selectedReminderModes.contains(e),
                    (value) {
                      if (value!) {
                        controller.selectedReminderModes.add(e);
                      } else {
                        controller.selectedReminderModes.remove(e);
                      }
                    },
                    label: _translateConsultationReminderModeEnum(e),
                    labelTextSty:
                        Storage.currentTheme.dialogStyle.messageTextStyle,
                  )))
              .toList()),
    );
  }

  Widget _createPaticipantWidge(ConsultationMemberDTO memberDTO) {
    return Row(
      children: [
        CircleAvatar(
          radius: 25.s,
          backgroundImage: buildUserAvatarProvider(memberDTO.headImageToken),
        ),
        SizedBox(
          width: 10.s,
        ),
        FISText(
          memberDTO.name ?? '',
          overflow: TextOverflow.ellipsis,
          style: Storage.currentTheme.dialogStyle.messageTextStyle.copyWith(
              fontSize: Storage
                  .currentTheme.dialogStyle.messageTextStyle.fontSize!.s),
        ),
        FISIconButton(
          icon: const Icon(Icons.delete),
          color: Storage.currentTheme.dialogStyle.messageTextStyle.color,
          onPressed: () {
            controller.paticipants.remove(memberDTO);
          },
        )
      ],
    );
  }

  List<Widget> _createPaticipantsWidget() {
    List<Widget> participantsListWidget = []; //参与人员列表
    for (var paticipant in controller.paticipants) {
      participantsListWidget.add(_createPaticipantWidge(paticipant));
    }

    Widget addParticipants = SizedBox(
        //之所以外面套一层SizedBox是因为在安卓网页，会大于该高度，导致与标题参与人员没有保持一致高度
        height: 24.s,
        child: CustomTextButton(
            onPressed: () async {
              Get.put<AddParticipantController>(AddParticipantController());
              var addParticipantController =
                  Get.find<AddParticipantController>();
              addParticipantController.currentConsultation =
                  controller.currentConsultation;
              await addParticipantController.initializeAsync();
              Get.dialog(
                  AddParticipantDialog(controller.updatePaticipants,
                      existExpertList: controller.paticipants),
                  barrierDismissible: false);
            },
            style: CustomTextButton.styleFrom(
                padding: const EdgeInsets.all(0),
                alignment: Alignment.topCenter),
            child: Row(
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                Icon(
                  Icons.add_box,
                  size: 24.s,
                  color:
                      Storage.currentTheme.dialogStyle.messageTextStyle.color,
                ),
                FISText(
                  i18nBook.realTimeConsultation.addParticipants.t,
                  style: Storage.currentTheme.dialogStyle.messageTextStyle
                      .copyWith(
                          fontSize: Storage.currentTheme.dialogStyle
                              .messageTextStyle.fontSize!.s),
                )
              ],
            )));

    participantsListWidget.add(addParticipants);

    return participantsListWidget;
  }

  String _translateConsultationReminderModeEnum(
      ConsultationReminderMode consultationReminderModeEnum) {
    switch (consultationReminderModeEnum) {
      case ConsultationReminderMode.SMS:
        return i18nBook.realTimeConsultation.SMSReminder.t;
      case ConsultationReminderMode.Application:
        return i18nBook.realTimeConsultation.inAppReminder.t;
      default:
        return "";
    }
  }
}
