import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/liveConsultation.m.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/controllers/addparticipantcontroller.dart';
import 'package:flyinsonolite/controls/custom/customcheckbox.dart';
import 'package:flyinsonolite/controls/dialog/fisdialog.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/utils.dart';
import 'package:get/get.dart';

class AddParticipantDialog extends GetView<AddParticipantController> {
  /// 确认后的回调函数
  final Function confirmCallback;

  AddParticipantDialog(this.confirmCallback,
      {List<ConsultationMemberDTO>? existExpertList, super.key}) {
    controller.existExpertList = existExpertList;
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      return FISDialog(
        i18nBook.realTimeConsultation.participants.t,
        _createContentView(context),
        true,
        true,
        contentPadding: EdgeInsets.all(5.s),
        onConfirm: () {
          confirmCallback(controller.selectedExpertList);
          Get.close(1);
        },
      );
    });
  }

  Widget _createContentView(BuildContext context) {
    var width = Get.width > 600.s ? 600.s : Get.width;
    var height = (Get.height - 132.s) / 5 * 4;
    return SizedBox(
      height: height,
      width: width,
      child: Row(
        children: [
          SizedBox(
            width: width * 0.4,
            child: ListView(
              children: controller.hospitals
                  .map((e) => SizedBox(
                        height: 60.s,
                        child: Obx(() => FISLinearGradientElevatedButtonAsync(
                                () async {
                              controller.selectedHospital.value = e;
                              await controller
                                  .getAllSelectableParticipantsAsync();
                            },
                                e.organizationName ?? '',
                                controller.selectedHospital.value?.organizationCode ==
                                        e.organizationCode
                                    ? Storage
                                        .currentTheme
                                        .listViewButtonItemStyle
                                        .selectedGradient
                                    : Storage.currentTheme
                                        .listViewButtonItemStyle.normalGradient,
                                textStyle: controller.selectedHospital.value?.organizationCode ==
                                        e.organizationCode
                                    ? Storage.currentTheme.normalTextStyle.copyWith(
                                        color: Storage.currentTheme.listViewButtonItemStyle.selectedTextColor,
                                        fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s)
                                    : Storage.currentTheme.normalTextStyle.copyWith(fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s))),
                      ))
                  .toList(),
            ),
          ),
          VerticalDivider(
            width: 5.s,
          ),
          Obx(() => Expanded(
                  child: ListView(
                children: controller.selectableExpertList
                    .map((e) => InkWell(
                          child: Container(
                            padding: EdgeInsets.all(10.s),
                            margin: EdgeInsets.all(5.s),
                            decoration: BoxDecoration(
                                borderRadius:
                                    BorderRadius.all(Radius.circular(5.s)),
                                border: Border.all(
                                    width: 1.s,
                                    color: Storage.currentTheme
                                        .loginContainStyle.borderColor)),
                            child: Row(
                              children: [
                                Expanded(
                                    child: Row(
                                  children: [
                                    CircleAvatar(
                                      radius: 25.s,
                                      backgroundImage: buildUserAvatarProvider(
                                          e.headImageUrl),
                                    ),
                                    SizedBox(
                                      width: 10.s,
                                    ),
                                    Column(
                                      crossAxisAlignment:
                                          CrossAxisAlignment.start,
                                      children: [
                                        FISText(
                                          e.displayName ?? '',
                                          style: Storage.currentTheme
                                              .dialogStyle.messageTextStyle
                                              .copyWith(
                                                  fontWeight: FontWeight.bold,
                                                  fontSize: 22.s),
                                        ),
                                        SizedBox(
                                          width: 5.s,
                                        ),
                                        FISText(e.roleName ?? '',
                                            style: Storage.currentTheme
                                                .dialogStyle.messageTextStyle
                                                .copyWith(
                                                    fontSize: Storage
                                                        .currentTheme
                                                        .dialogStyle
                                                        .messageTextStyle
                                                        .fontSize!
                                                        .s)),
                                        SizedBox(
                                          width: 5.s,
                                        ),
                                        RichText(
                                          text: TextSpan(children: [
                                            TextSpan(
                                              text: i18nBook.user
                                                  .skilledAreaWithDevider.t,
                                              style: Storage.currentTheme
                                                  .dialogStyle.messageTextStyle
                                                  .copyWith(
                                                      fontSize: Storage
                                                          .currentTheme
                                                          .dialogStyle
                                                          .messageTextStyle
                                                          .fontSize!
                                                          .s),
                                            ),
                                            TextSpan(
                                              text: e.fieldList == null
                                                  ? ''
                                                  : e.fieldList?.join("; "),
                                              style: Storage.currentTheme
                                                  .dialogStyle.messageTextStyle
                                                  .copyWith(
                                                      fontSize: Storage
                                                          .currentTheme
                                                          .dialogStyle
                                                          .messageTextStyle
                                                          .fontSize!
                                                          .s),
                                            ),
                                          ]),
                                        ),
                                      ],
                                    )
                                  ],
                                )),
                                Transform.scale(
                                  scale: 1.5,
                                  child: CustomCheckbox(
                                    value: controller.selectedExpertList.any(
                                        (element) =>
                                            element.userCode == e.userCode),
                                    onChanged: ((value) {
                                      if (value!) {
                                        controller.selectedExpertList.add(e);
                                      } else {
                                        var exist = controller
                                            .selectedExpertList
                                            .firstWhereOrNull((element) =>
                                                element.userCode == e.userCode);
                                        if (exist != null) {
                                          controller.selectedExpertList
                                              .remove(exist);
                                        }
                                      }
                                    }),
                                    shape: const CircleBorder(),
                                  ),
                                ),
                              ],
                            ),
                          ),
                          onTap: () {
                            if (controller.selectedExpertList.any(
                                (element) => element.userCode == e.userCode)) {
                              var exist = controller.selectedExpertList
                                  .firstWhereOrNull((element) =>
                                      element.userCode == e.userCode);
                              if (exist != null) {
                                controller.selectedExpertList.remove(exist);
                              }
                            } else {
                              controller.selectedExpertList.add(e);
                            }
                          },
                        ))
                    .toList(),
              )))
        ],
      ),
    );
  }
}
