import 'package:fis_common/extensions/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flyinsonolite/consultation/records/controllers/applyconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/records/views/choosepatientdialog.dart';
import 'package:flyinsonolite/consultation/records/views/choosescanpositiondialog.dart';
import 'package:flyinsonolite/controls/custom/customdropdownbutton.dart';
import 'package:flyinsonolite/controls/select/fisdropdownbutton.dart';
import 'package:flyinsonolite/controls/select/fbuildagetype.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/text/fislabelinputfield.dart';
import 'package:flyinsonolite/controls/text/datetimetextfield.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/fistextfield.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/localization/localization.dart';
import 'package:flyinsonolite/consultation/records/models/selectmodel.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';

class ApplyConsultationPage extends GetView<ApplyConsultationController> {
  final _consultationConstant = 'consultation';
  EdgeInsets get _labelFieldMargin => EdgeInsets.all(5.s);
  double get _labelWidth => i18nBook.isCurrentChinese ? 90.s : 140.s;
  EdgeInsets get _labelPadding => EdgeInsets.only(top: 12.s);
  const ApplyConsultationPage({super.key});

  @override
  Widget build(BuildContext context) {
    Get.lazyPut<ApplyConsultationController>(
        () => ApplyConsultationController());
    return FutureBuilder(
        future: controller.initializeAsync(),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return LayoutBuilder(builder: (context, constraint) {
              return Scaffold(
                resizeToAvoidBottomInset: false,
                backgroundColor:
                    Storage.currentTheme.themeData.scaffoldBackgroundColor,
                body: _buildBody(context),
              );
            });
          } else {
            return Scaffold(
              resizeToAvoidBottomInset: false,
              backgroundColor:
                  Storage.currentTheme.themeData.scaffoldBackgroundColor,
            );
          }
        });
  }

  Widget _buildBody(BuildContext context) {
    double headerHeight = 72.s;
    double minBodyHeight = Get.height - headerHeight;
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        _createHeader(headerHeight),
        Expanded(
            child: SingleChildScrollView(
                child: Container(
          constraints: BoxConstraints(minHeight: minBodyHeight),
          padding: EdgeInsets.fromLTRB(10.s, 10.s, 10.s, 10.s),
          color: Storage.currentTheme.bgContainerColor,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              FISText(
                i18nBook.remedical.patientDetail.t,
                style: Storage.currentTheme.titleTextStyle.copyWith(
                    fontSize: Storage.currentTheme.titleTextStyle.fontSize!.s),
              ),
              SizedBox(
                height: 10.s,
              ),
              _buildPatinetInfo(context),
              Container(
                margin: EdgeInsets.symmetric(vertical: 15.s),
                child: Divider(
                  height: 10.s,
                  thickness: 2.s,
                ),
              ),
              FISText(
                i18nBook.realTimeConsultation.appointmentInformation.t,
                style: Storage.currentTheme.titleTextStyle.copyWith(
                    fontSize: Storage.currentTheme.titleTextStyle.fontSize!.s),
              ),
              SizedBox(
                height: 10.s,
              ),
              _buildConsultationInfo(context),
              SizedBox(
                height: 10.s,
              ),
            ],
          ),
        )))
      ],
    );
  }

  Widget _createHeader(double height) {
    return Container(
      color: Storage.currentTheme.homepageHeaderColor,
      height: height,
      child: Row(mainAxisAlignment: MainAxisAlignment.spaceBetween, children: [
        FISIconButtonAsync(
            onPressed: () async {
              await controller.createOrUpdateOrImproveConsultationAsync();
            },
            icon: const Icon(
              Icons.done,
            ),
            iconSize: 56.s,
            alignment: Alignment.centerLeft,
            padding: EdgeInsets.only(left: 20.s)),
        FISIconButton(
            onPressed: () {
              Get.close(1);
            },
            icon: const Icon(
              Icons.close,
            ),
            iconSize: 56.s,
            padding: EdgeInsets.only(right: 20.s))
      ]),
    );
  }

  Widget _buildPatinetInfo(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      double wrapWidth = constraints.maxWidth;
      if (Storage.configurationFromerver == null) {
        return const SizedBox();
      } else {
        return Wrap(
          children: [
            ...Storage
                .configurationFromerver!.consultationServerConfig.patientKeyList
                .map((e) {
              switch (e.showType) {
                case "input": //手机号，社保卡号，身份证等
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    FISTextField(
                      uniqueKey: GlobalKey(),
                      enabled: !controller.isEdit.value,
                      borderColor: Storage.currentTheme.fisLabelInputFieldStyle
                          .inputTextBorderColor,
                      enabledBorderColor: Storage.currentTheme
                          .fisLabelInputFieldStyle.inputTextBorderColor,
                      textStyle: Storage
                          .currentTheme.fisLabelInputFieldStyle.inputTextStyle
                          .copyWith(
                              fontSize: Storage
                                  .currentTheme
                                  .fisLabelInputFieldStyle
                                  .inputTextStyle
                                  .fontSize!
                                  .s),
                      controller: controller.getTextEditingController(
                          e.key, controller.patient[e.key] ?? ""),
                      onChanged: (value) {
                        controller.patient[e.key] = value.toString().trim();
                      },
                      fillColor: Storage
                          .currentTheme.fisLabelInputFieldStyle.fillColor,
                      inputFormatters: [
                        _getInputFormatter(e.key),
                        LengthLimitingTextInputFormatter(20),
                      ],
                    ),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelPadding: _labelPadding,
                    labelWidth: _labelWidth,
                  );
                case "AgeType":
                  _handleAgeUnitSource(e.source ?? []);
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    const FBuildAgeType(false),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    labelPadding: _labelPadding,
                  );
                case "select": //性别，宠物种类等
                  _handleSexSource(e.source ?? []);
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    Obx(() => controller.isEdit.value
                        ? FISTextField(
                            uniqueKey: GlobalKey(),
                            enabled: false,
                            borderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            enabledBorderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            textStyle: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextStyle
                                .copyWith(
                                    fontSize: Storage
                                        .currentTheme
                                        .fisLabelInputFieldStyle
                                        .inputTextStyle
                                        .fontSize!
                                        .s),
                            controller: TextEditingController(
                                text: controller.selectedSex.value == null
                                    ? ''
                                    : controller.selectedSex.value!.value),
                            fillColor: Storage
                                .currentTheme.fisLabelInputFieldStyle.fillColor,
                          )
                        : Row(children: [
                            Expanded(
                                child: FISDropdownButton(
                              value: controller.selectedSex.value,
                              items: controller.sexList.value
                                  .map<CustomDropdownMenuItem<FISSelectModel>>(
                                      (FISSelectModel selectModel) {
                                return CustomDropdownMenuItem<FISSelectModel>(
                                  value: selectModel,
                                  child: FISText(selectModel.value),
                                );
                              }).toList(),
                              onChanged: (value) {
                                controller.selectedSex.value = value;
                                controller.patient[e.key] =
                                    (value as FISSelectModel).key;
                              },
                            )),
                            controller.selectedSex.value == null
                                ? const SizedBox()
                                : FISIconButton(
                                    icon: const Icon(Icons.close),
                                    onPressed: () {
                                      controller.selectedSex.value = null;
                                      controller.patient[e.key] = '';
                                    }),
                          ])),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    labelPadding: _labelPadding,
                  );
                case "inputButton":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    FISTextField(
                      uniqueKey: GlobalKey(),
                      enabled: !controller.isEdit.value,
                      readOnly: true, //病人姓名不支持输入
                      borderColor: Storage.currentTheme.fisLabelInputFieldStyle
                          .inputTextBorderColor,
                      enabledBorderColor: Storage.currentTheme
                          .fisLabelInputFieldStyle.inputTextBorderColor,
                      textStyle: Storage
                          .currentTheme.fisLabelInputFieldStyle.inputTextStyle
                          .copyWith(
                              fontSize: Storage
                                  .currentTheme
                                  .fisLabelInputFieldStyle
                                  .inputTextStyle
                                  .fontSize!
                                  .s),
                      controller: controller.getTextEditingController(
                          e.key, controller.patient[e.key] ?? ""),
                      onTap: controller.isEdit.value
                          ? null
                          : () {
                              Get.dialog(const ChoosePatientDialog(false),
                                  barrierDismissible: false);
                            },
                      fillColor: Storage
                          .currentTheme.fisLabelInputFieldStyle.fillColor,
                    ),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    labelPadding: _labelPadding,
                  );
                case 'textarea':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    FISTextField(
                      uniqueKey: GlobalKey(),
                      enabled: !controller.isEdit.value,
                      borderColor: Storage.currentTheme.fisLabelInputFieldStyle
                          .inputTextBorderColor,
                      enabledBorderColor: Storage.currentTheme
                          .fisLabelInputFieldStyle.inputTextBorderColor,
                      textStyle: Storage
                          .currentTheme.fisLabelInputFieldStyle.inputTextStyle
                          .copyWith(
                              fontSize: Storage
                                  .currentTheme
                                  .fisLabelInputFieldStyle
                                  .inputTextStyle
                                  .fontSize!
                                  .s),
                      controller: controller.getTextEditingController(
                          e.key, controller.patient[e.key] ?? ""),
                      onChanged: (value) {
                        controller.patient[e.key] = value.toString().trim();
                      },
                      maxLines: 6,
                      contentPadding:
                          EdgeInsets.fromLTRB(12.s, 10.s, 12.s, 10.s),
                      fillColor: Storage
                          .currentTheme.fisLabelInputFieldStyle.fillColor,
                    ),
                    wrapWidth,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    height: 150.s,
                  );
                default:
                  return const SizedBox();
              }
            }).toList()
          ],
        );
      }
    });
  }

  Widget _buildConsultationInfo(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      double wrapWidth = constraints.maxWidth;
      if (Storage.configurationFromerver == null) {
        return const SizedBox();
      } else {
        return Wrap(
          children: [
            ...Storage.configurationFromerver!.consultationServerConfig
                .appointmentKeyList
                .map((e) {
              switch (e.key) {
                case "Device":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    controller.isImprove.value
                        ? FISTextField(
                            uniqueKey: GlobalKey(),
                            enabled: false,
                            borderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            enabledBorderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            textStyle: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextStyle
                                .copyWith(
                                    fontSize: Storage
                                        .currentTheme
                                        .fisLabelInputFieldStyle
                                        .inputTextStyle
                                        .fontSize!
                                        .s),
                            controller: TextEditingController(
                                text: controller
                                    .selectedDeviceInfo.value!.displayName),
                            fillColor: Storage
                                .currentTheme.fisLabelInputFieldStyle.fillColor,
                          )
                        : Obx(() => FISDropdownButton(
                              value: controller.selectedDeviceInfo.value,
                              items: controller.deviceInfoList
                                  .map<CustomDropdownMenuItem<DeviceInfoDTO>>(
                                      (DeviceInfoDTO selectedDeviceInfo) {
                                return CustomDropdownMenuItem<DeviceInfoDTO>(
                                  value: selectedDeviceInfo,
                                  child:
                                      FISText(selectedDeviceInfo.displayName!),
                                );
                              }).toList(),
                              onChanged: (value) {
                                controller.selectedDeviceInfo.value = value;
                                controller.appointmentInformation
                                        .appointmentDevice =
                                    controller
                                        .selectedDeviceInfo.value!.deviceCode;
                              },
                            )),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    labelPadding: _labelPadding,
                  );
                case "ConsultationTime":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    controller.isImprove.value
                        ? FISTextField(
                            uniqueKey: GlobalKey(),
                            enabled: false,
                            borderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            enabledBorderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            textStyle: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextStyle
                                .copyWith(
                                    fontSize: Storage
                                        .currentTheme
                                        .fisLabelInputFieldStyle
                                        .inputTextStyle
                                        .fontSize!
                                        .s),
                            controller: TextEditingController(
                                text: DateFormat('yyyy-MM-dd')
                                    .format(controller.appointmentInformation
                                            .appointmentTime ??
                                        DateTime.now())
                                    .toString()),
                            fillColor: Storage
                                .currentTheme.fisLabelInputFieldStyle.fillColor,
                          )
                        : DateTimeTextField(
                            null,
                            controller.appointmentInformation.appointmentTime,
                            textStyle: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextStyle
                                .copyWith(
                                    fontSize: Storage
                                        .currentTheme
                                        .fisLabelInputFieldStyle
                                        .inputTextStyle
                                        .fontSize!
                                        .s),
                            borderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            enabledBorderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            fillColor: Storage
                                .currentTheme.fisLabelInputFieldStyle.fillColor,
                            dateChanged: (value) {
                              controller.appointmentInformation
                                  .appointmentTime = value;
                            },
                            selectableDayCheck: true,
                          ),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    labelPadding:
                        i18nBook.isCurrentChinese ? _labelPadding : null,
                  );
                case "ScanUser":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    controller.isImprove.value
                        ? FISTextField(
                            uniqueKey: GlobalKey(),
                            enabled: false,
                            borderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            enabledBorderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            textStyle: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextStyle
                                .copyWith(
                                    fontSize: Storage
                                        .currentTheme
                                        .fisLabelInputFieldStyle
                                        .inputTextStyle
                                        .fontSize!
                                        .s),
                            controller: TextEditingController(
                                text: controller
                                    .selectedScanDoctor.value!.displayName),
                            fillColor: Storage
                                .currentTheme.fisLabelInputFieldStyle.fillColor,
                          )
                        : Obx(() => FISDropdownButton(
                              value: controller.selectedScanDoctor.value,
                              items: controller.scanDoctorList
                                  .map<CustomDropdownMenuItem<UserExtendDTO>>(
                                      (UserExtendDTO selectedScanDoctor) {
                                return CustomDropdownMenuItem<UserExtendDTO>(
                                  value: selectedScanDoctor,
                                  child:
                                      FISText(selectedScanDoctor.displayName!),
                                );
                              }).toList(),
                              onChanged: (value) {
                                controller.selectedScanDoctor.value = value;
                                controller.appointmentInformation
                                        .appointmentScanDoctor =
                                    (value as UserExtendDTO).userCode;
                              },
                            )),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    labelPadding: _labelPadding,
                  );
                case "Diseases":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    FISTextField(
                      uniqueKey: GlobalKey(),
                      borderColor: Storage.currentTheme.fisLabelInputFieldStyle
                          .inputTextBorderColor,
                      enabledBorderColor: Storage.currentTheme
                          .fisLabelInputFieldStyle.inputTextBorderColor,
                      textStyle: Storage
                          .currentTheme.fisLabelInputFieldStyle.inputTextStyle
                          .copyWith(
                              fontSize: Storage
                                  .currentTheme
                                  .fisLabelInputFieldStyle
                                  .inputTextStyle
                                  .fontSize!
                                  .s),
                      controller: TextEditingController(
                          text: controller
                              .appointmentInformation.appointmentDiseaseName),
                      onChanged: (value) {
                        controller.appointmentInformation
                            .appointmentDiseaseName = value.toString().trim();
                      },
                      fillColor: Storage
                          .currentTheme.fisLabelInputFieldStyle.fillColor,
                    ),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    labelPadding: _labelPadding,
                  );
                case 'ScanPosition':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    FISTextField(
                      uniqueKey: GlobalKey(),
                      readOnly: true, //扫查部位不支持输入
                      borderColor: Storage.currentTheme.fisLabelInputFieldStyle
                          .inputTextBorderColor,
                      enabledBorderColor: Storage.currentTheme
                          .fisLabelInputFieldStyle.inputTextBorderColor,
                      textStyle: Storage
                          .currentTheme.fisLabelInputFieldStyle.inputTextStyle
                          .copyWith(
                              fontSize: Storage
                                  .currentTheme
                                  .fisLabelInputFieldStyle
                                  .inputTextStyle
                                  .fontSize!
                                  .s),
                      controller: controller.scanPositionTextEditingController,
                      onTap: () {
                        Get.dialog(const ChooseScanPositionDialog(),
                            barrierDismissible: false);
                      },
                      fillColor: Storage
                          .currentTheme.fisLabelInputFieldStyle.fillColor,
                    ),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    labelPadding: _labelPadding,
                  );
                case 'ExpertOrganization':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    controller.isImprove.value
                        ? FISTextField(
                            uniqueKey: GlobalKey(),
                            enabled: false,
                            borderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            enabledBorderColor: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextBorderColor,
                            textStyle: Storage.currentTheme
                                .fisLabelInputFieldStyle.inputTextStyle
                                .copyWith(
                                    fontSize: Storage
                                        .currentTheme
                                        .fisLabelInputFieldStyle
                                        .inputTextStyle
                                        .fontSize!
                                        .s),
                            controller: TextEditingController(
                                text: controller.selectedParentOrganization
                                    .value!.organizationName),
                            fillColor: Storage
                                .currentTheme.fisLabelInputFieldStyle.fillColor,
                          )
                        : Obx(() => FISDropdownButton(
                              value:
                                  controller.selectedParentOrganization.value,
                              items: controller.parentOrganizationList.map<
                                      CustomDropdownMenuItem<
                                          OrganizationBaseDTO>>(
                                  (OrganizationBaseDTO selectedHospital) {
                                return CustomDropdownMenuItem<
                                        OrganizationBaseDTO>(
                                    value: selectedHospital,
                                    child: FISText(
                                        selectedHospital.organizationName!));
                              }).toList(),
                              onChanged: (value) async {
                                var newOrganization =
                                    value as OrganizationBaseDTO;
                                if (controller.selectedParentOrganization.value
                                        ?.organizationCode ==
                                    newOrganization.organizationCode) {
                                  return;
                                }
                                controller.selectedParentOrganization.value =
                                    value;
                                controller.appointmentInformation
                                        .appointmentConsultationInstitution =
                                    newOrganization.organizationCode;
                                await controller.updateExpertsAsync(controller
                                    .appointmentInformation
                                    .appointmentConsultationInstitution!);
                              },
                              disabledHint: FISText(
                                  fisI18nBook.common
                                      .consultationInstitutionlistisempty.t,
                                  overflow: TextOverflow.ellipsis,
                                  style: Storage.currentTheme.normalTextStyle
                                      .copyWith(
                                          fontSize: Storage.currentTheme
                                              .normalTextStyle.fontSize!.s)),
                            )),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    labelPadding: _labelPadding,
                  );
                case 'Expert':
                  return FISLabelInputField(
                      e.required,
                      getLanguageValues(_consultationConstant, e.key),
                      controller.isImprove.value
                          ? FISTextField(
                              uniqueKey: GlobalKey(),
                              enabled: false,
                              borderColor: Storage.currentTheme
                                  .fisLabelInputFieldStyle.inputTextBorderColor,
                              enabledBorderColor: Storage.currentTheme
                                  .fisLabelInputFieldStyle.inputTextBorderColor,
                              textStyle: Storage.currentTheme
                                  .fisLabelInputFieldStyle.inputTextStyle
                                  .copyWith(
                                      fontSize: Storage
                                          .currentTheme
                                          .fisLabelInputFieldStyle
                                          .inputTextStyle
                                          .fontSize!
                                          .s),
                              controller: TextEditingController(
                                  text: controller
                                      .selectedExpert.value!.displayName),
                              fillColor: Storage.currentTheme
                                  .fisLabelInputFieldStyle.fillColor,
                            )
                          : Obx(() => Row(children: [
                                Expanded(
                                    child: FISDropdownButton(
                                  disabledHint: Obx(() => controller
                                              .selectedParentOrganization
                                              .value !=
                                          null
                                      ? FISText(fisI18nBook.common.expertListIsEmpty.t,
                                          overflow: TextOverflow.ellipsis,
                                          style: Storage.currentTheme.normalTextStyle
                                              .copyWith(
                                                  fontSize: Storage
                                                      .currentTheme
                                                      .normalTextStyle
                                                      .fontSize!
                                                      .s))
                                      : FISText(fisI18nBook.common.chooseconsultationInstitutionfirst.t,
                                          overflow: TextOverflow.ellipsis,
                                          style: Storage.currentTheme.normalTextStyle
                                              .copyWith(fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s))),
                                  value: controller.selectedExpert.value,
                                  items: controller.expertList.value.map<
                                          CustomDropdownMenuItem<
                                              OrganizationExpertDTO>>(
                                      (OrganizationExpertDTO selectedExpert) {
                                    return CustomDropdownMenuItem<
                                        OrganizationExpertDTO>(
                                      value: selectedExpert,
                                      child:
                                          FISText(selectedExpert.displayName!),
                                    );
                                  }).toList(),
                                  onChanged: (value) {
                                    controller.selectedExpert.value = value;
                                    controller.appointmentInformation
                                            .appointmentCompetentExperts =
                                        (value as OrganizationExpertDTO)
                                            .userCode;
                                  },
                                )),
                                controller.selectedExpert.value == null
                                    ? const SizedBox()
                                    : FISIconButton(
                                        icon: const Icon(Icons.close),
                                        onPressed: () {
                                          controller.selectedExpert.value =
                                              null;
                                          controller.appointmentInformation
                                                  .appointmentCompetentExperts =
                                              null;
                                        }),
                              ])),
                      Storage.isPortraitMode.value
                          ? wrapWidth
                          : wrapWidth * 0.5,
                      padding: _labelFieldMargin,
                      labelPadding: _labelPadding,
                      labelWidth: _labelWidth);
                case 'ApplyUser':
                  return Storage.user.isApplyUserField
                      ? FISLabelInputField(
                          e.required,
                          getLanguageValues(_consultationConstant, e.key),
                          Obx(() => FISDropdownButton(
                                value: controller.selectedApplicant.value,
                                items: controller.applicantList.map<
                                        CustomDropdownMenuItem<
                                            OrganizationExpertDTO>>(
                                    (OrganizationExpertDTO selectedApplicant) {
                                  return CustomDropdownMenuItem<
                                      OrganizationExpertDTO>(
                                    value: selectedApplicant,
                                    child:
                                        FISText(selectedApplicant.displayName!),
                                  );
                                }).toList(),
                                onChanged: (value) {
                                  controller.selectedApplicant.value = value;
                                  controller.appointmentInformation
                                          .appointmentApplicant =
                                      (value as OrganizationExpertDTO).userCode;
                                },
                              )),
                          Storage.isPortraitMode.value
                              ? wrapWidth
                              : wrapWidth * 0.5,
                          padding: _labelFieldMargin,
                          labelWidth: _labelWidth,
                          labelPadding: _labelPadding,
                        )
                      : const SizedBox();
                case 'ApplyOrganization':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    FISText(Storage.user.organizationName!),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                  );
                case 'Operator':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    FISText(Storage.user.displayName!),
                    Storage.isPortraitMode.value ? wrapWidth : wrapWidth * 0.5,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                  );
                case 'PrimaryDiagnosis':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consultationConstant, e.key),
                    FISTextField(
                      uniqueKey: GlobalKey(),
                      borderColor: Storage.currentTheme.fisLabelInputFieldStyle
                          .inputTextBorderColor,
                      enabledBorderColor: Storage.currentTheme
                          .fisLabelInputFieldStyle.inputTextBorderColor,
                      textStyle: Storage
                          .currentTheme.fisLabelInputFieldStyle.inputTextStyle
                          .copyWith(
                              fontSize: Storage
                                  .currentTheme
                                  .fisLabelInputFieldStyle
                                  .inputTextStyle
                                  .fontSize!
                                  .s),
                      controller: TextEditingController(
                          text: controller
                              .appointmentInformation.preliminaryDiagnosis),
                      onChanged: (value) {
                        controller.appointmentInformation.preliminaryDiagnosis =
                            value.toString().trim();
                      },
                      maxLines: 6,
                      contentPadding:
                          EdgeInsets.fromLTRB(12.s, 10.s, 12.s, 10.s),
                      fillColor: Storage
                          .currentTheme.fisLabelInputFieldStyle.fillColor,
                    ),
                    wrapWidth,
                    padding: _labelFieldMargin,
                    labelWidth: _labelWidth,
                    height: 150.s,
                  );
                default:
                  return const SizedBox();
              }
            }).toList()
          ],
        );
      }
    });
  }

  void _handleSexSource(List source) {
    List<FISSelectModel> selectList = [];
    for (int i = 0; i < source.length; i++) {
      selectList.add(
        FISSelectModel(
          source[i],
          getLanguageValues(
            "exam",
            source[i],
          ),
        ),
      );
    }

    controller.selectedSex.value = selectList.firstWhereNullable(
        (element) => element.key == controller.patient["Sex"]);
    controller.sexList.value = selectList;
  }

  void _handleAgeUnitSource(List source) {
    List<FISSelectModel> selectList = [];
    for (int i = 0; i < source.length; i++) {
      selectList.add(
        FISSelectModel(
          source[i],
          getLanguageValues(
            "exam",
            source[i],
          ),
        ),
      );
    }
    controller.ageUnitList.value = selectList;
    controller.selectedAgeUnit.value = selectList.firstWhereNullable(
        (element) => element.key == controller.patient["AgeUnits"]);
  }

  TextInputFormatter _getInputFormatter(String key) {
    if (key.toLowerCase() == "phone") {
      return FilteringTextInputFormatter.digitsOnly;
    } else if (key.toLowerCase() == "identitycard") {
      return FilteringTextInputFormatter.allow(RegExp(r'[A-Za-z0-9]'));
    } else {
      return FilteringTextInputFormatter.allow(RegExp(r'[A-Za-z0-9]'));
    }
  }
}
