import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/authentication.m.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/controllers/applyconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/choosepatientcontroller.dart';
import 'package:flyinsonolite/consultation/emergency/controllers/emergencypreparednesscontroller.dart';
import 'package:flyinsonolite/controls/custom/customcheckbox.dart';
import 'package:flyinsonolite/controls/dialog/fisdialog.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/fistextfield.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/consultation/records/models/patientinfo.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/utils.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/consultation.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:get/get.dart';

class ChoosePatientDialog extends GetView<ChoosePatientController> {
  final bool isEmergencyMode;
  const ChoosePatientDialog(this.isEmergencyMode, {super.key});

  @override
  Widget build(BuildContext context) {
    Get.lazyPut<ChoosePatientController>(() => ChoosePatientController());
    controller.selectedPatient.value = null;
    return FutureBuilder(
        future: controller.initializeAsync(isEmergencyMode),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return LayoutBuilder(builder: (context, constraint) {
              return FISDialog(
                i18nBook.realTimeConsultation.pleaseSelectPatient.t,
                _createChoosePatientView(context),
                false,
                false,
              );
            });
          } else {
            return Container();
          }
        });
  }

  Widget _createChoosePatientView(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        Row(
          children: [
            Expanded(
              child: FISTextField(
                uniqueKey: GlobalKey(),
                placeHolder: i18nBook.remedical.searchKeyword.t,
                onChanged: controller.keyword,
                onSubmitted: onSubmittedAsync,
                suffixIcon: FISIconButtonAsync(
                  onPressed: () async {
                    await controller.searchPatientAsync();
                  },
                  icon: Icon(
                    Icons.search,
                    size: 40.s,
                    color: const Color.fromRGBO(93, 95, 111, 1),
                  ),
                  padding: EdgeInsets.fromLTRB(0, 0, 10.s, 10.s),
                ),
              ),
            ),
            SizedBox(
              width: 10.s,
            ),
            Obx(() => FISLinearGradientElevatedButtonAsync(
                  controller.keyword.value.isEmpty
                      ? null
                      : () async {
                          await createPatientAsync();
                        },
                  i18nBook.remedical.createPatient.t,
                  Storage.currentTheme.dialogStyle.buttonGradient,
                  textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                      fontSize:
                          Storage.currentTheme.normalTextStyle.fontSize!.s),
                ))
          ],
        ),
        SizedBox(
          height: 5.s,
        ),
        SizedBox(
          height: (Get.height - 150.s) / 5 * 4,
          width: 600.s,
          child: _createListView(),
        )
      ],
    );
  }

  Future onSubmittedAsync(String value) async {
    try {
      if (Storage.platform == Platform.Windows && Storage.isShowSoftKeyBoard) {
        Storage.isShowSoftKeyBoard = false;
        await jsonRpcProxyForFISLib.consultation.operateSoftKeyBoard(
            FISControlPositionInfo(isShowSoftwareKeyBoard: false));
      }
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "OperateSoftKeyBoard error: $ex");
    }
    controller.searchPatientAsync();
  }

  Widget _createListView() {
    var scrollController = ScrollController();
    scrollController.addListener(() {
      if (scrollController.position.pixels ==
              scrollController.position.maxScrollExtent &&
          scrollController.position.axisDirection == AxisDirection.down) {
        if (controller.patientList.length < controller.totalCount) {
          controller.searchMorePatientAsync();
        }
      }
    });
    return Obx(
      () => ListView.separated(
        controller: scrollController,
        padding: EdgeInsets.zero,
        itemCount: controller.patientList.length,
        itemBuilder: (BuildContext context, int index) {
          return createListViewItem(controller.patientList[index]);
        },
        separatorBuilder: (context, index) {
          return Divider(
            height: 3.s,
          );
        },
      ),
    );
  }

  Widget createListViewItem(PatientInfo patientInfo) {
    return ConsultationPatientListViewItem(
      patientInfo: patientInfo,
      controller: controller,
    );
  }

  Future createPatientAsync() async {
    await controller.createPatientAsync(true);
  }
}

class ConsultationPatientListViewItem extends StatelessWidget {
  final PatientInfo patientInfo;
  final ChoosePatientController controller;
  const ConsultationPatientListViewItem(
      {super.key, required this.patientInfo, required this.controller});

  @override
  Widget build(BuildContext context) {
    var isHover = false.obs;
    return InkWell(
        onHover: (value) {
          isHover.value = value;
        },
        hoverColor: Storage.currentTheme.listViewStyle.hoverColor,
        onTap: () {
          _clickPatientCard();
        },
        child: Obx(
          () => Container(
              color: isHover.value
                  ? Storage.currentTheme.listViewStyle.itembgColor
                  : null,
              padding: EdgeInsets.symmetric(
                horizontal: 15.s,
                vertical: 8.s,
              ),
              child: Row(
                children: [
                  Expanded(
                      child: Row(
                    children: [
                      CircleAvatar(
                        radius: 25.s,
                        backgroundImage:
                            buildUserAvatarProvider(patientInfo.headImgUrl),
                      ),
                      SizedBox(
                        width: 20.s,
                      ),
                      Column(
                        mainAxisAlignment: MainAxisAlignment.center,
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          FISText(
                            patientInfo.name ?? "",
                            style: Storage
                                .currentTheme.dialogStyle.messageTextStyle
                                .copyWith(
                                    fontSize: Storage.currentTheme.dialogStyle
                                        .messageTextStyle.fontSize!.s),
                          ),
                          SizedBox(
                            width: 8.s,
                          ),
                          FISText(
                            patientInfo.phone ?? "",
                            style: Storage
                                .currentTheme.dialogStyle.messageTextStyle
                                .copyWith(
                                    fontSize: Storage.currentTheme.dialogStyle
                                        .messageTextStyle.fontSize!.s),
                          )
                        ],
                      )
                    ],
                  )),
                  Obx(() => Transform.scale(
                        scale: 1.5,
                        child: CustomCheckbox(
                          value:
                              controller.selectedPatient.value?.patientCode ==
                                  patientInfo.patientCode,
                          onChanged: ((value) {
                            _clickPatientCard();
                          }),
                          shape: const CircleBorder(),
                        ),
                      ))
                ],
              )),
        ));
  }

  void _clickPatientCard() {
    try {
      controller.selectedPatient.value = patientInfo;
      if (!controller.isEmergencyMode) {
        var applyConsultationController =
            Get.find<ApplyConsultationController>();
        applyConsultationController.updatePatientInfo(patientInfo);
      } else {
        var emergencyPreparednessController =
            Get.find<EmergencyPreparednessController>();
        emergencyPreparednessController.updatePatientInfo(patientInfo);
      }
    } catch (ex) {
      handlerErrorAsync(ex, "choose patient controller create patient error");
    } finally {
      Get.back();
    }
  }
}
