import 'package:fis_common/extensions/string.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/authentication.m.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/controllers/choosescanpositioncontroller.dart';
import 'package:flyinsonolite/controls/custom/customcheckbox.dart';
import 'package:flyinsonolite/controls/dialog/fisdialog.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/fistextfield.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/consultation.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/localization/localization.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class ChooseScanPositionDialog extends GetView<ChooseScanPositionController> {
  const ChooseScanPositionDialog({super.key});

  @override
  Widget build(BuildContext context) {
    Get.lazyPut<ChooseScanPositionController>(
        () => ChooseScanPositionController());
    controller.initialize();
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      return FISDialog(
        i18nBook.realTimeConsultation.pleaseSelectInspectionSite.t,
        _createChooseScanPositionView(context),
        true,
        true,
        onConfirm: () {
          controller.confirm();
        },
      );
    });
  }

  Widget _createChooseScanPositionView(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        Row(
          children: [
            Expanded(
              child: FISTextField(
                uniqueKey: GlobalKey(),
                onChanged: controller.newScanPosition,
                onSubmitted: onSubmittedAsync,
              ),
            ),
            SizedBox(
              width: 10.s,
            ),
            Obx(() => FISLinearGradientElevatedButton(
                  controller.newScanPosition.value.isNullOrWhiteSpace
                      ? null
                      : controller.addCustomScanPosition,
                  i18nBook.common.add.t,
                  Storage.currentTheme.dialogStyle.buttonGradient,
                  textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                      fontSize:
                          Storage.currentTheme.normalTextStyle.fontSize!.s),
                )),
            SizedBox(
              width: 10.s,
            ),
          ],
        ),
        SizedBox(
          height: 5.s,
        ),
        SizedBox(
          height: (Get.height - 180.s) / 5 * 4,
          width: 600.s,
          child: _createListView(),
        )
      ],
    );
  }

  Future onSubmittedAsync(String value) async {
    try {
      if (Storage.platform == Platform.Windows && Storage.isShowSoftKeyBoard) {
        Storage.isShowSoftKeyBoard = false;
        await jsonRpcProxyForFISLib.consultation.operateSoftKeyBoard(
            FISControlPositionInfo(isShowSoftwareKeyBoard: false));
      }
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "OperateSoftKeyBoard error: $ex");
    }
    controller.addCustomScanPosition();
  }

  Widget _createListView() {
    return Obx(
      () => ListView.separated(
        padding: EdgeInsets.zero,
        itemCount: controller.scanPositionList.value.length,
        itemBuilder: (BuildContext context, int index) {
          return createListViewItem(controller.scanPositionList.value[index]);
        },
        separatorBuilder: (context, index) {
          return Divider(
            height: 3.s,
          );
        },
      ),
    );
  }

  Widget createListViewItem(String scanPosition) {
    return ScanPositionListViewItem(
      scanPosition: scanPosition,
      controller: controller,
    );
  }
}

class ScanPositionListViewItem extends StatelessWidget {
  final _consulttationConstant = 'consultation';
  final String scanPosition;
  final ChooseScanPositionController controller;
  const ScanPositionListViewItem(
      {super.key, required this.scanPosition, required this.controller});

  @override
  Widget build(BuildContext context) {
    var isHover = false.obs;
    return InkWell(
      onHover: (value) {
        isHover.value = value;
      },
      hoverColor: Storage.currentTheme.listViewStyle.hoverColor,
      onTap: () {
        _clickScanPosition();
      },
      child: Obx(() => Container(
          color: isHover.value
              ? Storage.currentTheme.listViewStyle.itembgColor
              : null,
          padding: EdgeInsets.symmetric(
            horizontal: 15.s,
            vertical: 8.s,
          ),
          child: Row(
            children: [
              Expanded(
                child: FISText(
                  getLanguageValues(_consulttationConstant, scanPosition),
                  style: Storage.currentTheme.dialogStyle.messageTextStyle
                      .copyWith(
                          fontSize: Storage.currentTheme.dialogStyle
                              .messageTextStyle.fontSize!.s),
                ),
              ),
              Obx(() => Transform.scale(
                    scale: 1.5,
                    child: CustomCheckbox(
                      value: controller.selectedScanPositionList.value
                          .contains(scanPosition),
                      onChanged: ((value) {
                        _clickScanPosition();
                      }),
                      shape: const CircleBorder(),
                    ),
                  ))
            ],
          ))),
    );
  }

  void _clickScanPosition() {
    if (controller.selectedScanPositionList.value.contains(scanPosition)) {
      controller.selectedScanPositionList.value.remove(scanPosition);
    } else {
      controller.selectedScanPositionList.value.add(scanPosition);
    }

    controller.selectedScanPositionList.refresh();
  }
}
