import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/images/views/consultationimagesview.dart';
import 'package:flyinsonolite/consultation/records/views/consultationinfoview.dart';
import 'package:flyinsonolite/consultation/reports/views/consultationreportview.dart';
import 'package:flyinsonolite/consultation/visits/views/consultationvisitview.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class ConsultationDetailView extends GetView<ConsultationDetailController> {
  const ConsultationDetailView({super.key});

  @override
  Widget build(BuildContext context) {
    return Obx(() {
      final tabs = controller.tabTitles
          .map((e) => SizedBox(
              height: 48.s,
              child: Center(
                child: FISText(
                  e,
                  textAlign: TextAlign.center,
                ),
              )))
          .toList();
      return Column(
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          TabBar(
            indicatorColor: Storage.currentTheme.themeData.indicatorColor,
            onTap: controller.onTabChangedAsync,
            tabs: tabs,
            labelPadding: EdgeInsets.symmetric(horizontal: 16.s),
            indicatorWeight: 2.s,
            controller: controller.tabController,
          ),
          _buildConsultationDetailInfo(),
        ],
      );
    });
  }

  Widget _buildConsultationDetailInfo() {
    switch (controller.currentTabIndex.value) {
      case 0:
        return const ConsultationInfoView();

      case 1:
        return ConsultationImageView(controller);

      case 2:
        return const ConsultationReportView();

      case 3:
        return ConsultationVisitView();

      default:
        return const SizedBox();
    }
  }
}
