import 'package:fis_common/extensions/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/authentication.m.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/controllers/acceptconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/records/views/acceptconsultationdialog.dart';
import 'package:flyinsonolite/consultation/records/controllers/applyconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/records/views/memberavatars.dart';
import 'package:flyinsonolite/consultation/records/views/refuseconsultationdialog.dart';
import 'package:flyinsonolite/consultation/reports/views/reporteditpage.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fislabelinputfield.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/localization/localization.dart';
import 'package:flyinsonolite/consultation/records/models/patientmodel.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/routeconstants.dart';
import 'package:get/get.dart';

class ConsultationInfoView extends GetView<ConsultationDetailController> {
  final _consulttationConstant = 'consultation';
  EdgeInsets get _labelFieldMargin => EdgeInsets.fromLTRB(0, 8.s, 0, 8.s);
  double get _labelWidth =>
      i18nBook.isCurrentChinese || Storage.isPortraitMode.value ? 90.s : 140.s;

  const ConsultationInfoView({super.key});

  @override
  Widget build(BuildContext context) {
    return Expanded(
        child: SingleChildScrollView(
            child: Container(
                padding: EdgeInsets.fromLTRB(15.s, 10.s, 15.s, 10.s),
                color: Storage.currentTheme.listViewStyle.itembgColor,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: [
                    FISText(
                      i18nBook.remedical.patientDetail.t,
                      style: Storage.currentTheme.titleTextStyle.copyWith(
                          fontSize:
                              Storage.currentTheme.titleTextStyle.fontSize!.s),
                    ),
                    SizedBox(
                      height: 10.s,
                    ),
                    _buildPatinetInfo(),
                    Divider(
                      height: 20.s,
                      thickness: 2.s,
                    ),
                    FISText(
                      i18nBook.realTimeConsultation.appointmentInformation.t,
                      style: Storage.currentTheme.titleTextStyle.copyWith(
                          fontSize:
                              Storage.currentTheme.titleTextStyle.fontSize!.s),
                    ),
                    SizedBox(
                      height: 10.s,
                    ),
                    _buildConsultationInfo(),
                    SizedBox(
                      height: 20.s,
                    ),
                    Obx(() => controller.consultationDetail.value == null
                        ? const SizedBox()
                        : Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: [
                              //开始会诊
                              Obx(() => controller.consultationDetail.value!
                                          .isInitiateShow &&
                                      ((Storage.platform == Platform.Windows &&
                                              controller.consultationDetail
                                                      .value!.deviceCode ==
                                                  Storage.usDeviceCode) ||
                                          Storage.platform != Platform.Windows)
                                  ? FISLinearGradientElevatedButtonAsync(
                                      () async {
                                      if (Storage.isConsultating.value) {
                                        PromptBox.toast(
                                            i18nBook.errorCodes.errorCode852.t);
                                        return;
                                      }
                                      await controller.startConsultationAsyn();
                                    }, i18nBook.realTimeConsultation.starttConsultation.t,
                                      Storage.currentTheme.dialogStyle.buttonGradient,
                                      textStyle: Storage.currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller
                                      .consultationDetail.value!.isJoinInShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //进入会诊
                              Obx(() => controller.consultationDetail.value!
                                          .isJoinInShow &&
                                      ((Storage.platform == Platform.Windows &&
                                              controller.consultationDetail
                                                      .value!.deviceCode ==
                                                  Storage.usDeviceCode) ||
                                          Storage.platform != Platform.Windows)
                                  ? FISLinearGradientElevatedButtonAsync(
                                      () async {
                                      if (Storage.isConsultating.value) {
                                        PromptBox.toast(
                                            i18nBook.errorCodes.errorCode861.t);
                                        return;
                                      }
                                      await controller.startConsultationAsyn(
                                          isJoin: true);
                                    }, i18nBook.realTimeConsultation.joinConsultation.t,
                                      Storage.currentTheme.dialogStyle.buttonGradient,
                                      textStyle: Storage.currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller
                                      .consultationDetail.value!.isImproveShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //完善信息
                              Obx(() => controller
                                      .consultationDetail.value!.isImproveShow
                                  ? FISLinearGradientElevatedButton(() {
                                      Get.lazyPut<ApplyConsultationController>(
                                          () => ApplyConsultationController());
                                      final applyConsultationController = Get
                                          .find<ApplyConsultationController>();
                                      applyConsultationController
                                          .isImprove.value = true;
                                      applyConsultationController.isEdit.value =
                                          false;
                                      Get.toNamed(RouteConstants
                                          .consultationRouteConstants
                                          .applyconsultation);
                                    },
                                      i18nBook.common.completeInformation.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller
                                      .consultationDetail.value!.isRevokeShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),

                              //撤回
                              Obx(() => controller
                                      .consultationDetail.value!.isRevokeShow
                                  ? FISLinearGradientElevatedButton(() {
                                      controller.withdraw();
                                    },
                                      i18nBook.realTimeConsultation.withdraw.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller.consultationDetail.value!
                                      .isUpdateConsultationShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //编辑
                              Obx(() => controller.consultationDetail.value!
                                      .isUpdateConsultationShow
                                  ? FISLinearGradientElevatedButton(() {
                                      Get.lazyPut<ApplyConsultationController>(
                                          () => ApplyConsultationController());
                                      final applyConsultationController = Get
                                          .find<ApplyConsultationController>();
                                      applyConsultationController.isEdit.value =
                                          true;
                                      applyConsultationController
                                          .isImprove.value = false;
                                      Get.toNamed(RouteConstants
                                          .consultationRouteConstants
                                          .applyconsultation);
                                    },
                                      i18nBook.common.edit.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller
                                      .consultationDetail.value!.isApprovalShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //同意
                              Obx(() => controller
                                      .consultationDetail.value!.isApprovalShow
                                  ? FISLinearGradientElevatedButton(() {
                                      Get.lazyPut<AcceptConsultationController>(
                                          () => AcceptConsultationController());
                                      var acceptConsultationController = Get
                                          .find<AcceptConsultationController>();
                                      acceptConsultationController
                                              .currentConsultation =
                                          controller.consultationDetail.value!;
                                      Get.dialog(
                                          const AcceptConsultationDialog(),
                                          barrierDismissible: false);
                                    },
                                      i18nBook.remedical.agree.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller
                                      .consultationDetail.value!.isRejectShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //拒绝
                              Obx(() => controller
                                      .consultationDetail.value!.isRejectShow
                                  ? FISLinearGradientElevatedButton(() {
                                      Get.dialog(RefuseConsultationDialog(),
                                          barrierDismissible: false);
                                    },
                                      i18nBook.user.refuse.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller.consultationDetail.value!
                                          .isAcceptInvitationShow &&
                                      controller.consultationDetail.value!
                                          .isRejectInvitationShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //接受邀请
                              Obx(() => controller.consultationDetail.value!
                                          .isAcceptInvitationShow &&
                                      controller.consultationDetail.value!
                                          .isRejectInvitationShow
                                  ? FISLinearGradientElevatedButtonAsync(
                                      () async {
                                      await controller.agreeInvitationAsync();
                                    },
                                      i18nBook.realTimeConsultation
                                          .acceptInvitation.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller.consultationDetail.value!
                                          .isAcceptInvitationShow &&
                                      controller.consultationDetail.value!
                                          .isRejectInvitationShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //拒绝邀请
                              Obx(() => controller.consultationDetail.value!
                                          .isAcceptInvitationShow &&
                                      controller.consultationDetail.value!
                                          .isRejectInvitationShow
                                  ? FISLinearGradientElevatedButton(() {
                                      Get.dialog(
                                          RefuseConsultationDialog(
                                            isThirdParty: true,
                                          ),
                                          barrierDismissible: false);
                                    },
                                      i18nBook.realTimeConsultation
                                          .declineInvitation.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller
                                      .consultationDetail.value!.isReportShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //第三方已同意邀约，可切换至拒绝
                              Obx(() => !controller.consultationDetail.value!
                                          .isAcceptInvitationShow &&
                                      controller.consultationDetail.value!
                                          .isRejectInvitationShow
                                  ? FISLinearGradientElevatedButton(() {
                                      Get.dialog(
                                          RefuseConsultationDialog(
                                            isThirdParty: true,
                                          ),
                                          barrierDismissible: false);
                                    },
                                      i18nBook.realTimeConsultation
                                          .changeToDeclineInvitation.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller.consultationDetail.value!
                                          .isAcceptInvitationShow &&
                                      controller.consultationDetail.value!
                                          .isRejectInvitationShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //第三方已拒绝邀约，可切换至同意
                              Obx(() => controller.consultationDetail.value!
                                          .isAcceptInvitationShow &&
                                      !controller.consultationDetail.value!
                                          .isRejectInvitationShow
                                  ? FISLinearGradientElevatedButtonAsync(
                                      () async {
                                      await controller.agreeInvitationAsync();
                                    },
                                      i18nBook.realTimeConsultation
                                          .changeToAcceptInvitation.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                              Obx(() => controller
                                      .consultationDetail.value!.isReportShow
                                  ? SizedBox(
                                      width: 10.s,
                                    )
                                  : const SizedBox()),
                              //撰写报告
                              Obx(() => controller
                                      .consultationDetail.value!.isReportShow
                                  ? FISLinearGradientElevatedButton(() {
                                      Get.to(ReportEditPage(controller
                                          .consultationDetail.value!));
                                    },
                                      i18nBook.remedical.writeReport.t,
                                      Storage.currentTheme.dialogStyle
                                          .buttonGradient,
                                      textStyle: Storage
                                          .currentTheme.normalTextStyle
                                          .copyWith(
                                              fontSize: Storage.currentTheme
                                                  .normalTextStyle.fontSize!.s))
                                  : const SizedBox()),
                            ],
                          ))
                  ],
                ))));
  }

  Widget _buildPatinetInfo() {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      double wrapWidth = constraints.maxWidth;
      if (Storage.configurationFromerver == null) {
        return const SizedBox();
      } else {
        return Wrap(
          children: [
            ...Storage
                .configurationFromerver!.consultationServerConfig.patientKeyList
                .map((e) {
              switch (e.showType) {
                case "input":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.patient.value[e.key].isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller.patient.value[e.key]!,
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case "AgeType":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() {
                      String ageText = '';
                      switch (controller.patient.value['AgeUnits'] ?? '') {
                        case 'YearAndMonth':
                          if (controller.patient.value["AgeYear"]
                                  .isNullOrWhiteSpace &&
                              controller.patient.value["AgeMonth"]
                                  .isNullOrWhiteSpace) {
                            ageText = i18nBook.remedical.notFill.t;
                          } else {
                            ageText =
                                "${controller.patient.value["AgeYear"]}${i18nBook.remedical.years.t}${controller.patient.value["AgeMonth"]}${i18nBook.remedical.month.t}";
                          }
                          break;
                        case 'MonthAndWeek':
                          if (controller.patient.value["AgeMonth"]
                                  .isNullOrWhiteSpace &&
                              controller.patient.value["AgeWeek"]
                                  .isNullOrWhiteSpace) {
                            ageText = i18nBook.remedical.notFill.t;
                          } else {
                            ageText =
                                "${controller.patient.value["AgeMonth"]}${i18nBook.remedical.month.t}${controller.patient.value["AgeWeek"]}${i18nBook.remedical.week.t}";
                          }
                          break;
                        case 'WeekAndDay':
                          if (controller.patient.value["AgeWeek"]
                                  .isNullOrWhiteSpace &&
                              controller
                                  .patient.value["AgeDay"].isNullOrWhiteSpace) {
                            ageText = i18nBook.remedical.notFill.t;
                          } else {
                            ageText =
                                "${controller.patient.value["AgeWeek"]}${i18nBook.remedical.week.t}${controller.patient.value["AgeDay"]}${i18nBook.remedical.day.t}";
                          }
                          break;
                        default:
                          if (controller
                              .patient.value["AgeYear"].isNullOrWhiteSpace) {
                            ageText = i18nBook.remedical.notFill.t;
                          } else {
                            ageText =
                                "${controller.patient.value["AgeYear"]}${i18nBook.remedical.years.t}";
                          }
                          break;
                      }

                      return RichText(
                        text: TextSpan(
                            style: Storage.currentTheme.normalTextStyle
                                .copyWith(
                                    fontSize: Storage.currentTheme
                                        .normalTextStyle.fontSize!.s),
                            children: [
                              TextSpan(
                                text: ageText,
                                style: Storage.currentTheme.normalTextStyle
                                    .copyWith(
                                        fontSize: Storage.currentTheme
                                            .normalTextStyle.fontSize!.s),
                              ),
                            ]),
                      );
                    }),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case "select":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          _getPatientDataString(e),
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case "inputButton":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.patient.value[e.key].isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller.patient.value[e.key]!,
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case 'textarea':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.patient.value[e.key].isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller.patient.value[e.key]!,
                          maxLines: 6,
                          softWrap: true,
                        )),
                    wrapWidth,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                default:
                  return const SizedBox();
              }
            }).toList()
          ],
        );
      }
    });
  }

  String _getPatientDataString(PatientModel patientModel) {
    if (patientModel.key == "Sex" &&
        controller.patient.value.containsKey(patientModel.key)) {
      var sex = getLanguageValues(
          "exam", controller.patient.value[patientModel.key]!);
      return sex.isNullOrWhiteSpace ? i18nBook.remedical.notFill.t : sex;
    }

    return controller.patient.value[patientModel.key].isNullOrWhiteSpace
        ? i18nBook.remedical.notFill.t
        : controller.patient.value[patientModel.key]!;
  }

  Widget _buildConsultationInfo() {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      double wrapWidth = constraints.maxWidth;
      if (Storage.configurationFromerver == null) {
        return const SizedBox();
      } else {
        return Wrap(
          children: [
            ...Storage.configurationFromerver!.consultationServerConfig
                .appointmentKeyList
                .map((e) {
              switch (e.key) {
                case "Device":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.consultationDetail.value?.displayName ??
                              controller.consultationDetail.value?.deviceName ??
                              i18nBook.remedical.notFill.t,
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case "ConsultationTime":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.getConsultationTime(),
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case "ScanUser":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!
                                      .scanUserName.isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller
                                  .consultationDetail.value!.scanUserName!,
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case "Diseases":
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!.diseases
                                      .isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller.consultationDetail.value!.diseases!,
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case 'ScanPosition':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!
                                          .scanPositions ==
                                      null ||
                                  controller.consultationDetail.value!
                                      .scanPositions!.isEmpty
                              ? i18nBook.remedical.notFill.t
                              : controller
                                  .consultationDetail.value!.scanPositions!
                                  .map((data) => getLanguageValues(
                                      _consulttationConstant, data))
                                  .join(";"),
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case 'ExpertOrganization':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!
                                      .expertOrganizationName.isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller.consultationDetail.value!
                                  .expertOrganizationName!,
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case 'Expert':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!
                                      .expertUserName.isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller
                                  .consultationDetail.value!.expertUserName!,
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case 'ApplyUser':
                  return Storage.user.isApplyUserField
                      ? FISLabelInputField(
                          e.required,
                          getLanguageValues(_consulttationConstant, e.key),
                          Obx(() => FISText(
                                controller.consultationDetail.value == null ||
                                        controller.consultationDetail.value!
                                            .applyUserName.isNullOrWhiteSpace
                                    ? i18nBook.remedical.notFill.t
                                    : controller.consultationDetail.value!
                                        .applyUserName!,
                                softWrap: true,
                              )),
                          wrapWidth * 0.5,
                          labelWidth: _labelWidth,
                          labelColor:
                              Storage.currentTheme.normalTextStyle.color,
                          padding: _labelFieldMargin,
                        )
                      : const SizedBox();
                case 'ApplyOrganization':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!
                                      .applyOrganizationName.isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller.consultationDetail.value!
                                  .applyOrganizationName!,
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case 'Operator':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!
                                      .operateUserName.isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller
                                  .consultationDetail.value!.operateUserName!,
                          softWrap: true,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                case 'PrimaryDiagnosis':
                  return FISLabelInputField(
                    e.required,
                    getLanguageValues(_consulttationConstant, e.key),
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!
                                      .primaryDiagnosis.isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller
                                  .consultationDetail.value!.primaryDiagnosis!,
                          maxLines: 6,
                          softWrap: true,
                        )),
                    wrapWidth,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  );
                default:
                  return const SizedBox();
              }
            }).toList(),
            Obx(() =>
                controller.consultationDetail.value?.consultationMembers !=
                            null &&
                        controller.consultationDetail.value!
                            .consultationMembers!.isNotEmpty
                    ? FISLabelInputFieldWithWrap(
                        false,
                        i18nBook.realTimeConsultation.participants.t,
                        Obx(
                          () => MemberAvatars(
                            members: controller
                                .consultationDetail.value!.consultationMembers!,
                          ),
                        ),
                        wrapWidth,
                        labelWidth: _labelWidth,
                        labelColor: Storage.currentTheme.normalTextStyle.color,
                        padding: _labelFieldMargin,
                      )
                    : const SizedBox()),
            Obx(() => controller.isShowDescriptionAndParticipants()
                ? FISLabelInputField(
                    false,
                    fisI18nBook.common.consultationDescription.t,
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!
                                      .description.isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller
                                  .consultationDetail.value!.description!,
                        )),
                    wrapWidth,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  )
                : const SizedBox()),
            Obx(() => controller.consultationDetail.value != null &&
                    controller.consultationDetail.value!.rejectReason
                        .isNullOrWhiteSpace
                ? const SizedBox()
                : FISLabelInputField(
                    false,
                    i18nBook.realTimeConsultation.reasonsForRefusal.t,
                    Obx(() => FISText(
                          controller.consultationDetail.value == null ||
                                  controller.consultationDetail.value!
                                      .rejectReason.isNullOrWhiteSpace
                              ? i18nBook.remedical.notFill.t
                              : controller
                                  .consultationDetail.value!.rejectReason!,
                        )),
                    wrapWidth * 0.5,
                    labelWidth: _labelWidth,
                    labelColor: Storage.currentTheme.normalTextStyle.color,
                    padding: _labelFieldMargin,
                  ))
          ],
        );
      }
    });
  }
}
