import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/filters/controllers/filterdrawercontroller.dart';
import 'package:flyinsonolite/consultation/fispanel/consultationfispanel.dart';
import 'package:flyinsonolite/consultation/records/controllers/applyconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/records/views/consultationdetaildialog.dart';
import 'package:flyinsonolite/consultation/records/views/consultationdetailview.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationlistcontroller.dart';
import 'package:flyinsonolite/consultation/filters/views/filterdrawer.dart';
import 'package:flyinsonolite/controls/dialog/exitorhideConfirmDialog.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/consultation/records/models/features.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/home/controllers/homecontroller.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/icons/fisicons.dart';
import 'package:flyinsonolite/managers/interfaces/inetworkerrorhandlermanager.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/routeconstants.dart';
import 'package:flyinsonolite/consultation/records/models/consultationviewmodel.dart';
import 'package:get/get.dart';

class ConsultationListPage extends GetView<ConsultationListController> {
  final INetworkErrorHandlerManager _networkErrorHandlerManager =
      Get.find<INetworkErrorHandlerManager>();

  final GlobalKey _globalKey = GlobalKey();
  final HomeController _homeController = Get.find<HomeController>();
  ScaffoldState get _scaffoldState => _globalKey.currentState as ScaffoldState;
  ConsultationListPage({super.key});

  @override
  Widget build(BuildContext context) {
    Get.lazyPut<ConsultationListController>(() => ConsultationListController());
    Get.lazyPut<FilterDrawerController>(() => FilterDrawerController());
    Get.lazyPut<ConsultationFilter>(() => ConsultationFilter());
    Get.lazyPut<ConsultationDetailController>(
        () => ConsultationDetailController());
    return FutureBuilder(
        future: controller.initilizeAsync(),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return LayoutBuilder(builder: (context, constraint) {
              return Scaffold(
                resizeToAvoidBottomInset: false,
                backgroundColor:
                    Storage.currentTheme.themeData.scaffoldBackgroundColor,
                key: _globalKey,
                body: _buildBody(),
                endDrawer: FilterDrawer(
                  closeDrawer: () {
                    _scaffoldState.closeEndDrawer();
                  },
                ),
              );
            });
          } else {
            return Scaffold(
              resizeToAvoidBottomInset: false,
              backgroundColor:
                  Storage.currentTheme.themeData.scaffoldBackgroundColor,
              key: _globalKey,
            );
          }
        });
  }

  Widget _buildBody() {
    return Obx(() => Stack(
          children: [
            Flex(
              direction: Axis.vertical,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                _createHeader(),
                Expanded(
                    child: Container(
                  color: Storage.currentTheme.bgContainerColor,
                  child: Flex(
                    direction: Axis.horizontal,
                    children: [
                      Flexible(
                          flex: 4,
                          child: Container(
                            padding: EdgeInsets.fromLTRB(8.s, 5.s, 5.s, 5.s),
                            child: _createListView(),
                          )),
                      Storage.isPortraitMode.value
                          ? const SizedBox()
                          : VerticalDivider(
                              width: 5.s,
                            ),
                      Storage.isPortraitMode.value
                          ? const SizedBox()
                          : Flexible(
                              flex: 6,
                              child: Obx(
                                () => controller.selectedItem.value == null
                                    ? const SizedBox()
                                    : Container(
                                        alignment: Alignment.topLeft,
                                        padding: EdgeInsets.all(5.s),
                                        child: const ConsultationDetailView(),
                                      ),
                              )),
                    ],
                  ),
                )),
              ],
            ),
            Positioned(
                right: 0,
                bottom: 0,
                child: Obx(() => Storage.isUseFISForConsultation.value ||
                        (!Storage.isUseFISForConsultation.value &&
                            Storage.isConsultating.value &&
                            Storage.hasMultiScreens)
                    ? const ConsulationFISPanel()
                    : const SizedBox()))
          ],
        ));
  }

  Widget _createListView() {
    var scrollController = ScrollController();
    scrollController.addListener(() {
      if (scrollController.position.pixels ==
              scrollController.position.maxScrollExtent &&
          scrollController.position.axisDirection == AxisDirection.down) {
        if (controller.consultationList.length < controller.totalCount) {
          controller.searchMoreAsync();
        }
      }
    });
    return Obx(
      () => ListView.separated(
        controller: scrollController,
        padding: EdgeInsets.zero,
        itemCount: controller.consultationList.length,
        itemBuilder: (BuildContext context, int index) {
          return createListViewItem(controller.consultationList[index]);
        },
        separatorBuilder: (context, index) {
          return SizedBox(
            height: 5.s,
          );
        },
      ),
    );
  }

  Widget createListViewItem(ConsultationViewModel itemViewModel) {
    return ConsultationListViewItem(
      itemViewModel: itemViewModel,
      controller: controller,
    );
  }

  Widget _createHeader() {
    return Container(
        color: Storage.currentTheme.homepageHeaderColor,
        height: 72.s,
        child: Flex(
          direction: Axis.horizontal,
          children: [
            Flexible(
              fit: FlexFit.tight,
              child: Row(
                children: [
                  SizedBox(
                    width: 20.s,
                  ),
                  FISIconButton(
                    onPressed: () {
                      Get.back();
                      if (Get.currentRoute == RouteConstants.home) {
                        _homeController.isHomePage.value = true;
                      }
                    },
                    icon: const Icon(Icons.arrow_back),
                    label: i18nBook.common.back.t,
                  ),
                  ..._showoptionbutton(true),
                ],
              ),
            ),
            Row(
              mainAxisAlignment: MainAxisAlignment.end,
              children: [
                FISText(
                  Storage.user.displayName!,
                  style: Storage.currentTheme.normalTextStyle
                      .copyWith(fontSize: 24.s),
                ),
                SizedBox(
                  width: 15.s,
                ),
                FISIconButtonAsync(
                  onPressed: () async {
                    await controller.logoutAsync();
                  },
                  icon: const Icon(Icons.logout),
                  label: i18nBook.auth.signOut.t,
                ),
                Obx(() => Storage.isShowCloseButton.value
                    ? SizedBox(
                        width: 20.s,
                      )
                    : const SizedBox()),
                Obx(() => Storage.isShowCloseButton.value
                    ? FISIconButton(
                        onPressed: () {
                          Get.dialog(const ExitOrHideConfirmDialog(),
                              barrierDismissible: false);
                        },
                        icon: const Icon(
                          Icons.close,
                        ),
                        iconSize: 56.s,
                      )
                    : const SizedBox()),
              ],
            ),
          ],
        ));
  }

  List<Widget> _showoptionbutton(bool isShow) {
    return isShow
        ? [
            SizedBox(
              width: 20.s,
            ),
            Obx(() => !_networkErrorHandlerManager.isNetworkAbnormal.value &&
                    Storage.user.hasFeature(FeatureKeys.ApplyConsultation)
                ? FISIconButton(
                    onPressed: () {
                      Get.lazyPut<ApplyConsultationController>(
                          () => ApplyConsultationController());
                      final applyConsultationController =
                          Get.find<ApplyConsultationController>();
                      applyConsultationController.isEdit.value = false;
                      applyConsultationController.isImprove.value = false;
                      Get.toNamed(RouteConstants
                          .consultationRouteConstants.applyconsultation);
                    },
                    icon: const Icon(Icons.add_circle_outline),
                    label: fisI18nBook.common.createApplication.t)
                : const SizedBox()),
            Obx(() => !_networkErrorHandlerManager.isNetworkAbnormal.value &&
                    Storage.user.hasFeature(FeatureKeys.ApplyConsultation)
                ? SizedBox(
                    width: 20.s,
                  )
                : const SizedBox()),
            Obx(() => !_networkErrorHandlerManager.isNetworkAbnormal.value &&
                    Storage.user.hasFeature(FeatureKeys.EmergencyConsultation)
                ? FISIconButtonAsync(
                    onPressed: () async {
                      await controller.startEmergencyAsync();
                    },
                    icon: const Icon(FISICons.realtimeconsultation),
                    label: i18nBook.realTimeConsultation.emergency.t,
                  )
                : const SizedBox()),
            Obx(() => _networkErrorHandlerManager.isNetworkAbnormal.value
                ? FISText(
                    i18nBook.remoteControl.networkConnectionError.t,
                    style: Storage.currentTheme.normalTextStyle.copyWith(
                        color: Colors.red,
                        fontSize:
                            Storage.currentTheme.normalTextStyle.fontSize!.s),
                  )
                : const SizedBox()),
            Obx(() => !_networkErrorHandlerManager.isNetworkAbnormal.value &&
                    Storage.user.hasFeature(FeatureKeys.EmergencyConsultation)
                ? SizedBox(
                    width: 20.s,
                  )
                : const SizedBox()),
            Obx(() => _networkErrorHandlerManager.isNetworkAbnormal.value
                ? const SizedBox()
                : FISIconButtonAsync(
                    onPressed: () async {
                      var filterDrawerController =
                          Get.find<FilterDrawerController>();
                      await filterDrawerController.initializeAsync();
                      _scaffoldState.openEndDrawer();
                    },
                    icon: const Icon(Icons.filter_alt_rounded),
                    label: i18nBook.common.filter.t,
                  )),
            SizedBox(width: 20.s),
            Obx(() => _networkErrorHandlerManager.isNetworkAbnormal.value
                ? const SizedBox()
                : FISIconButtonAsync(
                    onPressed: () async {
                      await controller.refreshAsync();
                    },
                    icon: const Icon(Icons.refresh_rounded),
                    label: i18nBook.common.refresh.t,
                  )),
          ]
        : [];
  }
}

class ConsultationListViewItem extends StatelessWidget {
  final ConsultationViewModel itemViewModel;
  final ConsultationListController controller;
  const ConsultationListViewItem(
      {super.key, required this.itemViewModel, required this.controller});

  @override
  Widget build(BuildContext context) {
    return MouseRegion(
        cursor: SystemMouseCursors.click,
        child: GestureDetector(
            onTap: () async {
              await controller.updateSelectedConsultationAsync(itemViewModel);
              if (Storage.isPortraitMode.value) {
                Get.dialog(const ConsultationDetailDialog(),
                    barrierDismissible: false);
              }
            },
            child: Container(
              color: Storage.currentTheme.listViewStyle.itembgColor,
              child: Row(mainAxisAlignment: MainAxisAlignment.start, children: [
                Obx(() => controller.selectedItem.value?.consultationCode ==
                        itemViewModel.consultationCode
                    ? Container(
                        width: 10.s,
                        height: 80.s,
                        color: Storage
                            .currentTheme.listViewStyle.selecteLeftBorderColodr)
                    : Container(
                        width: 10.s,
                        height: 80.s,
                        color: Storage.currentTheme.listViewStyle.itembgColor)),
                SizedBox(
                  width: 10.s,
                ),
                Expanded(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 10.s,
                      ),
                      FISText(
                        itemViewModel.patientName ?? "",
                        style: Storage.currentTheme.normalTextStyle.copyWith(
                            fontWeight: FontWeight.bold,
                            fontSize: Storage
                                .currentTheme.normalTextStyle.fontSize!.s),
                      ),
                      SizedBox(
                        height: 10.s,
                      ),
                      FISText(
                        itemViewModel.displayInfo,
                        overflow: TextOverflow.ellipsis,
                      ),
                      SizedBox(
                        height: 10.s,
                      ),
                    ],
                  ),
                ),
                Obx(() => controller.selectedItem.value?.consultationCode ==
                        itemViewModel.consultationCode
                    ? Row(
                        children: [
                          Obx(() => FISText(
                              _translateState(itemViewModel.state.value!))),
                          FISIconButtonAsync(
                              onPressed: () async {
                                await controller.switchToLocalPatientAsync(
                                    itemViewModel.consultationCode);
                              },
                              icon: const Icon(Icons.account_circle),
                              iconSize: 55.s,
                              color: Storage.currentTheme.listViewStyle
                                  .selecteLeftBorderColodr)
                        ],
                      )
                    : Column(
                        crossAxisAlignment: CrossAxisAlignment.end,
                        children: [
                          Obx(() => FISText(
                              _translateState(itemViewModel.state.value!))),
                          SizedBox(
                            height: 10.s,
                          ),
                          FISText(itemViewModel.timeInfo),
                        ],
                      )),
                SizedBox(
                  width: 15.s,
                )
              ]),
            )));
  }

  String _translateState(TransactionStatusEnum statusEnum) {
    switch (statusEnum) {
      case TransactionStatusEnum.Applied:
        return i18nBook.user.underVerification.t;
      case TransactionStatusEnum.Withdrawn:
        return i18nBook.realTimeConsultation.withdrawn.t;
      case TransactionStatusEnum.Rejected:
        return i18nBook.realTimeConsultation.rejected.t;
      case TransactionStatusEnum.ToStart:
        return i18nBook.realTimeConsultation.toBeStarted.t;
      case TransactionStatusEnum.InProgress:
        return i18nBook.realTimeConsultation.haveInHand.t;
      case TransactionStatusEnum.PendingReport:
        return i18nBook.realTimeConsultation.toBeReported.t;
      case TransactionStatusEnum.End:
        return i18nBook.realTimeConsultation.closed.t;
      case TransactionStatusEnum.Expired:
        return i18nBook.realTimeConsultation.expired.t;
      default:
        return i18nBook.common.all.t;
    }
  }
}
