import 'package:fis_jsonrpc/services/liveConsultation.m.dart';
import 'package:fis_jsonrpc/services/notification.m.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/views/texttooltip.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';

/// 会诊成员头像列表
class MemberAvatars extends StatelessWidget {
  const MemberAvatars({
    super.key,
    required this.members,
  });
  final List<ConsultationMemberDTO> members;

  /// 状态角标尺寸
  double get _iconSize => 16.s;

  /// 详情页内单个元素占位尺寸
  Size get _itemSizeInDetail => Size(120.s, 50.s);

  /// 头像容器占位尺寸(上方要多10，给角标留位置)
  Size get _avatarContainerSize => Size(50.s, 40.s);

  /// 头像尺寸
  double get _avatarSize => 30.s;

  /// 角标偏移量()
  Offset get _iconOffset => Offset(3.s, 3.s);

  /// 滚动控制器

  @override
  Widget build(BuildContext context) {
    return Wrap(
      children: [
        ...members
            .map((e) => _buildDetailMemberItem(
                  e.name ?? "",
                  e.headImageToken ?? "",
                  e.memberStatus,
                ))
            .toList(),
      ],
    );
  }

  /// 详情页的参与人展示
  Widget _buildDetailMemberItem(
      String name, String avatar, LiveConsultationMemberStatus status) {
    return TextTooltip(
      message: name,
      height: 26.s,
      padding: EdgeInsets.symmetric(horizontal: 10.s),
      textStyle: TextStyle(
        color: Colors.white,
        fontSize: 12.s,
      ),
      position: DisplayPosition.bottom,
      child: Container(
        width: _itemSizeInDetail.width,
        height: _itemSizeInDetail.height,
        decoration: BoxDecoration(
          border: Border.all(color: Colors.black38, width: 1.s),
          borderRadius: BorderRadius.circular(4.s),
        ),
        margin: EdgeInsets.only(right: 5.s),
        child: Stack(
          children: [
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                _buildAvatar(avatar, name),
                Expanded(
                  child: Align(
                    alignment: Alignment.centerLeft,
                    child: FISText(
                      name,
                      overflow: TextOverflow.ellipsis,
                      style: TextStyle(
                          fontSize: 14.s,
                          color: Storage.currentTheme.normalTextStyle.color),
                    ),
                  ),
                ),
              ],
            ),
            Positioned(
              right: _iconOffset.dx,
              top: _iconOffset.dy,
              child: _buildStatusIcon(status),
            )
          ],
        ),
      ),
    );
  }

  Widget _buildAvatar(String avatar, String name) {
    Widget avatarWidget;
    if (avatar == '') {
      avatarWidget = Image(
        image: const AssetImage("assets/images/avatar_man.png"),
        width: _avatarSize,
        height: _avatarSize,
      );
    } else {
      avatarWidget = Image(
        image: NetworkImage(avatar),
        width: _avatarSize,
        height: _avatarSize,
        errorBuilder: (context, error, stackTrace) {
          return Image.asset(
            'assets/images/avatar_man.png',
            fit: BoxFit.cover,
          );
        },
      );
    }
    return SizedBox(
      width: _avatarContainerSize.width,
      height: _avatarContainerSize.height,
      child: Center(
        child: Container(
            width: _avatarSize,
            height: _avatarSize,
            decoration: BoxDecoration(
              borderRadius: BorderRadius.circular(4),
            ),
            clipBehavior: Clip.antiAlias,
            child: avatarWidget),
      ),
    );
  }

  Widget _buildStatusIcon(LiveConsultationMemberStatus status) {
    /// 状态映射
    Color iconColor = Colors.transparent;
    Color bgColor = Colors.transparent;
    IconData iconData = Icons.check_circle;
    switch (status) {
      case LiveConsultationMemberStatus.Default:
        iconColor = Colors.orange;
        bgColor = Colors.orange.shade100;
        iconData = Icons.info;
        break;
      case LiveConsultationMemberStatus.Accepted:
        iconColor = Colors.green;
        bgColor = Colors.green.shade100;
        iconData = Icons.check_circle;
        break;
      case LiveConsultationMemberStatus.Rejected:
        iconColor = Colors.red;
        bgColor = Colors.red.shade100;
        iconData = Icons.do_not_disturb_on;
        break;
      case LiveConsultationMemberStatus.Left:
        iconColor = Colors.grey;
        bgColor = Colors.grey.shade100;
        iconData = Icons.cancel;
        break;
      default:
        iconColor = Colors.transparent;
        break;
    }
    return Container(
        width: _iconSize,
        height: _iconSize,
        decoration: BoxDecoration(
          color: iconColor,
          borderRadius: BorderRadius.circular(
            _iconSize,
          ),
        ),
        child: Icon(
          iconData,
          color: bgColor,
          size: _iconSize,
        ));
  }
}
