import 'dart:convert';
import 'package:fis_common/event/event_type.dart';
import 'package:fis_common/extensions/index.dart';
import 'package:fis_common/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationlistcontroller.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/element_tag_names.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/input_image_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/input_image_list_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/input_text_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/single_selected_info.dart';
import 'package:flyinsonolite/consultation/reports/views/reporteditstate.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/helpers/pdfhelper.dart';
import 'package:flyinsonolite/consultation/reports/controllers/thesauruscontroller.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/managers/interfaces/ireportmanager.dart';
import 'package:flyinsonolite/consultation/reports/models/reporteditsetting.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/services/localreporteditorservice.dart';
import 'package:flyinsonolite/infrastructure/filestorage.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';

class ReportEditController extends GetxController
    with GetSingleTickerProviderStateMixin {
  static const String aiRporterName = "VAid";
  late final ThesaurusController thesaurusController;

  /// 本地报告编辑器持久化缓存服务
  late final LocalReportEditorService localReportEditorService =
      Get.find<LocalReportEditorService>();
  final IReportManager _reportManager = Get.find<IReportManager>();
  final IAppointmentManager _appointmentManager =
      Get.find<IAppointmentManager>();
  final state = ReportEditState();
  final String? reportCode;
  ReportDTO? _serverReport;
  ConsultationDetailDTO consultationDetailDTO;
  ReportEditorSetting reportEditorSetting = ReportEditorSetting.ins;
  bool get isReloadFromLocalCache =>
      reportEditorSetting.reportDraft.isNotEmpty &&
      reportEditorSetting.reportDraftRecordCode ==
          consultationDetailDTO.consultationCode;

  /// 标签页列表
  List<String> get tabTitles => [
        i18nBook.remedical.cloudImageShort.t,
        i18nBook.realTimeConsultation.consultationImage.t,
      ];

  /// tab选择器
  late TabController tabController = TabController(
    vsync: this,
    length: tabTitles.length,
    initialIndex: 0,
  );

  ///图像选择时通知报告
  final FEventHandler<String> onSelect = FEventHandler<String>();

  final _fileStorage = FileStorage.createInstance("remedical/report");

  String get reportName => _getReportName();
  ReportDataPipe get fileStorage => ReportDataPipe(
      cacheGet: _fileStorage.getFile, cacheSave: _fileStorage.setFile);
  Future<bool> Function(List<int> data, String fileName)? get shellExporter =>
      TextMediaFileExporterForShell.exportFile;

  ReportEditController(this.consultationDetailDTO, {this.reportCode}) {
    thesaurusController = ThesaurusController(this);
    state.reportCode.value = reportCode;
  }

  @override
  void onReady() async {
    super.onReady();
    await _initDataAsync(false);
  }

  @override
  void onClose() {
    super.onClose();
    onSelect.dispose();
    thesaurusController.dispose();
    //reportCacheController.removeScheduledCacheReport();
    //FReportInfo.instance.onSpeechInput.removeListener(_openSpeechInput);
    //MeasureDataHelper.source = VidImageSource.Remedical;
    //labelInputController.dispose();
  }

  ///初始化报告模板、报告缓存、报告编辑器缓存
  Future _initDataAsync(bool isReload) async {
    try {
      EasyLoading.show(status: i18nBook.common.loading.t);
      //await initParameters();
      _initReportEditorCache();
      await thesaurusController.initDefaultThesaurusAsync();
      await _initPatientInfoAsync();
      String creatorName = consultationDetailDTO.expertUserName ?? '';
      if (reportCode.isNotNullOrEmpty) {
        await _initServerReport();
      } else {
        if (!isReload) {
          await _initUserDefaultTemplateAsync();
          var reportTemplateJsonStr =
              state.selectedReportTemplate.value?.reportTemplateJson;
          if (isReloadFromLocalCache) {
            final json =
                jsonDecode(reportEditorSetting.reportDraft) as List<dynamic>;
            bool checkIsSameTemplate =
                _checkIsSameTemplate(reportTemplateJsonStr, json);
            if (checkIsSameTemplate) {
              await Future.delayed(const Duration(milliseconds: 500), () {
                FReportInfo.instance.fromJson(json);
              });
              //FReportInfo.instance.fromJson(json);
            }
          }
        }

        _initReportInfo(creatorName);
      }
      _initImageInputChangeListener();
      //await getRemedicalList();
      _autoFillCurrentUserSignature();
      //_addReportInfoListen();
      await _initReportTemplatesAsync();
      await _refreshImagesAsync();
      onSelect.removeListener(_onSelectCallBack);
      onSelect.addListener(_onSelectCallBack);
    } catch (e) {
      await handlerErrorAsync(e, 'ReportEditController initData: $e');
    } finally {
      EasyLoading.dismiss();
    }
  }

  void initImageInput() {
    _initImageInputChangeListener();
    state.selectedImages.clear();
    onSelect.removeListener(_onSelectCallBack);
    onSelect.addListener(_onSelectCallBack);
  }

  void _initImageInputChangeListener() {
    Future.delayed(const Duration(milliseconds: 1000), () {
      final imagesInput =
          FReportInfo.instance.getElementInfoByTagName(TagNames.SELECTIMAGES);
      if (imagesInput != null) {
        var imagesInfo = imagesInput as InputImageListInfo;
        imagesInfo.onRemove.addListener((sender, e) {
          final selectedImages = state.selectedImages;
          if (selectedImages.contains(e)) {
            state.selectedImages.remove(e);
          }
        });
        imagesInfo.onSelectedInputChange.addListener((sender, e) {
          //inputImagesIsSelected = e;
          if (e) {
            state.selectedImages.clear();
            state.selectedImages.addAll(imagesInfo.selectedImages);
          } else {
            state.selectedImages.clear();
          }
          //onSelectedInputChange.emit(this, state.selectedImages);
        });
      }
    });
  }

  Future _refreshImagesAsync() async {
    var consultationImageInfos = await _appointmentManager
        .getConsultationImagesAsync(consultationDetailDTO.consultationCode!);
    List<ConsultationFileDTO> consultationImages = [];
    //List<RemedicalMeasuredInfoDTO> measureImages = [];
    List<RemedicalInfoDTO> remedicalImages = [];
    for (var image in consultationImageInfos) {
      consultationImages.addAll(image.screenshotList ?? []);
      //measureImages.addAll(image.measurementImageList ?? []);
      remedicalImages.addAll(image.ultrasoundImageList ?? []);
    }
    state.remedicalItemList.value = remedicalImages;
    state.consultationImages.value = consultationImages;
  }

  ///从Server加载报告
  Future _initServerReport() async {
    _serverReport = await _reportManager.findReportByCodeAsync(reportCode!);
    if (_serverReport == null) {
      return;
    }

    final serverTempalte = ReportTemplateDTO(
      reportTemplateName: _serverReport!.reportTemplateName,
      reportTemplateJson: _serverReport!.reportTemplateJson,
    );
    state.selectedReportTemplate.value = serverTempalte;
    if (_serverReport!.reportTemplateJson.isNullOrWhiteSpace) {
      await _initUserDefaultTemplateAsync();
    }

    final jsonStr = _serverReport!.reportDatasJson ?? '';
    final json = jsonDecode(jsonStr) as List<dynamic>;
    //FReportInfo.instance.fromJson(json);
    await Future.delayed(const Duration(milliseconds: 500), () {
      FReportInfo.instance.fromJson(json);
    });
  }

  Future refreshImagesAsync() async {
    EasyLoading.show(status: i18nBook.common.loading.t);
    try {
      await _refreshImagesAsync();
    } catch (e) {
      await handlerErrorAsync(
          e, 'ReportEditController refreshImagesAsync error: e');
    } finally {
      EasyLoading.dismiss();
    }
  }

  /// 自动填充当前用户的电子签名
  void _autoFillCurrentUserSignature() {
    try {
      final signature = Storage.user.signature;
      if (signature.isNullOrWhiteSpace) return;
      Future.delayed(const Duration(seconds: 1), () {
        final signatureElement = FReportInfo.instance
            .getElementInfoByTagName(TagNames.DIGITALSIGNATURE);
        if (signatureElement != null) {
          final signatureArea = signatureElement as InputImageInfo;
          signatureArea.selectedImage = signature!;
        }
      });
    } catch (e) {
      loggerAsyn(FISDeviceLogCategory.Error,
          'ReportEditController _autoFillCurrentUserSignature ex: $e');
    }
  }

  //初始化报告信息
  void _initReportInfo(String creatorName) {
    if (Storage.user.organizationName.isNotNullOrEmpty) {
      Future.delayed(
        const Duration(milliseconds: 800),
        () async {
          final examDoctorInput = FReportInfo.instance
              .getElementInfoByTagName(TagNames.INSPHYSICIAN);
          if (examDoctorInput != null) {
            final examDoctor = examDoctorInput as InputTextInfo;
            if (examDoctor.text == aiRporterName ||
                examDoctor.text.isNullOrWhiteSpace) {
              examDoctor.text = consultationDetailDTO.scanUserName ?? '';
            }
          }

          final hospitalInput = FReportInfo.instance
              .getElementInfoByTagName(TagNames.HOSPITALNAME);
          if (hospitalInput != null) {
            final hospital = hospitalInput as InputTextInfo;
            if (hospital.text.isEmpty) {
              hospital.text = Storage.user.organizationName ?? '';
            }
          }

          final reportPhysicianInput =
              FReportInfo.instance.getElementInfoByTagName(TagNames.RPHYSICIAN);
          if (reportPhysicianInput != null) {
            final doctor = reportPhysicianInput as InputTextInfo;
            if (doctor.text != creatorName || doctor.text == aiRporterName) {
              doctor.text = creatorName;
            }
          }

          final patientIdInputElement = FReportInfo.instance
              .getElementInfoByTagName(TagNames.PATIENT_ID_TAG);
          if (patientIdInputElement != null) {
            final patientIdInput = patientIdInputElement as InputTextInfo;
            patientIdInput.text = state.devicePatientId.value;
          }

          final genderSelect = FReportInfo.instance
              .getElementInfoByTagName(TagNames.PATIENT_SEX_TAG);
          if (genderSelect != null && genderSelect is SingleSelectedInfo) {
            final items = genderSelect.items ?? [];
            var gender = state.patientSex.value;
            if (gender == 'AnimalInfoMale' || gender == 'Male') {
              if (items.contains('男')) {
                genderSelect.selectedItem = '男';
              } else if (items.contains('雄性')) {
                genderSelect.selectedItem = '雄性';
              } else if (items.contains('Male')) {
                genderSelect.selectedItem = 'Male';
              }
            } else if (gender == 'AnimalInfoFemale' || gender == 'Female') {
              if (items.contains('女')) {
                genderSelect.selectedItem = '女';
              } else if (items.contains('Female')) {
                genderSelect.selectedItem = 'Female';
              } else if (items.contains('雌性')) {
                genderSelect.selectedItem = '雌性';
              }
            }
          } else {
            print('null  genderSelect');
          }
        },
      );
    }
  }

  /// 初始化报告编辑器本地缓存
  void _initReportEditorCache() {
    final localReportEditorService = Get.find<LocalReportEditorService>();
    localReportEditorService.init();
  }

  ///初始化报告模板
  Future<void> _initUserDefaultTemplateAsync() async {
    final defaultTemplateInfo =
        await _reportManager.getDefaultReportTemplateAsync();
    if (defaultTemplateInfo != null) {
      state.selectedReportTemplate.value = defaultTemplateInfo;
    }
  }

  Future _initPatientInfoAsync() async {
    //final patientInfo = await getPatientInfo(state.patientId);
    //var devicePatientIDs = patientInfo?.devicePatientIDs ?? [];
    //var devicePatientId = devicePatientIDs.isNotEmpty
    //    ? patientInfo?.devicePatientIDs?.first ?? ''
    //    : '';
    //state.devicePatientId = devicePatientId;
    //if (patientInfo != null) {
    //state.patientName =
    //    FEncryptHelper.decryptBase64(patientInfo.encryptFullName ?? '');
    state.patientName.value = consultationDetailDTO.patientName ?? '';
    var patientData = consultationDetailDTO.patientDatas ?? [];
    if (patientData.isNotEmpty) {
      var ageInfo =
          patientData.firstWhereOrNull((element) => element.key == 'AgeYear');
      state.patientAge.value = ageInfo == null ? '' : ageInfo.value ?? '';
      var patientSexInfo = patientData.firstWhereOrNull(
          (element) => element.key == 'Sex' || element.key == 'AnimalInfoSex');
      state.patientSex.value =
          patientSexInfo == null ? '' : patientSexInfo.value ?? '';
    }
    //}
  }

  //校验草稿中的所有组件是否都存在，不存在则不加载草稿
  bool _checkIsSameTemplate(String? reportTemplateJsonStr, List json) {
    if (reportTemplateJsonStr != null) {
      for (var item in json) {
        var tag = item['Key'] ?? '';
        if (!reportTemplateJsonStr.contains(tag)) {
          return false;
        }
      }
    }
    return true;
  }

  Future onTabChangedAsync(int index) async {
    state.selectImageTabIndex.value = index;
    EasyLoading.show(status: i18nBook.common.loading.t);
    try {
      tabController.index = index;

      switch (index) {
        case 0:
          break;
        case 1:
          break;
      }
    } catch (e) {
      printError(
          info: "ConsultationDetailController onTabChangedAsync exception:$e");
      await handlerErrorAsync(
          e, "ConsultationDetailController onTabChangedAsync exception: $e");
    } finally {
      EasyLoading.dismiss();
    }
  }

  void onConsultationImageSelected(String coverImageUrl) {
    onSelect.emit(this, coverImageUrl);
    if (state.selectedImages.contains(coverImageUrl)) {
      state.selectedImages.remove(coverImageUrl);
    } else {
      state.selectedImages.add(coverImageUrl);
    }
  }

  //提交报告
  Future submitReportAsync() async {
    EasyLoading.show(status: i18nBook.common.submitting.t);
    try {
      final reportInfo = FReportInfo.instance;
      reportInfo.toJson();
      final reportInfoJson = jsonEncode(reportInfo.jsonItems);
      final templateJson =
          state.selectedReportTemplate.value?.reportTemplateJson;
      String recordCode = consultationDetailDTO.consultationCode!;

      final patientNameElementInfo = FReportInfo.instance
          .getElementInfoByTagName(TagNames.PATIENT_NAME_TAG);
      var name = '';
      if (patientNameElementInfo != null) {
        final patientNameInfo = patientNameElementInfo as InputTextInfo;
        name = patientNameInfo.text;
      }
      bool result = false;
      if (reportCode.isNotNullOrEmpty) {
        result = await _reportManager.modifyReportAsync(
            reportCode!,
            _serverReport!.reportOrgan,
            consultationDetailDTO.consultationCode!,
            templateJson ?? '',
            name,
            reportInfoJson);
      } else {
        final addResult = await _reportManager.addReportAsync(
            recordCode, templateJson ?? '', name, reportInfoJson);
        if (addResult.isNotNullOrEmpty) {
          result = true;
          //state.reportCode = addResult;
        }
      }
      if (result) {
        //_reportManager.onSubmitReport.emit(this, result);
        /// 提交则清空缓存
        reportEditorSetting.setReportDraft("");
        await localReportEditorService.saveAsync();
        PromptBox.toast(i18nBook.remedical.submitSuccess.t);
      }
      PromptBox.toast(i18nBook.remedical.submitSuccess.t);
      await Future.delayed(const Duration(milliseconds: 500));
      Get.close(1);
      var consultationListController = Get.find<ConsultationListController>();
      await consultationListController.refreshAsync();
      var consultationDetailController =
          Get.find<ConsultationDetailController>();
      if (reportCode.isNotNullOrEmpty) {
        await consultationDetailController.onTabChangedAsync(2);
      } else {
        await consultationDetailController.onTabChangedAsync(0);
      }
    } catch (ex) {
      await handlerErrorAsync(
          ex, 'ReportEditController submitReportAsync error: $ex');
    } finally {
      EasyLoading.dismiss();
    }
  }

  /// 直接打印报告
  void printReportDirectly() {
    try {
      PDFHelper.printPDF(reportName, fileStorage);
    } catch (e) {
      loggerAsyn(FISDeviceLogCategory.Error, "print report error: $e");
    }
  }

  ///不打开报告预览页面，直接导出
  void exportReportDirectly() {
    try {
      PDFHelper.downloadPDF(
        reportName,
        fileStorage,
        shellExporter,
      );
    } catch (e) {
      loggerAsyn(FISDeviceLogCategory.Error, "export report error: $e");
    }
  }

  String _getReportName() {
    var reportInfo = FReportInfo.instance;
    InputTextInfo inputTextInfo;
    String patientName = '';
    var patientElement =
        reportInfo.getElementInfoByTagName(TagNames.PATIENT_NAME_TAG);
    if (patientElement != null) {
      inputTextInfo = patientElement as InputTextInfo;
      patientName = inputTextInfo.text;
    } else {
      patientName = "";
    }
    var creatTime = reportInfo.reportDate ?? DateTime.now();
    var reportCode = reportInfo.id ?? '';
    DateFormat format = DateFormat("yyyMMddHHmmss");
    var dateStr = format.format(creatTime);
    return "[$patientName]-[$dateStr]-[$reportCode].pdf";
  }

  Future<void> _initReportTemplatesAsync() async {
    var templatePage = await _reportManager.getReportTemplatePage();
    state.reportTemplates.value = templatePage.pageData
            ?.map(
              (e) => ReportTemplateDTO(
                reportTemplateName: e.reportTemplateName ?? '',
                reportTemplateCode: e.reportTemplateCode,
              ),
            )
            .toList() ??
        [];
  }

  Future<void> changeSelectedTemplateAsync(
      ReportTemplateDTO reportTemplateDTO) async {
    var reportTemplate = await _reportManager.findReportTemplateByCode(
        code: reportTemplateDTO.reportTemplateCode ?? '');
    state.selectedReportTemplate.value = reportTemplate;
    FReportInfo.instance.reload(
      reportTemplate.reportTemplateJson ?? '',
    );
    state.selectedImages.value = [];
    await _initDataAsync(true);
    await thesaurusController.initDefaultThesaurusAsync();
  }

  void _onSelectCallBack(Object sender, String e) {
    FReportInfo.instance.selectedInputImage(e);
  }
}
