import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/report.m.dart';
import 'package:flyinsonolite/consultation/reports/controllers/reporteditcontroller.dart';
import 'package:flyinsonolite/consultation/reports/views/reporteditstate.dart';
import 'package:flyinsonolite/consultation/reports/views/vocabularyentryreportdialog.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/index.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/managers/interfaces/ireportmanager.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:get/get.dart';

class ThesaurusController {
  final ReportEditController controller;
  late final ReportEditState state;
  ThesaurusAllDTO? defaultThesaurus;
  final IReportManager _reportManager = Get.find<IReportManager>();
  ThesaurusController(this.controller) {
    state = controller.state;
    initAllThesaurus();
  }

  /// 初始化词条数据
  Future<void> getDefaultThesaurus(String thesaurusCode) async {
    final allThesaurusItems =
        await _reportManager.getThesaurusItems(thesaurusCode: thesaurusCode);
    if (allThesaurusItems.isEmpty) {
      return;
    } else {
      controller.state.allThesaurusList.value = allThesaurusItems;
    }
  }

  ///初始化默认词条
  Future initDefaultThesaurusAsync() async {
    try {
      defaultThesaurus = await _reportManager.getDefaultThesaurusContentAsync();
      var thesaurusCode = defaultThesaurus?.baseInfo!.thesaurusCode ?? '';
      if (thesaurusCode.isNotEmpty) {
        state.selectedEntryCode.value = thesaurusCode;
        await getDefaultThesaurus(thesaurusCode);
      }
    } catch (e) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          'ThesaurusController initDefaultThesaurusAsync ex: $e');
    }
    FReportInfo.instance.onDiagnosticTap.removeListener(_onDiagnosticTap);
    //对选择词条监听
    FReportInfo.instance.onDiagnosticTap.addListener(_onDiagnosticTap);
  }

  void dispose() {
    FReportInfo.instance.onDiagnosticTap.removeListener(_onDiagnosticTap);
  }

  void _onDiagnosticTap(Object sender, String e) {
    if (defaultThesaurus == null) {
      PromptBox.toast(i18nBook.remedical.noEntrys.t);
    } else {
      Get.dialog(const VocabularyEntryReport(), barrierDismissible: false);
    }
  }

  Future<void> initAllThesaurus() async {
    if (Storage.user.token.isEmpty) {
      return;
    }
    var allEntrys = await _reportManager.getThesaurusPage();
    var userDefaultEnty =
        allEntrys.firstWhereOrNull((element) => element.isUserDefault);
    if (userDefaultEnty != null) {
      state.entryInfos.value = [userDefaultEnty];
      var normalEntrys = allEntrys.where((element) => !element.isUserDefault);
      state.entryInfos.addAll(normalEntrys);
    } else {
      state.entryInfos.value = allEntrys;
    }
  }

  ///切换词条
  void changeEntry() {
    getDefaultThesaurus(state.selectedEntryCode.value);
  }
}
