import 'package:flutter/foundation.dart';
import 'package:flyinsonolite/consultation/reports/pdf/pdfimagelist.dart';
import 'package:flyinsonolite/consultation/reports/pdf/pdfleafelement.dart';
import 'package:flyinsonolite/consultation/reports/pdf/pdfparagraph.dart';
import 'package:flyinsonolite/consultation/reports/pdf/pdfrttable.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/block_element_info_interface.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/date_time_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/element_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/input_image_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/input_image_list_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/input_text_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/line_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/multi_selected_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/paragraph_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/rt_table_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/single_selected_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/static_image_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/static_text_info.dart';
import 'package:pdf/widgets.dart' as pw;

abstract class PdfElement {
  ///模板主要列表结构的信息转换为PDF组件列表
  @protected
  pw.Widget generatePDFElement() {
    return pw.Container();
  }

  pw.Widget generateElementByBlockInfo(IBlockElementInfo elementInfo,
      {bool parentIsRTCell = false, bool parentIsTable = false}) {
    pw.Widget ouputWidget = pw.Text("*");
    if (elementInfo is RTTableInfo) {
      var pdfTable = PdfRTTable(elementInfo, parentIsTable: parentIsTable);
      ouputWidget = pdfTable.generatePDFElement();
    }
    if (elementInfo is ParagraphInfo) {
      var pdfParagraph = PdfParagraph(elementInfo);
      ouputWidget =
          pdfParagraph.generatePDFElement(isParentRTCell: parentIsRTCell);
    }
    if (elementInfo is InputImageListInfo) {
      var pdfInputImageList = PdfImageList(elementInfo);
      ouputWidget = pdfInputImageList.generatePDFElementAsync();
    }

    return ouputWidget;
  }

  ///根据模板具体element信息生成对应element的pdf组件
  pw.Widget generateElementByElementType(ElementInfo elementInfo,
      {bool parentElementHasNoContainer = false,
      bool parentIsTable = false,
      bool parentIsRTCell = false}) {
    if (elementInfo is StaticTextInfo) {
      return PdfLeafElement.getTextPdfWidget(
          elementInfo, parentIsTable, parentIsRTCell);
    }
    if (elementInfo is LineInfo) {
      return PdfLeafElement.getLinePdfWidget(elementInfo);
    }
    if (elementInfo is DateTimeInfo) {
      return PdfLeafElement.getDateTimePdfWidget(elementInfo);
    }
    if (elementInfo is InputTextInfo) {
      return PdfLeafElement.getInputTextPdfWidget(
          elementInfo, parentElementHasNoContainer);
    }
    if (elementInfo is SingleSelectedInfo) {
      return PdfLeafElement.getSingleSelectPdfWidget(elementInfo);
    }
    if (elementInfo is MulitiSelectedInfo) {
      return PdfLeafElement.getMultiSelectPdfWidget(elementInfo);
    }
    if (elementInfo is StaticImageInfo) {
      return PdfLeafElement.getStaticImagePdfWidget(elementInfo);
    }
    if (elementInfo is InputImageInfo) {
      return PdfLeafElement.getInputImagePdfWidget(elementInfo);
    }

    return pw.Text("UnKnown- ${elementInfo.elementType!.name}");
  }
}
