import 'package:flyinsonolite/consultation/reports/pdf/pdfbody.dart';
import 'package:flyinsonolite/consultation/reports/pdf/pdffooter.dart';
import 'package:flyinsonolite/consultation/reports/pdf/pdfheader.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/block_element_info_interface.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/pdf/resourcescache.dart';
import 'package:pdf/pdf.dart';
import 'package:pdf/widgets.dart' as pw;

class PdfPageCreator {
  pw.ThemeData them;
  PdfPageFormat format;
  FReportInfo reportInfo;
  PDFTypeEnum pdfType;
  pw.Font defaultFont;

  PdfPageCreator(
      this.reportInfo, this.pdfType, this.defaultFont, this.them, this.format);

  ///根据传入的具体PDF类型生成对应的PDF页面
  Future<pw.Document> getPage() async {
    var doc = pw.Document(pageMode: PdfPageMode.outlines);
    doc.addPage(_generatePdfPage());
    return doc;
  }

  static Future<pw.Document> getPageFromJPGE(
    List<String> jpges,
    PdfPageFormat format,
  ) async {
    var doc = pw.Document(pageMode: PdfPageMode.outlines);
    for (var j in jpges) {
      var body = _generateJPGReportBody(j);
      var page = pw.Page(
          pageFormat: format.copyWith(
              marginLeft: 0 * PdfPageFormat.cm,
              marginRight: 0 * PdfPageFormat.cm,
              marginTop: 0 * PdfPageFormat.cm,
              marginBottom: 0 * PdfPageFormat.cm),
          build: (pw.Context context) {
            return body;
          });
      doc.addPage(page);
    }
    return doc;
  }

  pw.Page _generatePdfPage() {
    if (pdfType == PDFTypeEnum.report) {
      return pw.MultiPage(
          pageFormat: format.copyWith(
            marginLeft: reportInfo.reportTemplate!.pagePadding!.left! *
                (1.76 / 5) *
                PdfPageFormat.mm,
            marginRight: reportInfo.reportTemplate!.pagePadding!.right! *
                (1.76 / 5) *
                PdfPageFormat.mm,
            marginTop: reportInfo.reportTemplate!.pagePadding!.top! *
                (1.76 / 5) *
                PdfPageFormat.mm,
            marginBottom: reportInfo.reportTemplate!.pagePadding!.bottom! *
                (1.76 / 5) *
                PdfPageFormat.mm,
          ),
          textDirection: pw.TextDirection.ltr,
          crossAxisAlignment: pw.CrossAxisAlignment.stretch,
          footer: _generateFooterPDFElement,
          header: _generateHeaderElement,
          build: _generateBodyElements);
    }

    //当没有指定的pdf类型时的缺省页面
    return pw.Page(build: (context) {
      return pw.Text('No pdf type was set',
          style: pw.TextStyle(
              font: defaultFont, fontSize: 18, fontWeight: pw.FontWeight.bold));
    });
  }

  List<pw.Widget> _generateBodyElements(pw.Context context) {
    List<IBlockElementInfo> blocks = [];
    blocks.addAll(reportInfo.blocks);
    var body = PdfBody(blocks);
    var bodyWidgets = body.generatedPDFElements();
    if (bodyWidgets.isEmpty) {
      bodyWidgets.add(pw.Text(""));
    }
    return bodyWidgets;
  }

  pw.Widget _generateFooterPDFElement(pw.Context context) {
    var pdfFooter = PdfFooter(reportInfo.footers);
    return pdfFooter.getFooter();
  }

  pw.Widget _generateHeaderElement(pw.Context context) {
    var header = PdfHeader(reportInfo.headers);
    return header.getHeader();
  }

  static pw.Widget _generateJPGReportBody(String jpegReport) {
    var jpg = ResourcesCache.getCacheCloudImages(jpegReport);
    return pw.Image(jpg!, fit: pw.BoxFit.fitWidth);
  }
}

///Pdf文件种类枚举
///默认unDefined
enum PDFTypeEnum {
  ///未指定类型
  unDefined,

  ///报告类型的PDF
  report,

  ///指定pdf二进制文件
  pdfBytes,

  ///纯jpg图片类型的PDF
  pdfJpeg,
}
