import 'package:fis_common/event/event_type.dart';
import 'package:flyinsonolite/converters/datetostringconverter.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/text_element_info.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/date_time_element.dart';

class DateTimeInfo extends TextElementInfo {
  ///UI绑定的文本
  String get text => _text;
  set text(String v) {
    _text = v;
    displayValue = v;
    try {
      final date = DateTime.tryParse(_text.trim());
      if (date != null) {
        displayValue = DateToStringConverter.dateTimeConvert(
                dateTimeFormat!, date.millisecondsSinceEpoch)
            .trim();
      } else {
        _text = DateTime.now().toString();
        displayValue = _text;
      }
    } catch (e) {}
    onTextChange.emit(this, v);
  }

  String displayValue = '';

  ///文本改变通知UI变化
  FEventHandler<String> onTextChange = FEventHandler<String>();

  ///日期格式
  String? dateTimeFormat = 'yyyy-MM-dd';

  String _text = '';

  DateTimeInfo.fromElement(DateTimeElement element)
      : super.fromElement(element) {
    dateTimeFormat = element.dateTimeFormat;
    text = DateTime.now().toString();
  }

  @override
  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['Key'] = id;
    map['Value'] = text;
    map['Name'] = tag == null ? '' : tag!.name;
    map['TagId'] = tag == null ? '' : tag!.id;
    FReportInfo.instance.jsonItems.add(map);
    return map;
  }
}
