import 'package:flutter/material.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';

class ExpandableIconText extends StatefulWidget {
  final IconData icon;
  final String text;
  final bool isSelected;
  final bool isExpand;
  final VoidCallback onPressed;
  final bool isExpandChild;
  final void Function(bool)? onHoverChange;
  final double horizontalMargin;
  final bool isDistinguishChinese;

  const ExpandableIconText({
    super.key,
    required this.icon,
    required this.text,
    this.isSelected = false,
    this.isExpand = false,
    this.isExpandChild = true,
    this.isDistinguishChinese = true,
    this.horizontalMargin = 0,
    this.onHoverChange,
    required this.onPressed,
  });

  @override
  _ExpandableIconTextState createState() => _ExpandableIconTextState();

  String get pageName => 'ExpandableIconText';
}

class _ExpandableIconTextState extends State<ExpandableIconText> {
  bool _isSelected = false;
  bool _isHovered = false;
  bool _isExpand = false;

  @override
  void initState() {
    _isSelected = widget.isSelected;
    _isExpand = widget.isExpand;
    super.initState();
  }

  @override
  void didUpdateWidget(oldWidget) {
    if (_isSelected != widget.isSelected) {
      setState(() {
        _isSelected = widget.isSelected;
      });
    }
    if (_isExpand != widget.isExpand) {
      setState(() {
        _isExpand = widget.isExpand;
      });
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    var child = MouseRegion(
      cursor: SystemMouseCursors.click,
      onEnter: (event) {
        setState(() {
          _isHovered = true;
        });
        widget.onHoverChange?.call(_isHovered);
      },
      onExit: (event) {
        setState(() {
          _isHovered = false;
        });
        widget.onHoverChange?.call(_isHovered);
      },
      child: _buildContentView(),
    );
    if (_isSelected || _isHovered) {
      return child;
    }
    if (widget.isExpandChild) {
      return Expanded(child: child);
    }
    return child;
  }

  Widget _buildContentView() {
    Color back = Colors.black;
    if (_isSelected) {
      back = Storage.currentTheme.themeData.indicatorColor;
    } else if (_isHovered) {
      back = Storage.currentTheme.themeData.indicatorColor;
    }
    return Container(
      margin: EdgeInsets.symmetric(horizontal: widget.horizontalMargin.s),
      padding: EdgeInsets.only(bottom: 7.s),
      constraints: BoxConstraints(
        minWidth: 70.s, // 设置最小宽度为100像素
      ),
      decoration: BoxDecoration(
        border: Border(
          bottom: BorderSide(
            color: (_isSelected || _isHovered) ? back : Colors.transparent,
            width: 2.s,
          ),
        ),
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          SizedBox(
            width: 10.s,
          ),
          if (!i18nBook.isCurrentChinese && widget.isDistinguishChinese)
            GestureDetector(
              onTap: () {
                if (!_isSelected) {
                  setState(() {
                    _isSelected = true;
                  });
                }
                widget.onPressed();
              },
              child: Icon(
                widget.icon,
                color: back,
              ),
            ),
          if (_isExpand ||
              !widget.isDistinguishChinese ||
              (i18nBook.isCurrentChinese && widget.isDistinguishChinese)) ...[
            GestureDetector(
              onTap: () {
                if (!_isSelected) {
                  setState(() {
                    _isSelected = true;
                  });
                }
                widget.onPressed();
              },
              child: FISText(
                widget.text,
                style: TextStyle(color: back, fontSize: 14.s),
              ),
            ),
          ],
          SizedBox(
            width: 10.s,
          ),
        ],
      ),
    );
  }
}
