import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/authentication.m.dart';
import 'package:fis_jsonrpc/services/remedical.m.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/consultation/reports/views/consultationreportpreviewpage.dart';
import 'package:flyinsonolite/consultation/reports/views/qrcodesharedialog.dart';
import 'package:flyinsonolite/consultation/reports/views/reporteditpage.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/consultation/records/models/features.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/managers/interfaces/ireportmanager.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class ConsultationReportView extends GetView<ConsultationDetailController> {
  const ConsultationReportView({super.key});

  @override
  Widget build(BuildContext context) {
    return Obx(() =>
        controller.reports.isEmpty ? _mockEmptyView() : _buildRepotsView());
  }

  ///缺省显示
  Widget _mockEmptyView() {
    return Container(
      margin: EdgeInsets.only(top: 40.s),
      child: Column(
        children: [
          Center(
            child: Image.asset(
              "assets/images/no_data.png",
              width: 300.s,
              height: 300.s,
              fit: BoxFit.cover,
            ),
          ),
          FISText(i18nBook.common.noneDataTip.t),
        ],
      ),
    );
  }

  Widget _buildRepotsView() {
    return Expanded(
        child: SingleChildScrollView(
            child: Container(
                padding: EdgeInsets.fromLTRB(15.s, 10.s, 15.s, 10.s),
                color: Storage.currentTheme.listViewStyle.itembgColor,
                child: ListView.separated(
                    padding: EdgeInsets.zero,
                    shrinkWrap: true,
                    itemCount: controller.reports.length,
                    itemBuilder: (BuildContext context, int index) {
                      return ConsultationReportListViewItem(
                          reportInfo: controller.reports[index],
                          controller: controller,
                          canEditReport: controller
                              .consultationDetail.value!.isEditReportShow);
                    },
                    separatorBuilder: (context, index) {
                      return Divider(
                        height: 3.s,
                      );
                    }))));
  }
}

class ConsultationReportListViewItem extends StatelessWidget {
  final ReportDTO reportInfo;
  final _reportManager = Get.find<IReportManager>();
  final bool canEditReport;
  late ConsultationDetailController controller;
  ConsultationReportListViewItem(
      {super.key,
      required this.reportInfo,
      required this.controller,
      required this.canEditReport});

  @override
  Widget build(BuildContext context) {
    return Stack(children: [
      Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          _buildRecordContent(context),
          Container(
              width: double.infinity,
              padding: EdgeInsets.all(10.s),
              margin: EdgeInsets.only(right: 15.s, top: 10.s),
              decoration: BoxDecoration(
                border: Border(
                  top: BorderSide(
                    color: const Color.fromRGBO(216, 216, 216, 1),
                    width: 0.5.s,
                  ),
                ),
              )),
          _buildOperateButtons(),
        ],
      )
    ]);
  }

  ///操作按钮
  Widget _buildOperateButtons() {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        FISLinearGradientElevatedButton(
          () {
            _openReportPreviewPage();
          },
          i18nBook.common.view.t,
          Storage.currentTheme.dialogStyle.buttonGradient,
          textStyle: Storage.currentTheme.normalTextStyle.copyWith(
              fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s),
        ),
        if (canEditReport &&
            Storage.user.hasFeature(FeatureKeys.DiagnosisEditReport)) ...[
          SizedBox(
            width: 10.s,
          ),
          FISLinearGradientElevatedButton(
            () {
              Get.to(ReportEditPage(
                controller.consultationDetail.value!,
                reportCode: reportInfo.reportCode,
              ));
            },
            i18nBook.common.modify.t,
            Storage.currentTheme.dialogStyle.buttonGradient,
            textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s),
          ),
        ],
        SizedBox(
          width: 10.s,
        ),
        Storage.platform == Platform.Windows
            ? FISLinearGradientElevatedButtonAsync(
                () async {
                  await controller.printReportDirectlyAsync(reportInfo);
                },
                i18nBook.remedical.print.t,
                Storage.currentTheme.dialogStyle.buttonGradient,
                textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s),
              )
            : const SizedBox(),
        Storage.platform == Platform.Windows
            ? SizedBox(
                width: 10.s,
              )
            : const SizedBox(),
        FISLinearGradientElevatedButton(
          () {
            Get.dialog(QRCodeShareDialog(reportInfo.reportCode!),
                barrierDismissible: false);
          },
          i18nBook.remedical.share.t,
          Storage.currentTheme.dialogStyle.buttonGradient,
          textStyle: Storage.currentTheme.normalTextStyle.copyWith(
              fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s),
        ),
      ],
    );
  }

  Widget _buildRecordContent(BuildContext context) {
    bool isAIReport = false;
    final diagnosisLabels = reportInfo.diagnosisLabels ?? [];
    if (diagnosisLabels.contains('VAid')) {
      isAIReport = true;
    }
    return Row(
      children: [
        Expanded(
            child: Row(
          mainAxisSize: MainAxisSize.min,
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Container(
              margin: EdgeInsets.only(
                bottom: 15.s,
              ),
              child: _generateReportIcon(_generateNailUrl(reportInfo)),
            ),
            SizedBox(width: 15.s),
            Expanded(
              child: Column(
                mainAxisSize: MainAxisSize.min,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  FISText(reportInfo.patientName ?? ""),
                  SizedBox(height: 5.s),
                  FISText(
                    isAIReport
                        ? i18nBook.remedical.aiReport.t
                        : i18nBook.remedical.cloudReport.t,
                  ),
                  SizedBox(
                    height: 5.s,
                  ),
                  Row(
                    children: [
                      FISText(i18nBook.remedical.reporter.t),
                      FISText(reportInfo.reportUserName ?? ""),
                    ],
                  ),
                  _buildReportTags(context)
                ],
              ),
            ),
            Column(
              children: [
                Container(
                  margin: EdgeInsets.only(
                    right: 15.s,
                  ),
                  child: FISText(
                    _reportManager.getTime(
                      reportInfo.updateTime.toString(),
                    ),
                  ),
                ),
              ],
            ),
          ],
        )),
      ],
    );
  }

  String? _generateNailUrl(ReportDTO reportList) {
    if (reportList.reportPreviewList == null ||
        reportList.reportPreviewList!.isEmpty) {
      return null;
    }
    return reportList.reportPreviewList!.first.thumbnailToken;
  }

  Widget _generateReportIcon(String? reportNailUrl) {
    var useDefaultImage = reportNailUrl == null || reportNailUrl.isEmpty;
    return Center(
      child: useDefaultImage
          ? Icon(
              Icons.content_paste,
              size: 70.s,
            )
          : Image.network(
              reportNailUrl,
              fit: BoxFit.fill,
              width: 70.s,
              height: 100.s,
            ),
    );
  }

  ///报告标签列表
  Widget _buildReportTags(BuildContext context) {
    final tags = reportInfo.reportLabels ?? [];
    return SizedBox(
      width: MediaQuery.of(context).size.width,
      child: Wrap(
        direction: Axis.horizontal,
        alignment: WrapAlignment.start,
        children: tags
            .map(
              (e) => Container(
                height: 22.s,
                margin: EdgeInsets.only(right: 15.s, top: 3.s),
                child: Container(
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.all(
                      Radius.circular(
                        4.s,
                      ),
                    ), //圆角弧度
                    border: Border.all(
                        width: 1.s,
                        color: Storage.currentTheme.iconButtonIconColor),
                  ),
                  child: Container(
                    margin: EdgeInsets.symmetric(horizontal: 5.s),
                    child: FISText(e),
                  ),
                ),
              ),
            )
            .toList(),
      ),
    );
  }

  void _openReportPreviewPage() {
    Get.to(ConsultationReportPreviewPage(reportInfo));
  }
}
