import 'package:fis_common/extensions/index.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/images/views/cloudimageview.dart';
import 'package:flyinsonolite/consultation/images/views/consultationimagepage.dart';
import 'package:flyinsonolite/consultation/images/views/consultationimagesview.dart';
import 'package:flyinsonolite/consultation/reports/controllers/reporteditcontroller.dart';
import 'package:flyinsonolite/consultation/reports/views/reporteditview.dart';
import 'package:flyinsonolite/controls/custom/customdropdownbutton.dart';
import 'package:flyinsonolite/controls/select/fisdropdownbutton.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class ReportEditPage extends GetView<ReportEditController> {
  int _index = 0;
  double get _leftMargin => 20.s;
  double get _topMargin => 8.s;
  double get _imageWidth => 190.s;
  final String? reportCode;
  ConsultationDetailDTO consultationDetailDTO;
  ReportEditPage(this.consultationDetailDTO, {super.key, this.reportCode});

  @override
  Widget build(BuildContext context) {
    Get.lazyPut<ReportEditController>(() =>
        ReportEditController(consultationDetailDTO, reportCode: reportCode));
    ScrollController scrollerController = ScrollController();
    return Scaffold(
        resizeToAvoidBottomInset: false,
        body: Obx(() => Storage.isPortraitMode.value
            ? Container(
                padding: EdgeInsets.all(5.s),
                child: Column(
                  children: [
                    Container(
                        color: Colors.white,
                        child: Scrollbar(
                            thickness: 8.s,
                            radius: Radius.circular(8.s),
                            interactive: true,
                            controller: scrollerController,
                            thumbVisibility: true,
                            child: SingleChildScrollView(
                              scrollDirection: Axis.horizontal,
                              child: _buildReportContentView(context),
                            ))),
                    Divider(
                      height: 5.s,
                    ),
                    Expanded(child: _buildImagesView())
                  ],
                ),
              )
            : Container(
                padding: EdgeInsets.all(5.s),
                child: Row(
                  children: [
                    Container(
                      color: Colors.white,
                      child: _buildReportContentView(context),
                    ),
                    VerticalDivider(
                      width: 5.s,
                    ),
                    Expanded(child: _buildImagesView())
                  ],
                ),
              )));
  }

  ///报告内容视图
  Widget _buildReportContentView(BuildContext context) {
    final state = controller.state;
    var body = Obx(() {
      final jsonStr =
          state.selectedReportTemplate.value?.reportTemplateJson ?? '';
      if (jsonStr.isEmpty) {
        return Storage.isPortraitMode.value
            ? SizedBox(height: Get.height * 0.55)
            : SizedBox(
                width: Get.width * 0.55,
              );
      }
      return Storage.isPortraitMode.value
          ? SizedBox(
              height: Get.height * 0.55,
              child: SingleChildScrollView(
                  child: ReportEditView(jsonStr, controller)))
          : Expanded(
              child: SingleChildScrollView(
                  child: ReportEditView(jsonStr, controller)));
    });

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Obx(() {
          ///已提交的报告不支持修改模板
          if (controller.state.reportTemplates.isEmpty ||
              controller.state.reportCode.value.isNotNullOrEmpty) {
            return const SizedBox();
          }
          return Container(
              width: 320.s,
              margin: EdgeInsets.only(top: 5.s, left: 5.s),
              decoration: BoxDecoration(
                border: Border.all(color: Colors.black, width: 1.s),
                borderRadius: BorderRadius.circular(4.s),
              ),
              padding: EdgeInsets.only(left: 10.s),
              child: Obx(() => FISDropdownButton(
                    value: controller.state.reportTemplates.firstWhereOrNull(
                        (element) =>
                            element.reportTemplateCode ==
                            controller.state.selectedReportTemplate.value
                                ?.reportTemplateCode),
                    items: controller.state.reportTemplates
                        .map<CustomDropdownMenuItem<ReportTemplateDTO>>(
                            (ReportTemplateDTO selectedReportTemplate) {
                      return CustomDropdownMenuItem<ReportTemplateDTO>(
                        value: selectedReportTemplate,
                        child: FISText(
                            selectedReportTemplate.reportTemplateName!,
                            style: Storage.currentTheme.normalTextStyle
                                .copyWith(
                                    fontSize: Storage.currentTheme
                                        .normalTextStyle.fontSize!.s,
                                    color: Colors.black)),
                      );
                    }).toList(),
                    onChanged: (value) async {
                      await controller.changeSelectedTemplateAsync(value);
                    },
                    dropdownColor: Colors.white,
                    underline: Container(color: Colors.white),
                    textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                      color: Colors.black,
                      fontSize:
                          Storage.currentTheme.normalTextStyle.fontSize!.s,
                    ),
                  )));
        }),
        body,
      ],
    );
  }

  Widget _buildImagesView() {
    final tabs = controller.tabTitles
        .map((e) => SizedBox(
            height: 50.s,
            child: Center(
              child: FISText(
                e,
                softWrap: true,
                textAlign: TextAlign.center,
              ),
            )))
        .toList();
    return Column(
      children: [
        Expanded(
          child: Container(
            alignment: Alignment.topLeft,
            child: SingleChildScrollView(
                child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                TabBar(
                  indicatorColor: Storage.currentTheme.themeData.indicatorColor,
                  onTap: controller.onTabChangedAsync,
                  tabs: tabs,
                  controller: controller.tabController,
                ),
                _buildImagsView(),
              ],
            )),
          ),
        ),
        Center(
          child: Wrap(
            spacing: 10.s,
            runSpacing: 5.s,
            children: [
              FISLinearGradientElevatedButtonAsync(() async {
                await controller.submitReportAsync();
              }, i18nBook.common.submit.t,
                  Storage.currentTheme.dialogStyle.buttonGradient,
                  textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                      fontSize:
                          Storage.currentTheme.normalTextStyle.fontSize!.s)),
              Storage.platform == Platform.Windows
                  ? FISLinearGradientElevatedButton(() {
                      controller.printReportDirectly();
                    }, i18nBook.remedical.print.t,
                      Storage.currentTheme.dialogStyle.buttonGradient,
                      textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                          fontSize:
                              Storage.currentTheme.normalTextStyle.fontSize!.s))
                  : const SizedBox(),
              FISLinearGradientElevatedButtonAsync(() async {
                await controller.refreshImagesAsync();
              }, i18nBook.common.refresh.t,
                  Storage.currentTheme.dialogStyle.buttonGradient,
                  textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                      fontSize:
                          Storage.currentTheme.normalTextStyle.fontSize!.s)),
              FISLinearGradientElevatedButton(() {
                Get.close(1);
              }, i18nBook.common.back.t,
                  Storage.currentTheme.dialogStyle.buttonGradient,
                  textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                      fontSize:
                          Storage.currentTheme.normalTextStyle.fontSize!.s)),
            ],
          ),
        )
      ],
    );
  }

  ///图像展示
  Widget _buildImagsView() {
    return Obx(
      () {
        switch (controller.state.selectImageTabIndex.value) {
          case 0:
            return _buildCloudImages(controller.state.remedicalItemList);
          case 1:
            return ScreenshotImags(
              controller.state.consultationImages,
              controller: controller,
            );
          default:
            return const SizedBox();
        }
      },
    );
  }

  Widget _buildCloudImages(List<RemedicalInfoDTO> images) {
    _index = 0;
    return Container(
      alignment: Alignment.topLeft,
      margin: EdgeInsets.only(
        left: _leftMargin,
        top: _topMargin,
      ),
      child: Wrap(
        spacing: 8.s, // 主轴(水平)方向间距
        runSpacing: 8.s, // 纵轴（垂直）方向间距
        direction: Axis.horizontal,
        alignment: WrapAlignment.start,
        children: List<Widget>.generate(
          images.length,
          (i) {
            var index = _index;
            var imageInfo = images[i];
            var consultationFileDTO =
                convertRemedicalInfoDTOToconsultationFileDTO(imageInfo);
            final image = Obx(() => CloudImageItem(
                  index,
                  imageInfo,
                  isSelected: controller.state.selectedImages
                      .contains(consultationFileDTO.coverImageUrl),
                  onTap: () {
                    controller.onConsultationImageSelected(
                        consultationFileDTO.coverImageUrl ?? '');
                  },
                  onDoubleTap: () {
                    Get.to(ConsultationImagePage(consultationFileDTO));
                  },
                  imageWidth: _imageWidth,
                ));
            _index++;
            return image;
          },
        ),
      ),
    );
  }

  ConsultationFileDTO convertRemedicalInfoDTOToconsultationFileDTO(
      RemedicalInfoDTO imageInfo) {
    return ConsultationFileDTO(
        sourceUrl: imageInfo.terminalImages?.imageUrl,
        previewImageUrl: imageInfo.terminalImages?.previewUrl,
        coverImageUrl: imageInfo.terminalImages?.coverImageUrl,
        createTime: imageInfo.createTime,
        creatorCode: imageInfo.deviceCode,
        creatorName: "",
        fileDataType: imageInfo.fileDataType,
        consultationFileType: ConsultationFileTypeEnum.UltrasoundImage,
        remedicalCode: imageInfo.remedicalCode,
        remedicalMeasureCode: "");
  }

  ConsultationFileDTO convertRemedicalMeasuredInfoDTOToconsultationFileDTO(
      RemedicalMeasuredInfoDTO imageInfo) {
    return ConsultationFileDTO(
        sourceUrl: "",
        previewImageUrl: imageInfo.previewFileToken,
        coverImageUrl: imageInfo.measuredFileToken,
        createTime: imageInfo.createTime,
        creatorCode: imageInfo.userCode,
        creatorName: "",
        fileDataType: RemedicalFileDataTypeEnum.VinnoVidSingle,
        consultationFileType: ConsultationFileTypeEnum.MeasurementImage,
        remedicalCode: imageInfo.remedicalCode,
        remedicalMeasureCode: imageInfo.remedicalMeasuredInfoCode);
  }
}
