import 'dart:convert';
import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/reports/controllers/reporteditcontroller.dart';
import 'package:flyinsonolite/converters/pttopxconverter.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/consultation/reports/widgets/blockelementview.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_event_args.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/interfaces/block_element.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/report_template_document.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/rt_thickness.dart';

class ReportEditView extends StatefulWidget {
  final bool isReload;

  ReportEditView(this.reportTemplateJson, this.controller,
      {super.key, this.isReload = true});

  late String reportTemplateJson;
  late ReportEditController controller;

  @override
  State<ReportEditView> createState() {
    return _ReportEditViewState();
  }
}

class _ReportEditViewState extends State<ReportEditView> {
  bool isInit = false;
  ReportTemplateDocument _reportTemplate = ReportTemplateDocument();
  double _height = 0;
  double _width = 0;
  EdgeInsetsGeometry _padding = EdgeInsets.all(PtToPxConverter.ptToPx(56.83));
  List<IBlockElement> _blocks = [];
  List<IBlockElement> _header = [];
  List<IBlockElement> _footer = [];
  bool _isReload = true;
  final bool _isPortraitMode = Storage.isPortraitMode.value;

  @override
  initState() {
    _isReload = widget.isReload;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    if (widget.reportTemplateJson.isEmpty) {
      return const SizedBox();
    }
    if (_isReload) {
      FReportInfo.instance.toJson();
      var oldData = FReportInfo.instance.jsonItems;
      _initTemplate();
      _initListentener();
      FReportInfo.instance.fromJson(oldData);
      widget.controller.initImageInput();
      _isReload = false;
    }
    var margin = EdgeInsets.symmetric(vertical: 2.s);
    return Container(
      width: _width,
      padding: _padding,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.start,
        children: [
          ..._header.map((head) {
            return Container(
              margin: margin,
              child: FBlockElementPage(element: head),
            );
          }),
          ..._blocks.map((block) {
            return Container(
              margin: margin,
              child: FBlockElementPage(element: block),
            );
          }),
          ..._footer.map((footer) {
            return Container(
              margin: margin,
              constraints: BoxConstraints(minHeight: 24.s),
              child: FBlockElementPage(element: footer),
            );
          }),
        ],
      ),
    );
  }

  void _initPage() {
    try {
      _width = PtToPxConverter.ptToPx(_reportTemplate.pageSize!.width);
      _height = PtToPxConverter.ptToPx(_reportTemplate.pageSize!.height);
      final pagePadding =
          _reportTemplate.pagePadding ?? RTThickness.uniform(56);
      _padding = EdgeInsets.only(
        left: PtToPxConverter.ptToPx(pagePadding.left),
        right: PtToPxConverter.ptToPx(pagePadding.right),
        top: PtToPxConverter.ptToPx(pagePadding.top),
        bottom: PtToPxConverter.ptToPx(pagePadding.bottom),
      );
      _footer = _reportTemplate.footer ?? [];
      _blocks = _reportTemplate.blocks ?? [];
      _header = _reportTemplate.header ?? [];
    } catch (e) {
      _height = 841.8897637795275.s;
      _width = 595.275590551181.s;
    }
  }

  void _initListentener() {
    FReportInfo.instance.onReload.addListener(_onReloadCallback);
  }

  void _initTemplate({ReportEventArgs? args}) {
    String jsonStr = widget.reportTemplateJson;
    if (args != null) {
      jsonStr = args.jsonStr;
      FReportInfo.instance.onReload.addListener(_onReloadCallback);
    }
    final reportMap = jsonDecode(jsonStr);

    setState(() {
      _reportTemplate = ReportTemplateDocument.fromJson(reportMap);
      FReportInfo.instance.init(
        _reportTemplate,
        DateTime.now(),
        Storage.user.displayName ?? '',
        patientName: widget.controller.state.patientName.value,
        patientAge: widget.controller.state.patientAge.value,
        patientSex: widget.controller.state.patientSex.value,
        patinentId: widget.controller.state.devicePatientId.value,
        revoke: i18nBook.common.revoke.t,
        selectEntry: i18nBook.remedical.selectWord.t,
        selectImageHint: i18nBook.remedical.clickAndSelectImage.t,
      );
      if (args != null) {
        FReportInfo.instance.isASROn = args.isASROn;
      }
      _initPage();
    });

    isInit = true;
  }

  @override
  void dispose() {
    FReportInfo.instance.onReload.removeListener(_onReloadCallback);
    if (_isPortraitMode == Storage.isPortraitMode.value) {
      FReportInfo.instance.clear();
    }
    super.dispose();
  }

  void _onReloadCallback(Object sender, ReportEventArgs e) {
    _header.clear();
    _blocks.clear();
    _footer.clear();
    _initTemplate(args: e);
    FReportInfo.instance.onReloadFinsh.emit(this, '');
  }
}
