import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flyinsonolite/consultation/reports/controllers/reporteditcontroller.dart';
import 'package:flyinsonolite/controls/dialog/fisdialog.dart';
import 'package:flyinsonolite/controls/swipecard.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/searchtextfield.dart';
import 'package:flyinsonolite/controls/treeview/node.dart' as controls;
import 'package:flyinsonolite/controls/treeview/treeview.dart';
import 'package:flyinsonolite/controls/treeview/treeviewcontroller.dart';
import 'package:flyinsonolite/controls/treeview/treeviewtheme.dart';
import 'package:flyinsonolite/consultation/reports/models/reporteditsetting.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/expandable_icon_text.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/services/localreporteditorservice.dart';
import 'package:get/get.dart';

///报告中词条选择
class VocabularyEntryReport extends StatefulWidget {
  @override
  State<VocabularyEntryReport> createState() => _VocabularyEntryReportState();

  const VocabularyEntryReport({
    super.key,
  });

  final String pageName = 'VocabularyEntryReport';
}

class _VocabularyEntryReportState extends State<VocabularyEntryReport> {
  /// 词条树顶层结构
  late List<controls.Node<dynamic>> treeViewTopData = [];

  final entrysScrollController = ScrollController();

  double _maxWidth = 0;

  /// 当前选中项
  String selectedKey = '';

  /// 关键字搜索
  String keyWords = '';

  late final TextEditingController _textEditingController =
      TextEditingController(text: keyWords);

  /// 当前词条分类的词条卡片内容
  List<ThesaurusItemDTO> vocabularyCard = [];

  /// 序列化存储的词条列表
  List<dynamic> defaultThesaurusList = [];

  /// 树形选择器控制器
  TreeViewController _treeViewController = TreeViewController();

  /// 本地报告编辑器服务
  LocalReportEditorService localReportEditorService =
      Get.find<LocalReportEditorService>();

  ReportEditorSetting reportEditorSetting = ReportEditorSetting.ins;

  String? _focusItem;

  void _listenKeyboardEvent(RawKeyEvent event) {
    if (event.isKeyPressed(LogicalKeyboardKey.enter) ||
        event.isKeyPressed(LogicalKeyboardKey.numpadEnter)) {
      getEntryByKeyWords();
    }
  }

  /// 报告控制器
  final reportEditController = Get.find<ReportEditController>();

  /// 检查是否需要重新打开上次打开的节点
  void checkReopenNode() {
    /// 上次打开的词条分类和词条项
    final lastEntryCategoryCode = reportEditorSetting.lastEntryCategoryCode;
    final lastEntryCode = reportEditorSetting.lastEntryCode;
    var entryCategoryCodeToReopen = '';
    var entryCodeToReopen = '';

    for (var element in treeViewTopData) {
      if (element.key == lastEntryCategoryCode) {
        entryCategoryCodeToReopen = element.key;
      }
      for (var child in element.children) {
        if (child.key == lastEntryCode) {
          entryCodeToReopen = child.key;
        }
      }
    }
    if (entryCategoryCodeToReopen == '') return;
    List<controls.Node> newdata =
        _treeViewController.expandToNode(entryCategoryCodeToReopen);
    if (entryCodeToReopen == '') {
      _treeViewController = _treeViewController.copyWith(children: newdata);
    } else {
      _treeViewController = _treeViewController.copyWith(
          children: newdata, selectedKey: entryCodeToReopen);
      vocabularyCard = reportEditController.state.allThesaurusList
          .where((element) => element.parentItemCode == entryCodeToReopen)
          .toList();
    }
  }

  /// 下拉项点击
  Future onExpansionChanged(String code, bool isDrop) async {
    reportEditorSetting.setLastEntryCategoryCode(code);
    await localReportEditorService.saveAsync();
    if (isDrop) {
      List<controls.Node> newdata = _treeViewController.expandToNode(code);
      _treeViewController = _treeViewController.copyWith(children: newdata);
    } else {
      List<controls.Node> newdata = _treeViewController.collapseToNode(code);
      _treeViewController = _treeViewController.copyWith(children: newdata);
    }
    setState(() {});
  }

  /// 词条分类的选中
  Future onNodeTapAsync(String code) async {
    reportEditorSetting.setLastEntryCode(code);
    await localReportEditorService.saveAsync();
    _treeViewController = _treeViewController.copyWith(selectedKey: code);
    if (reportEditController.state.allThesaurusList.isNotEmpty) {
      vocabularyCard = reportEditController.state.allThesaurusList
          .where((element) => element.parentItemCode == code)
          .toList();
    }
    setState(() {
      selectedKey = code;
    });
  }

  /// 搜索的方法
  void getEntryByKeyWords() {
    selectedKey = "";
    vocabularyCard = [];
    List<ThesaurusItemDTO> thesaurusItem = [];
    var allThesaurusList = reportEditController.state.allThesaurusList;
    for (var e in allThesaurusList) {
      if (e.thesaurusItemName?.contains(keyWords) == true ||
          e.thesaurusItemConclusion?.contains(keyWords) == true ||
          e.thesaurusItemDescription?.contains(keyWords) == true) {
        if (e.parentItemCode != '') {
          for (var element in allThesaurusList) {
            if (element.thesaurusItemCode == e.parentItemCode) {
              thesaurusItem.add(element);
              if (element.parentItemCode != '') {
                for (var key in allThesaurusList) {
                  if (key.thesaurusItemCode == element.parentItemCode) {
                    thesaurusItem.add(key);
                  }
                }
              }
            }
          }
        }
        thesaurusItem.add(e);
      }
    }
    thesaurusItem = thesaurusItem.toSet().toList();
    treeViewTopData = getThesaurusItemsAsyncList(thesaurusItem);
    setState(() {
      _treeViewController = TreeViewController(
        children: treeViewTopData,
        selectedKey: selectedKey,
      );
    });
  }

  void _initThesaurusList() {
    treeViewTopData = getThesaurusItemsAsyncList(
      reportEditController.state.allThesaurusList,
    );
    if (treeViewTopData.isNotEmpty) {
      var firstThesaurus = treeViewTopData.first;
      treeViewTopData[0] = firstThesaurus.copyWith(
        expanded: true,
      );
      if (firstThesaurus.children.isNotEmpty) {
        selectedKey = firstThesaurus.children.first.key;
      } else {
        selectedKey = firstThesaurus.key;
      }
    }
    _treeViewController = TreeViewController(
      children: treeViewTopData,
      selectedKey: selectedKey,
    );

    vocabularyCard = reportEditController.state.allThesaurusList
        .where((element) => element.parentItemCode == selectedKey)
        .toList();
  }

  /// 数据处理
  List<controls.Node> getThesaurusItemsAsyncList(
    List<ThesaurusItemDTO> allThesaurusItemsAsyncList,
  ) {
    List<controls.Node> treeView = [];
    for (var element in allThesaurusItemsAsyncList) {
      List<controls.Node> treeviewChild = [];
      final lista = allThesaurusItemsAsyncList.where(
        (e) => e.parentItemCode == element.thesaurusItemCode,
      );
      for (var element in lista) {
        if (element.thesaurusItemType != ThesaurusItemTypeEnum.Contents) {
          treeviewChild.add(
            controls.Node(
              key: element.thesaurusItemCode ?? '',
              label: element.thesaurusItemName ?? '',
            ),
          );
        }
      }
      if (element.parentItemCode == '' || element.parentItemCode == null) {
        treeView.add(
          controls.Node(
            key: element.thesaurusItemCode!,
            label: element.thesaurusItemName!,
            children: treeviewChild,
          ),
        );
      }
    }
    return treeView;
  }

  @override
  void initState() {
    super.initState();
    RawKeyboard.instance.addListener(_listenKeyboardEvent);
    entrysScrollController.addListener(_scrollListener);
    _checkIsShowRightScrol();

    /// 初始化形成所需的树结构
    _initThesaurusList();
  }

  @override
  void didUpdateWidget(VocabularyEntryReport oldWidget) {
    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    RawKeyboard.instance.removeListener(_listenKeyboardEvent);
    entrysScrollController.dispose();
    super.dispose();
  }

  /// 左边能否滑动
  bool _isLeftCanScroll = false;

  /// 右边能否滑动
  bool _isRightCanScroll = false;

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      _maxWidth = Get.width * (Storage.isPortraitMode.value ? 0.8 : 0.4);
      return FISDialog(
        i18nBook.remedical.wordSelectBox.t,
        Column(
          children: [
            _buildEntrysSelected(),
            SizedBox(
              height: 8.s,
            ),
            Row(
              children: [
                SizedBox(
                  width: 16.s,
                ),
                Expanded(
                  child: SearchTextField(
                    uniqueKey: GlobalKey(),
                    controller: _textEditingController,
                    placeHolder: i18nBook.remedical.searchKeyword.t,
                    onChanged: (value) {
                      keyWords = value;
                    },
                    () {
                      getEntryByKeyWords();
                    },
                  ),
                ),
                SizedBox(
                  width: 16.s,
                ),
              ],
            ),
            SizedBox(
              height: 8.s,
            ),
            _buildVocabularyTreeview(context),
          ],
        ),
        false,
        false,
        alignment: Alignment.centerRight,
        bgColor: Colors.white,
      );
    });
  }

  Widget _buildVocabularyTreeview(BuildContext context) {
    return Center(
      child: Container(
        width: _maxWidth,
        height: Get.height - 250.s,
        decoration: BoxDecoration(
          border: Border(
            top: BorderSide(
              color: const Color.fromRGBO(211, 211, 211, 1),
              width: 1.s,
            ),
          ),
        ),
        margin: EdgeInsets.symmetric(
          vertical: 4.s,
          horizontal: 8.s,
        ),
        child: Column(
          children: [
            Row(
              children: [
                Container(
                  height: Get.height - 260.s,
                  width: _maxWidth * 0.33,
                  decoration: BoxDecoration(
                    border: Border(
                      right: BorderSide(
                          color: const Color.fromRGBO(211, 211, 211, 1),
                          width: 1.s),
                    ),
                  ),
                  child: _buildEntryTree(),
                ),
                SizedBox(
                  height: Get.height - 260.s,
                  width: _maxWidth * 0.67,
                  child: _EntriesItem(
                    vocabularyCard: vocabularyCard,
                  ),
                ),
              ],
            ),
          ],
        ),
      ),
    );
  }

  Widget _buildEntryTree() {
    return Obx(() {
      if (reportEditController.state.allThesaurusList.isEmpty) {
        return const SizedBox();
      }
      return SizedBox(
        height: Get.height - 180.s,
        child: TreeView(
          theme: TreeViewTheme(
            levelPadding: 15.s,
            horizontalSpacing: 15.s,
            colorScheme: ColorScheme.dark(
              primary: Storage.currentTheme.themeData.indicatorColor,
            ),
          ),
          onNodeTap: (code) async {
            await onNodeTapAsync(code);
          },
          onExpansionChanged: (code, isDrop) {
            onExpansionChanged(
              code,
              isDrop,
            );
          },
          controller: _treeViewController,
          nodeBuilder: (buildContext, node) {
            return _EntryTreeView(
              node: node,
              selectedKey: selectedKey,
            );
          },
        ),
      );
    });
  }

  void _scrollListener() {
    if (entrysScrollController.offset <=
        entrysScrollController.position.minScrollExtent) {
      setState(() {
        _isLeftCanScroll = false;
        _isRightCanScroll = true;
      });
    } else if (entrysScrollController.offset >=
        entrysScrollController.position.maxScrollExtent) {
      setState(() {
        _isLeftCanScroll = true;
        _isRightCanScroll = false;
      });
    } else {
      setState(() {
        _isLeftCanScroll = true;
        _isRightCanScroll = true;
      });
    }
  }

  Widget _buildEntrysSelected() {
    return Obx(() {
      var entryInfos = reportEditController.state.entryInfos;
      if (entryInfos.isEmpty) {
        return const SizedBox();
      }
      return Container(
          height: 40.s,
          alignment: Alignment.center,
          width: _maxWidth - 20.s,
          margin: EdgeInsets.symmetric(horizontal: 5.s),
          child: Row(
            children: [
              InkWell(
                child: Icon(
                  Icons.chevron_left_rounded,
                  color: _isLeftCanScroll ? Colors.black : Colors.grey,
                  size: 50.s,
                ),
                onTap: () {
                  var target = entrysScrollController.offset - 60.s;
                  entrysScrollController.jumpTo(target > 0 ? target : 0);
                },
              ),
              SizedBox(
                width: 10.s,
              ),
              Expanded(
                child: ListView(
                  shrinkWrap: true,
                  scrollDirection: Axis.horizontal,
                  controller: entrysScrollController,
                  children: entryInfos
                      .map(
                        (e) => ExpandableIconText(
                          icon: Icons.book,
                          isDistinguishChinese: false,
                          isExpandChild: false,
                          horizontalMargin: 10.s,
                          isSelected: reportEditController
                                  .state.selectedEntryCode.value ==
                              e.thesaurusCode,
                          onPressed: () async {
                            reportEditController.state.selectedEntryCode.value =
                                e.thesaurusCode ?? '';
                            await reportEditController.thesaurusController
                                .getDefaultThesaurus(e.thesaurusCode ?? '');
                            _initThesaurusList();
                            setState(() {});
                          },
                          onHoverChange: (v) {
                            if (v) {
                              setState(() {
                                _focusItem = e.thesaurusCode;
                              });
                            } else {
                              setState(() {
                                _focusItem = reportEditController
                                    .state.selectedEntryCode.value;
                              });
                            }
                          },
                          text: e.thesaurusName ?? '',
                          isExpand: _focusItem == e.thesaurusCode,
                        ),
                      )
                      .toList(),
                ),
              ),
              SizedBox(width: 10.s),
              InkWell(
                child: Icon(
                  Icons.chevron_right_rounded,
                  size: 50.s,
                  color: _isRightCanScroll ? Colors.black : Colors.grey,
                ),
                onTap: () {
                  var target = entrysScrollController.offset + 60.s;
                  var allLength =
                      entrysScrollController.position.maxScrollExtent;
                  entrysScrollController.jumpTo(
                      target > allLength ? allLength.toDouble() : target);
                },
              ),
            ],
          ));
    });
  }

  void _checkIsShowRightScrol() {
    Future.delayed(const Duration(milliseconds: 500), () {
      var target = entrysScrollController.offset + 1;
      var allLength = entrysScrollController.position.maxScrollExtent;
      entrysScrollController
          .jumpTo(target > allLength ? allLength.toDouble() : target);
      entrysScrollController.jumpTo(0);
    });
  }
}

class _EntryTreeView extends StatefulWidget {
  const _EntryTreeView({Key? key, required this.node, this.selectedKey = ''})
      : super(key: key);

  /// 当前下拉节点
  final controls.Node<dynamic> node;
  final String pageName = 'EntryTreeView';

  /// 当前选中项
  final String? selectedKey;
  @override
  State<_EntryTreeView> createState() => _EntryTreeViewState();
}

class _EntryTreeViewState extends State<_EntryTreeView> {
  final EdgeInsets _padding = EdgeInsets.symmetric(vertical: 4.s);
  bool hovering = false;
  @override
  Widget build(BuildContext context) {
    var isNormalOption = widget.selectedKey != widget.node.key;
    return Container(
      margin: EdgeInsets.only(right: 15.s),
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        onEnter: (e) {
          setState(() {
            hovering = true;
          });
        },
        onExit: (e) {
          setState(() {
            hovering = false;
          });
        },
        child: GestureDetector(
          child: Column(
            children: [
              hovering
                  ? Row(
                      children: [
                        Expanded(
                          child: _SelectText(
                            label: widget.node.label,
                            isSelected: isNormalOption,
                            padding: _padding,
                          ),
                        ),
                      ],
                    )
                  : Row(
                      children: [
                        Expanded(
                          child: Container(
                            padding: _padding,
                            child: FISText(
                              widget.node.label,
                              softWrap: true,
                              style: widget.selectedKey == widget.node.key
                                  ? TextStyle(
                                      color: Colors.white, fontSize: 14.s)
                                  : TextStyle(
                                      color: Colors.black, fontSize: 14.s),
                            ),
                          ),
                        ),
                      ],
                    ),
            ],
          ),
        ),
      ),
    );
  }
}

/// 词条分类选中样式
class _SelectText extends StatelessWidget {
  const _SelectText({
    required this.isSelected,
    required this.label,
    required this.padding,
  });
  final bool isSelected;
  final String label;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: padding,
      child: FISText(
        label,
        softWrap: true,
        style: TextStyle(
            color: isSelected ? Colors.black : Colors.white, fontSize: 14.s),
      ),
    );
  }
}

class _EntriesItem extends StatelessWidget {
  const _EntriesItem({this.vocabularyCard});

  /// 词条卡片内容
  final List<ThesaurusItemDTO>? vocabularyCard;
  @override
  Widget build(BuildContext context) {
    var selectedKey = '';
    return ReorderableListView(
      shrinkWrap: true,
      buildDefaultDragHandles: false,
      children: vocabularyCard!
          .map(
            (item) => Container(
              key: Key(
                item.thesaurusItemCode.toString(),
              ),
              padding: EdgeInsets.only(top: 15.s),
              child: SwipeCard(
                isForbiddenAnimation: true,
                forbiddenAnimationMargin: const EdgeInsets.all(0),
                forbiddenAnimationPadding:
                    EdgeInsets.only(left: 5.s, top: 5.s, right: 5.s),
                onTap: () {
                  selectedKey = item.thesaurusItemCode ?? '';
                  FReportInfo.instance.insertDiagnostic(
                    item.thesaurusItemDescription ?? '',
                    item.thesaurusItemConclusion ?? '',
                  );
                },
                isActive: false,
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    FISText(
                      i18nBook.remedical.description.t,
                      softWrap: true,
                      style: selectedKey == item.thesaurusItemCode
                          ? TextStyle(
                              fontWeight: FontWeight.bold,
                              color: Colors.white,
                              fontSize: 14.s)
                          : TextStyle(
                              fontWeight: FontWeight.bold,
                              color: Colors.black,
                              fontSize: 14.s),
                    ),
                    Row(
                      mainAxisSize: MainAxisSize.max,
                      children: [
                        Expanded(
                          child: Container(
                            margin: EdgeInsets.only(right: 15.s),
                            child: FISText(
                              item.thesaurusItemDescription!,
                              textAlign: TextAlign.left,
                              softWrap: true,
                              style: selectedKey == item.thesaurusItemCode
                                  ? TextStyle(
                                      color: Colors.white, fontSize: 14.s)
                                  : TextStyle(
                                      color: Colors.black, fontSize: 14.s),
                            ),
                          ),
                        )
                      ],
                    ),
                    SizedBox(
                      height: 5.s,
                    ),
                    FISText(
                      i18nBook.remedical.conclusion.t,
                      softWrap: true,
                      style: selectedKey == item.thesaurusItemCode
                          ? TextStyle(
                              fontWeight: FontWeight.bold,
                              color: Colors.white,
                              fontSize: 14.s)
                          : TextStyle(
                              fontWeight: FontWeight.bold,
                              color: Colors.black,
                              fontSize: 14.s),
                    ),
                    Row(
                      mainAxisSize: MainAxisSize.max,
                      children: [
                        Expanded(
                          child: Container(
                            margin: EdgeInsets.only(right: 15.s),
                            child: FISText(
                              softWrap: true,
                              item.thesaurusItemConclusion!,
                              textAlign: TextAlign.left,
                              style: selectedKey == item.thesaurusItemCode
                                  ? TextStyle(
                                      color: Colors.white, fontSize: 14.s)
                                  : TextStyle(
                                      color: Colors.black, fontSize: 14.s),
                            ),
                          ),
                        )
                      ],
                    ),
                    SizedBox(
                      height: 10.s,
                    ),
                  ],
                ),
              ),
            ),
          )
          .toList(),
      onReorder: (int oldIndex, int newIndex) {
        /// 拖拽的方法
      },
    );
  }
}
