import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/custom/customdatepicker.dart';
import 'package:flyinsonolite/controls/custom/customtextbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/converters/alignmentconverter.dart';
import 'package:flyinsonolite/converters/datetostringconverter.dart';
import 'package:flyinsonolite/converters/pttopxconverter.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/date_time_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/date_time_element.dart';

class FDateTime extends StatefulWidget {
  final DateTimeElement dateTimeElement;
  const FDateTime(this.dateTimeElement, {Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _FDateTimeState();
  }
}

class _FDateTimeState extends State<FDateTime> {
  DateTime selectedDate = DateTime.now();

  _FDateTimeState();

  String _dateTimeStr = '';
  double _fontSize = 14.s;
  double _width = 0;
  TextStyle _style = TextStyle(fontSize: 14.s);
  Color _fontColor = Colors.black;
  Color _backgroundColor = const Color.fromARGB(255, 255, 255, 255);
  EdgeInsets _margin = const EdgeInsets.all(0);
  DateTimeInfo? _dateTimeInfo;

  @override
  initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final dateTimeElement = widget.dateTimeElement;
    _initDatas();
    return Container(
        margin: _margin,
        constraints: BoxConstraints(minHeight: 24.s),
        padding: EdgeInsets.symmetric(horizontal: 4.s),
        height: _fontSize + 10.s,
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(4.s),
          border: Border.all(
            width: 1.s,
            color:
                dateTimeElement.isReadOnly ? Colors.transparent : Colors.grey,
          ),
        ),
        width: _width,
        alignment: AlignmentConverter.verticalLayoutConvert(
            dateTimeElement.verticalAlignment!),
        child: Row(
          mainAxisAlignment: AlignmentConverter.horizontalToMainConvert(
              dateTimeElement.horizontalAlignment),
          children: [
            Container(
              alignment: AlignmentConverter.verticalLayoutConvert(
                  dateTimeElement.verticalAlignment!),
              child: FISText(
                _dateTimeStr,
                style: _style,
                textAlign: AlignmentConverter.horizontalAlignmentConvert(
                    dateTimeElement.horizontalAlignment),
              ),
            ),
            if (!dateTimeElement.isReadOnly) ...[
              const Expanded(child: SizedBox()),
              GestureDetector(
                onTap: () => _selectDate(context),
                child: Icon(
                  Icons.date_range_outlined,
                  size: 17.s,
                ),
              ),
            ],
          ],
        ));
  }

  _selectDate(BuildContext context) async {
    final DateTime? picked = await showCustomDatePicker(
      initialEntryMode: DatePickerEntryMode.calendarOnly,
      context: context,
      initialDate: selectedDate, // Refer step 1
      firstDate: DateTime(2000),
      lastDate: DateTime(4000),
      locale: i18nBook.locale,
      currentDate: DateTime.now(),
      builder: (context, child) {
        return Theme(
            data: ThemeData(
                fontFamily: Storage.currentTheme.fontFamily,
                // 更改右侧时间字体的大小
                textTheme: TextTheme(
                  titleSmall: TextStyle(
                      fontSize: 18.s,
                      color: const Color.fromRGBO(211, 211, 211, 1)),
                  titleMedium: TextStyle(
                      fontSize: 18.s,
                      color: const Color.fromRGBO(211, 211, 211, 1)),
                  titleLarge: TextStyle(
                      fontSize: 18.s,
                      color: const Color.fromRGBO(211, 211, 211, 1)),
                  bodySmall: TextStyle(
                      fontSize: 18.s,
                      color: const Color.fromRGBO(211, 211, 211, 1)),
                  bodyMedium: TextStyle(
                      fontSize: 18.s,
                      color: const Color.fromRGBO(211, 211, 211, 1)),
                  bodyLarge: TextStyle(
                      fontSize: 18.s,
                      color: const Color.fromRGBO(211, 211, 211, 1)),
                  labelSmall: TextStyle(
                    fontSize: 18.s,
                    color: const Color.fromRGBO(211, 211, 211, 1),
                  ),
                  labelMedium: TextStyle(
                      fontSize: 18.s,
                      color: const Color.fromRGBO(211, 211, 211, 1)),
                  labelLarge: TextStyle(
                      fontSize: 18.s,
                      color: const Color.fromRGBO(211, 211, 211, 1)),
                  headlineSmall: TextStyle(
                    fontSize: 20.s,
                    color: const Color.fromRGBO(211, 211, 211, 1),
                  ),
                  headlineMedium: TextStyle(
                    fontSize: 20.s,
                    color: const Color.fromRGBO(211, 211, 211, 1),
                  ),
                  headlineLarge: TextStyle(
                    fontSize: 20.s,
                    color: const Color.fromRGBO(211, 211, 211, 1),
                  ),
                  displaySmall: TextStyle(
                    fontSize: 20.s,
                    color: const Color.fromRGBO(211, 211, 211, 1),
                  ),
                  displayMedium: TextStyle(
                    fontSize: 20.s,
                    color: const Color.fromRGBO(211, 211, 211, 1),
                  ),
                  displayLarge: TextStyle(
                    fontSize: 20.s,
                    color: const Color.fromRGBO(211, 211, 211, 1),
                  ),
                ),
                inputDecorationTheme: InputDecorationTheme(
                  labelStyle: TextStyle(
                    fontSize: 22.s,
                    color: const Color.fromRGBO(211, 211, 211, 1),
                  ),
                ),
                dialogBackgroundColor:
                    Storage.currentTheme.datePickerStyle.backgroundColor,
                colorScheme: ColorScheme.light(
                    primary: Storage.currentTheme.datePickerStyle.leftSizeColor,
                    onSurface: Storage.currentTheme.normalTextStyle.color!),
                textButtonTheme: TextButtonThemeData(
                    style: CustomTextButton.styleFrom(
                        foregroundColor:
                            Storage.currentTheme.normalTextStyle.color,
                        textStyle: Storage.currentTheme.normalTextStyle))),
            child: child!);
      },
    );
    if (picked != null && picked != selectedDate) {
      final format = widget.dateTimeElement.dateTimeFormat;
      setState(() {
        selectedDate = picked;
        //_dateTimeConvert(format!, selectedDate.millisecondsSinceEpoch);
        _dateTimeInfo!.text = selectedDate.toString();
      });
    }
  }

  void _onTextChangeCallback(Object sender, String e) {
    final dateTimeElement = widget.dateTimeElement;
    final format = dateTimeElement.dateTimeFormat;
    if (mounted) {
      setState(() {
        final date = DateTime.parse(_dateTimeInfo!.text.trim());
        _dateTimeConvert(format!, date.millisecondsSinceEpoch);
      });
    }
  }

  void _initDatas() {
    final dateTimeElement = widget.dateTimeElement;
    final format = dateTimeElement.dateTimeFormat;
    final dateTimeInfo =
        FReportInfo.instance.getElementInfo(dateTimeElement) as DateTimeInfo?;
    if (dateTimeInfo != null && dateTimeInfo != _dateTimeInfo) {
      if (_dateTimeInfo != null) {
        _dateTimeInfo!.onTextChange.removeListener(_onTextChangeCallback);
      }
      _dateTimeInfo = dateTimeInfo;
      _dateTimeInfo!.onTextChange.addListener(_onTextChangeCallback);
    }
    if (_dateTimeInfo!.text.isNotEmpty) {
      final date = DateTime.parse(_dateTimeInfo!.text.trim());
      final dateStr = DateToStringConverter.dateTimeConvert(
          format!, date.millisecondsSinceEpoch);
      if (dateStr != _dateTimeStr) {
        _dateTimeStr = dateStr.trim();
      }
    }
    //TODO(Loki):常规模板暂未设置fontStyles，后续再支持
    final fontStyles = dateTimeElement.fontStyles;
    final fontColor = dateTimeElement.fontColor;
    if (fontColor != null) {
      _fontColor = Color.fromARGB(
          fontColor.a!, fontColor.r!, fontColor.g!, fontColor.b!);
    }
    final backgroundColor = dateTimeElement.background;
    if (backgroundColor != null) {
      _backgroundColor = Color.fromARGB(backgroundColor.a!, backgroundColor.r!,
          backgroundColor.g!, backgroundColor.b!);
      if (_backgroundColor == Colors.white) {
        _backgroundColor = Colors.transparent;
      }
    }
    final margin = dateTimeElement.margin;
    if (margin != null) {
      _margin = EdgeInsets.only(
          top: margin.top ?? 0,
          bottom: margin.bottom ?? 0,
          left: margin.left ?? 0,
          right: margin.right ?? 0);
    }
    _fontSize = PtToPxConverter.ptToPx(dateTimeElement.fontSize ?? 14);
    _style = TextStyle(
      fontSize: _fontSize,
      color: _fontColor,
      backgroundColor: _backgroundColor,
    );
    _width = (_fontSize * 2 / 3) * _dateTimeStr.length;
    if (!dateTimeElement.isReadOnly) {
      var contentWidth = PtToPxConverter.ptToPx(dateTimeElement.lineWidth);
      _width = contentWidth;
    }
  }

  void _dateTimeConvert(String format, int timestamp) {
    _dateTimeStr =
        DateToStringConverter.dateTimeConvert(format, timestamp).trim();
  }
}
