import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/custom/customcheckbox.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/converters/pttopxconverter.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/interfaces/multi_selected.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';

class FOptionRow extends StatefulWidget {
  const FOptionRow(
    this.multiSelected,
    this.label, {
    Key? key,
    this.isActive = false,
    this.height,
    this.onTap,
    this.isSelected = false,
  }) : super(key: key);

  final Function? onTap;
  final IMultiSelected multiSelected;
  final String label;
  final bool isActive;
  final double? height;
  final bool isSelected;

  @override
  State<StatefulWidget> createState() {
    return _FOptionRowState();
  }
}

class _FOptionRowState extends State<FOptionRow> {
  _FOptionRowState();
  bool? isSelected = false;

  @override
  void initState() {
    isSelected = widget.isSelected;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final color = widget.multiSelected.fontColor!;
    final fontSize = PtToPxConverter.ptToPx(widget.multiSelected.fontSize);
    final text = FISText(
      widget.label,
      style: TextStyle(
        fontSize: fontSize > 18.s ? 18.s : fontSize,
        color: Color.fromARGB(color.a!, color.r!, color.g!, color.b!),
      ),
    );
    if (widget.height != null) {
      return GestureDetector(
        onTap: () {
          if (mounted) {
            setState(() {
              isSelected = !isSelected!;
            });
          }
          if (widget.onTap != null) {
            widget.onTap!.call();
          }
        },
        child: Container(
          alignment: Alignment.centerLeft,
          height: widget.height,
          width: PtToPxConverter.ptToPx(widget.multiSelected.lineWidth),
          child: Row(
            children: [
              CustomCheckbox(
                onChanged: (bool? value) {
                  if (mounted) {
                    setState(() {
                      isSelected = value ?? false;
                    });
                  }
                  if (widget.onTap != null) {
                    widget.onTap!.call();
                  }
                },
                value: isSelected,
              ),
              Expanded(child: text),
            ],
          ),
        ),
      );
    } else {
      return text;
    }
  }
}
