import 'package:flutter/material.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/consultation/images/views/contentimageview.dart';
import 'package:flyinsonolite/controls/custom/customiconbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/converters/marginconverter.dart';
import 'package:flyinsonolite/converters/pttopxconverter.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/cell_position_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/input_image_list_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/cell_postion.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/input_image_list.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';

class FInputImageList extends StatefulWidget {
  final InputImageList inputImageList;
  final String pageName = 'FInputImageList';
  final CellPostion? cellPostion;
  final String parentTableId;
  const FInputImageList(this.inputImageList, this.cellPostion,
      {Key? key, this.parentTableId = ""})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _FInputImageListState();
  }
}

class _FInputImageListState extends State<FInputImageList> {
  InputImageList? inputImageList;

  InputImageListInfo? _inputImageListInfo;
  Color _borderColor = Colors.grey;
  bool _isSelected = false;
  List<String> _images = [];
  double _height = 0.0;

  Widget _child = FISText(FReportInfo.instance.selectImageHint,
      style: Storage.currentTheme.normalTextStyle.copyWith(
        fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
      ));

  @override
  initState() {
    _initDatas();
    super.initState();
  }

  @override
  void didUpdateWidget(covariant FInputImageList oldWidget) {
    super.didUpdateWidget(oldWidget);
    _heightAdaptation();
  }

  @override
  Widget build(BuildContext context) {
    _checkInputImageListInfo();
    //行数
    final rowCount = (_images.length / inputImageList!.column!).ceil();
    return SizedBox(
      width: PtToPxConverter.ptToPx(
          inputImageList!.imageWidth! * inputImageList!.column!),
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        child: GestureDetector(
          onTap: () {
            _onSelectedInputChange();
          },
          child: Container(
            padding: EdgeInsets.symmetric(vertical: 2.s, horizontal: 2.s),
            height: _height + 4.s - rowCount.s,
            width: PtToPxConverter.ptToPx(
                inputImageList!.imageWidth! * inputImageList!.column!),
            alignment: Alignment.center,
            margin: MarginConverter.marginConvert(inputImageList!.margin),
            decoration: BoxDecoration(
                border: _isSelected
                    ? Border.all(
                        width: 0.5.s,
                        color: _borderColor,
                      )
                    : null,
                color: Colors.transparent),
            child: _child,
          ),
        ),
      ),
    );
  }

  @override
  void dispose() {
    _inputImageListInfo!.onSelect.removeListener(_onSelected);
    super.dispose();
  }

  Widget _getImageListChild() {
    if (_images.isNotEmpty) {
      final imageHeight = PtToPxConverter.ptToPx(inputImageList!.imageHeight!);
      final rowCount = (_images.length / inputImageList!.column!).ceil();
      _height = rowCount * imageHeight + rowCount;
      return GridView.builder(
        primary: true,
        physics: const NeverScrollableScrollPhysics(),
        gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
          crossAxisCount: inputImageList!.column!,
          crossAxisSpacing: 1,
          mainAxisSpacing: 1,
          childAspectRatio:
              inputImageList!.imageWidth! / inputImageList!.imageHeight!,
        ),
        itemCount: _images.length,
        itemBuilder: (BuildContext context, int index) {
          return Container(
            height: imageHeight,
            color: Colors.black,
            child: Stack(
              alignment: Alignment.center,
              children: [
                ContentImage(
                  fit: BoxFit.fitHeight,
                  isPureImage: true,
                  fileDataType: RemedicalFileDataTypeEnum.Image,
                  terminalImage: TerminalImage(
                    coverImageUrl: _images[index],
                    previewUrl: _images[index],
                    imageUrl: _images[index],
                  ),
                  onTap: () {
                    _onSelectedInputChange();
                  },
                  ifShowIndex: false,
                ),
                Positioned(
                  right: 2.s,
                  top: 2.s,
                  child: CustomIconButton(
                    icon: const Icon(
                      Icons.highlight_off,
                      color: Colors.white,
                    ),
                    iconSize: 30.s,
                    padding: const EdgeInsets.all(0),
                    constraints: const BoxConstraints(),
                    onPressed: () {
                      var image = _images[index];
                      _inputImageListInfo!.onRemove.emit(this, image);
                      _onSelected(this, image);
                    },
                  ),
                ),
              ],
            ),
          );
        },
      );
    } else {
      _height = PtToPxConverter.ptToPx(inputImageList!.imageHeight!);
    }

    return FISText(FReportInfo.instance.selectImageHint,
        style: Storage.currentTheme.normalTextStyle.copyWith(
            fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
            color: Colors.black));
  }

  void _onSelectedChangeCallBack(Object sender, List<String> e) {
    if (mounted) {
      setState(() {
        _images = e;
        _child = _getImageListChild();
      });
    }
  }

  void _addSelectImageListening() {
    try {
      _inputImageListInfo!.onSelectedChange
          .addListener(_onSelectedChangeCallBack);
      _heightAdaptation();
    } catch (e) {
      print(e);
    }
  }

  void _addOnSelectedListening() {
    _inputImageListInfo!.onSelect.removeListener(_onSelected);
    _inputImageListInfo!.onSelect.addListener(_onSelected);
  }

  void _onSelected(Object sender, String e) {
    if (mounted) {
      setState(() {
        if (_images.contains(e)) {
          _images.remove(e);
          if (_inputImageListInfo!.selectedImages.contains(e)) {
            _inputImageListInfo!.selectedImages.remove(e);
          }
        } else {
          _images.add(e);
          if (!_inputImageListInfo!.selectedImages.contains(e)) {
            _inputImageListInfo!.selectedImages.add(e);
          }
        }
        _child = _getImageListChild();
      });
      _heightAdaptation();
    }
  }

  void _initDatas() {
    inputImageList = widget.inputImageList;
    _height = PtToPxConverter.ptToPx(inputImageList!.imageHeight);
    final element = FReportInfo.instance.getBlockElement(inputImageList!);
    if (element != null) {
      _inputImageListInfo = element as InputImageListInfo;
      //  _addOnSelectedListening();
      _addSelectImageListening();
    } else {
      final element =
          FReportInfo.instance.getElementInfoById(inputImageList!.id ?? '');
      if (element != null) {
        _inputImageListInfo = element as InputImageListInfo;
        _addSelectImageListening();
      }
    }
  }

  void _checkInputImageListInfo() {
    if (_inputImageListInfo != null) {
      final imageInfo =
          FReportInfo.instance.getBlockElement(widget.inputImageList);
      if (imageInfo != null) {
        imageInfo as InputImageListInfo;
      } else {
        final element =
            FReportInfo.instance.getElementInfoById(inputImageList!.id ?? '');
        if (element != null) {
          _inputImageListInfo = element as InputImageListInfo;
          _addSelectImageListening();
        }
      }
      if (imageInfo != _inputImageListInfo && imageInfo is InputImageListInfo) {
        if (_inputImageListInfo != null) {
          _inputImageListInfo!.onSelectedChange
              .removeListener(_onSelectedChangeCallBack);
        }
        _inputImageListInfo = imageInfo;
        _addSelectImageListening();

        _addOnSelectedListening();
        if (_images != _inputImageListInfo!.selectedImages) {
          _images = _inputImageListInfo!.selectedImages;
        }
        _isSelected = false;
        _child = _getImageListChild();
      } else {
        _addOnSelectedListening();
        if (_images != _inputImageListInfo!.selectedImages) {
          setState(() {
            _images = _inputImageListInfo!.selectedImages;
            _child = _getImageListChild();
          });
          _heightAdaptation();
        }
      }
    }
  }

  void _onSelectedInputChange() {
    if (_isSelected) {
      setState(() {
        _borderColor = Colors.grey;
        _isSelected = false;
        _inputImageListInfo!.isSelected = false;
      });
    } else {
      setState(() {
        _borderColor = const Color.fromARGB(255, 64, 159, 248);
        _isSelected = true;
        _inputImageListInfo!.isSelected = true;
      });
    }
    _heightAdaptation();
  }

  void _heightAdaptation() {
    var cellPostion = widget.cellPostion;
    if (cellPostion != null) {
      final rowCount = (_images.length / inputImageList!.column!).ceil();
      //触发高度适配
      if (widget.parentTableId.isNotEmpty) {
        FReportInfo.instance.cellRowChange.emit(
          this,
          TableScaleInfo(
            rowCount == 0 ? 1 : rowCount.toDouble(),
            cellPostion,
            widget.parentTableId,
          ),
        );
      }
    }
  }
}
