import 'package:fis_jsonrpc/services/authentication.m.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/custom/customiconbutton.dart';
import 'package:flyinsonolite/controls/defaultinputdecoration.dart';
import 'package:flyinsonolite/converters/pttopxconverter.dart';
import 'package:flyinsonolite/converters/textsizeconverter.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/consultation.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/consultation/reports/widgets/autojumpintent.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/cell_position_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/element_tag_names.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/input_text_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/cell_postion.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/element_tag.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/input_text.dart';
import 'package:get/get.dart';

class FInputText extends StatefulWidget {
  final InputText inputText;
  final CellPostion? cellPostion;
  final String parentTableId;
  const FInputText({
    Key? key,
    required this.inputText,
    this.cellPostion,
    this.parentTableId = "",
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _FInputTextState();
  }
}

class _FInputTextState extends State<FInputText> {
  _FInputTextState();
  final _controller = CustomTextEditingController();
  final FocusNode _focusNode = FocusNode();
  double _lineWidth = 136.s;
  double _fontSize = 22.s;
  double _allHeight = 0.0;
  bool _textWrap = false;
  Color _fontColor = const Color.fromARGB(255, 0, 0, 0);
  TextStyle? _textStyle;
  InputTextInfo? _inputTextInfo;
  bool _isReadOnly = false;
  bool _widthLock = false;
  int _rowCount = 1;
  InputDecoration _inputDecoration = DefaultInputDecoration2();
  double? _freezeHeight; // 输入框高度冻结
  /// 组件Key
  final GlobalKey _textFieldConatinerKey = GlobalKey();
  final GlobalKey _key = GlobalKey();

  final String route = Get.currentRoute;
  @override
  initState() {
    _focusNode.addListener(_textFieldFocusNodeListener);
    _controller.addComposeListener(_composeListener);
    _initStyle();
    WidgetsBinding.instance.addPostFrameCallback((_) {
      _heightAdaptation();
    });
    super.initState();
  }

  /// 焦点事件监听
  void _textFieldFocusNodeListener() {
    if (!_focusNode.hasFocus) {
      _onInputChanged(_textStyle!, _controller.text);
    }
  }

  // 开始进行中文输入时触发
  void _composeListener(bool isComposing) {
    if (isComposing) {
      final RenderBox renderBox = _textFieldConatinerKey.currentContext!
          .findRenderObject() as RenderBox;
      setState(() {
        _freezeHeight = renderBox.size.height;
      });
    } else {
      setState(() {
        _freezeHeight = null;
      });
    }
  }

  @override
  void didUpdateWidget(FInputText oldWidget) {
    _initStyle();
    WidgetsBinding.instance.addPostFrameCallback((_) {
      _heightAdaptation();
    });
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    _checkInputTextInfo();
    final textField = Shortcuts(
      shortcuts: {
        LogicalKeySet(LogicalKeyboardKey.tab): AutoJumpIntent(),
      },
      child: Actions(
        actions: <Type, Action<Intent>>{
          AutoJumpIntent: CallbackAction<AutoJumpIntent>(
            onInvoke: (AutoJumpIntent intent) {
              intent.handleAutoJump(_controller);
              return null;
            },
          ),
        },
        child: TextField(
          key: _key,
          scrollPadding: EdgeInsets.all(20.0.s),
          cursorWidth: 2.s,
          cursorHeight: 25.s,
          enableInteractiveSelection: Storage.platform == Platform.Windows,
          textAlignVertical: TextAlignVertical.top,
          decoration: _isReadOnly ? null : _inputDecoration,
          readOnly: _isReadOnly,
          focusNode: _focusNode,
          minLines: _textWrap ? _rowCount : 1,
          maxLines:
              (_textWrap && widget.cellPostion == null) ? null : _rowCount,
          controller: _controller,
          onEditingComplete: () {
            _onInputChanged(_textStyle!, _controller.text);
          },
          onChanged: (value) {
            _inputTextInfo!.writeOnly = value;
          },
          onTap: () async {
            if (route != Get.currentRoute ||
                Storage.platform == Platform.Android) {
              return;
            }
            Offset position = const Offset(0, 0);
            Size screenSize = const Size(0, 0);

            final RenderBox renderBox =
                _key.currentContext!.findRenderObject() as RenderBox;
            position = renderBox.localToGlobal(Offset.zero);
            screenSize = MediaQuery.of(_key.currentContext!).size;
            var positionY =
                position.dy + renderBox.size.height > screenSize.height
                    ? screenSize.height
                    : position.dy + renderBox.size.height;
            if (positionY < 0) {
              positionY = 0;
            }
            var fiscontrolPositionInfo = FISControlPositionInfo(
                isShowSoftwareKeyBoard: true,
                screenHeight: screenSize.height,
                screenWidth: screenSize.width,
                positionX: position.dx,
                positionY: positionY);
            await operateSoftKeyBoard(fiscontrolPositionInfo);
          },
          onTapOutside: (e) async {
            _focusNode.unfocus();
            if (route != Get.currentRoute ||
                Storage.platform == Platform.Android) {
              return;
            }
            var fiscontrolPositionInfo = FISControlPositionInfo(
                isShowSoftwareKeyBoard: false,
                screenHeight: 0,
                screenWidth: 0,
                positionX: 0,
                positionY: 0);
            await operateSoftKeyBoard(fiscontrolPositionInfo);
          },
          style: _textStyle,
        ),
      ),
    );
    final suffixIcon = _buildSufficIcon();
    Widget body;
    if (suffixIcon != null) {
      body = Stack(
        children: [
          textField,
          Positioned(
            right: 6,
            bottom: 6,
            child: suffixIcon,
          ),
        ],
      );
    } else {
      body = textField;
    }
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Container(
            key: _textFieldConatinerKey,
            width: _lineWidth,
            height: _textWrap ? _freezeHeight : _allHeight,
            alignment: Alignment.center,
            child: body),
        if (_textWrap) ...[
          SizedBox(height: 3.s),
        ],
        if (_inputTextInfo != null &&
            _inputTextInfo!.tag != null &&
            _inputTextInfo!.tag!.elementTagType ==
                ElementTagType.DiagnosticEntry) ...[
          Align(
            alignment: Alignment.centerRight,
            child: Container(
              margin: EdgeInsets.only(right: 5.s),
              child: Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  FISLinearGradientElevatedButton(
                    () {
                      FReportInfo.instance.onDiagnosticTap
                          .emit(this, _inputTextInfo!.tag!.name ?? '');
                    },
                    FReportInfo.instance.selectEntry,
                    Storage.currentTheme.dialogStyle.buttonGradient,
                    textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                        fontSize:
                            Storage.currentTheme.normalTextStyle.fontSize!.s),
                    padding: EdgeInsets.all(5.s),
                  ),
                  SizedBox(
                    width: 10.s,
                  ),
                  FISLinearGradientElevatedButton(
                    () {
                      setState(() {
                        _inputTextInfo!.text = '';
                      });
                    },
                    FReportInfo.instance.revoke,
                    Storage.currentTheme.dialogStyle.buttonGradient,
                    textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                        fontSize:
                            Storage.currentTheme.normalTextStyle.fontSize!.s),
                    padding: EdgeInsets.all(5.s),
                  ),
                ],
              ),
            ),
          ),
        ],
      ],
    );
  }

  Future operateSoftKeyBoard(
      FISControlPositionInfo fisControlPositionInfo) async {
    try {
      if (fisControlPositionInfo.isShowSoftwareKeyBoard ||
          Storage.isShowSoftKeyBoard !=
              fisControlPositionInfo
                  .isShowSoftwareKeyBoard) //因为OnTapOutSide一个页面如果有多个TextField控件会都触发
      {
        Storage.isShowSoftKeyBoard =
            fisControlPositionInfo.isShowSoftwareKeyBoard;
        await jsonRpcProxyForFISLib.consultation
            .operateSoftKeyBoard(fisControlPositionInfo);
      }
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "OperateSoftKeyBoard error: $ex");
    }
  }

  //onchange 事件
  void _onInputChanged(TextStyle textStyle, String value) {
    if (_inputTextInfo != null) {
      _inputTextInfo!.text = value;
    }
  }

  @override
  void dispose() {
    _inputTextInfo?.onTextChange.removeListener(_onTextChangeCallback);
    _controller.dispose();
    _focusNode.removeListener(_textFieldFocusNodeListener);
    _controller.removeComposeListener(_composeListener);
    super.dispose();
  }

  void _initDatas() {
    final inputText = widget.inputText;
    _controller.text = '';

    final inputTextInfo = FReportInfo.instance.getElementInfo(inputText);
    _inputTextInfo = inputTextInfo as InputTextInfo;
    if (_inputTextInfo!.isReadOnly!) {
      if (_inputTextInfo!.tag!.name == TagNames.RPHYSICIAN) {
        _controller.text = FReportInfo.instance.reporter ?? '';
      }
    }

    final fontColor = inputText.fontColor;
    if (fontColor != null) {
      _fontColor = Color.fromARGB(
          fontColor.a!, fontColor.r!, fontColor.g!, fontColor.b!);
    }
    final backgroundColor = inputText.background;
    if (backgroundColor != null) {}
    var sufficIcon = _buildSufficIcon();
    _inputDecoration = DefaultInputDecoration2(
        contentPadding: EdgeInsets.all(_textWrap ? 8.s : 4.s));
    FReportInfo.instance.onReloadFinsh.addListener((sender, e) {
      _initDatas();
    });
    setState(() {});
  }

  void _syncWidth(InputText inputText, String value) {
    final fontSize = _textStyle!.fontSize ?? 18.s;
    final width = fontSize * value.length + 15.s;

    if (inputText.tag != null && inputText.tag!.name == TagNames.HOSPITALNAME) {
      if (_lineWidth < width) {
        setState(() {
          if (width < 480.s) {
            _lineWidth = width;
            _widthLock = true;
          }
        });
      } else if (_controller.text.isEmpty) {
        setState(() {
          //重置长度
          _lineWidth = PtToPxConverter.ptToPx(inputText.lineWidth);
          _widthLock = false;
        });
      } else if (TextSizeConverter.getTextSize(_controller.text, _textStyle!)
              .width >
          inputText.lineWidth!) {
        setState(() {
          if (width < 480.s) {
            _lineWidth = width;
            _widthLock = true;
          }
        });
      }
    }
  }

  void _initStyle() {
    final inputText = widget.inputText;
    _textStyle ??= TextStyle(
      fontSize: PtToPxConverter.ptToPx(inputText.fontSize),
      color: _fontColor,
    );
    _isReadOnly = inputText.isReadOnly;
    //TODO(Loki):set FontName in TextField
    final fontName = inputText.fontName;
    //TODO(Loki):常规模板暂未设置fontStyles，后续再支持
    final fontStyles = inputText.fontStyles;
    if (!_widthLock) {
      _lineWidth = PtToPxConverter.ptToPx(inputText.lineWidth);
    }
    _widthLock = false;
    _textWrap = inputText.textWrap ?? false;
    _rowCount = _textWrap ? 6 : 1;

    if (_inputTextInfo != null && _inputTextInfo!.text.isNotEmpty) {
      final text = _inputTextInfo!.text;
      _syncWidth(
        widget.inputText,
        text,
      );
    }
    _fontSize = PtToPxConverter.ptToPx(inputText.fontSize);
    _allHeight = _textWrap ? (10.s + _fontSize) * _rowCount : _fontSize;
    _allHeight += 10.s;
    if (_inputTextInfo != null && _textWrap) {
      _allHeight += 16.s;
    }
    _inputDecoration = DefaultInputDecoration2(
        contentPadding: EdgeInsets.all(_textWrap ? 8.s : 4.s));
  }

  void _checkInputTextInfo() {
    try {
      final inputTextInfo =
          FReportInfo.instance.getElementInfo(widget.inputText);
      if (_inputTextInfo != inputTextInfo && inputTextInfo != null) {
        if (_inputTextInfo != null) {
          _inputTextInfo!.onTextChange.removeListener(_onTextChangeCallback);
        }
        _inputTextInfo = inputTextInfo as InputTextInfo;
        _addListining();
      } else if (inputTextInfo != null) {
        _inputTextInfo!.onTextChange.removeListener(_onTextChangeCallback);
        _addListining();
      }
      if (!_inputTextInfo!.isListening!) {
        _initDatas();
        _inputTextInfo!.isListening = true;
      }
      if (_inputTextInfo!.text != _controller.text) {
        _controller.text = _inputTextInfo!.text;
      }
    } catch (e) {
      print(e);
    }
  }

  void _onTextChangeCallback(Object sender, String e) {
    setState(() {
      _controller.text = e;
      _syncWidth(
        widget.inputText,
        e,
      );
    });
  }

  void _addListining() {
    _inputTextInfo!.onTextChange.addListener(_onTextChangeCallback);
  }

  Widget? _buildSufficIcon() {
    if (_inputTextInfo == null || _inputTextInfo!.featureTag.isEmpty) {
      return null;
    }
    if (_inputTextInfo!.featureTag.contains(TagNames.SPEECHINPUT) &&
        FReportInfo.instance.isASROn) {
      return CustomIconButton(
        icon: const Icon(
          Icons.mic,
        ),
        iconSize: 30.s,
        padding: const EdgeInsets.all(0),
        constraints: const BoxConstraints(),
        onPressed: () {
          FReportInfo.instance.onSpeechInput.emit(this, _inputTextInfo!);
        },
      );
    }
    return null;
  }

  void _heightAdaptation() {
    var cellPostion = widget.cellPostion;
    if (cellPostion != null) {
      final rowHeightScale = _rowCount > 5 ? 1.5 : 1.0;
      //触发高度适配
      if (widget.parentTableId.isNotEmpty) {
        FReportInfo.instance.cellRowChange.emit(
          this,
          TableScaleInfo(
            rowHeightScale < 1 ? 1 : rowHeightScale,
            cellPostion,
            widget.parentTableId,
          ),
        );
      }
    }
  }
}

/// 自定义TextEditingController，会在输入法开始输入时触发监听
class CustomTextEditingController extends TextEditingController {
  final List<ValueChanged<bool>> _composeListeners = [];

  void addComposeListener(ValueChanged<bool> listener) {
    _composeListeners.add(listener);
  }

  void removeComposeListener(ValueChanged<bool> listener) {
    _composeListeners.remove(listener);
  }

  @override
  set value(TextEditingValue newValue) {
    final bool isComposingBefore = value.composing.isValid;
    final bool isComposingAfter = newValue.composing.isValid;

    if (isComposingBefore != isComposingAfter) {
      _notifyComposeListeners(isComposingAfter);
    }

    super.value = newValue;
  }

  void _notifyComposeListeners(bool isComposing) {
    for (final listener in _composeListeners) {
      listener(isComposing);
    }
  }
}
