import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/select/customdropdownbutton2.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/converters/pttopxconverter.dart';
import 'package:flyinsonolite/consultation/reports/widgets/foptionrow.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/multi_selected_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/multi_selected.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/rt_color.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';

class FMultiSelected extends StatefulWidget {
  final MultiSelected multiSelected;
  const FMultiSelected(this.multiSelected, {Key? key}) : super(key: key);
  @override
  State<StatefulWidget> createState() {
    return _RRMultiSelectedState();
  }
}

class _RRMultiSelectedState extends State<FMultiSelected> {
  _RRMultiSelectedState();
  final String _value = '';
  String? _selectedItemView = '';
  List<String> _items = [];
  List<String> _selectedItems = [];
  double _width = 0.0;
  double _height = 0.0;
  double _fontSize = 0;
  MulitiSelectedInfo? _mulitiSelectedInfo;
  final EdgeInsetsGeometry _padding = EdgeInsets.only(left: 4.s, right: 25.s);

  TextStyle _textStyle = TextStyle(fontSize: 14.s);
  @override
  initState() {
    _initDatas();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    final multiSelected = widget.multiSelected;
    final background = multiSelected.background ?? RTColor(255, 255, 255, 255);
    return Stack(
      children: [
        Container(
          decoration: BoxDecoration(
            border: Border.all(color: Colors.grey, width: 1.s),
            borderRadius: BorderRadius.circular(4.s),
            color: Color.fromARGB(
                background.a!, background.r!, background.g!, background.b!),
          ),
          padding: _padding,
          width: _width,
          height: _height,
          child: FISText(
            _selectedItemView!,
            maxLines: 1,
            style: _textStyle,
          ),
        ),
        _buildDropdownButton(),
      ],
    );
  }

  List<DropdownMenuItem<String>> buildItems() {
    return _items.map((String e) {
      final active = e == _value;
      return DropdownMenuItem<String>(
        value: e,
        child: FOptionRow(
          widget.multiSelected,
          e,
          isActive: active,
          height: _height,
          onTap: () {
            if (mounted) {
              setState(() {
                _onSelectedItem(e);
              });
            }
            if (_mulitiSelectedInfo != null) {
              _mulitiSelectedInfo!.selectedItems = _selectedItems;
            }
          },
          isSelected: _selectedItems.contains(e),
        ),
      );
    }).toList();
  }

  _onSelectedItem(String value) {
    if (mounted) {
      setState(() {
        if (_selectedItems.contains(value)) {
          _selectedItems.remove(value);
        } else {
          _selectedItems.add(value);
        }
        String text = '';
        _getSelectedItems();
      });
    }
  }

  Widget _buildDropdownButton() {
    if (_items.isEmpty) {
      return const SizedBox();
    }
    var buttonDecoration = BoxDecoration(
      border: Border.all(color: Colors.transparent, width: 1.s),
      borderRadius: BorderRadius.circular(4.s),
      color: Colors.transparent,
    );
    return SizedBox(
      height: _height,
      width: _width,
      child: DropdownButtonHideUnderline(
        child: DropdownButton2<String>(
          dropdownMaxHeight: 300.s,
          buttonWidth: _width,
          itemPadding: EdgeInsets.symmetric(horizontal: 0, vertical: 2.s),
          itemHeight: 28.s,
          iconSize: 18.s,
          isExpanded: false,
          dropdownOverButton: false,
          scrollbarAlwaysShow: true,
          scrollbarThickness: 4,
          scrollbarRadius: Radius.circular(2.s),
          //    buttonPadding: const EdgeInsets.symmetric(horizontal: 0, vertical: 0),
          dropdownDecoration: BoxDecoration(
            borderRadius: BorderRadius.circular(0),
          ),
          buttonDecoration: buttonDecoration,
          selectedItemBuilder: (_) => _selectedItems.map((e) {
            return SizedBox(
              width: _width - 34.s,
              child: FISText(
                e,
                maxLines: 1,
                style: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
                    color: Colors.black),
              ),
            );
          }).toList(),
          value: null,
          items: buildItems(),
          onChanged: (v) {},
        ),
      ),
    );
  }

  void _onSelectedChangeCallback(Object sender, List<String> e) {
    if (mounted) {
      setState(() {
        _selectedItems = e;
        _getSelectedItems();
      });
    }
  }

  void _initDatas() {
    final multiSelected = widget.multiSelected;
    final selectInfo = FReportInfo.instance.getElementInfo(multiSelected);
    if (selectInfo != null && _mulitiSelectedInfo != selectInfo) {
      if (_mulitiSelectedInfo != null) {
        _mulitiSelectedInfo!.onSelectedChange
            .removeListener(_onSelectedChangeCallback);
      }
      _mulitiSelectedInfo = selectInfo as MulitiSelectedInfo;
      _mulitiSelectedInfo!.onSelectedChange
          .removeListener(_onSelectedChangeCallback);
      _mulitiSelectedInfo!.onSelectedChange
          .addListener(_onSelectedChangeCallback);
    }
    if (multiSelected.items!.isNotEmpty) {
      _items = multiSelected.items!;
    }
    if (_mulitiSelectedInfo != null &&
        _mulitiSelectedInfo!.selectedItems != _selectedItems) {
      _selectedItems = _mulitiSelectedInfo!.selectedItems;
      _getSelectedItems();
    }
    _width = PtToPxConverter.ptToPx(multiSelected.lineWidth);
    final fontColor = multiSelected.fontColor ?? RTColor.Black;
    _fontSize = PtToPxConverter.ptToPx(multiSelected.fontSize);
    _height = _fontSize + 10.s;
    _textStyle = TextStyle(
      color: Color.fromARGB(
          fontColor.a!, fontColor.r!, fontColor.g!, fontColor.b!),
      fontSize: _fontSize > 18.s ? 18.s : _fontSize,
    );
  }

  void _getSelectedItems() {
    String text = '';
    for (var element in _selectedItems) {
      final index = _selectedItems.indexOf(element);
      if (index > 0) {
        text = '$text;$element';
      } else if (index == 0) {
        text = element;
      } else if (_selectedItems.isEmpty) {
        text = '';
      }
    }
    _selectedItemView = text;
  }
}
