import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/converters/alignmentconverter.dart';
import 'package:flyinsonolite/consultation/reports/widgets/datetime.dart';
import 'package:flyinsonolite/consultation/reports/widgets/input_image.dart';
import 'package:flyinsonolite/consultation/reports/widgets/input_text.dart';
import 'package:flyinsonolite/consultation/reports/widgets/line.dart';
import 'package:flyinsonolite/consultation/reports/widgets/multi_selected.dart';
import 'package:flyinsonolite/consultation/reports/widgets/multi_selected_vertical.dart';
import 'package:flyinsonolite/consultation/reports/widgets/page_number.dart';
import 'package:flyinsonolite/consultation/reports/widgets/single_select.dart';
import 'package:flyinsonolite/consultation/reports/widgets/space.dart';
import 'package:flyinsonolite/consultation/reports/widgets/static_image.dart';
import 'package:flyinsonolite/consultation/reports/widgets/static_text.dart';
import 'package:flyinsonolite/consultation/reports/widgets/testboxdecoration.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/cell_postion.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/date_time_element.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/element_type.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/input_image.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/input_text.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/interfaces/element.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/line.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/multi_selected.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/multi_selected_vertical.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/page_number.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/paragraph.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/rt_Cell.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/rt_table.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/single_selected.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/space.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/static_image.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/static_text.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';

class FParagraphPage extends StatefulWidget {
  final Paragraph paragraph;
  final CellPostion? cellPostion;
  final String parentTableId;
  const FParagraphPage(
      {Key? key,
      required this.paragraph,
      this.cellPostion,
      this.parentTableId = ""})
      : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _ParagraphState();
  }
}

class _ParagraphState extends State<FParagraphPage> {
  List<IElement> _elements = [];
  int _itemCount = 0;

  @override
  initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    _elements = widget.paragraph.elements ?? [];
    _itemCount = _elements.length;
    if (_itemCount == 0) {
      return Container(
        constraints: const BoxConstraints(minHeight: 0),
      );
    } else if (_itemCount == 2 &&
        _elements[0].elementType!.name == ElementType.staticText.name &&
        (_elements[1].elementType!.name == ElementType.singleSelected.name ||
            _elements[1].elementType!.name == ElementType.multiSelected.name ||
            _elements[1].elementType!.name == ElementType.dateTime.name ||
            _elements[1].elementType!.name == ElementType.inputText.name)) {
      try {
        final staticText = _elements[0];
        if (staticText.parent != null) {
          final paragraph = staticText.parent as Paragraph;
          if (paragraph.parent != null) {
            final cell = paragraph.parent as RTCell;
            if (cell.parent != null) {
              final table = cell.parent as RTTable;
              final values = table.cells!.values.toList();
              for (var i = 0; i < values.length; i++) {
                if (values[i] == cell) {
                  final key = table.cells!.keys.toList()[i];
                  final column = key.column;
                  final width = table.columnDefinitions![column!].width;
                  var valueWidth = 0.0;
                  if (_elements[1].elementType!.name ==
                      ElementType.inputText.name) {
                    final input = _elements[1] as InputText;
                    valueWidth = input.lineWidth ?? 0;
                  }
                  if (_elements[1].elementType!.name ==
                      ElementType.singleSelected.name) {
                    final input = _elements[1] as SingleSelected;
                    valueWidth = input.lineWidth ?? 0;
                    input.fontSize = 9;
                    if ((input.items?.any((element) => element.length > 7) ??
                            false) &&
                        i18nBook.isCurrentChinese) {
                      valueWidth += 13;
                      input.lineWidth = input.lineWidth! + 13;
                    }
                  }
                  if (_elements[1].elementType!.name ==
                      ElementType.multiSelected.name) {
                    final input = _elements[1] as MultiSelected;
                    valueWidth = input.lineWidth ?? 0;
                  }
                  if (_elements[1].elementType!.name ==
                      ElementType.dateTime.name) {
                    final dateTime = _elements[1] as DateTimeElement;
                    if (!dateTime.isReadOnly) {
                      valueWidth = dateTime.lineWidth ?? 0;
                    }
                  }
                  if (valueWidth > 0) {
                    final textElement = staticText as StaticText;
                    final textWidth = width! - valueWidth;
                    textElement.lineWidth = textWidth;
                  }
                }
              }
            }
          }
        }
      } catch (e) {
        print(e);
      }
    }
    final margin = widget.paragraph.margin!;
    return Container(
      margin: EdgeInsets.only(
        top: margin.top!.s,
        bottom: margin.bottom!.s,
        left: margin.left!.s,
        right: margin.right!.s,
      ),
      alignment: AlignmentConverter.horizontalConvert(
          widget.paragraph.horizontalAlignment),
      child: Wrap(
        alignment: WrapAlignment.start,
        direction: Axis.horizontal,
        crossAxisAlignment: WrapCrossAlignment.center,
        children: [
          ..._elements.map((element) => _buildElementWidget(element)),
        ],
      ),
    );
  }

  Widget _buildElementWidget(IElement element) {
    Widget? eleWidget;
    final typeName = element.elementType!.name;
    if (typeName == ElementType.inputText.name) {
      InputText inputText = element as InputText;
      eleWidget = FInputText(
        inputText: inputText,
        cellPostion: widget.cellPostion,
        parentTableId: widget.parentTableId,
      );
    } else if (typeName == ElementType.staticText.name) {
      StaticText staticText = element as StaticText;
      eleWidget = FStaticText(staticText);
    } else if (typeName == ElementType.singleSelected.name) {
      SingleSelected singleSelected = element as SingleSelected;
      eleWidget = FSingleSelected(singleSelected);
    } else if (typeName == ElementType.line.name) {
      Line line = element as Line;
      eleWidget = FLine(line);
    } else if (typeName == ElementType.dateTime.name) {
      final dateTime = element as DateTimeElement;
      eleWidget = FDateTime(dateTime);
    } else if (typeName == ElementType.multiSelected.name) {
      final multiSelected = element as MultiSelected;
      eleWidget = FMultiSelected(multiSelected);
    } else if (typeName == ElementType.multiSelectedVertical.name) {
      final multiSelected = element as MultiSelectedVertical;
      eleWidget = FMultiSelectedVertical(multiSelected);
    } else if (typeName == ElementType.inputImage.name) {
      final inputImage = element as InputImage;
      eleWidget = FInputImage(inputImage);
    } else if (typeName == ElementType.staticImage.name) {
      final staticImage = element as StaticImage;
      eleWidget = FStaticImage(staticImage);
    } else if (typeName == ElementType.space.name) {
      final space = element as Space;
      eleWidget = FSpace(space);
    } else if (typeName == ElementType.pageNumber.name) {
      final pageNumber = element as PageNumber;
      eleWidget = FPageNumber(pageNumber);
    }
    if (eleWidget == null) {
      return Container(
        height: 30.s,
        width: 180.s,
        decoration: TestBoxDecoration.buildDecoration(),
        child: FISText('Unknown component placeholder',
            style: Storage.currentTheme.normalTextStyle.copyWith(
                fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s,
                color: Colors.black)),
      );
    }
    return eleWidget;
  }
}
