import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/converters/pttopxconverter.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/report_info.dart';
import 'package:flyinsonolite/consultation/reports/reportinfo/single_selected_info.dart';
import 'package:flyinsonolite/consultation/reports/reporttemplate/single_selected.dart';
import 'package:flyinsonolite/controls/select/customdropdownbuttonformfield.dart'
    as custom_widget;

class FSingleSelected extends StatefulWidget {
  final SingleSelected singleSelected;
  const FSingleSelected(this.singleSelected, {Key? key}) : super(key: key);
  @override
  State<StatefulWidget> createState() {
    return _RSingleSelectState();
  }
}

class _RSingleSelectState extends State<FSingleSelected> {
  late SingleSelected singleSelected;
  _RSingleSelectState();
  String _value = '';
  List<String>? _items = [];
  SingleSelectedInfo? _singleSelectedInfo;
  late final double _width;
  double _height = 24.s;
  double _fontSize = 0;

  @override
  initState() {
    _width = PtToPxConverter.ptToPx(widget.singleSelected.lineWidth);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    singleSelected = widget.singleSelected;
    final selectInfo = FReportInfo.instance.getElementInfo(singleSelected);
    if (selectInfo != null && _singleSelectedInfo != selectInfo) {
      _singleSelectedInfo = selectInfo as SingleSelectedInfo;
      _singleSelectedInfo!.onSelectedChange.addListener((sender, e) {
        if (_singleSelectedInfo!.items!.contains(e)) {
          if (mounted) {
            setState(() {
              _value = _singleSelectedInfo!.selectedItem;
            });
          }
        }
      });
    }
    if (singleSelected.items!.isNotEmpty) {
      _items = singleSelected.items;
    }
    if (_items!.isEmpty) {
      return const SizedBox();
    }
    final background = singleSelected.background!;
    if (_singleSelectedInfo != null &&
        _singleSelectedInfo!.selectedItem.isNotEmpty &&
        _singleSelectedInfo!.selectedItem != _value) {
      _value = _singleSelectedInfo!.selectedItem;
    }
    _fontSize = PtToPxConverter.ptToPx(singleSelected.fontSize);
    _height = _fontSize + 10.s;
    return SizedBox(
      height: _height,
      child: ButtonTheme(
        /// 为了让下拉框的宽度和输入框一样
        alignedDropdown: true,
        child: DropdownButtonHideUnderline(
          child: custom_widget.DropdownButtonFormField<String>(
            itemHeight: _height,
            iconSize: 18.s,
            isExpanded: false,
            decoration: InputDecoration(
                suffixIconConstraints: BoxConstraints(
                    minWidth: 48.s,
                    minHeight: 48.s,
                    maxHeight: 48.s,
                    maxWidth: 48.s),
                contentPadding:
                    EdgeInsets.symmetric(horizontal: 0, vertical: 2.s),
                constraints: BoxConstraints(
                  maxWidth: _width,
                ),
                border: OutlineInputBorder(
                    borderSide: BorderSide(color: Colors.grey, width: 0.8.s),
                    borderRadius: BorderRadius.all(Radius.circular(4.s)),
                    gapPadding: 4.s),
                enabledBorder: OutlineInputBorder(
                    borderSide: BorderSide(color: Colors.grey, width: 0.8.s),
                    borderRadius: BorderRadius.all(Radius.circular(4.s)),
                    gapPadding: 4.s),
                focusedBorder: OutlineInputBorder(
                    borderSide: BorderSide(color: Colors.grey, width: 0.8.s),
                    borderRadius: BorderRadius.all(Radius.circular(4.s)),
                    gapPadding: 4.s),
                fillColor: Color.fromARGB(background.a!, background.r!,
                    background.g!, background.b!)),
            selectedItemBuilder: (_) => _items!
                .map((e) => _OptionRow(
                      singleSelected,
                      e,
                      height: _height,
                      maxWidth: _width,
                    ))
                .toList(),
            value: _value.isEmpty ? null : _value,
            items: buildItems(),
            onChanged: (v) {
              setState(() {
                if (_singleSelectedInfo != null) {
                  _singleSelectedInfo!.selectedItem = v!;
                }
                _value = v!;
              });
            },
          ),
        ),
      ),
    );
  }

  List<custom_widget.DropdownMenuItem<String>>? buildItems() {
    return _items!.map((String e) {
      final active = e == _value;
      return custom_widget.DropdownMenuItem<String>(
        value: e,
        child: _OptionRow(
          singleSelected,
          e,
          isActive: active,
          height: _height,
        ),
      );
    }).toList();
  }
}

class _OptionRow extends StatelessWidget {
  const _OptionRow(
    this.singleSelected,
    this.label, {
    Key? key,
    this.isActive = false,
    this.height,
    this.maxWidth,
  }) : super(key: key);

  final SingleSelected singleSelected;
  final String label;
  final bool isActive;
  final double? height;
  final double? maxWidth;
  @override
  Widget build(BuildContext context) {
    final color = singleSelected.fontColor!;

    final text = FISText(
      label,
      overflow: TextOverflow.ellipsis,
      style: TextStyle(
        fontSize: PtToPxConverter.ptToPx(singleSelected.fontSize),
        color: Color.fromARGB(color.a!, color.r!, color.g!, color.b!),
      ),
    );
    var defaultPadding = EdgeInsets.only(left: 4.s);
    if (height != null) {
      return Container(
        alignment: Alignment.centerLeft,
        padding: defaultPadding,
        height: height,
        constraints: BoxConstraints(
          maxWidth: maxWidth == null ? double.infinity : maxWidth! - 26.s,
        ),
        color: isActive ? Storage.currentTheme.themeData.indicatorColor : null,
        child: text,
      );
    } else {
      return Container(
        padding: defaultPadding,
        child: text,
      );
    }
  }
}
