import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/records/controllers/consultationdetailcontroller.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class ConsultationVisitView extends GetView<ConsultationDetailController> {
  final scrollController = ScrollController();
  RxInt currentPageIndex = RxInt(1);
  RxInt totalPageCount = RxInt(1);

  bool get _isFirstPage => currentPageIndex.value == 1;

  bool get _isLastPage => currentPageIndex.value == totalPageCount.value;

  ConsultationVisitView({super.key});

  @override
  Widget build(BuildContext context) {
    return Obx(() => controller.followUpVisits.isEmpty
        ? _mockEmptyView()
        : _buildFollowUpVisitsView());
  }

  ///创建底部widget
  Widget _buildBottomWidget() {
    return Stack(
      children: [
        FISText("${currentPageIndex.value} / ${totalPageCount.value}"),
        Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            FISLinearGradientElevatedButton(
                _isFirstPage ? null : _previous,
                i18nBook.realTimeConsultation.previous.t,
                Storage.currentTheme.dialogStyle.buttonGradient,
                textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize:
                        Storage.currentTheme.normalTextStyle.fontSize!.s)),
            SizedBox(
              width: 15.s,
            ),
            FISLinearGradientElevatedButton(
                _isLastPage ? null : _next,
                i18nBook.realTimeConsultation.next.t,
                Storage.currentTheme.dialogStyle.buttonGradient,
                textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize:
                        Storage.currentTheme.normalTextStyle.fontSize!.s)),
          ],
        ),
      ],
    );
  }

  Widget _buildCommonWidget(String title, String contentText) {
    return Flex(
      direction: Axis.horizontal,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Expanded(
          flex: 1,
          child: FISText(
            title,
            overflow: TextOverflow.ellipsis,
          ),
        ),
        Expanded(
          flex: 3,
          child: FISText(
            maxLines: 15,
            contentText,
            overflow: TextOverflow.ellipsis,
          ),
        ),
      ],
    );
  }

  ///分隔线
  Widget _buildLine() {
    return Container(
      margin: EdgeInsets.only(bottom: 10.s),
      color: const Color.fromRGBO(216, 216, 216, 1),
      height: 0.5.s,
    );
  }

  ///行与行之间的间隔
  Widget _buildSpaceWidget() {
    return Container(
      height: 20.s,
    );
  }

  //点击下一页
  void _next() {
    if (currentPageIndex.value < totalPageCount.value) {
      currentPageIndex.value++;
    }
  }

  ///点击上一页
  void _previous() {
    if (currentPageIndex.value > 1) {
      currentPageIndex.value--;
    }
  }

  ///缺省显示
  Widget _mockEmptyView() {
    return Container(
      margin: EdgeInsets.only(top: 40.s),
      child: Column(
        children: [
          Center(
            child: Image.asset(
              "assets/images/no_data.png",
              width: 300.s,
              height: 300.s,
              fit: BoxFit.cover,
            ),
          ),
          FISText(i18nBook.common.noneDataTip.t),
        ],
      ),
    );
  }

  Widget _buildFollowUpVisitsView() {
    var occurredTime =
        controller.followUpVisits[currentPageIndex.value - 1].occurredTime ??
            DateTime.now();
    totalPageCount.value = controller.followUpVisits.length;
    return Expanded(
        child: SingleChildScrollView(
            child: Container(
                color: Storage.currentTheme.listViewStyle.itembgColor,
                padding: EdgeInsets.fromLTRB(15.s, 10.s, 15.s, 10.s),
                child: Obx(
                  () => Column(
                    children: [
                      _buildCommonWidget(
                          i18nBook.common.fullName.t,
                          controller.followUpVisits[currentPageIndex.value - 1]
                                  .patientName ??
                              ""),
                      _buildSpaceWidget(),
                      _buildCommonWidget(
                          i18nBook.common.mobileNumber.t,
                          controller.followUpVisits[currentPageIndex.value - 1]
                                  .patientPhone ??
                              ""),
                      _buildSpaceWidget(),
                      _buildCommonWidget(
                          i18nBook.realTimeConsultation.followUpPerson.t,
                          controller.followUpVisits[currentPageIndex.value - 1]
                                  .doctorName ??
                              ""),
                      _buildSpaceWidget(),
                      _buildCommonWidget(
                          i18nBook.realTimeConsultation.followUpTime.t,
                          occurredTime.toLocal().toString().substring(0, 16)),
                      _buildSpaceWidget(),
                      _buildCommonWidget(
                          i18nBook.realTimeConsultation.followUpRecord.t,
                          controller.followUpVisits[currentPageIndex.value - 1]
                                  .generalCase ??
                              ""),
                      _buildSpaceWidget(),
                      _buildLine(),
                      _buildBottomWidget(),
                    ],
                  ),
                ))));
  }
}
