import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/custom/customiconbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';

class FISIconButton extends StatelessWidget {
  final VoidCallback? onPressed;
  final Widget icon;
  final EdgeInsetsGeometry? padding;
  final double? iconSize;
  final AlignmentGeometry? alignment;
  final String? label;
  final Color? color;
  bool _isBusy = false;
  FISIconButton(
      {super.key,
      this.onPressed,
      required this.icon,
      this.label,
      this.padding,
      this.iconSize,
      this.alignment,
      this.color});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        CustomIconButton(
            onPressed: () {
              if (_isBusy) {
                return;
              }
              try {
                _isBusy = true;
                if (onPressed != null) {
                  onPressed!();
                }
              } finally {
                _isBusy = false;
              }
            },
            icon: icon,
            padding: padding ?? const EdgeInsets.all(0),
            iconSize: iconSize ?? 36.s,
            alignment: Alignment.center,
            color: color ?? Storage.currentTheme.iconButtonIconColor),
        label == null
            ? const SizedBox()
            : InkWell(
                onTap: onPressed,
                child: FISText(
                  label!,
                  style: Storage.currentTheme.normalTextStyle
                      .copyWith(fontSize: 17.s),
                ),
              )
      ],
    );
  }
}

class FISIconButtonAsync extends StatelessWidget {
  final AsyncVoidCallback? onPressed;
  final Widget icon;
  final EdgeInsetsGeometry? padding;
  final double? iconSize;
  final AlignmentGeometry? alignment;
  final String? label;
  final Color? color;
  bool _isBusy = false;
  FISIconButtonAsync(
      {super.key,
      this.onPressed,
      required this.icon,
      this.label,
      this.padding,
      this.iconSize,
      this.alignment,
      this.color});

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        CustomIconButton(
            onPressed: () async {
              if (_isBusy) {
                return;
              }
              try {
                _isBusy = true;
                if (onPressed != null) {
                  await onPressed!();
                }
              } finally {
                _isBusy = false;
              }
            },
            icon: icon,
            padding: padding ?? const EdgeInsets.all(0),
            iconSize: iconSize ?? 36.s,
            alignment: Alignment.center,
            color: color ?? Storage.currentTheme.iconButtonIconColor),
        label == null
            ? const SizedBox()
            : InkWell(
                onTap: onPressed,
                child: FISText(
                  label!,
                  style: Storage.currentTheme.normalTextStyle
                      .copyWith(fontSize: 17.s),
                ),
              )
      ],
    );
  }
}
