import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/custom/customelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/utils.dart';

class FISLinearGradientElevatedButton extends StatelessWidget {
  double? width;
  double? height;
  bool _isBusy = false;
  final VoidCallback? onPressed;
  final Gradient? gradient;
  String text;
  EdgeInsetsGeometry? padding;
  TextStyle? textStyle;
  FISLinearGradientElevatedButton(this.onPressed, this.text, this.gradient,
      {super.key, this.width, this.height, this.padding, this.textStyle});

  @override
  Widget build(BuildContext context) {
    return Container(
        width: width,
        height: height ?? 40.s,
        decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4.s), gradient: gradient),
        child: CustomElevatedButton(
          onPressed: onPressed == null
              ? null
              : () {
                  if (_isBusy) {
                    return;
                  }
                  try {
                    _isBusy = true;
                    if (onPressed != null) {
                      onPressed!();
                    }
                  } finally {
                    _isBusy = false;
                  }
                },
          style: CustomElevatedButton.styleFrom(
            elevation: 0,
            padding: padding ??
                (kIsMobile
                    ? EdgeInsets.symmetric(vertical: 6.s, horizontal: 20.s)
                    : EdgeInsets.symmetric(vertical: 16.s, horizontal: 20.s)),
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(4.s),
            ),
            backgroundColor: Colors.transparent,
            disabledBackgroundColor: Colors.grey,
          ),
          child: FISText(
            text,
            style: textStyle ??
                Storage.currentTheme.normalTextStyle.copyWith(
                    color: Storage.currentTheme
                        .elevatedButtonLinearGradientStyle.textColor,
                    fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s),
          ),
        ));
  }
}

typedef AsyncVoidCallback = Future Function();

class FISLinearGradientElevatedButtonAsync extends StatelessWidget {
  double? width;
  double? height;
  bool _isBusy = false;
  final AsyncVoidCallback? onPressed;
  final Gradient? gradient;
  String text;
  EdgeInsetsGeometry? padding;
  TextStyle? textStyle;
  FISLinearGradientElevatedButtonAsync(this.onPressed, this.text, this.gradient,
      {super.key, this.width, this.height, this.padding, this.textStyle});

  @override
  Widget build(BuildContext context) {
    return Container(
        width: width,
        height: height ?? 40.s,
        decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(4.s), gradient: gradient),
        child: CustomElevatedButton(
          onPressed: onPressed == null
              ? null
              : () async {
                  if (_isBusy) {
                    return;
                  }
                  try {
                    _isBusy = true;
                    if (onPressed != null) {
                      await onPressed!();
                    }
                  } finally {
                    _isBusy = false;
                  }
                },
          style: CustomElevatedButton.styleFrom(
            elevation: 0,
            padding: padding ??
                (kIsMobile
                    ? EdgeInsets.symmetric(vertical: 6.s, horizontal: 20.s)
                    : EdgeInsets.symmetric(vertical: 16.s, horizontal: 20.s)),
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(4.s),
            ),
            backgroundColor: Colors.transparent,
            disabledBackgroundColor: Colors.grey,
          ),
          child: FISText(
            text,
            style: textStyle ??
                Storage.currentTheme.normalTextStyle.copyWith(
                    color: Storage.currentTheme
                        .elevatedButtonLinearGradientStyle.textColor,
                    fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s),
          ),
        ));
  }
}
