import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class FISSquareIconButton extends StatelessWidget {
  final _pressed = false.obs;
  bool _isBusy = false;
  final VoidCallback? onPressed;
  final double width;
  final double height;
  final IconData iconData;
  final double size;
  final String text;
  FISSquareIconButton(this.onPressed, this.iconData, this.text,
      {super.key, this.width = 100, this.height = 100, this.size = 40});
  @override
  Widget build(BuildContext context) {
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      child: GestureDetector(
        onTap: () {
          if (_isBusy) {
            return;
          }
          try {
            _isBusy = true;
            _pressed.value = true;
            Future.delayed(const Duration(seconds: 1), () {
              _pressed.value = false;
            });
            if (onPressed != null) {
              onPressed!();
            }
          } finally {
            _isBusy = false;
          }
        },
        child: Obx(() => Container(
            width: width.s,
            height: height.s,
            decoration: BoxDecoration(
                border: Border.all(
                    color: Storage.currentTheme.loginContainStyle.borderColor,
                    width: 1.s),
                gradient: _pressed.value
                    ? Storage.currentTheme.squareIconButtonStyle
                        .focusedLinearGradient
                    : Storage
                        .currentTheme.squareIconButtonStyle.backgroundGradient,
                borderRadius: BorderRadius.all(Radius.circular(10.s))),
            alignment: Alignment.center,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Icon(
                  iconData,
                  size: size.s,
                  color: Storage.currentTheme.squareIconButtonStyle.iconColor,
                ),
                SizedBox(
                  height: 5.s,
                ),
                FISText(text)
              ],
            ))),
      ),
    );
  }
}

class FISSquareIconButtonAsync extends StatelessWidget {
  final _pressed = false.obs;
  bool _isBusy = false;
  final AsyncVoidCallback? onPressed;
  final double width;
  final double height;
  final IconData iconData;
  final double size;
  final String text;
  FISSquareIconButtonAsync(this.onPressed, this.iconData, this.text,
      {super.key, this.width = 100, this.height = 100, this.size = 40});
  @override
  Widget build(BuildContext context) {
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      child: GestureDetector(
        onTap: () async {
          if (_isBusy) {
            return;
          }
          try {
            _isBusy = true;
            _pressed.value = true;
            Future.delayed(const Duration(milliseconds: 500), () {
              _pressed.value = false;
            });
            if (onPressed != null) {
              await onPressed!();
            }
          } finally {
            _isBusy = false;
          }
        },
        child: Obx(() => Container(
            width: width.s,
            height: height.s,
            decoration: BoxDecoration(
                border: Border.all(
                    color: Storage.currentTheme.loginContainStyle.borderColor,
                    width: 1.s),
                gradient: _pressed.value
                    ? Storage.currentTheme.squareIconButtonStyle
                        .focusedLinearGradient
                    : Storage
                        .currentTheme.squareIconButtonStyle.backgroundGradient,
                borderRadius: BorderRadius.all(Radius.circular(10.s))),
            alignment: Alignment.center,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Icon(
                  iconData,
                  size: size.s,
                  color: Storage.currentTheme.squareIconButtonStyle.iconColor,
                ),
                SizedBox(
                  height: 5.s,
                ),
                FISText(text)
              ],
            ))),
      ),
    );
  }
}
