import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/dialog/fisdialog.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/index.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class ExitOrHideConfirmDialog extends GetView<ExitOrHideConfirmController> {
  const ExitOrHideConfirmDialog({super.key});

  @override
  Widget build(BuildContext context) {
    Get.put<ExitOrHideConfirmController>(ExitOrHideConfirmController());
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      return FISDialog(
          i18nBook.common.tip.t, _createContentView(), false, false,
          minHeight: 150.s);
    });
  }

  Widget _createContentView() {
    return Column(
      children: [
        Container(
            constraints: const BoxConstraints(maxWidth: 300),
            child: FISText(
              fisI18nBook.common.exitOrHideTips.t,
              style: Storage.currentTheme.dialogStyle.messageTextStyle.copyWith(
                  fontSize: Storage
                      .currentTheme.dialogStyle.messageTextStyle.fontSize!.s),
              softWrap: true,
            )),
        SizedBox(
          height: 50.s,
        ),
        Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            FISLinearGradientElevatedButtonAsync(() async {
              Get.close(1);
              await controller.closeFlyinsonoPage();
              await Storage.user.handleLogOutAsync();
            }, i18nBook.common.close.t,
                Storage.currentTheme.dialogStyle.buttonGradient,
                textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize:
                        Storage.currentTheme.normalTextStyle.fontSize!.s)),
            SizedBox(
              width: 50.s,
            ),
            FISLinearGradientElevatedButtonAsync(() async {
              Get.close(1);
              await controller.hideFlyinsonoPage();
            }, fisI18nBook.common.minimize.t,
                Storage.currentTheme.dialogStyle.buttonGradient,
                textStyle: Storage.currentTheme.normalTextStyle.copyWith(
                    fontSize: Storage.currentTheme.normalTextStyle.fontSize!.s))
          ],
        )
      ],
    );
  }
}

class ExitOrHideConfirmController extends GetxController {
  Future hideFlyinsonoPage() async {
    try {
      await jsonRpcProxyForFISLib.connection.hideFlyinsonoPageForFlutter();
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "Hide FlyinsonoPage Error$ex");
    }
  }

  Future closeFlyinsonoPage() async {
    try {
      await jsonRpcProxyForFISLib.connection.closeFlyinsonoPageForFlutter();
    } catch (ex) {
      await loggerAsyn(
          FISDeviceLogCategory.Error, "Close FlyinsonoPage Error$ex");
    }
  }
}
