import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/controls/custom/customdialog.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class FISDialog extends StatelessWidget {
  final String title;
  final Widget contentChild;
  final double? minWidth;
  final double? elevation;
  final EdgeInsets? insetPadding;
  final Clip clipBehavior;
  final bool showConfirmButton;
  final bool showCancelButton;
  final bool showCloseButton;
  final VoidCallback? onConfirm;
  final EdgeInsets? contentPadding;
  final AlignmentGeometry? alignment;
  final Color? bgColor;
  final bool hasScrollView;
  final double minHeight;
  final EdgeInsets _defaultInsetPadding = EdgeInsets.all(12.s);

  FISDialog(this.title, this.contentChild, this.showConfirmButton,
      this.showCancelButton,
      {super.key,
      this.showCloseButton = true,
      this.minWidth,
      this.onConfirm,
      this.elevation,
      this.contentPadding,
      this.insetPadding,
      this.clipBehavior = Clip.none,
      this.alignment = Alignment.center,
      this.bgColor,
      this.hasScrollView = true,
      this.minHeight = 0});

  @override
  Widget build(BuildContext context) {
    return hasScrollView
        ? Row(children: [
            Expanded(child: SingleChildScrollView(child: _dialogChild()))
          ])
        : _dialogChild();
  }

  Widget _dialogChild() {
    return CustomDialog(
      elevation: elevation,
      alignment: alignment,
      insetPadding: insetPadding ?? _defaultInsetPadding,
      clipBehavior: clipBehavior,
      shape: RoundedRectangleBorder(borderRadius: BorderRadius.circular(5.s)),
      child: IntrinsicWidth(
          child: ConstrainedBox(
              constraints: BoxConstraints(minWidth: minWidth ?? 280.s),
              child: Column(
                mainAxisSize: MainAxisSize.min,
                crossAxisAlignment: CrossAxisAlignment.stretch,
                children: [
                  ////Title
                  Container(
                    padding: EdgeInsets.all(5.s),
                    decoration: BoxDecoration(
                      color: Storage.currentTheme.dialogStyle.headerColor,
                      borderRadius: BorderRadius.only(
                          topLeft: Radius.circular(5.s),
                          topRight: Radius.circular(5.s)),
                    ),
                    child: Row(
                      children: [
                        SizedBox(
                          width: 5.s,
                        ),
                        Expanded(
                            child: FISText(
                          title,
                          style: Storage.currentTheme.dialogStyle.titleTextStyle
                              .copyWith(
                                  fontSize: Storage.currentTheme.dialogStyle
                                      .titleTextStyle.fontSize!.s),
                        )),
                        showCloseButton
                            ? FISIconButton(
                                onPressed: () {
                                  Get.close(1);
                                },
                                icon: const Icon(
                                  Icons.close,
                                ),
                                iconSize: 52.s,
                              )
                            : const SizedBox()
                      ],
                    ),
                  ),

                  ///content
                  Container(
                      decoration: BoxDecoration(
                          borderRadius: BorderRadius.only(
                            bottomLeft: Radius.circular(5.s),
                            bottomRight: Radius.circular(5.s),
                          ),
                          gradient: bgColor == null
                              ? Storage
                                  .currentTheme.dialogStyle.backgroundGradient
                              : const LinearGradient(
                                  begin: Alignment.centerLeft,
                                  end: Alignment.centerRight,
                                  colors: [Colors.white, Colors.white],
                                  stops: [0, 1])),
                      padding: contentPadding ?? EdgeInsets.all(5.s),
                      child: Column(
                        children: [
                          Container(
                            alignment: Alignment.center,
                            constraints: BoxConstraints(
                                minWidth: 200.s, minHeight: minHeight),
                            child: contentChild,
                          ),
                          showConfirmButton || showCancelButton
                              ? SizedBox(
                                  height: 10.s,
                                )
                              : const SizedBox(),
                          showConfirmButton || showCancelButton
                              ? Row(
                                  mainAxisAlignment: MainAxisAlignment.end,
                                  children: [
                                    showConfirmButton
                                        ? FISLinearGradientElevatedButton(() {
                                            onConfirm!();
                                          },
                                            i18nBook.common.confirm.t,
                                            Storage.currentTheme.dialogStyle
                                                .buttonGradient,
                                            textStyle: Storage
                                                .currentTheme.normalTextStyle
                                                .copyWith(
                                                    fontSize: Storage
                                                        .currentTheme
                                                        .normalTextStyle
                                                        .fontSize!
                                                        .s))
                                        : const SizedBox(),
                                    showConfirmButton
                                        ? SizedBox(
                                            width: 10.s,
                                          )
                                        : const SizedBox(),
                                    showCancelButton
                                        ? FISLinearGradientElevatedButton(() {
                                            Get.close(1);
                                          },
                                            i18nBook.common.cancel.t,
                                            Storage.currentTheme.dialogStyle
                                                .buttonGradient,
                                            textStyle: Storage
                                                .currentTheme.normalTextStyle
                                                .copyWith(
                                                    fontSize: Storage
                                                        .currentTheme
                                                        .normalTextStyle
                                                        .fontSize!
                                                        .s))
                                        : const SizedBox(),
                                  ],
                                )
                              : const SizedBox()
                        ],
                      )),
                ],
              ))),
    );

    ///bottom buttons
  }
}
