import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flyinsonolite/consultation/records/controllers/applyconsultationcontroller.dart';
import 'package:flyinsonolite/consultation/emergency/controllers/emergencypreparednesscontroller.dart';
import 'package:flyinsonolite/controls/custom/customdropdownbutton.dart';
import 'package:flyinsonolite/controls/select/fisdropdownbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/fistextfield.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/managers/interfaces/ipatientController.dart';
import 'package:flyinsonolite/consultation/records/models/fmtedduration.dart';
import 'package:flyinsonolite/consultation/records/models/patientinfo.dart';
import 'package:flyinsonolite/consultation/records/models/selectmodel.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class FBuildAgeType extends StatefulWidget {
  const FBuildAgeType(
    this.isEmergencyMode, {
    Key? key,
  }) : super(key: key);

  final bool isEmergencyMode;
  @override
  State<FBuildAgeType> createState() => _BuildAgeTypeState();
}

class _BuildAgeTypeState extends State<FBuildAgeType> {
  late IPatientController controller = (widget.isEmergencyMode
      ? Get.find<EmergencyPreparednessController>()
      : Get.find<ApplyConsultationController>()) as IPatientController;

  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    Widget child = Obx(() => Storage.isPortraitMode.value
        ? SizedBox(
            height: 96.s,
            child: Column(
              children: [
                Expanded(
                  child: Obx(
                    () => _buildAgeType(controller.selectedAgeUnit.value?.key),
                  ),
                ),
                Expanded(
                  child: _buildAgeTypeSelect(),
                )
              ],
            ))
        : SizedBox(
            height: 48.s,
            child: Row(
              children: [
                Expanded(
                  flex: 2,
                  child: Obx(
                    () => _buildAgeType(controller.selectedAgeUnit.value?.key),
                  ),
                ),
                Expanded(
                  child: _buildAgeTypeSelect(),
                )
              ],
            )));

    return child;
  }

  Widget _buildSizedBox() {
    return SizedBox(
      width: 10.s,
    );
  }

  Widget _buildAgeType(String? currentAgeType) {
    switch (currentAgeType ?? 'Year') {
      case 'YearAndMonth':
        return _buildYearAndMonth();
      case 'MonthAndWeek':
        return _buildMonthAndWeek();
      case 'WeekAndDay':
        return _buildWeekAndDay();
      default:
        return _buildYear();
    }
  }

  Widget _buildYear() {
    return Row(
      children: [
        _buildAgeTypeInput(
          (value) {
            controller.currentPatient.ageYear = value;
            controller.patient['AgeYear'] = value;
            controller.currentPatient.birthday =
                FmtedDuration.countDurYearAndMonthBirthDay(value);
          },
          "AgeYear",
          [FilteringTextInputFormatter.digitsOnly],
        ),
        _buildSizedBox(),
        FISText(i18nBook.remedical.years.t),
        _buildSizedBox(),
      ],
    );
  }

  Widget _buildYearAndMonth() {
    return Row(
      children: [
        _buildAgeTypeInput(
          (value) {
            controller.currentPatient.ageYear = value;
            controller.patient['AgeYear'] = value;
            controller.currentPatient.birthday =
                FmtedDuration.countDurYearAndMonthBirthDay(
                    value, controller.currentPatient.ageMonth ?? '');
          },
          "AgeYear",
          [FilteringTextInputFormatter.digitsOnly],
        ),
        _buildSizedBox(),
        FISText(i18nBook.remedical.years.t),
        _buildSizedBox(),
        _buildAgeTypeInput(
          (value) {
            controller.currentPatient.ageMonth = value;
            controller.patient['AgeMonth'] = value;
            controller.currentPatient.birthday =
                FmtedDuration.countDurYearAndMonthBirthDay(
                    controller.currentPatient.ageYear ?? '', value);
          },
          "AgeMonth",
          [
            FilteringTextInputFormatter.allow(
              RegExp(r'^(0?[1-9]|1[0-1])$'),
            ),
          ],
        ),
        _buildSizedBox(),
        FISText(i18nBook.remedical.month.t),
        _buildSizedBox(),
      ],
    );
  }

  Widget _buildMonthAndWeek() {
    return Row(
      children: [
        _buildAgeTypeInput(
          (value) {
            controller.currentPatient.ageMonth = value;
            controller.patient['AgeMonth'] = value;
            controller.currentPatient.birthday =
                FmtedDuration.countDurMonthAndWeekBirthDay(
                    value, controller.currentPatient.ageWeek ?? '');
          },
          "AgeMonth",
          [
            FilteringTextInputFormatter.allow(
              RegExp(r'^(0?[1-9]|1[0-1])$'),
            ),
          ],
        ),
        _buildSizedBox(),
        FISText(i18nBook.remedical.month.t),
        _buildSizedBox(),
        _buildAgeTypeInput(
          (value) {
            controller.currentPatient.ageWeek = value;
            controller.patient['AgeWeek'] = value;
            controller.currentPatient.birthday =
                FmtedDuration.countDurMonthAndWeekBirthDay(
                    controller.currentPatient.ageMonth ?? '', value);
          },
          "AgeWeek",
          [
            FilteringTextInputFormatter.allow(
              RegExp(r'^([0-4])$'),
            ),
          ],
        ),
        _buildSizedBox(),
        FISText(i18nBook.remedical.week.t),
        _buildSizedBox(),
      ],
    );
  }

  Widget _buildWeekAndDay() {
    return Row(
      children: [
        _buildAgeTypeInput(
          (value) {
            controller.currentPatient.ageWeek = value;
            controller.patient['AgeWeek'] = value;
            controller.currentPatient.birthday =
                FmtedDuration.countDurWeekAndDayBirthDay(
                    value, controller.currentPatient.ageDay ?? '');
          },
          "AgeWeek",
          [FilteringTextInputFormatter.digitsOnly],
        ),
        _buildSizedBox(),
        FISText(i18nBook.remedical.week.t),
        _buildSizedBox(),
        _buildAgeTypeInput(
          (value) {
            controller.currentPatient.ageDay = value;
            controller.patient['AgeDay'] = value;
            controller.currentPatient.birthday =
                FmtedDuration.countDurWeekAndDayBirthDay(
                    controller.currentPatient.ageWeek ?? '', value);
          },
          "AgeDay",
          [
            FilteringTextInputFormatter.allow(
              RegExp(r'^([1-9]|[1-2]\d|3[0-1])$'),
            ),
          ],
        ),
        _buildSizedBox(),
        FISText(i18nBook.remedical.day.t),
        _buildSizedBox(),
      ],
    );
  }

  Widget _buildAgeTypeSelect() {
    void onSelectChanged(String? value) {
      PatientInfo currentPatient = controller.currentPatient;
      Map<String, String> patient = controller.patient;
      currentPatient.ageUnits = value;
      bool hasBirthDay = controller.currentPatient.birthday != null &&
          (controller.currentPatient.birthday?.isNotEmpty ?? false);
      switch (value ?? 'Year') {
        case 'YearAndMonth':
          if (hasBirthDay) {
            var year = FmtedDuration.countDurYearAndMonth(
              DateTime.parse(
                currentPatient.birthday!,
              ),
            )[0]
                .toString();
            var month = FmtedDuration.countDurYearAndMonth(
              DateTime.parse(
                currentPatient.birthday!,
              ),
            )[1]
                .toString();
            currentPatient.ageYear = year;
            currentPatient.ageMonth = month;
            patient["AgeYear"] = year;
            patient["AgeMonth"] = month;
          } else {
            currentPatient.ageYear = null;
            currentPatient.ageMonth = null;
            patient.removeWhere((key, value) => key == "AgeYear");
            patient.removeWhere((key, value) => key == "AgeMonth");
          }
          currentPatient.ageWeek = null;
          currentPatient.ageDay = null;
          patient.removeWhere((key, value) => key == "AgeWeek");
          patient.removeWhere((key, value) => key == "AgeDay");

          break;
        case 'MonthAndWeek':
          if (hasBirthDay) {
            var month = FmtedDuration.countDurMonthAndWeek(
              DateTime.parse(
                currentPatient.birthday!,
              ),
            )[0]
                .toString();
            var week = FmtedDuration.countDurMonthAndWeek(
              DateTime.parse(
                currentPatient.birthday!,
              ),
            )[1]
                .toString();

            currentPatient.ageMonth = month;
            currentPatient.ageWeek = week;
            patient["AgeMonth"] = month;
            patient["AgeWeek"] = week;
          } else {
            currentPatient.ageMonth = null;
            currentPatient.ageWeek = null;
            patient.removeWhere((key, value) => key == "AgeMonth");
            patient.removeWhere((key, value) => key == "AgeWeek");
          }
          currentPatient.ageYear = null;
          currentPatient.ageDay = null;
          patient.removeWhere((key, value) => key == "AgeYear");
          patient.removeWhere((key, value) => key == "AgeDay");
          break;
        case 'WeekAndDay':
          if (hasBirthDay) {
            var week = FmtedDuration.countDurWeekAndDay(
              DateTime.parse(
                currentPatient.birthday!,
              ),
            )[0]
                .toString();
            var day = FmtedDuration.countDurWeekAndDay(
              DateTime.parse(
                currentPatient.birthday!,
              ),
            )[1]
                .toString();

            currentPatient.ageWeek = week;
            currentPatient.ageDay = day;
            patient["AgeWeek"] = week;
            patient["AgeDay"] = day;
          } else {
            currentPatient.ageWeek = null;
            currentPatient.ageDay = null;
            patient.removeWhere((key, value) => key == "AgeWeek");
            patient.removeWhere((key, value) => key == "AgeDay");
          }
          currentPatient.ageYear = null;
          currentPatient.ageMonth = null;
          patient.removeWhere((key, value) => key == "AgeYear");
          patient.removeWhere((key, value) => key == "AgeMonth");
          break;
        default:
          if (hasBirthDay) {
            var year = FmtedDuration.countDurYearAndMonth(
              DateTime.parse(
                currentPatient.birthday!,
              ),
            )[0]
                .toString();
            currentPatient.ageYear = year;
            patient["AgeYear"] = year;
          } else {
            currentPatient.ageYear = null;
            patient.removeWhere((key, value) => key == "AgeYear");
          }
          currentPatient.ageMonth = null;
          currentPatient.ageWeek = null;
          currentPatient.ageDay = null;
          patient.removeWhere((key, value) => key == "AgeMonth");
          patient.removeWhere((key, value) => key == "AgeWeek");
          patient.removeWhere((key, value) => key == "AgeDay");
          break;
      }
      setState(() {});
    }

    final select = Obx(() {
      return FISDropdownButton(
        value: controller.selectedAgeUnit.value,
        items: controller.ageUnitList.value
            .map<CustomDropdownMenuItem<FISSelectModel>>(
                (FISSelectModel selectModel) {
          return CustomDropdownMenuItem<FISSelectModel>(
            value: selectModel,
            child: FISText(selectModel.value),
          );
        }).toList(),
        onChanged: (value) {
          controller.selectedAgeUnit.value = value;
          controller.patient["AgeUnits"] = (value as FISSelectModel).key;
          onSelectChanged(value.key);
        },
      );
    });
    return Obx(() {
      if (!controller.isEdit.value) {
        return select;
      } else {
        return const SizedBox();
      }
    });
  }

  Widget _buildAgeTypeInput(Function(dynamic)? onChanged, String key,
      [List<TextInputFormatter>? inputFormatters]) {
    String? defaultValue;
    switch (key) {
      case "AgeYear":
        defaultValue = controller.currentPatient.ageYear;
        break;
      case "AgeMonth":
        defaultValue = controller.currentPatient.ageMonth;
        break;
      case "AgeWeek":
        defaultValue = controller.currentPatient.ageWeek;
        break;
      case "AgeDay":
        defaultValue = controller.currentPatient.ageDay;
        break;
    }
    return Expanded(
        child: FISTextField(
      uniqueKey: GlobalKey(),
      enabled: !controller.isEdit.value,
      borderColor:
          Storage.currentTheme.fisLabelInputFieldStyle.inputTextBorderColor,
      enabledBorderColor:
          Storage.currentTheme.fisLabelInputFieldStyle.inputTextBorderColor,
      textStyle: Storage.currentTheme.fisLabelInputFieldStyle.inputTextStyle
          .copyWith(
              fontSize: Storage.currentTheme.fisLabelInputFieldStyle
                  .inputTextStyle.fontSize!.s),
      controller: controller.getTextEditingController(key, defaultValue ?? ""),
      onChanged: onChanged,
      inputFormatters: inputFormatters,
      fillColor: Storage.currentTheme.fisLabelInputFieldStyle.fillColor,
    ));
  }
}
