import 'package:flutter/material.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';

class FISLabelInputField extends StatelessWidget {
  /// 是否必填
  final bool isRequired;
  final String label;

  /// 组件
  final Widget widget;
  final double width;
  final double? height;
  final double? labelWidth;
  final EdgeInsetsGeometry? labelPadding;
  final EdgeInsetsGeometry? padding;
  final Color? labelColor;
  final double minHeight;

  const FISLabelInputField(this.isRequired, this.label, this.widget, this.width,
      {super.key,
      this.height,
      this.padding,
      this.labelWidth,
      this.labelColor,
      this.minHeight = 0,
      this.labelPadding});

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.symmetric(horizontal: 5.s),
      padding: padding,
      width: width - 11.s, //多1是因为缩放会有误差
      child: Flex(
        direction: Axis.horizontal,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Container(
            padding: labelPadding ?? const EdgeInsets.all(0),
            width: labelWidth,
            alignment: Alignment.centerLeft,
            child: RichText(
              text: TextSpan(
                style: labelColor == null
                    ? Storage
                        .currentTheme.fisLabelInputFieldStyle.labelTextStyle
                        .copyWith(
                            fontSize: Storage
                                .currentTheme
                                .fisLabelInputFieldStyle
                                .labelTextStyle
                                .fontSize!
                                .s)
                    : Storage
                        .currentTheme.fisLabelInputFieldStyle.labelTextStyle
                        .copyWith(
                            color: labelColor,
                            fontSize: Storage
                                .currentTheme
                                .fisLabelInputFieldStyle
                                .labelTextStyle
                                .fontSize!
                                .s),
                children: [
                  TextSpan(
                    text: label,
                  ),
                  if (isRequired) ...[
                    TextSpan(
                      text: ' * ',
                      style: Storage
                          .currentTheme.fisLabelInputFieldStyle.labelTextStyle
                          .copyWith(
                              color: Colors.red,
                              fontSize: Storage
                                  .currentTheme
                                  .fisLabelInputFieldStyle
                                  .labelTextStyle
                                  .fontSize!
                                  .s),
                    )
                  ]
                ],
              ),
              textAlign: TextAlign.left,
            ),
          ),
          Expanded(
              child: Container(
            constraints: BoxConstraints(minHeight: minHeight),
            margin: EdgeInsets.fromLTRB(5.s, 0, 0, 0),
            height: height,
            child: widget,
          ))
        ],
      ),
    );
  }
}

class FISLabelInputFieldWithWrap extends StatelessWidget {
  /// 是否必填
  final bool isRequired;
  final String label;

  /// 组件
  final Widget widget;
  final double? width;
  final double? height;
  final double? labelWidth;
  final EdgeInsetsGeometry? labelPadding;
  final EdgeInsetsGeometry? padding;
  final Color? labelColor;
  final double minHeight;

  const FISLabelInputFieldWithWrap(
      this.isRequired, this.label, this.widget, this.width,
      {super.key,
      this.height,
      this.padding,
      this.labelWidth,
      this.labelColor,
      this.minHeight = 0,
      this.labelPadding});

  @override
  Widget build(BuildContext context) {
    return Container(
        padding: padding,
        width: width,
        child: Row(crossAxisAlignment: CrossAxisAlignment.start, children: [
          Container(
            padding: labelPadding ?? EdgeInsets.fromLTRB(0, 12.s, 0, 0),
            width: labelWidth,
            alignment: Alignment.centerLeft,
            child: RichText(
              text: TextSpan(
                style: labelColor == null
                    ? Storage
                        .currentTheme.fisLabelInputFieldStyle.labelTextStyle
                        .copyWith(
                            fontSize: Storage
                                .currentTheme
                                .fisLabelInputFieldStyle
                                .labelTextStyle
                                .fontSize!
                                .s)
                    : Storage
                        .currentTheme.fisLabelInputFieldStyle.labelTextStyle
                        .copyWith(
                            color: labelColor,
                            fontSize: Storage
                                .currentTheme
                                .fisLabelInputFieldStyle
                                .labelTextStyle
                                .fontSize!
                                .s),
                children: [
                  TextSpan(text: label),
                  if (isRequired) ...[
                    TextSpan(
                      text: ' * ',
                      style: Storage
                          .currentTheme.fisLabelInputFieldStyle.labelTextStyle
                          .copyWith(
                              color: Colors.red,
                              fontSize: Storage
                                  .currentTheme
                                  .fisLabelInputFieldStyle
                                  .labelTextStyle
                                  .fontSize!
                                  .s),
                    )
                  ]
                ],
              ),
              textAlign: TextAlign.left,
            ),
          ),
          Expanded(child: widget)
        ]));
  }
}
