import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flyinsonolite/consultation/emergency/views/emergencypreparednessdialog.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/consultation/records/models/features.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/index.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/icons/fisicons.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/interfaces/iconfigurationmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/routeconstants.dart';
import 'package:get/get.dart';

class HomeController extends GetxController {
  RxBool isHomePage = true.obs;
  List<FeatureViewModel> featureList = [];
  final _configurationManager = Get.find<IConfigurationManager>();

  void updateFeatureList() {
    clear();
    if (Storage.user.hasFeature(FeatureKeys.Consultation)) {
      featureList.add(FeatureViewModel(
          FeatureKeys.Consultation,
          fisI18nBook.common.appointmentConsultation.t,
          FISICons.consultation,
          RouteConstants.consultationRouteConstants.consultationList));
    }

    if (Storage.user.hasFeature(FeatureKeys.EmergencyConsultation)) {
      featureList.add(FeatureViewModel(
          FeatureKeys.EmergencyConsultation,
          i18nBook.realTimeConsultation.emergency.t,
          FISICons.realtimeconsultation,
          ""));
    }
  }

  Future<void> checkUpdateAsync() async {
    try {
      if (featureList.isNotEmpty) {
        EasyLoading.show(status: i18nBook.common.loading.t);
        await _configurationManager.checkUpdateAsync();
      }
    } catch (ex) {
      loggerAsyn(
          FISDeviceLogCategory.Error, "check update Error:${ex.toString()}");
    } finally {
      EasyLoading.dismiss();
    }
    if (Storage.user.passwordExpired) {
      PromptBox.toast(fisI18nBook.common.theCurrentPasswordHasExpired.t);
      if (featureList.isEmpty) {
        await Future.delayed(const Duration(seconds: 1));
        PromptBox.toast(fisI18nBook.common.accountHasNoPermission.t);
      }
    } else if (featureList.isEmpty) {
      PromptBox.toast(fisI18nBook.common.accountHasNoPermission.t);
    }
  }

  void clear() {
    featureList.clear();
  }

  Future logoutAsync() async {
    await Storage.user.handleLogOutAsync();
  }
}

class FeatureViewModel {
  final String featureName;
  final IconData iconData;
  final String url;
  final String displayName;

  FeatureViewModel(this.featureName, this.displayName, this.iconData, this.url);

  Future onClickAsync() async {
    try {
      if (featureName == FeatureKeys.EmergencyConsultation) {
        final liveConsultationManager = Get.find<ILiveConsultationManager>();
        var datas = await liveConsultationManager.queryEmergencyDataAsync();
        if (datas.deviceInfoList!.isNotEmpty && datas.expertList!.isNotEmpty) {
          var emergencySetting =
              await liveConsultationManager.getEmergencySettingAsync();
          Get.dialog(EmergencyPreparednessDialog(datas, emergencySetting),
              barrierDismissible: false);
        }
        return;
      }
      var homeController = Get.find<HomeController>();
      homeController.isHomePage.value = false;
      Get.toNamed(url);
    } catch (ex) {
      await handlerErrorAsync(
          ex, 'HomeController clice Emergency button error: $ex');
    }
  }
}
