import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/consultation/fispanel/consultationfispanel.dart';
import 'package:flyinsonolite/controls/dialog/exitorhideConfirmDialog.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/button/fisquareiconbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/home/controllers/homecontroller.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/managers/interfaces/inetworkerrorhandlermanager.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class HomePage extends GetView<HomeController> {
  final INetworkErrorHandlerManager _networkErrorHandlerManager =
      Get.find<INetworkErrorHandlerManager>();
  HomePage({super.key}) {
    Get.lazyPut<HomeController>(() => HomeController());
    controller.isHomePage.value = true;
    controller.updateFeatureList();
    controller.checkUpdateAsync();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        resizeToAvoidBottomInset: false,
        backgroundColor: Storage.currentTheme.themeData.scaffoldBackgroundColor,
        body: Stack(
          children: [
            Flex(
              direction: Axis.vertical,
              children: [
                SizedBox(
                    height: 72.s,
                    child: Container(
                      color: Storage.currentTheme.homepageHeaderColor,
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.end,
                        children: [
                          Expanded(
                              child: Center(
                            child: Obx(() => _networkErrorHandlerManager
                                    .isNetworkAbnormal.value
                                ? FISText(
                                    i18nBook
                                        .remoteControl.networkConnectionError.t,
                                    style: Storage.currentTheme.normalTextStyle
                                        .copyWith(
                                            color: Colors.red,
                                            fontSize: Storage.currentTheme
                                                .normalTextStyle.fontSize),
                                  )
                                : const SizedBox()),
                          )),
                          FISText(
                            Storage.user.displayName ?? "",
                            textAlign: TextAlign.center,
                            style:
                                Storage.currentTheme.normalTextStyle.copyWith(
                              fontSize: 24.s,
                            ),
                          ),
                          SizedBox(
                            width: 15.s,
                          ),
                          FISIconButtonAsync(
                            onPressed: () async {
                              await controller.logoutAsync();
                            },
                            icon: const Icon(
                              Icons.logout,
                            ),
                            label: i18nBook.auth.signOut.t,
                          ),
                          Obx(() => Storage.isShowCloseButton.value
                              ? SizedBox(
                                  width: 20.s,
                                )
                              : const SizedBox()),
                          Obx(() => Storage.isShowCloseButton.value
                              ? FISIconButton(
                                  onPressed: () {
                                    Get.dialog(const ExitOrHideConfirmDialog(),
                                        barrierDismissible: false);
                                  },
                                  icon: const Icon(
                                    Icons.close,
                                  ),
                                  iconSize: 56.s,
                                )
                              : const SizedBox()),
                        ],
                      ),
                    )),
                SizedBox(
                  height: 20.s,
                ),
                Flexible(
                  fit: FlexFit.tight,
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 120.s,
                        child: ListView.builder(
                            scrollDirection: Axis.horizontal,
                            itemCount: controller.featureList.length,
                            itemBuilder: (BuildContext context, int index) {
                              return Container(
                                margin: EdgeInsets.all(10.s),
                                child: FISSquareIconButtonAsync(() async {
                                  await controller.featureList[index]
                                      .onClickAsync();
                                }, controller.featureList[index].iconData,
                                    controller.featureList[index].displayName),
                              );
                            }),
                      )
                    ],
                  ),
                )
              ],
            ),
            Positioned(
                right: 0,
                bottom: 0,
                child: Obx(() => (Storage.isUseFISForConsultation.value ||
                            (!Storage.isUseFISForConsultation.value &&
                                Storage.isConsultating.value &&
                                Storage.hasMultiScreens)) &&
                        controller.isHomePage.value
                    ? const ConsulationFISPanel()
                    : const SizedBox()))
          ],
        ));
  }
}
