import 'package:fis_common/extensions/index.dart';
import 'package:fis_common/logger/logger.dart';
import 'package:fis_common/web/web_asset_bundle.dart';
import 'package:fis_jsonrpc/notifications/index.dart';
import 'package:fis_jsonrpc/services/authentication.m.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:flyinsonolite/consultation/liveconsultation/models/consultationpopup.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/connection.m.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclient.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/jsplatform/jsplatformlistener.dart';
import 'package:flyinsonolite/localization/localization.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/managers/appointmentmanager.dart';
import 'package:flyinsonolite/managers/configurationmanager.dart';
import 'package:flyinsonolite/managers/devicemanager.dart';
import 'package:flyinsonolite/managers/interfaces/iappointmentmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iconfigurationmanager.dart';
import 'package:flyinsonolite/managers/interfaces/idevicemanager.dart';
import 'package:flyinsonolite/managers/interfaces/iliveconsultationmanager.dart';
import 'package:flyinsonolite/managers/interfaces/inetworkerrorhandlermanager.dart';
import 'package:flyinsonolite/managers/interfaces/ireportmanager.dart';
import 'package:flyinsonolite/managers/interfaces/irtmpmockrtcmanager.dart';
import 'package:flyinsonolite/managers/interfaces/iserverconfigmanager.dart';
import 'package:flyinsonolite/managers/liveconsultationmanager.dart';
import 'package:flyinsonolite/managers/networkerrorhandlermanager.dart';
import 'package:flyinsonolite/managers/reportmanager.dart';
import 'package:flyinsonolite/managers/rtmpmockrtcmanager.dart';
import 'package:flyinsonolite/managers/serverconfigmanager.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:flyinsonolite/infrastructure/user.dart';
import 'package:flyinsonolite/notifications/notifications.dart';
import 'package:flyinsonolite/helpers/parameterhelper.dart';
import 'package:flyinsonolite/services/localreporteditorservice.dart';
import 'package:flyinsonolite/services/sharedstorageservice.dart';
import 'package:get/get.dart';
import 'package:universal_html/js.dart' as js;

/// 全局设置
class Global {
  static AssetBundle? _assetBundle;

  /// 资源包
  static AssetBundle get assetBundle => _assetBundle!;

  /// 初始化
  static Future<void> initAsync() async {
    await logger.init();
    await Localization.intializeAsync();
    jsonrpcclientForFISLib.updateServer("127.0.0.1:8090");
    try {
      var usMachineInfo =
          await jsonRpcProxyForFISLib.connection.getUSMachineInfoForFlutter();
      if (usMachineInfo.serverUrl.isNotNullOrEmpty && usMachineInfo.isLogon) {
        jsonrpcclient.updateServer(
            usMachineInfo.serverUrl!, usMachineInfo.useSSL);
      }
      Storage.usDeviceCode = usMachineInfo.deviceCode;
      Storage.hasMultiScreens = usMachineInfo.hasMultiScreens;
      Storage.isShowCloseButton.value = usMachineInfo.isShowCloseButton;
      switch (usMachineInfo.platform) {
        case FISPlatform.Android:
          Storage.platform = Platform.Android;
          break;
        case FISPlatform.Windows:
          Storage.platform = Platform.Windows;
          break;
        case FISPlatform.Ios:
          Storage.platform = Platform.Ios;
          break;
      }
    } catch (ex) {
      await handlerErrorAsync(ex, "getUSMachineInfoForFlutter Error: $ex");
    }
    //触发消息中心实例创建
    notificationCenter;
    await _initilzieManagersAsync();
    Storage.initLoginConfig();
    _initAssetBundle();
    await intializeWhenRouteIsLiveConsultationAsync();
    if (!Storage.isSecondView) {
      ///会诊呼叫弹窗监听
      ConsultationPopup.startListening();
      JsPlatformListener().run();
    }
  }

  static Future intializeWhenRouteIsLiveConsultationAsync() async {
    try {
      ///获取启动参数
      if (ParameterHelper.parameterMap.isEmpty) {
        ParameterHelper.getAppStartParameters();
      }

      var parameters = ParameterHelper.parameterMap;
      if (parameters.containsKey('page')) {
        var pageParameter = parameters['page'];
        if (pageParameter == 'liveconsultation') {
          Storage.isSecondView = true;
          if (parameters.containsKey('token')) {
            var token = parameters['token'] ?? '';
            Storage.user.setToken(token);
            await Storage.user
                .handleLoginAsync(token: token, isPasswordExpired: false);
            await Storage.user.handleAfterLoginAsync(type: WSType.Consultation);
          }

          if (parameters.containsKey('consultationCode')) {
            var appointmentManager = Get.find<IAppointmentManager>();
            var liveConsultationManager = Get.find<ILiveConsultationManager>();
            var consultationDetail = await appointmentManager
                .findConsultationDetailAsync(parameters['consultationCode']!);
            if (consultationDetail != null) {
              liveConsultationManager.currentConsultationCode =
                  consultationDetail.consultationCode!;
              liveConsultationManager.consultationDetail = consultationDetail;
            }
          }
        }
      }
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error,
          "intializeWhenRouteIsLiveConsultationAsync error: $ex");
    }
  }

  /// 初始化资源包
  static void _initAssetBundle() {
    if (kIsWeb) {
      final jsVersion = js.context["serviceWorkerVersion"];
      String version = jsVersion?.toString() ??
          "debug_${DateTime.now().millisecondsSinceEpoch}";
      _assetBundle = WebPlatformAssetBundle(
        rootPath: js.context["STATIC_ROOT"]!.toString(),
        version: version,
      );
    } else {
      _assetBundle = rootBundle;
    }
  }

  static Future _initilzieManagersAsync() async {
    Get.put(User());
    Get.put<IServerConfigManager>(ServerConfigManager());
    Get.put<IAppointmentManager>(AppointmentManager());
    await Get.putAsync<SharedStorageService>(
        () async => await SharedStorageService.initAsync());
    Get.put<IConfigurationManager>(ConfigurationManager());
    Get.put<IReportManager>(ReportManager());
    Get.put<ILiveConsultationManager>(LiveConsultationManager());
    Get.put<IDeviceManager>(DeviceManager());
    Get.put<INetworkErrorHandlerManager>(NetworkErrorHandlerManager());
    Get.lazyPut(() => LocalReportEditorService());
    Get.put<IRtmpMockRtcManager>(RtmpMockRtcManager());
  }
}
