import 'package:fis_common/index.dart';
import 'package:fis_common/logger/logger.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/controls/prompt.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';

Future loggerAsyn(FISDeviceLogCategory logType, String? msg) async {
  try {
    await jsonRpcProxyForFISLib.log
        .writeLogFromFlutter(FISLogEventArgs(logType: logType, msg: msg));
  } catch (ex) {
    logger.e(
        "Logger to fis sdk error:$ex,the logMsg：$msg,type:${logType.toString()}");
  }
}

Future handlerErrorAsync(Object ex, String logMsg) async {
  if (logMsg.isNotNullOrEmpty) {
    await loggerAsyn(FISDeviceLogCategory.Error, logMsg);
  }

  if (ex is JsonRpcException) {
    if (ex.data is JsonRpcNetworkException) {
      PromptBox.toast(i18nBook.common.networkInstability.t);
      return;
    }

    if (ex.data is JsonRpcServerError) {
      return;
    }
  }

  PromptBox.toast("${i18nBook.errorCodes.errorCodeUN.t}: ${ex.toString()}");
}
