import 'dart:convert';
import 'dart:ui';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/consultation/records/models/patientinfo.dart';
import 'package:flyinsonolite/consultation/records/models/patientmodel.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/infrastructure/user.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/login/models/loginparameter.dart';
import 'package:flyinsonolite/managers/configurationmanager.dart';
import 'package:flyinsonolite/services/sharedstorageservice.dart';
import 'package:flyinsonolite/themes/bluethemedata.dart';
import 'package:flyinsonolite/themes/customthemedata.dart';
import 'package:flyinsonolite/themes/graythemedata.dart';
import 'package:flyinsonolite/themes/purplethemedata.dart';
import 'package:get/get.dart';
import 'package:fis_common/extensions/string.dart';

class Storage {
  static bool isSecondView = false;

  static String loginRoute =
      '/login/${i18nBook.locale.languageCode}/${currentTheme.themeName}';

  /// 网络是否异常
  //static bool isNetworkAbnormal

  static User get user => Get.find();

  static CustomThemeData currentTheme = grayTheme;

  static List<PatientInfo>? patientList;

  static ConfigurationFromServer? configurationFromerver;

  ///当前超声机账号的devicecode
  static String? usDeviceCode;

  ///当前超声机账号的短码
  static String? usDeviceShortCode;

  static LoginParameter loginParameter = LoginParameter();

  static final RxBool _isRemeberPassword = false.obs;

  static RxBool get isRemeberPassword => _isRemeberPassword;

  ///超声机的Platform
  static Platform platform = Platform.Windows;

  ///超声机是否有多屏,如果超声机显示屏和主屏分开，则在主屏上显示直播画面
  static bool hasMultiScreens = false;

  /// 是否是基层本机会诊的情况（使用FIS独立进程进行会诊）
  static RxBool isUseFISForConsultation = false.obs;

  ///是否正在会诊
  static RxBool isConsultating = false.obs;

  ///是否显示软键盘
  static bool isShowSoftKeyBoard = false;

  ///显示关闭按钮
  static RxBool isShowCloseButton = false.obs;

  ///正在注销
  static bool isLogouting = false;

  ///是否是竖屏
  static RxBool isPortraitMode = false.obs;

  static void initLoginConfig() {
    try {
      var loginConfig =
          SharedStorageService.ins.getString(StoreKeys.loginConfig);
      if (loginConfig != null) {
        loginParameter = LoginParameter.fromJson(jsonDecode(loginConfig));
      }
      isRemeberPassword.value =
          SharedStorageService.ins.getBool(StoreKeys.remeberPwdConfig) ?? false;
    } catch (e) {
      loggerAsyn(
          FISDeviceLogCategory.Error, "Storage initLoginConfig error $e.");
    }
  }

  static void updateIsRemeberPassword(bool value) {
    if (_isRemeberPassword.value != value) {
      _isRemeberPassword.value = value;
      if (!_isRemeberPassword.value) {
        SharedStorageService.ins.setBool(StoreKeys.remeberPwdConfig, false);
      }
    }
  }

  static Future intializeLocaleAsync() async {
    var language = Get.parameters['language'];
    if (language.isNullOrWhiteSpace) {
      return;
    }

    switch (language) {
      case 'en':
        await FI18n.ins.switchLanguage(const Locale('en'));
        await FISI18n.ins.switchLanguage(const Locale('en'));
        break;
      case 'zh':
        await FI18n.ins.switchLanguage(const Locale('zh'));
        await FISI18n.ins.switchLanguage(const Locale('zh'));
        break;
      default:
        await FI18n.ins.switchLanguage(const Locale('zh'));
        await FISI18n.ins.switchLanguage(const Locale('zh'));
        break;
    }
  }

  static void initlizeTheme() {
    var theme = Get.parameters['theme'];
    if (theme.isNullOrWhiteSpace) {
      theme = 'purple';
    }
    switch (theme) {
      case "blue":
        Storage.currentTheme = blueTheme;
        break;
      case "purple":
        Storage.currentTheme = purpleTheme;
        break;
      case "gray":
        Storage.currentTheme = grayTheme;
        break;
    }
  }

  static void updateLoginParameter() {
    try {
      var loginConfig = jsonEncode(loginParameter.toJson());
      SharedStorageService.ins.setString(StoreKeys.loginConfig, loginConfig);
      SharedStorageService.ins
          .setBool(StoreKeys.remeberPwdConfig, isRemeberPassword.value);
    } catch (e) {
      loggerAsyn(
          FISDeviceLogCategory.Error, "Storage saveLoginConfig error $e.");
    }
  }
}

class ConfigurationFromServer {
  ////预约会诊显示的会诊申请信息字段是从Server获取的
  ConsultationServerConfig consultationServerConfig;

  LanguageServerConfig languageServerConfig;

  ConfigurationFromServer(
      this.consultationServerConfig, this.languageServerConfig);
}

class ConsultationServerConfig {
  ////预约会诊显示的会诊申请信息字段是从Server获取的
  List<AppointmentModel> appointmentKeyList;
  ////预约会诊显示的病人信息字段是从Server获取的
  List<PatientModel> patientKeyList;
  String version;
  ConsultationServerConfig(
      this.patientKeyList, this.appointmentKeyList, this.version);

  factory ConsultationServerConfig.fromJson(
      Map<String, dynamic> json, String version) {
    return ConsultationServerConfig(
      json['Patient'] == null
          ? []
          : (json['Patient'] as Map<String, dynamic>)
              .entries
              .map((e) => PatientModel.fromJson(e.value, e.key))
              .toList(),
      json['ConsultationBaseInfo'] == null
          ? []
          : (json['ConsultationBaseInfo'] as Map<String, dynamic>)
              .entries
              .map((e) => AppointmentModel.fromJson(e.value, e.key))
              .toList(),
      version,
    );
  }
}

class LanguageServerConfig {
  String? languageKey;
  List<LangugeDataResult> languageList;

  LanguageServerConfig(this.languageList, this.languageKey);

  factory LanguageServerConfig.fromJson(Map<String, dynamic> json) {
    return LanguageServerConfig(
      json['LanguageList'] == null
          ? []
          : (json['LanguageList'] as List<dynamic>)
              .map((e) => LangugeDataResult.fromJson(e))
              .toList(),
      json['LanguageKey'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = <String, dynamic>{};
    map['LanguageList'] = languageList.map((e) => e.toJson()).toList();
    map['LanguageKey'] = languageKey;
    return map;
  }

  /// 检查根据不同语言模块的父级Code获取集合
  ///
  /// [module] 当前模块
  /// [module] 翻译Code
  String findLanguageValueByCode(String module, String code) {
    try {
      String? languageContent = languageList
          .firstWhere((element) => element.langugeModule == module)
          .content;
      if (languageContent != null) {
        final String? children = jsonDecode(languageContent)["Language"][code];
        return children ?? code;
      }
      return code;
    } catch (e) {
      return code;
    }
  }
}
