import 'dart:collection';
import 'package:fis_common/extensions/type.dart';
import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_jsonrpc/interceptor.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/index.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';

typedef T ServiceBuilder<T extends JsonRpcClientBase>();

const C_SHELL_RPC_DEFAULT_HOST = 'platform.fis.plus';

/// JSON-RPC 代理
class JsonRpcProxyForFISLib {
  JsonRpcProxyForFISLib({
    String? host,
    this.platformHost = C_SHELL_RPC_DEFAULT_HOST,
  }) {
    _currentHost = host ?? "unknown";
    _currentProtocol = "http";
  }

  /// 宿主平台代理地址
  final String platformHost;

  /// 服务主机地址
  late String _currentHost;

  /// 服务主机协议
  late String _currentProtocol;

  /// 当前服务主机地址
  String get currentHostAddress => "$_currentProtocol://$_currentHost";

  final HashMap<Type, dynamic> _serviceCache = HashMap();

  /* 服务代理设置 Start */

  /// 平台服务

  LogService get log => findService(() => LogService(currentHostAddress));

  HardwareService get hardware =>
      findService(() => HardwareService(currentHostAddress));

  ConsultationService get consultation =>
      findService(() => ConsultationService(currentHostAddress));

  ConnectionService get connection =>
      findService(() => ConnectionService(currentHostAddress));

  /* 服务代理设置 End */

  /// 设置服务主机地址
  void setServerHost(String address, [bool useSSL = false]) {
    loggerAsyn(FISDeviceLogCategory.Info,
        'JsonRpcProxyForFISLib setServerHost :$address');
    _currentProtocol = useSSL ? "https" : "http";
    _currentHost = address;
  }

  /// 添加拦截器
  void addInterceptor(JsonRpcInterceptor interceptor) =>
      jsonRpcInterceptHost.addInterceptor(interceptor);

  /// 清空缓存
  void clearCache() => _serviceCache.clear();

  /// 查找Service实例
  T findService<T extends JsonRpcClientBase>(ServiceBuilder<T> builder) {
    Type serviceType = typeOf<T>();
    if (!_serviceCache.containsKey(serviceType)) {
      _serviceCache[serviceType] = builder.call();
    }
    return _serviceCache[serviceType] as T;
  }
}
