import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'consultation.m.dart';

import 'log.m.dart';

class ConsultationService extends JsonRpcClientBase {
  ConsultationService(
    String host, {
    String serviceName = "IConsultationService",
    Map<String, String>? headers,
    int? timeout,
  }) : super(
          host,
          serviceName,
          headers: headers,
          timeout: timeout,
        ) {
    /// 注册响应实体反序列化处理器
    FJsonConvert.setDecoder((map) => FISResultInfoDTO.fromJson(map));
    FJsonConvert.setDecoder((map) => FISLiveConsultationState.fromJson(map));
  }

  Future<FISResultInfoDTO> sendFlutterLiveConsultationInfo(
      FISFlutterLiveConsultationInfo fisFlutterLiveConsultationInfo) async {
    var rpcRst = await call(
        "SendFlutterLiveConsultationInfo", fisFlutterLiveConsultationInfo);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> startLiveConsultationFromFlutter(
      FISLiveConsultationRequestDTO fisLiveConsultationRequestDTO) async {
    var rpcRst = await call(
        "StartLiveConsultationFromFlutter", fisLiveConsultationRequestDTO);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> hangupLiveConsultationFromFlutter(
      FISHangupLiveConsultationDTO fisHangupLiveConsultationDTO) async {
    var rpcRst = await call(
        "HangupLiveConsultationFromFlutter", fisHangupLiveConsultationDTO);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> liveConsultationMemberChangedFromFlutter(
      FISConsultationMemberChangeDTO fisConsultationMemberChangeDTO) async {
    var rpcRst = await call("LiveConsultationMemberChangedFromFlutter",
        fisConsultationMemberChangeDTO);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> changeLiveConsultationFromFlutter(
      FISChangeLiveConsultationDTO fisAcceptLiveConsultationRequestDTO) async {
    var rpcRst = await call("ChangeLiveConsultationFromFlutter",
        fisAcceptLiveConsultationRequestDTO);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> switchToLocalPatientFromFlutter(
      FISSwitchToLocalPatientRequestDTO
          fisSwitchToLocalPatientRequestDTO) async {
    var rpcRst = await call(
        "SwitchToLocalPatientFromFlutter", fisSwitchToLocalPatientRequestDTO);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> receiveWhiteBoardDataFromFlutter(
      FISWhiteBoardDataDTO fisWhiteBoardDataDTO) async {
    var rpcRst =
        await call("ReceiveWhiteBoardDataFromFlutter", fisWhiteBoardDataDTO);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> switchVideoPlayFromFlutter(
      FISSwtichVideoPlayDTO fisVideoPlayChangedDTO) async {
    var rpcRst =
        await call("SwitchVideoPlayFromFlutter", fisVideoPlayChangedDTO);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> startOnlyForRtmpPushingFromFlutter(
      FISStartOnlyForRtmpPushingDTO fisStartOnlyForRtmpPushingDTO) async {
    var rpcRst = await call(
        "StartOnlyForRtmpPushingFromFlutter", fisStartOnlyForRtmpPushingDTO);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> stopOnlyForRtmpPushingFromFlutter() async {
    var rpcRst = await call(
      "StopOnlyForRtmpPushingFromFlutter",
    );
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> changeConsultationCodeOnlyForRtmpPushingFromFlutter(
      FISChangeLiveConsultationDTO fisChangeLiveConsultationDTO) async {
    var rpcRst = await call(
        "ChangeConsultationCodeOnlyForRtmpPushingFromFlutter",
        fisChangeLiveConsultationDTO);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISLiveConsultationState> checkLiveConsultationState() async {
    var rpcRst = await call(
      "CheckLiveConsultationState",
    );
    var result =
        FISLiveConsultationState.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> operateSoftKeyBoard(
      FISControlPositionInfo fisPositionInfo) async {
    var rpcRst = await call("OperateSoftKeyBoard", fisPositionInfo);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> switchToFlyinsonoPage() async {
    var rpcRst = await call(
      "SwitchToFlyinsonoPage",
    );
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> informUserStatusFormFlutter(
      FISUserLoginStatusArgs userLoginStatusArgs) async {
    var rpcRst = await call("InformUserStatusFormFlutter", userLoginStatusArgs);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }
}
