import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'hardware.m.dart';

import 'log.m.dart';

class HardwareService extends JsonRpcClientBase {
  HardwareService(
    String host, {
    String serviceName = "IHardwareService",
    Map<String, String>? headers,
    int? timeout,
  }) : super(
          host,
          serviceName,
          headers: headers,
          timeout: timeout,
        ) {
    /// 注册响应实体反序列化处理器
    FJsonConvert.setDecoder((map) => FISHardwareInfo.fromJson(map));
    FJsonConvert.setDecoder((map) => FISResultInfoDTO.fromJson(map));
  }

  Future<FISHardwareInfo> getCurrentHardwareInfo() async {
    var rpcRst = await call(
      "GetCurrentHardwareInfo",
    );
    var result = FISHardwareInfo.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> setCurrentMicVolume(int volume) async {
    var rpcRst = await call("SetCurrentMicVolume", volume);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> setCurrentSpeakerVolume(int volume) async {
    var rpcRst = await call("SetCurrentSpeakerVolume", volume);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> setCurrentSpeakerMute(bool isMute) async {
    var rpcRst = await call("SetCurrentSpeakerMute", isMute);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }

  Future<FISResultInfoDTO> setCurrentMicMute(bool isMute) async {
    var rpcRst = await call("SetCurrentMicMute", isMute);
    var result = FISResultInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
    return result;
  }
}
