import 'log.m.dart';

class FISCameraCaptureCapability {
  int width;
  int height;
  int framerate;

  FISCameraCaptureCapability({
    this.width = 0,
    this.height = 0,
    this.framerate = 0,
  });

  factory FISCameraCaptureCapability.fromJson(Map<String, dynamic> map) {
    return FISCameraCaptureCapability(
      width: map['Width'],
      height: map['Height'],
      framerate: map['Framerate'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    map['Width'] = width;
    map['Height'] = height;
    map['Framerate'] = framerate;
    return map;
  }
}

class FISCameraInfo {
  String? id;
  String? name;
  List<String>? hardwareId;
  List<FISCameraCaptureCapability>? capabilities;

  FISCameraInfo({
    this.id,
    this.name,
    this.hardwareId,
    this.capabilities,
  });

  factory FISCameraInfo.fromJson(Map<String, dynamic> map) {
    return FISCameraInfo(
      id: map['Id'],
      name: map['Name'],
      hardwareId: map['HardwareId']?.cast<String>().toList(),
      capabilities: map['Capabilities'] != null
          ? (map['Capabilities'] as List)
              .map((e) => FISCameraCaptureCapability.fromJson(
                  e as Map<String, dynamic>))
              .toList()
          : null,
    );
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    if (id != null) {
      map['Id'] = id;
    }
    if (name != null) {
      map['Name'] = name;
    }
    if (hardwareId != null) {
      map['HardwareId'] = hardwareId;
    }
    if (capabilities != null) {
      map['Capabilities'] = capabilities;
    }
    return map;
  }
}

class FISMicInfo {
  String? id;
  String? name;
  List<String>? hardwareId;
  int volume;
  bool isMute;

  FISMicInfo({
    this.id,
    this.name,
    this.hardwareId,
    this.volume = 0,
    this.isMute = false,
  });

  factory FISMicInfo.fromJson(Map<String, dynamic> map) {
    return FISMicInfo(
      id: map['Id'],
      name: map['Name'],
      hardwareId: map['HardwareId']?.cast<String>().toList(),
      volume: map['Volume'],
      isMute: map['IsMute'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    if (id != null) {
      map['Id'] = id;
    }
    if (name != null) {
      map['Name'] = name;
    }
    if (hardwareId != null) {
      map['HardwareId'] = hardwareId;
    }
    map['Volume'] = volume;
    map['IsMute'] = isMute;
    return map;
  }
}

class FISSpeakerInfo {
  String? id;
  String? name;
  List<String>? hardwareId;
  int volume;
  bool isMute;

  FISSpeakerInfo({
    this.id,
    this.name,
    this.hardwareId,
    this.volume = 0,
    this.isMute = false,
  });

  factory FISSpeakerInfo.fromJson(Map<String, dynamic> map) {
    return FISSpeakerInfo(
      id: map['Id'],
      name: map['Name'],
      hardwareId: map['HardwareId']?.cast<String>().toList(),
      volume: map['Volume'],
      isMute: map['IsMute'],
    );
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    if (id != null) {
      map['Id'] = id;
    }
    if (name != null) {
      map['Name'] = name;
    }
    if (hardwareId != null) {
      map['HardwareId'] = hardwareId;
    }
    map['Volume'] = volume;
    map['IsMute'] = isMute;
    return map;
  }
}

class FISHardwareInfo {
  FISCameraInfo? cameraInfo;
  FISMicInfo? micInfo;
  FISSpeakerInfo? speakerInfo;

  FISHardwareInfo({
    this.cameraInfo,
    this.micInfo,
    this.speakerInfo,
  });

  factory FISHardwareInfo.fromJson(Map<String, dynamic> map) {
    return FISHardwareInfo(
      cameraInfo: map['CameraInfo'] != null
          ? FISCameraInfo.fromJson(map['CameraInfo'])
          : null,
      micInfo:
          map['MicInfo'] != null ? FISMicInfo.fromJson(map['MicInfo']) : null,
      speakerInfo: map['SpeakerInfo'] != null
          ? FISSpeakerInfo.fromJson(map['SpeakerInfo'])
          : null,
    );
  }

  Map<String, dynamic> toJson() {
    final map = Map<String, dynamic>();
    if (cameraInfo != null) {
      map['CameraInfo'] = cameraInfo;
    }
    if (micInfo != null) {
      map['MicInfo'] = micInfo;
    }
    if (speakerInfo != null) {
      map['SpeakerInfo'] = speakerInfo;
    }
    return map;
  }
}
