import 'dart:convert';
import 'dart:ui';

import 'package:flutter/cupertino.dart';
import 'package:flyinsonolite/localization/fis_common.dart';
import 'package:fis_i18n/types.dart'
    show FTrStr, IPropQueryable, ModuleBase, StringLocaleExtensions;
import 'package:fis_i18n/types.dart' as t show StringMapUtils;

/// 翻译字典
class FISTrKeybook implements IPropQueryable {
  FISTrKeybook._(
    this.locale, {
    required this.common,
    this.moduleMap,
  });

  /// 当前是否中文
  bool get isCurrentChinese => locale.languageCode.toLowerCase() == "zh";

  ///当前是否英文
  bool get isCurrentEnglish => locale.languageCode.toLowerCase() == "en";

  ///当前是否俄语
  bool get isCurrentRussian => locale.languageCode.toLowerCase() == "ru";

  Map<String, dynamic>? moduleMap;

  /// 字典语言
  final Locale locale;

  /// 公共模块
  final FISCommonModule common;

  /// 翻译字典 - 通过JsonMap构造
  ///
  /// [locale] 语言
  ///
  /// [map] 字典集
  factory FISTrKeybook(Locale locale, dynamic map) {
    if (map == null || map is! Map<String, dynamic>) {
      throw ArgumentError(
          "FISTrKeybook .ctor: argument-map must be a not null Map<String, dynamic>.");
    }

    return FISTrKeybook._(
      locale,
      common: map.pick(FISCommonModule.ModuleName, (v) => FISCommonModule(v)),
      moduleMap: map,
    );
  }

  /// 通过json字符串构造创建字典
  static FISTrKeybook createFromJson(String text) {
    final map = jsonDecode(text);
    final localeStr = map['locale'];
    if (localeStr == null) {
      throw ArgumentError(
          "FISTrKeybook: create from json error - item `locale` is not defined.");
    }
    final locale = (localeStr as String).toLocale();
    if (locale == null) {
      throw ArgumentError(
          "FISTrKeybook: create from json error - item `locale` is not valid.");
    }
    return FISTrKeybook(locale, map);
  }

  @override
  dynamic getProp(String propName) {
    switch (propName) {
      case FISCommonModule.ModuleName:
        return common;
    }
    return null;
  }

  /// 获取翻译单元
  ///
  /// [moduleName] 模块名
  ///
  /// [key] 翻译键
  FTrStr? getTrItem(String moduleName, String key) {
    if (moduleMap?.containsKey(moduleName) == false) {
      return null;
    }
    final map = moduleMap![moduleName]! as Map<String, dynamic>;
    final item = t.StringMapUtils.pickTrStr(map, key, "");
    return item;
  }
}

typedef T _ModuleBuilder<T>(Map<String, dynamic> v);

extension _TrMapPickExtension on Map<String, dynamic> {
  /// 剥取翻译值
  ///
  /// [key] 字典键
  ///
  /// [defaultValue] 默认翻译
  T pick<T extends ModuleBase>(String key, _ModuleBuilder<T> builder) {
    dynamic value = this[key];
    if (value == null || value is! Map<String, dynamic>) {
      throw UnimplementedError(
          'translation keybook is not complete: Module - $key.');
    }
    return builder(value);
  }
}
