import 'package:fis_common/extensions/string.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/services/authentication.m.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flyinsonolite/controls/custom/customiconbutton.dart';
import 'package:flyinsonolite/controls/diagonalpainter.dart';
import 'package:flyinsonolite/controls/dialog/exitorhideConfirmDialog.dart';
import 'package:flyinsonolite/controls/select/fischeckbox.dart';
import 'package:flyinsonolite/controls/button/fisiconbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/fistextfield.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/consultation.m.dart';
import 'package:flyinsonolite/jsonrpc/jsonrpcclientforFISLib.dart';
import 'package:flyinsonolite/login/controllers/logincontroller.dart';
import 'package:flyinsonolite/login/views/verificationcodefield.dart';
import 'package:flyinsonolite/controls/button/fislineargradientelevatedbutton.dart';
import 'package:flyinsonolite/jsonrpc/fisLib/services/log.m.dart';
import 'package:flyinsonolite/localization/fis_i18n.dart';
import 'package:flyinsonolite/infrastructure/logger.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';
import 'package:flyinsonolite/controls/qrcode/qrcodeimageview.dart';

class LoginPage extends GetView<LoginController> {
  final RxBool _isObscure = RxBool(true);
  bool hasInit = false;
  LoginPage({super.key});

  @override
  Widget build(BuildContext context) {
    Get.lazyPut<LoginController>(() => LoginController());
    return FutureBuilder(
        future: initAsync(),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return Scaffold(
              resizeToAvoidBottomInset: false,
              backgroundColor:
                  Storage.currentTheme.themeData.scaffoldBackgroundColor,
              body: _buildWideLayout(context),
            );
          } else {
            return Scaffold(
              resizeToAvoidBottomInset: false,
              backgroundColor:
                  Storage.currentTheme.themeData.scaffoldBackgroundColor,
            );
          }
        });
  }

  Future initAsync() async {
    try {
      if (hasInit) {
        return;
      }
      Storage.initlizeTheme();
      await Storage.intializeLocaleAsync();
      hasInit = true;
    } catch (ex) {
      await loggerAsyn(FISDeviceLogCategory.Error, 'LoginPage Init Error:$ex');
    }
  }

  Widget _buildWideLayout(BuildContext context) {
    return LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
      double containerWidth = Storage.isPortraitMode.value ? 450.s : 700.s;
      double containerHeight = 340.s;
      return Stack(
        children: [
          Center(
              child: Stack(
            children: [
              Container(
                  width: containerWidth,
                  height: containerHeight,
                  padding: Storage.isPortraitMode.value
                      ? EdgeInsets.all(10.s)
                      : EdgeInsets.all(20.s),
                  decoration: BoxDecoration(
                      border: Border.all(
                          color: Storage
                              .currentTheme.loginContainStyle.borderColor,
                          width: 1.s),
                      color: Storage
                          .currentTheme.loginContainStyle.backgroundColor,
                      borderRadius: BorderRadius.all(Radius.circular(10.s))),
                  alignment: Alignment.center,
                  child: Row(
                    mainAxisAlignment: Storage.isPortraitMode.value
                        ? MainAxisAlignment.center
                        : MainAxisAlignment.spaceEvenly,
                    children: [
                      Obx(() => Storage.isPortraitMode.value &&
                                  controller.isQrcode.value ||
                              !Storage.isPortraitMode.value
                          ? Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                FISText(i18nBook.auth.scanQRCodeLogin.t),
                                SizedBox(height: 10.s),
                                SizedBox(
                                  width: 300.s,
                                  child: FISText(
                                    i18nBook.auth.useFlyinsonoMobileScan.t,
                                    softWrap: true,
                                    textAlign: TextAlign.center,
                                    style: Storage.currentTheme.normalTextStyle
                                        .copyWith(
                                            overflow: null,
                                            fontSize: Storage.currentTheme
                                                .normalTextStyle.fontSize!.s),
                                  ),
                                ),
                                SizedBox(height: 20.s),
                                _buildQrView()
                              ],
                            )
                          : const SizedBox()),
                      Storage.isPortraitMode.value
                          ? const SizedBox()
                          : VerticalDivider(
                              width: 16.s,
                              thickness: 1.s,
                            ),
                      Obx(() => Storage.isPortraitMode.value &&
                                  !controller.isQrcode.value ||
                              !Storage.isPortraitMode.value
                          ? Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                _buildTabBar(),
                                SizedBox(
                                  height: 20.s,
                                ),
                                Obx(
                                  () => controller
                                          .isLoginWithVerificationCode.value
                                      ? SizedBox(
                                          width: 320.s,
                                          child: FISTextField(
                                            uniqueKey: GlobalKey(),
                                            controller: controller
                                                .verifyCodeAccountTextController,
                                            placeHolder: i18nBook
                                                .auth.hint4AccountDouble.t,
                                            onChanged: (value) {
                                              controller.updateText(
                                                  controller
                                                      .verifyCodeAccountTextController,
                                                  value);
                                            },
                                          ),
                                        )
                                      : SizedBox(
                                          width: 320.s,
                                          child: FISTextField(
                                            uniqueKey: GlobalKey(),
                                            controller: controller
                                                .userNameTextController,
                                            onSubmitted: onSubmitted,
                                            placeHolder: i18nBook
                                                .auth.hint4AccountTriple.t,
                                            onChanged: (value) {
                                              controller.updateText(
                                                  controller
                                                      .userNameTextController,
                                                  value);
                                              controller.passwordTextController
                                                  .text = '';
                                              controller.password.value = '';
                                            },
                                          ),
                                        ),
                                ),
                                SizedBox(
                                  height: 20.s,
                                ),
                                Obx(() =>
                                    controller.isLoginWithVerificationCode.value
                                        ? SizedBox(
                                            width: 320.s,
                                            child: VerificationCodeField(
                                                fetchCodeFunc: controller
                                                    .sendVerifyCodeAsync,
                                                verificationCodeTextController:
                                                    controller
                                                        .verifyCodeTextController,
                                                verificationCodeWidth: 180.s,
                                                onSubmitted: onSubmitted,
                                                onChanged: (value) {
                                                  controller.updateText(
                                                      controller
                                                          .verifyCodeTextController,
                                                      value);
                                                }),
                                          )
                                        : SizedBox(
                                            width: 320.s,
                                            child: Obx(() => FISTextField(
                                                  uniqueKey: GlobalKey(),
                                                  controller: controller
                                                      .passwordTextController,
                                                  obscureText: _isObscure.value,
                                                  placeHolder:
                                                      i18nBook.auth.hint4Pwd.t,
                                                  onChanged: (value) {
                                                    controller.updateText(
                                                        controller
                                                            .passwordTextController,
                                                        value);
                                                  },
                                                  suffixIcon: CustomIconButton(
                                                    onPressed: () {
                                                      _isObscure.value =
                                                          !_isObscure.value;
                                                    },
                                                    padding:
                                                        const EdgeInsets.all(0),
                                                    constraints:
                                                        const BoxConstraints(),
                                                    icon: Icon(
                                                      _isObscure.value
                                                          ? Icons.visibility_off
                                                          : Icons.visibility,
                                                      size: 24.s,
                                                    ),
                                                    color: Storage
                                                        .currentTheme
                                                        .themeData
                                                        .indicatorColor,
                                                  ),
                                                  onSubmitted: onSubmitted,
                                                  inputFormatters: [
                                                    FilteringTextInputFormatter
                                                        .deny(RegExp(
                                                            "[\u4e00-\u9fa5]")),
                                                    FilteringTextInputFormatter
                                                        .deny(RegExp("\\s")),
                                                  ],
                                                )),
                                          )),
                                Obx(
                                  () => controller
                                          .isLoginWithVerificationCode.value
                                      ? SizedBox(
                                          height: 20.s,
                                        )
                                      : Column(
                                          crossAxisAlignment:
                                              CrossAxisAlignment.start,
                                          children: [
                                            SizedBox(
                                              height: 10.s,
                                            ),
                                            Container(
                                                width: 320.s,
                                                alignment: Alignment.center,
                                                child: Obx(
                                                  () => FISCheckBox(
                                                    Storage.isRemeberPassword
                                                        .value,
                                                    (value) {
                                                      Storage
                                                          .updateIsRemeberPassword(
                                                              value ?? false);
                                                    },
                                                    label: fisI18nBook
                                                        .common.remeberPwd.t,
                                                  ),
                                                )),
                                            SizedBox(
                                              height: 10.s,
                                            )
                                          ],
                                        ),
                                ),
                                Obx(
                                  () => FISLinearGradientElevatedButtonAsync(
                                    ((!controller.isLoginWithVerificationCode.value &&
                                                    controller.userName
                                                        .value.isNotNullOrEmpty &&
                                                    controller.password.value
                                                        .isNotNullOrEmpty) ||
                                                (controller.isLoginWithVerificationCode
                                                        .value &&
                                                    controller
                                                        .verifyCodeAccount
                                                        .value
                                                        .isNotNullOrEmpty &&
                                                    controller.verifyCode.value
                                                        .isNotNullOrEmpty)) &&
                                            controller.loginBtnText.value !=
                                                i18nBook.auth.logining.t
                                        ? () async {
                                            await controller.loginAsync();
                                          }
                                        : null,
                                    controller.loginBtnText.value,
                                    Storage
                                        .currentTheme
                                        .elevatedButtonLinearGradientStyle
                                        .elevatedButtonLinearGradient,
                                    width: 320.s,
                                    height: 50.s,
                                    textStyle: Storage
                                        .currentTheme.normalTextStyle
                                        .copyWith(
                                            color: Storage
                                                .currentTheme
                                                .elevatedButtonLinearGradientStyle
                                                .textColor,
                                            fontSize: Storage.currentTheme
                                                .normalTextStyle.fontSize!.s),
                                  ),
                                ),
                              ],
                            )
                          : const SizedBox()),
                    ],
                  )),
              Storage.isPortraitMode.value
                  ? Positioned(
                      top: 0,
                      right: 0,
                      child: Container(
                        color: Colors.transparent,
                        width: 64.s,
                        height: 64.s,
                        child: GestureDetector(
                          onTap: () {
                            controller.isQrcode.value =
                                !controller.isQrcode.value;
                          },
                          child: AnimatedContainer(
                            duration: const Duration(milliseconds: 200),
                            child: CustomPaint(
                              painter: DiagonalPainter(Colors.white),
                              child: Align(
                                alignment: Alignment.topRight,
                                child: Obx(() => Icon(
                                      controller.isQrcode.value
                                          ? Icons.desktop_windows_rounded
                                          : Icons.qr_code,
                                      size: 35.s,
                                      color: Storage.currentTheme
                                          .loginContainStyle.backgroundColor,
                                    )),
                              ),
                            ),
                          ),
                        ),
                      ),
                    )
                  : const SizedBox(),
            ],
          )),
          Positioned(
              top: 0,
              right: 0,
              child: Obx(() => Storage.isShowCloseButton.value
                  ? FISIconButton(
                      onPressed: () {
                        Get.dialog(const ExitOrHideConfirmDialog(),
                            barrierDismissible: false);
                      },
                      icon: const Icon(
                        Icons.close,
                      ),
                      iconSize: 56.s,
                      padding: EdgeInsets.only(top: 10.s),
                    )
                  : const SizedBox())),
        ],
      );
    });
  }

  Widget _buildTabBar() {
    final tabs = controller.tabTitles
        .map((e) => SizedBox(
            height: 48.s,
            child: Center(
                child: FISText(
              e,
              textAlign: TextAlign.center,
            ))))
        .toList();
    return SizedBox(
        width: 330.s,
        child: TabBar(
          indicatorColor: Storage.currentTheme.themeData.indicatorColor,
          onTap: controller.onTabChanged,
          tabs: tabs,
          labelPadding: EdgeInsets.symmetric(horizontal: 16.s),
          indicatorWeight: 2.s,
          controller: controller.tabController,
        ));
  }

  Widget _buildQrView() {
    return Stack(
      children: [
        Container(
          width: 140.s,
          height: 140.s,
          color: Colors.white,
          padding: EdgeInsets.all(10.s),
          child: Obx(() => QrImageView(
                data: "${controller.qrCode.value}&",
                size: 120.s,
              )),
        ),
        Obx(() => controller.qrCodeExpired.value
            ? Positioned(
                left: 0,
                top: 0,
                bottom: 0,
                right: 0,
                child: MouseRegion(
                  cursor: SystemMouseCursors.click,
                  child: GestureDetector(
                    onTap: () async {
                      await controller.startCheckQrcodeExpiredAsync();
                    },
                    child: Container(
                      decoration: BoxDecoration(
                        color: Colors.black.withOpacity(0.5),
                      ),
                      child: Center(
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.center,
                          children: [
                            Icon(
                              Icons.refresh,
                              size: 30.s,
                              color: Colors.white,
                            ),
                            SizedBox(height: 8.s),
                            FISText(
                              i18nBook.common.refreshQRCode.t,
                              style: TextStyle(
                                  color: Colors.white,
                                  fontSize: Storage.currentTheme.normalTextStyle
                                      .fontSize!.s),
                            ),
                          ],
                        ),
                      ),
                    ),
                  ),
                ),
              )
            : const SizedBox()),
      ],
    );
  }

  Future<void> onSubmitted(String p1) async {
    if (((!controller.isLoginWithVerificationCode.value &&
                controller.userName.value.isNotNullOrEmpty &&
                controller.password.value.isNotNullOrEmpty) ||
            (controller.isLoginWithVerificationCode.value &&
                controller.verifyCodeAccount.value.isNotNullOrEmpty &&
                controller.verifyCode.value.isNotNullOrEmpty)) &&
        controller.loginBtnText.value != i18nBook.auth.logining.t) {
      try {
        if (Storage.platform == Platform.Windows &&
            Storage.isShowSoftKeyBoard) {
          Storage.isShowSoftKeyBoard = false;
          await jsonRpcProxyForFISLib.consultation.operateSoftKeyBoard(
              FISControlPositionInfo(isShowSoftwareKeyBoard: false));
        }
      } catch (ex) {
        await loggerAsyn(
            FISDeviceLogCategory.Error, "OperateSoftKeyBoard error: $ex");
      }
      await controller.loginAsync();
    }
  }
}
