import 'dart:async';
import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:flyinsonolite/controls/custom/customelevatedbutton.dart';
import 'package:flyinsonolite/controls/text/fistext.dart';
import 'package:flyinsonolite/controls/text/fistextfield.dart';
import 'package:flyinsonolite/infrastructure/scale.dart';
import 'package:flyinsonolite/infrastructure/utils.dart';
import 'package:flyinsonolite/login/controllers/verificationcodefieldcontroller.dart';
import 'package:flyinsonolite/infrastructure/storage.dart';
import 'package:get/get.dart';

class VerificationCodeField extends GetView<VerificationCodeFieldController> {
  double? verificationCodeWidth;
  FocusNode? focusNode;
  void Function(String)? onChanged;
  void Function(String)? onSubmitted;
  VerificationCodeField(
      {Key? key,
      required this.fetchCodeFunc,
      required this.verificationCodeTextController,
      required this.onChanged,
      this.focusNode,
      this.verificationCodeWidth,
      this.onSubmitted})
      : super(key: key);

  final Future<bool> Function() fetchCodeFunc;
  final TextEditingController verificationCodeTextController;

  @override
  Widget build(BuildContext context) {
    Get.lazyPut<VerificationCodeFieldController>(
        () => VerificationCodeFieldController());

    return Row(
      children: [
        SizedBox(
          width: verificationCodeWidth ?? 165.s,
          child: FISTextField(
            uniqueKey: GlobalKey(),
            placeHolder: i18nBook.auth.hint4VerificationCode.t,
            controller: verificationCodeTextController,
            onChanged: onChanged,
            onSubmitted: onSubmitted,
          ),
        ),
        SizedBox(
          width: 5.s,
        ),
        SizedBox(
            width: 135.s,
            height: 48.s,
            child: Obx(
              () => CustomElevatedButton(
                  onPressed: controller.isBtnEnabled.value
                      ? () {
                          fetchCodeFunc().then((value) {
                            if (value) {
                              controller.startCountDown();
                            }
                          });
                        }
                      : null,
                  style: Storage
                      .currentTheme.themeData.elevatedButtonTheme.style!
                      .copyWith(
                    shape: MaterialStateProperty.all(RoundedRectangleBorder(
                      borderRadius: BorderRadius.circular(4.s),
                    )),
                    padding: kIsMobile
                        ? MaterialStateProperty.all(EdgeInsets.symmetric(
                            vertical: 5.s, horizontal: 20.s))
                        : MaterialStateProperty.all(EdgeInsets.symmetric(
                            vertical: 15.s, horizontal: 20.s)),
                  ),
                  child: Obx(
                    () => FISText(controller.btnText.value,
                        style: Storage.currentTheme.normalTextStyle.copyWith(
                            fontSize: Storage
                                .currentTheme.normalTextStyle.fontSize!.s,
                            color: Storage.currentTheme
                                .elevatedButtonLinearGradientStyle.textColor,
                            overflow: TextOverflow.ellipsis)),
                  )),
            ))
      ],
    );
  }
}
