import 'package:fis_jsonrpc/rpc.dart';
import 'package:flyinsonolite/consultation/records/models/appointmentinformation.dart';

abstract class IAppointmentManager {
  ///分页查询会诊记录
  Future<PageResult<ConsultationPageDTO>> findConsultationsByPageAsync(
    ConsultationQueryTypeEnum consultationQueryType,
    int pageIndex,
    int pageSize, {
    String? keyword,
    DateTime? startDate,
    DateTime? endDate,
    QueryConsultationStatusEnum? consultationStatus,
    QueryEvaluateGradeEnum? evaluateGrade,
    String? language,
    List<String>? expertCodes,
    List<String>? applyOrganizationCodes,
    List<String>? expertOrganizationCodes,
    String? patientSex,
    String? patientDiseases,
    String? patientPrimaryDiagnosis,
    QueryPatientAgeLimitDTO? patientAgeLimit,
  });

  Future<ConsultationDetailDTO?> findConsultationDetailAsync(
      String consultationCode);

  ///获取随访信息
  Future<List<FollowUpVisitDTO>> getFollowUpVisitInfo(
      String currentConsultationCode);

  ///分页查询病人信息
  Future<PageResult<ClientPatientInfoBaseDTO>> findConsultationPatientAsync(
      String? keyword, int pageIndex, int pageSize);

  ///没有此病人时创建病人
  Future<ClientPatientInfoBaseDTO?> createPatientByUnregisteredAsync(
      String patientName);

  ///获取用户名下所有设备
  Future<PageCollection<DeviceInfoDTO>> getDeviceListByPersonRoleAsync();

  ///获取所有扫查医生
  Future<List<UserExtendDTO>> getScanDoctorsAsync();

  ///获取扫查部位
  Future<List<String>> getScanPositionsAsync();

  /// 获取会诊机构
  Future<List<OrganizationBaseDTO>> findParentOrganizationsAsync();

  ///获取会诊专家
  Future<PageResult<OrganizationExpertDTO>> findOrganizationExpertsAsync(
      String organizationCode);

  ///获取申请人列表
  Future<List<OrganizationExpertDTO>> findAssistantExpertsAsync();

  //创建会诊
  Future<String> applyConsultationAsync(
      String? patientCode,
      AppointmentInformation appointmentInformation,
      List<DataItemDTO>? patientDatas);

  //修改会诊
  Future<bool> updateConsultationAsync(
      String? consultationCode, AppointmentInformation appointmentInformation);

  //完善会诊， 急诊创建的会诊单需要完善，其他不需要
  Future<bool> improveConsultationInfoAsync(
      String? patientCode,
      String? consultationCode,
      AppointmentInformation appointmentInformation,
      List<DataItemDTO>? patientDatas);

  //撤回会诊
  Future<bool> withdrawConsultationAsync(String? consultationCode);

  //拒绝会诊
  Future<bool> refuseConsultationAsync(
      String? consultationCode, String? reason);

  ///接受/拒绝加入会诊
  Future<bool> acceptInvitationAsync(
      String? consultationCode, bool isAgree, String? reason);

  ///获取用户信息，用于获取助理医师信息
  Future<UserInfoByCodeDTO> getUserByCodeAsync(String? userCode);

  ///审批同意会诊申请
  Future<bool> approveConsultationAsync(
      ApproveConsultationInfo approveConsultationInfo);

  //会诊中添加时排除已存在的用户
  Future<List<UserExtendDTO>> getInviteableUserListAsyn(
      String consultationCode, String organizationCode);

  //获取会诊图像
  Future<List<ConsultationImagesDTO>> getConsultationImagesAsync(
      String consultationCode);

  ///获取病人数据
  List<DataItemDTO> getPatientDatas(Map<String, String> patient);
}
