import 'package:fis_jsonrpc/rpc.dart';

abstract class IReportManager {
  ///获取所有报告
  Future<List<ReportDTO>> findReportsAsync(String recordCode);

  ///获取默认词条
  Future<ThesaurusAllDTO> getDefaultThesaurusContentAsync();

  ///获取默认报告模板
  Future<ReportTemplateDTO?> getDefaultReportTemplateAsync();

  ///新增报告
  Future<String?> addReportAsync(String consultationCode, String templateJson,
      String name, String reportInfoJson);

  ///修改报告
  Future<bool> modifyReportAsync(
      String reportCode,
      DiagnosisOrganEnum reportOrgan,
      String consultationCode,
      String templateJson,
      String name,
      String reportInfoJson);

  ///跟据reportCode获取报告详情
  Future<ReportDTO> findReportByCodeAsync(String reportCode);

  /// 格式化时间
  ///
  /// [time] 后端返回的时间
  String getTime(String time);

  ///获取模板列表
  Future<PageResult<ReportTemplateDTO>> getReportTemplatePage({
    int pageIndex = 1,
    int pageSize = 100,
    String? keyword,
    bool? isDefault,
  });

  ///获取指定模板
  Future<ReportTemplateDTO> findReportTemplateByCode({String code = ''});

  ///获取词条库分页
  Future<List<ThesaurusDTO>> getThesaurusPage({String keyword = ''});

  ///获取词条项列表
  Future<List<ThesaurusItemDTO>> getThesaurusItems({
    String? thesaurusCode,
    String? keyword,
  });
}
