import 'package:flutter/material.dart';
import 'package:flyinsonolite/themes/customthemedata.dart';

const String _fontFamily = 'DroidSansFallback';

const _titleTextStyle = TextStyle(
    fontFamily: _fontFamily,
    fontSize: 22,
    color: Color.fromRGBO(211, 211, 211, 1));

const _normalTextStyle = TextStyle(
    fontFamily: _fontFamily,
    fontSize: 18,
    color: Color.fromRGBO(211, 211, 211, 1));
final _grayThemeData = ThemeData(
  fontFamily: _fontFamily,
  scaffoldBackgroundColor: const Color.fromRGBO(27, 30, 41, 1),
  textTheme: const TextTheme(bodyMedium: _normalTextStyle),
  tabBarTheme: const TabBarTheme(
    labelStyle: _normalTextStyle,
  ),
  elevatedButtonTheme: ElevatedButtonThemeData(
      style: ButtonStyle(
    textStyle: MaterialStateProperty.resolveWith<TextStyle>(
      (Set<MaterialState> states) {
        return _normalTextStyle.copyWith(fontSize: 16); // 默认状态下的颜色
      },
    ),
    backgroundColor: MaterialStateProperty.resolveWith<Color>(
      (Set<MaterialState> states) {
        if (states.contains(MaterialState.disabled)) {
          return Colors.grey;
        }
        return const Color.fromRGBO(50, 150, 250, 1); // 默认状态下的颜色
      },
    ),
  )),
  dividerColor: Colors.white,
  indicatorColor: const Color.fromRGBO(50, 150, 250, 1),
);

final _loginContainerStyle = LoginContainStyle(
    const Color.fromRGBO(47, 54, 71, 1), const Color.fromRGBO(66, 77, 95, 1));
final _elevatedButtonLinearGradientStylie = ElevatedButtonLinearGradientStyle(
    const LinearGradient(
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
        colors: [
          Color.fromRGBO(195, 195, 217, 1),
          Color.fromRGBO(55, 77, 152, 1),
          Color.fromRGBO(21, 59, 144, 1)
        ]),
    _normalTextStyle.color!);
final _squareIconButtonStyle = FISSquareIconButtonStyle(
    _normalTextStyle.color!,
    _normalTextStyle.color!,
    const LinearGradient(
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
        colors: [
          Color.fromRGBO(50, 150, 250, 1),
          Color.fromRGBO(50, 150, 250, 1),
          Color.fromRGBO(50, 150, 250, 1)
        ]),
    LinearGradient(
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
        colors: [
          _loginContainerStyle.backgroundColor,
          _loginContainerStyle.backgroundColor,
          _loginContainerStyle.backgroundColor,
        ]));
final _dropdownButtonStyle = DropdownButtonStyle(
    _normalTextStyle.color!, 20, const Color.fromRGBO(27, 29, 35, 1));
final _dialogStyle = DialogStyle(
  _grayThemeData.scaffoldBackgroundColor,
  TextStyle(color: _normalTextStyle.color, fontSize: 26),
  const LinearGradient(
      begin: Alignment.centerLeft,
      end: Alignment.centerRight,
      colors: [
        Color.fromRGBO(47, 54, 71, 1),
        Color.fromRGBO(47, 54, 71, 1),
        Color.fromRGBO(47, 54, 71, 1),
        Color.fromRGBO(47, 54, 71, 1),
        Color.fromRGBO(47, 54, 71, 1),
        Color.fromRGBO(47, 54, 71, 1),
        Color.fromRGBO(47, 54, 71, 1),
      ],
      stops: [
        0,
        0.2,
        0.4,
        0.6,
        0.8,
        0.9,
        1
      ]),
  _normalTextStyle,
  const LinearGradient(
      begin: Alignment.topCenter,
      end: Alignment.bottomCenter,
      colors: [
        Color.fromRGBO(195, 195, 217, 1),
        Color.fromRGBO(142, 140, 178, 1),
        Color.fromRGBO(117, 120, 173, 1),
        Color.fromRGBO(68, 86, 155, 1),
        Color.fromRGBO(21, 59, 144, 1),
      ],
      stops: [
        0,
        0.17,
        0.2,
        0.4,
        1
      ]),
);

final FISLabelInputFieldStyle _fisLabelInputFieldStyle =
    FISLabelInputFieldStyle(
  _normalTextStyle,
  _normalTextStyle,
  const Color.fromRGBO(73, 86, 102, 1),
  _loginContainerStyle.borderColor,
);
final ListViewStyle _listViewStyle = ListViewStyle(
    const Color.fromRGBO(33, 40, 57, 1),
    const Color.fromRGBO(50, 150, 250, 1),
    const Color.fromRGBO(62, 71, 92, 1));
final DatePickerStyle _datePickerStyle = DatePickerStyle(
    const Color.fromRGBO(39, 89, 180, 1), const Color.fromRGBO(27, 29, 35, 1));
final ListViewButtonItemStyle _listViewButtonItemStyle =
    ListViewButtonItemStyle(
        const LinearGradient(
            begin: Alignment.topCenter,
            end: Alignment.bottomCenter,
            colors: [
              Color.fromRGBO(42, 59, 85, 1),
              Color.fromRGBO(40, 61, 92, 1),
              Color.fromRGBO(44, 69, 106, 1),
              Color.fromRGBO(48, 78, 122, 1),
              Color.fromRGBO(34, 67, 115, 1),
              Color.fromRGBO(19, 54, 106, 1),
              Color.fromRGBO(17, 47, 91, 1),
              Color.fromRGBO(14, 38, 73, 1),
              Color.fromRGBO(11, 30, 58, 1),
            ],
            stops: [
              0,
              0.1,
              0.25,
              0.495,
              0.5,
              0.505,
              0.75,
              0.9,
              1
            ]),
        const LinearGradient(
            begin: Alignment.topCenter,
            end: Alignment.bottomCenter,
            colors: [
              Color.fromRGBO(40, 94, 153, 1),
              Color.fromRGBO(44, 72, 153, 1),
              Color.fromRGBO(51, 51, 118, 1),
              Color.fromRGBO(46, 67, 132, 1),
              Color.fromRGBO(39, 83, 145, 1),
              Color.fromRGBO(25, 75, 142, 1),
              Color.fromRGBO(11, 69, 140, 1),
              Color.fromRGBO(7, 80, 150, 1),
              Color.fromRGBO(2, 95, 162, 1),
              Color.fromRGBO(1, 99, 166, 1),
            ],
            stops: [
              0,
              0.1,
              0.25,
              0.4,
              0.495,
              0.5,
              0.505,
              0.75,
              0.9,
              1
            ]),
        _normalTextStyle.color!);

final CustomThemeData grayTheme = CustomThemeData(
    "gray",
    _fontFamily,
    _grayThemeData,
    _loginContainerStyle,
    _elevatedButtonLinearGradientStylie,
    _squareIconButtonStyle,
    _normalTextStyle.color!,
    _dropdownButtonStyle,
    const Color.fromRGBO(17, 17, 27, 1),
    _dialogStyle,
    _fisLabelInputFieldStyle,
    const Color.fromRGBO(17, 35, 79, 1),
    _listViewStyle,
    const Color.fromRGBO(221, 224, 242, 1),
    _datePickerStyle,
    _listViewButtonItemStyle,
    _titleTextStyle,
    _normalTextStyle);
