import 'package:flutter/material.dart';
import 'package:flyinsonolite/themes/customthemedata.dart';

const String _fontFamily = 'DroidSansFallback';

const _titleTextStyle = TextStyle(
    fontFamily: _fontFamily,
    fontSize: 22,
    color: Color.fromRGBO(211, 211, 211, 1));

const _normalTextStyle = TextStyle(
    fontFamily: _fontFamily,
    fontSize: 18,
    color: Color.fromRGBO(211, 211, 211, 1));

final _purpleThemeData = ThemeData(
  fontFamily: _fontFamily,
  scaffoldBackgroundColor: const Color.fromRGBO(31, 32, 52, 1),
  textTheme: const TextTheme(bodyMedium: _normalTextStyle),
  tabBarTheme: const TabBarTheme(
    labelStyle: _normalTextStyle,
  ),
  elevatedButtonTheme: ElevatedButtonThemeData(
      style: ButtonStyle(
    textStyle: MaterialStateProperty.resolveWith<TextStyle>(
      (Set<MaterialState> states) {
        return _normalTextStyle.copyWith(fontSize: 16); // 默认状态下的颜色
      },
    ),
    backgroundColor: MaterialStateProperty.resolveWith<Color>(
      (Set<MaterialState> states) {
        if (states.contains(MaterialState.disabled)) {
          return Colors.grey;
        }
        return const Color.fromRGBO(151, 141, 211, 1); // 默认状态下的颜色
      },
    ),
  )),
  indicatorColor: const Color.fromRGBO(151, 141, 211, 1),
);

final _loginContainerStyle = LoginContainStyle(
    const Color.fromRGBO(74, 77, 110, 1),
    const Color.fromRGBO(121, 130, 168, 1));
final _elevatedButtonLinearGradientStylie = ElevatedButtonLinearGradientStyle(
  const LinearGradient(
      begin: Alignment.topCenter,
      end: Alignment.bottomCenter,
      colors: [
        Color.fromRGBO(137, 150, 242, 1),
        Color.fromRGBO(178, 187, 246, 1),
        Color.fromRGBO(223, 227, 253, 1)
      ]),
  const Color.fromRGBO(52, 52, 63, 1),
);
final _squareIconButtonStyle = FISSquareIconButtonStyle(
    const Color.fromRGBO(221, 221, 221, 1),
    const Color.fromRGBO(226, 199, 181, 1),
    const LinearGradient(
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
        colors: [
          Color.fromRGBO(131, 135, 195, 1),
          Color.fromRGBO(141, 138, 203, 1),
          Color.fromRGBO(151, 141, 211, 1)
        ]),
    LinearGradient(
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
        colors: [
          _loginContainerStyle.backgroundColor,
          _loginContainerStyle.backgroundColor,
          _loginContainerStyle.backgroundColor,
        ]));
final _dropdownButtonStyle = DropdownButtonStyle(
    _normalTextStyle.color!, 20, const Color.fromRGBO(31, 32, 52, 1));
final _dialogStyle = DialogStyle(
    _purpleThemeData.scaffoldBackgroundColor,
    TextStyle(color: _normalTextStyle.color!, fontSize: 26),
    const LinearGradient(
        begin: Alignment.centerLeft,
        end: Alignment.centerRight,
        colors: [
          Color.fromRGBO(47, 54, 71, 1),
          Color.fromRGBO(47, 54, 71, 1),
          Color.fromRGBO(47, 54, 71, 1),
          Color.fromRGBO(47, 54, 71, 1),
          Color.fromRGBO(47, 54, 71, 1),
          Color.fromRGBO(47, 54, 71, 1),
          Color.fromRGBO(47, 54, 71, 1),
        ],
        stops: [
          0,
          0.2,
          0.4,
          0.6,
          0.8,
          0.9,
          1
        ]),
    _normalTextStyle,
    const LinearGradient(
        begin: Alignment.topCenter,
        end: Alignment.bottomCenter,
        colors: [
          Color.fromRGBO(85, 87, 174, 1),
          Color.fromRGBO(71, 70, 119, 1),
          Color.fromRGBO(64, 61, 90, 1),
          Color.fromRGBO(58, 55, 84, 1),
          Color.fromRGBO(44, 41, 70, 1),
        ],
        stops: [
          0,
          0.15,
          0.2,
          0.4,
          1
        ]));

final FISLabelInputFieldStyle _fisLabelInputFieldStyle =
    FISLabelInputFieldStyle(
        _normalTextStyle.copyWith(
            color: const Color.fromRGBO(179, 179, 179, 1)),
        _normalTextStyle,
        const Color.fromRGBO(89, 92, 118, 1),
        _loginContainerStyle.borderColor);
final ListViewStyle _listViewStyle = ListViewStyle(
    const Color.fromRGBO(63, 67, 96, 1),
    const Color.fromRGBO(237, 203, 178, 1),
    const Color.fromRGBO(78, 82, 107, 1));
final DatePickerStyle _datePickerStyle = DatePickerStyle(
    const Color.fromRGBO(63, 67, 96, 1), const Color.fromRGBO(17, 17, 28, 1));
final ListViewButtonItemStyle _listViewButtonItemStyle =
    ListViewButtonItemStyle(
        const LinearGradient(
            begin: Alignment.topCenter,
            end: Alignment.bottomCenter,
            colors: [
              Color.fromRGBO(80, 82, 155, 1),
              Color.fromRGBO(76, 76, 135, 1),
              Color.fromRGBO(66, 65, 96, 1),
              Color.fromRGBO(63, 61, 90, 1),
              Color.fromRGBO(57, 55, 84, 1),
              Color.fromRGBO(51, 48, 77, 1),
              Color.fromRGBO(45, 42, 71, 1),
            ],
            stops: [
              0,
              0.05,
              0.1,
              0.25,
              0.5,
              0.75,
              1
            ]),
        const LinearGradient(
            begin: Alignment.topCenter,
            end: Alignment.bottomCenter,
            colors: [
              Color.fromRGBO(131, 150, 242, 1),
              Color.fromRGBO(125, 145, 235, 1),
              Color.fromRGBO(120, 140, 230, 1),
              Color.fromRGBO(115, 135, 225, 1),
              Color.fromRGBO(110, 130, 220, 1),
              Color.fromRGBO(105, 125, 215, 1),
              Color.fromRGBO(100, 120, 210, 1),
            ],
            stops: [
              0,
              0.05,
              0.1,
              0.25,
              0.5,
              0.75,
              1
            ]),
        _normalTextStyle.color!);

final CustomThemeData purpleTheme = CustomThemeData(
    "purple",
    _fontFamily,
    _purpleThemeData,
    _loginContainerStyle,
    _elevatedButtonLinearGradientStylie,
    _squareIconButtonStyle,
    _normalTextStyle.color!,
    _dropdownButtonStyle,
    const Color.fromRGBO(17, 17, 28, 1),
    _dialogStyle,
    _fisLabelInputFieldStyle,
    const Color.fromRGBO(63, 67, 96, 1),
    _listViewStyle,
    const Color.fromRGBO(221, 224, 242, 1),
    _datePickerStyle,
    _listViewButtonItemStyle,
    _titleTextStyle,
    _normalTextStyle);
